#!/usr/bin/env python
# -*- coding:utf-8 -*-
#############################################################################
#Copyright (c): 2020, Huawei Tech. Co., Ltd.
#FileName     : GSroachCommon.py
#Version      : V1.0.0
#Date         : 2020-01-20
#Description  : GSroachCommon.py provides common functions for backup and restore.
#############################################################################

try:
    import sys
    import os
    import commands
    import fcntl
    import string
    import signal
    import time
    import re
    from stat import ST_MODE
    import subprocess
    import json
    import glob
    import struct
    import socket
    from datetime import datetime
    sys.path.append(sys.path[0] + "/../")
    from gspylib.common.ErrorCode import ErrorCode
    from gspylib.common.GaussLog import GaussLog
    from gspylib.common.DbClusterInfo import dbClusterInfo, MASTER_INSTANCE, \
        STANDBY_INSTANCE, DUMMY_STANDBY_INSTANCE
    from gspylib.common.Common import DefaultValue, ClusterCommand
    from gspylib.common.OMCommand import OMCommand
    from gspylib.threads.SshTool import SshTool
    from gspylib.os.gsfile import g_file
    from gspylib.os.gsOSlib import g_OSlib
    from gspylib.common.DbClusterStatus import DbClusterStatus
    from gspylib.common.VersionInfo import VersionInfo
    import struct
    from util.GSroachUtils import *
except ImportError as err:
    sys.exit('Error: Unable to import module: ' + str(err))


## Global contants
GAUSS_ROACH_VERSION = "Roach V500R001C00"
GS_ROACH_CONF_NAME = "gs_roach.conf"  #used only in python, to overwrite cmdline param values
LOG_FILE_NAME = "roach-controller.log"
LOG_PATH = DefaultValue.getEnv("GAUSSLOG")
DEFAULT_LOG_PATH = "%s/roach/controller" %(LOG_PATH)
Current_Path = os.path.dirname(os.path.realpath(__file__))
LOCAL_ROACH = os.path.normpath(Current_Path + "/../local/LocalRoach.py")
SCRIPT_ROACH_LOCAL = os.path.normpath(Current_Path + "/GSroachLocal.py")
SCRIPT_IP_UTILITY = os.path.normpath(Current_Path + "/../local/GSroachChangeIPUtility.py")
DEF_BKP_KEY_SIZE = 15
TIMEOUT_PSSH_KILL_ROACH = 60        #1min to killall gs_roach
TIMEOUT_PSSH_PG_STOP = 1800         #30min to call pg_stop_backup(), we must wait it to normal
TIMEOUT_PSSH_POSTCLEAN = 60         #1min to clean some files except media dir
TIMEOUT_PSSH_PRECHECK = 300         #5min to check python process is runing
TIMEOUT_SCP = 60
pssh_tout = '600000'
VERSION_FILE = "upgrade_version"
ETCD_TERM_FILE = "etcd_term"
ETCD_TERM_FILE_TMP = "etcd_term.tmp"
STATIC_CONFIG_FILE = "cluster_static_config"
PITR_MARK_FILE = 'pitr_mark_file'
ACTION_CLEAN_METAMEDIAPATH = "clean_metamedia_path"
ACTION_BACKUP = "backup"
DEFAULT_SOURCE_CMD = 'source /etc/profile; source ~/.bashrc'
CONNECTTIMEOUT = 300
RETRYTIMES = 3
ACTION_CBMRECYCLE = "cbm_recycle"
ACTION_RENAME_TO_RCH = "rename_to_rch"


## Global variables
g_logger = None
command_dict = dict()
list_of_ip = dict()
final_ip_param_dict = dict()
proc_id_dict = dict()
proc_list_dict = dict()
ip_retCode_dict = dict()
ip_agntOpt_dict = dict()
ip_par_dict = dict()
remove_pattern = string.maketrans('', '',)
master_host = commands.getstatusoutput('hostname')[1]
poll_wait = False
master_ip =  DefaultValue.GetHostIpOrName()
gauss_home_loc = '' #Value set from getter method below
backup_key = ""
g_roach_op_fail = False
cn_hosts = ''
orig_gopts = None
cmd_dict = {}
time_out = False
force_stop = False
has_killall = False #if all gs_roach are killed
CBM_RECYCLE_LEVEL_0 = 0
CBM_RECYCLE_LEVEL_1 = 1
CBM_RECYCLE_LEVEL_2 = 2
SSH_TOOL_TIMEOUT = 1800

if DefaultValue.getEnv("HOST_IP") is not None:
    master_host = master_ip

def getGaussHomeFromClustInfo():
    """
    Return GaussHome avoiding use of env variable
    """
    if "USER" in os.environ and os.environ["USER"] is not None:
        gauss_user = DefaultValue.getEnv("USER")
    else:
        print 'ERROR: "USER" variable does not exists. Aborting...'
        sys.exit(1)

    l_clusterInfo = dbClusterInfo()
    try:
        l_clusterInfo.initFromStaticConfig(gauss_user)
    except Exception, e:
        #Cluster clean may have been done so will try to generate info from
        #BAK file! Sample exception string:
        #'[GAUSS-50201] : The static configuration file \
        #[/abc/gaussdb/bin/cluster_static_config] of \
        #designated user [abc] does not exist.'
        try:
            clustFileNameBak = str(e).split('[')[2].split(']')[0] + '_bak'
            l_clusterInfo.initFromStaticConfig(gauss_user, clustFileNameBak)
        except (IndexError, Exception) as e:
            #Not possible to get cluster static file name
            #Skipping failure here - if possible, it will work or fail later...
            pass
    return os.path.realpath(l_clusterInfo.appPath)

#Calling the getter method to set global gauss_home_loc
gauss_home_loc = getGaussHomeFromClustInfo()

def getMasterIpByHost(hostName):
    """
    Get masterip
    """
    global master_ip
    try:
        local_clusterinfo = dbClusterInfo()
        filename = "%s/bin/cluster_static_config" %(gauss_home_loc)
        filename_bak = "%s/bin/cluster_static_config_bak" %(gauss_home_loc)
        if "USER" in os.environ and os.environ["USER"] is not None:
            gauss_user = DefaultValue.getEnv("USER")
        else:
            print 'ERROR: "USER" variable does not exists. Aborting...'
            sys.exit(1)

        if os.path.isfile(filename):
            local_clusterinfo.initFromStaticConfig(gauss_user)
        elif os.path.isfile(filename_bak):
            local_clusterinfo.initFromStaticConfig(gauss_user, filename_bak)
        elif commands.getoutput('hostname') == hostName:
            master_ip = commands.getstatusoutput('hostname i')[1].split()[0]
            return master_ip
        else:
            GaussLog.exitWithError("ERROR: IP Address not found for current local node")

        local_nodeinfo = local_clusterinfo.getDbNodeByName(hostName)
        if local_nodeinfo is None:
            GaussLog.exitWithError("ERROR: Not able to get local node info")
        elif local_nodeinfo.sshIps[0] is not None:
            master_ip = local_nodeinfo.sshIps[0]
        elif local_nodeinfo.backIps[0] is not None:
            master_ip = local_nodeinfo.backIps[0]
        return master_ip
    except Exception, err:
        GaussLog.exitWithError("ERROR: %s" % str(err))

def updateMaster_ip(g_opts, command_dict):
    """
    To update Master IP passed from command line
    """
    global master_ip
    if g_opts.masterIP != "":
        master_ip = g_opts.masterIP
        del(command_dict["--master-ip"])
    elif DefaultValue.getEnv("HOST_IP") is None:
        #we do not have to update master_ip in agent env
        master_ip = getMasterIpByHost(master_host)

def getMaster_ip():
    """
    To set master ip
    """
    global master_ip
    return master_ip    

def getRoachOpResult():
    """
    GaussRoach exit code
    """
    global g_roach_op_fail
    return g_roach_op_fail

def setRoachOpResult(result):
    """
    set GaussRoach exit code
    """
    global g_roach_op_fail
    g_roach_op_fail = result

#############################################################################
# Handle failure
#############################################################################
def sigint_handler(signum, frame):
    """
    Handle CTR + C signal
    """
    global force_stop

    print "Ctrl + c signal received."
    if signum == 15:
        force_stop = True
    sendSignal()
    GaussLog.exitWithError("ERROR : Aborting Controller subprocess...")


signal.signal(signal.SIGINT, sigint_handler)
signal.signal(signal.SIGTERM, sigint_handler)


def sendSignal():
    """
    Send ctrl + C signal
    Notice: when network card down, then ssh tool will fail to send command to that node!
    """
    global orig_gopts, force_stop, has_killall

    if (has_killall == True and force_stop == False):
        return

    g_logger.debug("sendSignal now, will kill all gs_roach, force stop is: %d" % force_stop)

    if force_stop:
        signal = 9
    else:
        signal = 2

    #Use SshTool to replace gs_ssh so that we can set timeout, while gs_ssh has default timeout 30min
    l_clusterInfo = dbClusterInfo()
    staticConfig = "%s/bin/cluster_static_config" % (gauss_home_loc)
    staticConfig_bak = "%s/bin/cluster_static_config_bak" % (gauss_home_loc)
    if (os.path.exists(staticConfig) == False):
        staticConfig = staticConfig_bak
    l_clusterInfo.initFromStaticConfigWithoutUser(staticConfig)
    l_sshTool = SshTool(l_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_KILL_ROACH)
    cluster_version = l_clusterInfo.getClusterVersion(staticConfig)

    #Donot merge two 'try' blocks, because both are needed even if one of them gets exception
    try:
        #kill all gs_roach on all nodes
        cmd = "ps ux |grep '%s/bin/gs_roach' |grep -v grep |awk '{print \$2}' |xargs -r kill -%d" \
            % (gauss_home_loc, signal)
        g_logger.debug("cmd to killall gs_roach is: %s" % cmd)
        if not isMiniaturizedDeployment(cluster_version):
            l_sshTool.executeCommand(cmd, "Kill all gs_roach")
        else:
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.debug("Failed to kill gs_roach, output: %s" % output)
        time.sleep(1)
        # prevent hang problem which multiple ctrl+c signal will cause
        print("Successfully send signal to kill all gs_roach.")
    except Exception, err:
        g_logger.debug("Warning: Failed to kill all gs_roach. Error: %s" % (str(err)))

    has_killall = True

    try:
        #call pg_stop_backup() and pg_disable_delay_ddl() to terminate kernel activity
        if (('backup' in [orig_gopts.action, orig_gopts.origAction]) and (not orig_gopts.resumeBkp)):
            cmd = "source %s && python %s -t %s -U %s -l %s" \
                % (orig_gopts.sourceFile, LOCAL_ROACH, 'stop', 
                orig_gopts.user, orig_gopts.logFile)
            g_logger.debug("Stopping the backup process at kernel side, cmd: %s" % cmd)
            if not isMiniaturizedDeployment(cluster_version):
                #reset sshTool to give another timeout value, in case of network card is down
                l_sshTool = SshTool(l_clusterInfo.getClusterNodeNames(), \
                    g_logger.logFile, TIMEOUT_PSSH_PG_STOP)
                l_sshTool.executeCommand(cmd, "stop pg backup", 
                    DefaultValue.SUCCESS, orig_gopts.nodeList)
            else:
                (status, output) = commands.getstatusoutput(cmd)
                if (status != 0):
                    g_logger.debug("Failed to stop kernel side process, output: %s" % output)
            g_logger.debug("Successfully stopped the backup process at kernel side")
    except Exception, err:
        g_logger.debug("Warning: Failed to call pg_stop_backup() or pg_disable_delay_ddl(). Error: %s" % (str(err)))


def signalAndExit(msg):
    """
    killall gs_roach, and exit with log & message
    """
    sendSignal()
    g_logger.logExit(msg)


#############################################################################
# Init log
#############################################################################
def gsinitLogger(g_opts):
    """
    Init logger
    """
    global g_logger
    if g_opts.loggingPath == "":
        g_opts.loggingPath = DEFAULT_LOG_PATH
    g_opts.logFile = createFilePath(g_opts.loggingPath, LOG_FILE_NAME)
    g_logger = GaussLog(g_opts.logFile)
    return g_logger


def parseMaster_generated_file(master_generated_file):
    """
    reading the roach_nodes.done file
    """
    del_key = ""
    file_roach_nodes = open(master_generated_file, 'r')
    lines = file_roach_nodes.readlines()
    file_roach_nodes.close()
    node_dict = dict()

    for line in lines:
        if line == '\n':
            lines.remove(line)
        else:
            res = {line.strip('\n').split(':')[0] : line.strip('\n').split(':')[1]}
            node_dict.update(res)
    for key in node_dict.keys():
        if key == 'BackupKey':
            del_key = key
        else:
            pass
    if del_key:
        global backup_key
        backup_key = node_dict[del_key]
        del node_dict[del_key]
    return (backup_key, node_dict)


##############################################################
#Parse roach conf
##############################################################
def create_dict(ip_details):
    """
    create ip details
    """
    ip_dict = dict()
    for l in ip_details:
        l = l.translate(remove_pattern, "[]").rstrip('\n')
        l = l.strip()
        if '=' in l and len(l) > 0:
            if len(l.split('=')[0].strip()) > 1024 or len(l.split('=')[1].strip()) > 1024:
                GaussLog.exitWithError("Error : invalid option specified in config file")
            if l.split('=')[0] not in ip_dict.keys():
                ip_dict.update({l.split('=')[0].strip(): l.split('=')[1].strip()})
        elif len(l.split(' ')) > 1:
            if l.split(' ')[0] not in ip_dict.keys():
                ip_dict.update({l.split(' ')[0].strip(): l.split(' ')[1].strip()})
        elif len(l) > 0:
            if l.split(' ')[0] not in ip_dict.keys():
                ip_dict.update({l.split(' ')[0].strip(): l.split(' ')[0].strip()})
    return ip_dict

def checkRoachConfigFile(g_opts):
    """
    Check gs_roach.conf file for gs_roach
    """

    if os.path.isdir(g_opts.roachConf):
        file_path = createFilePath(g_opts.roachConf, GS_ROACH_CONF_NAME)
    elif os.path.isfile(g_opts.roachConf):
        file_path = g_opts.roachConf
    else:
        GaussLog.exitWithError("Error : cannot access configuration file")
    try:
        #check for the file permission
        permission = os.stat(file_path)[ST_MODE]
        if oct(permission)[-3:] != '600':
            GaussLog.exitWithError("Error :Config file %s has invalid access rights." %(file_path))
    except Exception, e:
        GaussLog.exitWithError("Error :Config file %s has invalid access rights or not a proper config file, msg: %s" \
            %(file_path, str(e)))

    return file_path

def parseRoachConf(g_opts):
    """
    Parse Roach Conf file
    """

    # check gs_roach.conf file
    file_path = checkRoachConfigFile(g_opts)

    try:
        res = dict()
        file_roach_conf = open(file_path, 'r')
        file_lines = file_roach_conf.readlines()
        file_roach_conf.close()
        lines = [line for line in file_lines if line != '\n']
        for line in lines:
            if len(line) > 2049:
                GaussLog.exitWithError("Error : invalid option specified in config file")
        ipindex_list = list()
        counter = 0
        for idx, line in enumerate(lines):
            if line.startswith('['):
                counter = counter + 1
        if counter == 0:
            GaussLog.exitWithError("Error :No header([]) specified in the gs_roach.conf")

        for idx, line in enumerate(lines):
            line = line.strip()
            if not line.startswith('[') and not line.startswith('--'):
                lines[idx] = '--'+lines[idx]

            if line.startswith('['):
                ipindex_list.append(idx)

        if not ipindex_list:
            GaussLog.exitWithError("Node parameters not specified in gs_roach.conf")

        for i, node_ip in enumerate(ipindex_list):
            if i < len(ipindex_list)-1:
                ip_details = lines[node_ip+1 : ipindex_list[i+1]]
                ip_dict = create_dict(ip_details)
                if lines[node_ip].rstrip('\n').strip() not in res.keys():
                    res = {lines[node_ip].rstrip('\n').strip(): ip_dict}
                    list_of_ip.update(res)
            else:
                ip_details = lines[node_ip+1 : ]
                ip_dict = create_dict(ip_details)
                if lines[node_ip].rstrip('\n').strip() not in res.keys():
                    res = {lines[node_ip].rstrip('\n').strip(): ip_dict}
                    list_of_ip.update(res)
        for key in list_of_ip.keys():
            if key.startswith('['):
                l = key.translate(remove_pattern, "[]").strip()
                list_of_ip[l] = list_of_ip.pop(key)
        checkConfParams(list_of_ip)
    except Exception, e:
        file_roach_conf.close()
        GaussLog.exitWithError("Error: parse roach config file result: %s" % str(e))

def checkConfParams(list_of_ip):
    """
    check for restricted entries in gs_roach.conf
    """
    master_opts = ['--backup-key', '--tablename', '--clean', \
        '--media-type', '--master-port', '--metadata-file-wait-time', '--restore-configs', \
        '--restore-new-cluster', "--parallel-restore", "--prior-backup-key", "--restore-target-time"]
    for key in list_of_ip.keys():
        for param in master_opts:
            if param in list_of_ip[key].keys():
                GaussLog.exitWithError("Error :%s can only be specified from commandline" %param)


################################################################
# Consolidate parameters for the cluster nodes for gs_roach
################################################################
def makeRaochOptions(g_opts, make_ip, make_host, command_dict):
    """
    Prepare gs_roach parameters
    """

    #keepin the required parameters to be sent to gs_roach
    reqParams = ["--media-type", "--tablename", \
                 "--master-port", "--filesplit-size", "--backup-topology", \
                 "--dbname", "--username", "--restore-new-cluster", "--parallel-restore", \
                 "--getdata-waittime-afterthreshold", "--prior-backup-key", \
                 "--verbose", "--metadata-destination", "--pre-disk-space", \
                 "--backup-key", "--restore-buffer-threshold", \
                 "--media-destination", "--compression-level",
                 "--compression-type", "--max-backup-io-speed", "--local-reserve-file-count",
                 "--buffer-size", "--buffer-block-size", "--output-file", \
                 "--parallel-process", "--logging-level", "--logging-path", \
                 "-w", "-W", "-U", "-s", "-d", "--log-filecount", \
                 "--log-filesize", "--logging", "--enable-logging", "--failure-retry-count", \
                 "--cpu-relinquish-time", "--cpu-relinquish-size", \
                 "--max-memory-usage", "--retry-wait-time", "--agent-port", \
                 "--clean", "--continue", "--archive", \
                 "--resource-retry-count", "--contrib-config", "--create", \
                 "--validate-prior-backups", "--no-display", "--overwrite", \
                 "--cascade", "--related-backup-keys", "--restore-configs", \
                 "--restore-target-time", "--table-list", "--target-backup-key", \
                 "--cluster-unique-id", "--obs-server-ip", "--bucket-name", \
                 "--reader-thread-count", "--reader-thread-file-count", "--logical",\
                 "--reader-thread-file-size", "--generate-force", "--validation-type", \
                 "--remote-master-ip", "--remote-media-destination", \
                 "--remote-metadata-destination", "--remote-host-username", \
                 "--remote-host-gausshome",  "--pitr", "--single", \
                 "--archive-xlog-buffer-path", "--archive-xlog-buffer-size", \
                 "--archive-media-type", "--archive-media-destination", \
                 "--resume-backup", "--resume-restore", \
                 "--cluster-migration", "--remote-host-gphome", \
                 "--single-group-nodeid", "--single-group-datapath",
				 "--split-local-disk-storage", "--skip-restore-gtmcm", "--remote-host-gausslog"]
    gOptParamDest = {"--media-type":"mediaType", "--tablename":"tableName", \
                 "--master-port":"masterPort", "--filesplit-size":"filesplitSize", \
                 "--backup-topology":"backupTopology", "--dbname":"dbName", \
                 "--username":"userName", "--getdata-waittime-afterthreshold":"threshold", \
                 "--prior-backup-key":"priorBkpKey", "--metadata-destination":"metadataDestination", \
                 "--backup-key":"backupKey", "--max-backup-io-speed":"maxBackupSpeed",
                 "--local-reserve-file-count":"localReserveFileCount",
                 "--restore-buffer-threshold":"bufferPercent", "--media-destination":"mediaDest", \
                 "--compression-level":"compressionLevel", "--buffer-size":"bufferSize", \
                 "--compression-type":"compressionType", \
                 "--buffer-block-size":"bufferBlock", "--output-file":"outputFile", \
                 "--parallel-process":"parallelProcess", "--logging-level":"loggingLevel", \
                 "--logging-path":"loggingPath", "-U":"userName", "-d":"dbName", \
                 "--log-filecount":"logCount", "--log-filesize":"logSize", \
                 "--enable-logging":"enableLogging", "--failure-retry-count":"failureRetryCount", \
                 "--cpu-relinquish-time":"cpuRelqTim", "--cpu-relinquish-size":"cpuRelSize", \
                 "--max-memory-usage":"maxMemUsage", "--retry-wait-time":"retryWaitTime", \
                 "--agent-port":"agentPort", \
                 "--resource-retry-count":"resourceRetryCount", "--contrib-config":"contribConfig", \
                 "--validate-prior-backups":"validatePreviousBackup", "--archive":"archiveMode", \
                 "--restore-target-time":"restoreTargetTime", "--table-list":"tablelist", \
                 "--target-backup-key":"targetBackupKey", \
                 "--reader-thread-count":"readerthreadcount", \
                 "--reader-thread-file-count":"readerthreadfilecount", \
                 "--reader-thread-file-size":"readerthreadfilesize", "--validation-type":"validateType", \
                 "--remote-master-ip":"remoteMasterIp", "--remote-media-destination":"remoteMediaData", \
                 "--remote-metadata-destination":"remoteMetaData", \
                 "--remote-host-username":"remoteHostUserName", \
                     "--remote-host-gausshome": "remoteHostGaussHome",
                 "--remote-host-gphome":"remoteHostGPHome", \
                 "--archive-xlog-buffer-path":"archiveXlogBufferPath", \
                 "--archive-xlog-buffer-size":"archiveXlogBufferSize", \
                 "--archive-media-type":"archiveMediaType", \
                 "--archive-media-destination":"archiveMediaDestination", \
                 "--single-group-nodeid":"singleGroupNodeid", \
                 "--single-group-datapath": "singleGroupDatapath",
				 "--remote-host-gausslog":"remoteHostGaussLog", "-W":"password"}
    gOptBoolDest = {"--with-serverlog":"withServerlog", "--verbose":"verbose", "--logging":"logging", \
                 "--clean":"clean", "--pre-disk-space":"preDiskSpace", "--continue":"continueOpt", \
                 "--restore-new-cluster":"restoreNewCluster", "--restore-configs":"restoreConfigs", \
                 "--generate-force":"generateForce", "--related-backup-keys":"relatedBackupKeys", \
                 "--all-backups":"allBackups", "--no-display":"noDisplay", "--overwrite":"overwrite", \
                 "--cascade":"cascade", "--create":"create", "--logical":"logical", "--pitr":"pitr", \
                 "--resume-backup":"resumeBkp", "--resume-restore":"resumeRst", \
                 "--cluster-migration":"clusterMigration", "--single":"single",
				 "--split-local-disk-storage":"splitLocalDiskStorage", "--parallel-restore":"parallelRestore"}

    dictfilt = lambda x, y: dict([(i, x[i]) for i in x if i in set(y)])
    new_command_dict = dictfilt(command_dict, reqParams)

    param_map = {'-U':'--username', '-d':'--dbname'}
    param_map_key = param_map.keys()

    # consolidating short and long options mentioned
    for key in new_command_dict.keys():
        if key in param_map_key:
            new_command_dict[param_map[key]] = new_command_dict.pop(key)
    # creating individual node parameters
    common_params = dict()
    if g_opts.roachConf:
        if 'common'in list_of_ip.keys():
            common_params = list_of_ip['common'].copy()

    common_params.update(new_command_dict)

    #creating final dictionary with ip list and corresponding command
    temp_final_ip_param_dict = dict()
    if  make_ip in list_of_ip.keys():
        temp_common_params = common_params.copy()
        temp_common_params.update(list_of_ip[make_ip])
        temp_final_ip_param_dict[make_ip] = temp_common_params
        final_ip_param_dict.update(temp_final_ip_param_dict)
    elif make_host in list_of_ip.keys():
        temp_common_params = common_params.copy()
        temp_common_params.update(list_of_ip[make_host])
        temp_final_ip_param_dict[make_host] = temp_common_params
        final_ip_param_dict.update(temp_final_ip_param_dict)
    else:
        final_ip_param_dict[make_ip] = common_params.copy()

    #changing the value for boolean parameter
    changeBooleanParam()
    #set final logging params
    setlogParams(g_opts)

    if g_opts.pitr and g_opts.action == "backup":
        #check the value for archive parameter
        checkArchiveParamDefaultValue(g_opts)

    # keeping only the required parameters
    for key in final_ip_param_dict.keys():
        final_ip_param_dict[key] = dictfilt(final_ip_param_dict[key], reqParams)
        if key != master_ip:
            continue
        #Update the Master options into g_opts and command_dict
        for k,v in final_ip_param_dict[key].iteritems():
            if k in gOptParamDest.keys():
                g_opts.__dict__[gOptParamDest[k]] = v
            if k in gOptBoolDest.keys():
                g_opts.__dict__[gOptBoolDest[k]] = True
            if k not in command_dict.keys():
                command_dict[k] = v
    filterPythonOnlyParams(g_opts)
    return final_ip_param_dict

def dropUnwantedParams(l_opts):
    """
    conditionally drop parameters which are problematic for current operation
    """
    global final_ip_param_dict
    for key in final_ip_param_dict.keys():
        #unwanted options related to partial-DB or partition table
        if l_opts.action in ("validate", "delete"):
            for tempkey in final_ip_param_dict[key].keys():
                if tempkey in ("--table-list", "--validation-type", "--restore-target-time"):
                    del final_ip_param_dict[key][tempkey]

def changeBooleanParam():
    """
    change boolean parameter merge short and long options
    """
    param_list = ["--verbose", "--logging", "--clean", "--pre-disk-space", "--logical", "--parallel-restore", \
                  "--restore-new-cluster", "--continue", "--restore-configs", "--generate-force", "--create", \
                  "--related-backup-keys", "--all-backups", "--no-display", "--overwrite", "--cascade", "--pitr", \
                  "--resume-backup", "--resume-restore", "--cluster-migration", "--single", "--split-local-disk-storage"]
    global final_ip_param_dict
    for key in final_ip_param_dict.keys():
        for k, v in final_ip_param_dict[key].items():
            if (k in param_list):
                if v == None or v == '' or v.lower() == 'true':
                    final_ip_param_dict[key][k] = None
                elif v.lower() == 'false':
                    del final_ip_param_dict[key][k]

def checkArchiveParamDefaultValue(g_opts):
    """
    if final_ip_param_dict does not have archive param, add default value.
    """
    global final_ip_param_dict
    for key in final_ip_param_dict.keys():
        final_ip_param_keys = final_ip_param_dict[key].keys()
        if "--archive-media-destination" not in final_ip_param_keys:
            final_ip_param_dict[key]["--archive-media-destination"] = g_opts.archiveMediaDestination
        if "--archive-xlog-buffer-size" not in final_ip_param_keys:
            final_ip_param_dict[key]["--archive-xlog-buffer-size"] = g_opts.archiveXlogBufferSize

def filterPythonOnlyParams(g_opts):
    """
    filter out Python-only parameters and set into g_opts
    """
    param_dest = {"--validate-prior-backups":"validatePreviousBackup", "--archive":"archiveMode"}
    bool_dest = {'--generate-force': 'generateForce'}
    param_keys = param_dest.keys()
    bool_keys = bool_dest.keys()
    for key in final_ip_param_dict.keys():
        for item in param_keys:
            if item in final_ip_param_dict[key].keys():
                val = final_ip_param_dict[key].pop(item) #remove from final
                #Update the removed value from Master into g_opts
                if key == master_ip:
                    g_opts.__dict__[param_dest[item]] = val
                    if item not in command_dict.keys():
                        command_dict[item] = val
        for item in bool_keys:
            if item in final_ip_param_dict[key].keys():
                final_ip_param_dict[key].pop(item) #remove from final
                #Update the removed value from Master into g_opts
                if key == master_ip:
                    g_opts.__dict__[bool_dest[item]] = True
                    if item not in command_dict.keys():
                        command_dict[item] = ''

def setlogParams(g_opts):
    """
    # assigning the final value for logging parametes
    """
    global master_ip
    for key in final_ip_param_dict.keys():
        if key == master_ip:
            for k in final_ip_param_dict[key].keys():
                if k == '--logging-path':
                    if final_ip_param_dict[key][k] == '--logging-path':
                        GaussLog.exitWithError("No value specified for --logging-path in conf file")
                    else:
                        g_opts.loggingPath = final_ip_param_dict[key][k]
                if k == '--logging-level':
                    g_opts.loggingLevel = final_ip_param_dict[key][k]
                if k == '--log-filesize':
                    g_opts.logSize = final_ip_param_dict[key][k]
                if k == '--log-filecount':
                    g_opts.logCount = final_ip_param_dict[key][k]
                if k == '--logging':
                    g_opts.logging = True
    return g_opts


def validate_logparams():
    """
    validating log parameters
    """
    try:
        for key in final_ip_param_dict.keys():
            for k, v in final_ip_param_dict[key].items():
                if k == '--logging':
                    if not v == None:
                        GaussLog.exitWithError("Error : Invalid value for the --logging")
                if k == '--log-filesize':
                    if int(final_ip_param_dict[key][k]) < 5 or int(final_ip_param_dict[key][k]) > 20:
                        GaussLog.exitWithError("Error : Invalid value for the --log-filesize, range is [5~20]")
                if k == '--log-filecount':
                    if int(final_ip_param_dict[key][k]) < 5 or int(final_ip_param_dict[key][k]) > 1024:
                        GaussLog.exitWithError("Error : Invalid value for the --log-filecount, range is [5~1024]")
                if k == '--logging-level':
                    if not (v.lower() in ['error', 'fatal', 'warning', 'info', 'log', 'debug', 'debug2']):
                        GaussLog.exitWithError("Error : Invalid value for the --logging-level")
    except Exception, e:
        GaussLog.exitWithError("Error:!! INVALID ROACH PARAMETERS SPECIFIED!! %s" % str(e))

def create_command(g_opts):
    """
    create command for master and agent
    """
    global ip_par_dict
    ip_par_dict = final_ip_param_dict.copy()
    createPattern()
    final_value = ''
    for k in final_ip_param_dict.keys():
        for key, val in final_ip_param_dict[k].iteritems():
            if key == '--restore-target-time' and not g_opts.pitr:
                continue
            if val != None:
                if key == '--restore-target-time':
                    new_value = ''.join("%s \'\"%s\"\' " % (key, val))
                else:
                    new_value = ''.join("%s %r " % (key, val))
            else:
                new_value = ''.join("%s " % (key))
            final_value += new_value
        final_ip_param_dict[k] = final_value
        final_value = ""

def createPattern():
    """
    create command pattern
    """
    param_list = ["--media-type", "--tablename", "--dbname", "--username", "--metadata-destination", \
                  "--backup-key", "--prior-backup-key", "--media-destination", "--logging-path"]
    for k in final_ip_param_dict.keys():
        for key, val in final_ip_param_dict[k].items():
            if k != master_ip:
                if (key in param_list):
                    final_ip_param_dict[k][key] = '\"%s\"' % val

def checkStartingPitrArchive(g_opts):
    """
    check whether exist pitr mark file in pitr mode.
    """
    gauss_home_location = DefaultValue.getEnv("GAUSSHOME")
    if g_opts.pitr:
        pitr_mark_file = "%s/bin/pitr_mark_file" % gauss_home_location
        if os.path.exists(pitr_mark_file):
            return True
    return False

def checkPitrStopInProgress(g_opts):
    """
    check if pitr-stop in progress, to avoid switchover/failover check failed during stop.
    """
    gauss_home_location = DefaultValue.getEnv("GAUSSHOME")
    if g_opts.pitr:
        pitr_stop_label = "%s/bin/pitr_stop_label" % gauss_home_location
        if os.path.exists(pitr_stop_label):
            return True
    return False

def getClusterBalanceState(g_opts, g_logger):
    """
    get cluster balance state
    if sucessful, return value is "Yes" or "No"
    if error occurs, return value is "", i.e. empty string
    """
    g_logger.debug("Get the cluster balance status.")

    try:
        gaussHome = DefaultValue.getEnv("GAUSSHOME")
        tmpFilePath = os.path.join(gaussHome, "roach")
        cmd = 'source %s && if [ ! -d %s ]; then mkdir -p %s -m 700; fi' \
            % (g_opts.sourceFile, tmpFilePath, tmpFilePath)
        g_logger.debug("Creating $GAUSSHOME/roach path, cmd: %s" % cmd)
        (status, output) = commands.getstatusoutput(cmd)
        if(status != 0):
            g_logger.debug("Warning: Create $GAUSSHOME/roach path failed, cmd is %s, status = %d, output is %s" % (cmd, status, output))
            return ""

        tmpFilePath = os.path.join(gaussHome, "roach", "balance_tmp")
        cmd = "source %s;cm_ctl query > %s" % (g_opts.sourceFile, tmpFilePath)
        g_logger.debug("Command for checking cluster balance state: %s" % cmd)
        (status, output) = commands.getstatusoutput(cmd)
        if(status != 0):
            errorString = ErrorCode.GAUSS_534["GAUSS_53403"] + "status = %d, output = %s" % (status, output)
            agentErrorReport("roach_backup", errorString)
            return ""
        # get cluster instance
        clusterInfo = DbClusterStatus()
        clusterInfo.initFromFile(tmpFilePath)
        return clusterInfo.balanced
    except Exception, e:
        errorString = ErrorCode.GAUSS_534["GAUSS_53403"] + " error: %s." % str(e)
        agentErrorReport("roach_backup", errorString)
        return ""

def checkBalanceStateSame(g_opts, faileRetryTimes):
    """
    Check whether cluster is balance during poll gs_roach processes.
    In the future, we need to eliminate this restriction.
    """
    # check balance for resumeBkp or single cluster backup
    # we only kill gs_roach when we get explicit unbalanced state
    if ((g_opts.resumeBkp or (g_opts.remoteMasterIp == "" and not g_opts.mediaType.upper() == "OBS")) and (g_opts.tableName == "" and g_opts.tablelist == "") and getClusterBalanceState(g_opts, g_logger) == "No" ):
        # if pitr basebackup completed, no need to care switchover
        if g_opts.pitr and (checkStartingPitrArchive(g_opts) or checkPitrStopInProgress(g_opts)):
            faileRetryTimes = 0
        else:
            faileRetryTimes = faileRetryTimes + 1
            g_logger.debug("Warning: cluster state has become unbalanced at %s faileRetryTimes." % (str(faileRetryTimes)))

        if faileRetryTimes >= int(g_opts.checkBalanceRetryTimes):
            if g_opts.single:
                errorString = ErrorCode.GAUSS_534["GAUSS_53403"] + "backup cannot continue because single-mode cluster Switchover/Failover, pitr Basebackup Stop, SendSignal to make process quit!"
                agentErrorReport("roach_backup", errorString)
                sendSignal()
            else:
                errorString = ErrorCode.GAUSS_534["GAUSS_53403"] + "backup cannot continue because cluster state has become unbalanced, resume backup or single cluster backup cannot continue. SendSignal to make process quit!"
                agentErrorReport("roach_backup", errorString)
                sendSignal()

            if g_opts.pitr:
                g_logger.log("unset Pitr Crontab for %s." % g_opts.action)
                gaussHome = DefaultValue.getEnv("GAUSSHOME")
                mpprcFile = g_opts.sourceFile
                hostName = socket.gethostname()
                status = unsetRoachCron("backup", gaussHome, hostName, mpprcFile, "GaussRoach.py", g_opts.logFile)
                if (status != 0):
                    g_logger.logExit("Stop backup failed failed.")
                g_logger.log("unset Pitr Crontab successfully.") 
    else:
        faileRetryTimes = 0
    return faileRetryTimes

###############################################################################
# Polling
###############################################################################

def pollSubprocess(proc_list_dict, g_opts):
    """
    poll the created subprocess
    """
    global ip_retCode_dict
    global ip_agntOpt_dict
    global has_killall #if all gs_roach are killed

    g_logger.debug("[pollSubprocess enter] TotalActive[%d], RetCodes: %s" \
        % (len(proc_list_dict), str(ip_retCode_dict)))
    
    if len(proc_list_dict) == 0:
        g_logger.debug("pollSubprocess: no process in list")
        global poll_wait
        poll_wait = False
        return
    else:
        lPopItems = []
        for node_ip, sProc in proc_list_dict.iteritems():
            if sProc.poll() == None:
                #subprocess are running ,move to next subprocess
                continue
            else:
                if sProc.poll() != 0:
                    g_logger.debug("[pollSubprocess ret code abnormal] TotalActive[%d], RetCodes: %s" \
                                    % (len(proc_list_dict), str(ip_retCode_dict)))
                    #Some processes already failed then make others also exit!
                    if (has_killall == False):
                        g_logger.debug("pollSubprocess: node %s pid %d process failed return code %d, sendSignal to make others quit!"\
                                       % (node_ip, sProc.pid, sProc.returncode))
                        sendSignal()
                        has_killall = True
                    ip_retCode_dict.update({node_ip:sProc.poll()})
                    if node_ip != master_ip:
                        ip_agntOpt_dict.update({node_ip:sProc.stdout.readlines()})
            lPopItems.append(node_ip)
        for item in lPopItems:
            g_logger.debug("[pollSubprocess] Popped [%s]" % item)
            proc_list_dict.pop(item)
        if len(ip_retCode_dict) > 0 and len(proc_list_dict) == 0:
            handleFailure(ip_retCode_dict, ip_agntOpt_dict, g_opts)

def handleFailure(ip_retCode_dict, ip_agntOpt_dict, g_opts):
    """
    Handle Failure for subprocess
    """
    global g_roach_op_fail
    g_logger.debug("Handling the failure subprocess")
    g_roach_op_fail = True

    if ('backup' in [orig_gopts.action, orig_gopts.origAction]):
        if (g_opts.resumeBkp):
            g_logger.error("Error: now will rollback the backup because of agent error!")
        else:
            if (orig_gopts.origAction == 'backup' and orig_gopts.action == 'validate'):
                g_logger.error("Error: validate failed after an completed backup, please check backup")
            else:    
                performCleanUp()

    for node_ip, retCod in ip_retCode_dict.iteritems():
        g_logger.debug("Error: The return code for failed subprocess on %s is %s" %(node_ip, retCod))
    if len(ip_agntOpt_dict) > 0:
        for node_ip, agt_opt in ip_agntOpt_dict.iteritems():
            myopt = ""
            g_logger.debug("Output at agent %s:" %(node_ip))
            g_logger.debug(agt_opt)
            for line in agt_opt:
                try:
                    split_pattern = '%s: ' %(node_ip)
                    if split_pattern in line:
                        agent_op = line.split(split_pattern)[1].replace('\n', '')
                        myopt = myopt + agent_op
                except Exception, e:
                    g_logger.debug(e)
            g_logger.debug("Error at agent %s:" %(node_ip))
            g_logger.debug(myopt)

def performCleanUp():
    """
    perform clean up in failure case
    """
    g_logger.error("Error: now will rollback the backup because of agent error!")
    g_logger.debug("Perform cleanup of meta and media dir, this may need very long time.")

    #Generate a dict of dirs for cleanup
    clean_list = ['--media-destination', '--metadata-destination']
    path_info = {} #Key=IP, Value=list of media/meta dest's
    for item in clean_list:
        for node_ip in ip_par_dict.keys():
            tmp = getBackupPath(ip_par_dict[node_ip][str(item)], backup_key)
            if not path_info.get(node_ip):
                new_item = [tmp]
            else:
                new_item = path_info[node_ip] + [tmp]
            path_info[node_ip] = new_item

    #Delete meta and media destination dirs on all agents
    for node_ip in ip_par_dict.keys():
        if node_ip == master_ip:
            rm_cmd = ""
            for item in path_info[node_ip]:
                rm_cmd += '\"%s\" ' % (item)
            cmd = 'rm -rf %s' %(rm_cmd)

            #####################################################################
            #                  !! IMPORTANT SECURITY NOTE !!                    #
            # Regarding usage of subprocess.Popen([cmd], shell=True, bufsize=-1)#
            #  1.All the inputs from user are captured as individual options    #
            #  2.Direct inputs from the user are not passed in subprocess to    #
            #    gs_roach.                                                      #
            #  3.All the options are validated for the type as well as range    #
            #  4.The commands to be launched through subprocess are formed in   #
            #    code and passed as string to shell                             #
            #####################################################################

            g_logger.debug("deleting file in performCleanUp, cmd: %s" % cmd)
            subprocess.Popen([cmd], shell=True, bufsize=-1)
        else:
            rm_cmd = ""
            for item in path_info[node_ip]:
                rm_cmd += 'cd \\"%s\\"; rm -rf %s;' % (os.path.dirname(item), backup_key)
            cmd = 'pssh -t %s -H \"%s\" -P "%s"' % (pssh_tout, node_ip, rm_cmd)
            g_logger.debug("performCleanUp before pssh rm -rf, cmd=%s." %(cmd))
            subprocess.Popen([cmd], stdout=subprocess.PIPE, \
                         stderr=subprocess.PIPE, shell=True, bufsize=-1)
            g_logger.debug("performCleanUp after pssh rm -rf.")
    g_logger.debug("Perform cleanup of meta and media dir finished.")
    g_logger.log("Cleanup completed.")

def getBackupPath(path, key):
    """
    create backup path
    """
    if path.endswith('/'):
        backup_fil = '%sroach/%s' %(path, key)
    else:
        backup_fil = '%s/roach/%s' %(path, key)
    return backup_fil

def createFilePath(path, file_val):
    """
    Function to create file path
    """
    if path.endswith('/'):
        file_path = '%s%s' %(path, file_val)
    else:
        file_path = '%s/%s' %(path, file_val)
    return file_path

def updateCN_hosts(host_list, g_opts, cmds_dict):
    """
    update the cn host list
    """
    global cn_hosts, cmd_dict
    saveOrigOptions(g_opts)
    cn_hosts = host_list
    cmd_dict = cmds_dict

def check_gs_roach_running(g_opts):
    """
    Check if gs_roach isntance is already running
    """
    global gauss_home_loc
    if gauss_home_loc.endswith('/'):
        roach_path = '%sroach' % (gauss_home_loc)
    else:
        roach_path = '%s/roach' %(gauss_home_loc)
    gs_roachFile = '%s/roach_%s.lock' % (roach_path, g_opts.user)
    if os.path.exists(gs_roachFile):
        try:
            lock_file = open(gs_roachFile, 'r')
            fcntl.flock(lock_file, fcntl.LOCK_SH | fcntl.LOCK_NB)
            fcntl.flock(lock_file, fcntl.LOCK_UN)
            lock_file.close()
        except IOError as e:
            g_logger.debug("Error when check roach flock: %s" % str(e))
            if not lock_file.closed:
                lock_file.close()
            if e.errno == 11:
                g_logger.logExit("ERROR: Another instance of roach execution is in progress")

def getVersionDetail():
    """
    get the current verion string
    """
    return GAUSS_ROACH_VERSION

def validateGenerateHost(final_ip_param_dict):
    """
    delete parameters validation
    """
    for key in final_ip_param_dict.keys():
        if '--media-type' not in final_ip_param_dict[key].keys():
            g_logger.logExit("--media-type not specified")
        if '--media-destination' not in final_ip_param_dict[key].keys():
            g_logger.logExit("--media-destination not specified")
        if '--backup-key' not in final_ip_param_dict[key].keys():
            g_logger.logExit("--backup-key not specified")

def create_roachPath(ip, g_opts):
    """
    create the roach path in $GAUSSHOME
    for holding the temp files for Roach
    """
    global gauss_home_loc, master_ip
    path = gauss_home_loc
    roach_path = '%s/roach' % ("$GAUSSHOME")
    #check for local and remote ip to create path
    if ip == master_ip:
        create_cmd = 'source %s && if [ ! -e %s ]; then mkdir -p %s -m 700; fi' \
            % (g_opts.sourceFile, roach_path, roach_path)
    else:
        create_cmd = 'pssh -t 300 -H \"%s\" -P "source %s && if [ ! -e %s ]; then mkdir -p %s -m 700; fi"' \
            % (ip, g_opts.sourceFile, roach_path, roach_path)
    status, output = commands.getstatusoutput(create_cmd)
    if status != 0:
        GaussLog.exitWithError("ERROR: Unable to create roach path \n%s" %(output))

def saveOrigOptions(g_opts):
    """
    backup the original options
    """
    global orig_gopts
    orig_gopts = g_opts

def getCoHostPort(hostName):
    """
    Find CoordPort for provided CoordHost
    """
    l_clusterInfo = dbClusterInfo()
    l_clusterInfo.initFromStaticConfig(orig_gopts.user)
    dbNodeInfo = l_clusterInfo.getDbNodeByName(hostName)
    coPort = dbNodeInfo.coordinators[0].port
    return hostName, coPort

def getSingleDnHostPort(hostName):
    """
    Find dnHost for single Mode
    """
    l_clusterInfo = dbClusterInfo()
    l_clusterInfo.initFromStaticConfig(orig_gopts.user)
    dbNodeInfo = l_clusterInfo.getDbNodeByName(hostName)
    dnPort = dbNodeInfo.datanodes[0].port
    return dnPort

def reform_tablename(tablename):
    """
    function: to reform the table name with special characters
    input: table name
    output: formatted table name
    """

    def addquotes(val):
        """
        function: to add quotes around a given string value
        input: string value
        output: quoted string
        """
        val = '"' + val + '"'
        val = "'" + val + "'"
        return val

    skipchars = ["\\", "$", "`"]
    tempname = []
    failstatus = False
    finalname = ""
    dotCount = tablename.count('.')

    if ((tablename.count('"')) % 2) != 0:
        GaussLog.exitWithError('Error: \'"\' should be followed by \'"\' for: %s' % str(tablename))

    if dotCount == 0:
        if re.match(r'^"[^"]+"$', tablename):
            finalname = repr(tablename)
        elif re.match(r'^".*\"\".*"$', tablename):
            finalname = tablename
            finalname = "'" + finalname + "'"
        elif re.match(r'[^"]+\"\"[^"]+', tablename):
            finalname = tablename
        else:
            finalname = str(tablename.lower())

    elif dotCount == 1:
        namelist = tablename.split('.')
        for namel in namelist:
            if ((namel.count('"')) % 2) != 0:
                GaussLog.exitWithError('Error: \'"\' should be followed by \'"\' for: %s' % str(tablename))
                failstatus = True

            if re.match(r'^".*"$', namel):
                tempname.append(namel)
            else:
                tempname.append(namel.lower())

        if not failstatus:
            finalname = '.'.join(item for item in tempname)
            if re.match(r'^"[^"]+""[^"]+"\.[^"]+$', tablename):
                finalname = repr(tablename)

            elif re.match(r'(^"[^"]+"\.[^"]+$)|(^"[^"]+"\."[^"]+"$)|(^"[^"]+"\."[^"]+"$)', tablename):
                charcount = False
                for char in skipchars:
                    if char in tablename:
                        repchar = '\\' + char
                        tablename = tablename.replace(char, repchar)
                        charcount = True
                if not charcount:
                    finalname = repr(tablename)
                else:
                    tablename = tablename.replace('"', '\\"')
                    tablename = '"' + tablename + '"'
                    finalname = tablename

            elif re.match(r'[^"]+\."[^"]+""[^"]+"$', tablename):
                finalname = repr(tablename)
            elif re.match(r'.*"".*', tablename):
                finalname = "'" + finalname + "'"
            elif re.match(r'^"[^"]+"\."[^"]+"$', tablename):
                if "'" in tablename:
                    finalname = tablename
                else:
                    finalname = repr(tablename)

            elif re.match(r'^"[^"]+"\.[^"]+', tablename):
                if '\\' in tablename:
                    finalname = repr(tablename)
                    finalname = finalname.replace('\\\\', '\\')
                else:
                    finalname = repr(tablename)
            elif re.match(r'[^"]+\."[^"]+"$', tablename):
                finalname = repr(tablename)
            elif re.match(r'^"[^"]+"\.[^"]+$', tablename):
                finalname = tablename
            elif re.match(r'^"[^"]+"\."[^"]+"$', tablename):
                finalname = tablename

    elif dotCount >= 2:
        if re.match(r'^"[^"]+"$', tablename):
            GaussLog.exitWithError("Error: Ambiguous quoting! If special " \
                "chars present in schema name, please close the quotes at the end of schema name.")
            failstatus = True
        elif re.match(r'[^"]+\..*\.".*"$', tablename):
            GaussLog.exitWithError("Error: Schema should have quotes: %s" % str(tablename))
        elif re.match(r'^".*"\.[^"]+\.[^"]+', tablename):
            GaussLog.exitWithError("Error: Table should have quotes: %s" % str(tablename))
        elif re.match(r'(^".*\."\.[^"]+)', tablename):
            finalname = addquotes(tablename)
        elif re.match(r'([^"^\.]+\.".*"$)', tablename):
            finalname = addquotes(tablename)
        elif re.match(r'(^"[^"]+"\.[^"]+$)|(^"[^"]+"\."[^"]+"$)|(^"[^"]+"\."[^"]+"$)|(^"[^"]+\.[^"]+"\."[^"]+"$)', tablename):
            charcount = False
            for char in skipchars:
                if char in tablename:
                    charcount = True
            if not charcount:
                finalname = repr(tablename)
            else:
                tablename = tablename.replace('"', '\\"')
                tablename = '"' + tablename + '"'
                finalname = tablename

        elif re.match(r'(^".*\..*"\."[^"]+"$)', tablename):
            if re.match(r'.*"".*', tablename):
                finalname = "'" + tablename + "'"
        elif re.match(r'^"[^"]+"\.".*\..*"$', tablename):
            finalname = addquotes(tablename)
        elif re.match(r'^".*\."".*\.".*\."".*"$', tablename):
            finalname = "'" + tablename + "'"
        elif re.match(r'^"[^"]+\.[^"]+"\.[^"]+$', tablename):
            finalname = repr(tablename)
        elif re.match(r'^"[^"]+""[^"]+"\."[^"]+""[^"]+"$', tablename):
            finalname = repr(tablename)

    if finalname != '':
        return finalname
    else:
        GaussLog.exitWithError('Provide correct tablename with pattern')

def getNodesInfoByUpdatedFile(jsonFile):
    fp = open(jsonFile, 'r')
    infostr = fp.read()
    infostr = re.sub(r'(\d+)UL','\g<1>',infostr)
    fp.close()
    infodict = json.loads(infostr)
    infoList = infodict['Nodes']
    return infoList

def createLogPath(logPath):
    """
    create log path
    """
    cmd = 'python -c \\\"import os; os.makedirs(\'%s\') if not os.path.isdir(\'%s\') else \'pass\'\\\"' \
        % (logPath, logPath)
    return cmd

def getMetaDest(final_ip_param_dict):
    """
    to fetch the --metadata-destination path
    """
    metaDest = ""
    try:
        if master_ip in final_ip_param_dict.keys():
            if "--metadata-destination" in final_ip_param_dict[master_ip].keys():
                metaDest = final_ip_param_dict[master_ip]["--metadata-destination"]
    except OSError:
        g_logger.log(str(OSError))

    if not metaDest:
        return metaDest
    else:
        return os.path.realpath(metaDest)

def postExecCleanup(g_opts, g_logger):
    """
    clean data after backup
    """
    try:
        g_logger.debug("Parsing the configuration file when postExecCleanup.")
        l_clusterInfo = dbClusterInfo()
        staticConfig = "%s/bin/cluster_static_config" % (gauss_home_loc)
        staticConfig_bak = "%s/bin/cluster_static_config_bak" % (gauss_home_loc)
        if (os.path.exists(staticConfig) == False):
            staticConfig = staticConfig_bak
        l_clusterInfo.initFromStaticConfigWithoutUser(staticConfig)
        l_sshTool = SshTool(l_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_POSTCLEAN)
        g_logger.log("Cleaning temp files after backup or restore.")
        pitr_cmd = ""
        if (g_opts.pitr):
            pitr_cmd += "--pitr"
        cmd = "source %s && python %s -t %s %s -U %s -s %s -l %s" \
            % (g_opts.sourceFile, LOCAL_ROACH, 'postexec', pitr_cmd, g_opts.user, \
            g_opts.metadataDestination, g_opts.logFile)
        g_logger.debug("post clean up cmd = %s.", cmd)
        cluster_version = getClusterVersion(g_opts.user)
        if isMiniaturizedDeployment(cluster_version):
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.logExit("Fail to clean temp files after backup or restore. Error:\n%s." % output)
        else:
            l_sshTool.executeCommand(cmd, "post execution cleanup", DefaultValue.SUCCESS, g_opts.nodeList)
        g_logger.log("Successfully cleaned temp files after backup or restore.")
    except Exception, err:
        g_logger.log("Clean temp files after backup or restore not properly.")
        g_logger.debug("Exception in postExecCleanup is: %s" % str(err))

def rmMediaBackupDirs(g_opts, backupKey):
    '''
    clean local media file after backup
    '''
    try:
        if (g_opts.remoteMasterIp != ''):
            cmd = "source %s && gs_ssh -c \"rm -rf %s/roach/%s 2>&1\"" \
                % (g_opts.sourceFile, g_opts.mediaDest, backupKey)
            status, output = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.logExit("Warning: Remove local media data failed. Error:\n%s." % output)
            g_logger.log("Remove local media data sucessfully.")
    except Exception, err:
        g_logger.log("Clean local media file after backup not properly.")
        g_logger.debug("Exception in rmMediaBackupDirs is: %s" % str(err))

def getClusterVersion(user):
    """
    get cluster version from static config file
    """
    try:
        g_clusterInfo = None
        gaussHome = DefaultValue.getEnv("GAUSSHOME")
        staticConfig = "%s/bin/cluster_static_config" % gaussHome
        staticConfig_bak = "%s/bin/cluster_static_config_bak" % gaussHome
        if (os.path.exists(staticConfig) == False):
            staticConfig = staticConfig_bak
        g_clusterInfo = dbClusterInfo()
        g_clusterInfo.initFromStaticConfig(user, staticConfig)
        cluster_version = g_clusterInfo.getClusterVersion(staticConfig)
        return cluster_version
    except Exception, err:
        GaussLog.exitWithError("ERROR: failed to get cluster version, msg: %s" % str(err))

def isMiniaturizedDeployment(cluster_version):
    """
    judge whether is the miniaturized deployment 
    """
    if (cluster_version >= 101 and cluster_version <= 200):
        return True
    return False

def isSinglePrimaryMultiStandbyDeployment(cluster_version):
    """
    judge whether is the single primary multi standby deployment 
    """
    if (cluster_version >=201 and cluster_version <= 300):
        return True
    return False

def isSingleInstCluster(cluster_version):
    """
    judge whether is the single instance deployment
    """
    if (cluster_version >=301 and cluster_version <= 400):
        return True
    return False

def getBackupDetails(command_dict, final_ip_param_dict):
    """
    function: to return BackupDetails from the backups.ini in JSON format
    """
    metaDest = getMetaDest(final_ip_param_dict)
    retryTimes = 12
    for retryTime in range(retryTimes):
        try:
            backupsFile = os.path.join(metaDest, "backups.ini")
            filedata = loadMetaFileAsJson(backupsFile)
            filedata = filedata["BackupDetails"]
            return filedata
        except Exception as e:
            g_logger.debug("Error: get backup details from backups.ini failed, output is [%s] for %s times" % (str(e), str(retryTime)))
            if retryTime == retryTimes -1:
                g_logger.logExit("Error: get backup details from backups.ini failed, output is [%s]" % (str(e)))
            else:
                time.sleep(5)

def getParentBackupKey(backupKey, final_ip_param_dict, command_dict):
    """
    function: to get the parent backup key
    input: command_dict, backup key
    output: list of backup keys
    """
    bkpkeylist = []

    #inner function
    def searchBackupKey(bkpkey):
        """
        function: recursive fn to find the backup key in the json
        input: backup key
        output: list of backup keys
        """
        for bkpdict in bkpdetails:
            if bkpdict.has_key("BackupKey") and bkpdict["BackupKey"] == bkpkey:
                bkpkeylist.append(str(bkpkey))
                if bkpdict.has_key("PriorBackupKey") and (bkpdict["PriorBackupKey"] == "FULL"):
                    return bkpkeylist
                else:
                    searchBackupKey(bkpdict["PriorBackupKey"])
    
    bkpdetails = getBackupDetails(command_dict, final_ip_param_dict)

    keyExists = False
    for bkpdict in bkpdetails:
        if bkpdict.has_key("BackupKey") and bkpdict["BackupKey"] == backupKey:
            keyExists = True
            if bkpdict["BackupType"] == "INCREMENTAL":
                searchBackupKey(backupKey)
            else:
                bkpkeylist.append(str(backupKey))
                break
            if not keyExists:
                errorString = ErrorCode.GAUSS_534["GAUSS_53404"] % backupKey
                agentErrorReport("roach_backup", errorString)
                g_logger.logExit(errorString)

    return bkpkeylist

def getArchiveBackupKeys(command_dict, doShow=False):
    """
    function: to get backup key list of archived backup
    """
    #If --media-type is not set then use a dummy value!
    if "--media-type" not in command_dict.keys():
        command_dict["--media-type"] = " "

    #Validate the user-input --restore-target-time
    try:
        val = command_dict["--restore-target-time"]
        #First check if it is a valid date-time
        datetime.strptime(val.strip(), '%Y-%m-%d %H:%M:%S')
        #If valid, confirm that user specified correct format
        val1 = val.split()
        #2 fields (Date and Time) should be there
        if len(val1) != 2:
            raise Exception('Value specified: "%s". Please use the exact format "YYYY-MM-DD HH:MM:SS"' % val)
        #Check format of date field
        val1 = val1[0].split('-')
        if len(val1) != 3 or len(val1[0]) != 4 or len(val1[1]) != 2 or len(val1[2]) != 2:
            raise Exception('Value specified: "%s". Please use the exact format "YYYY-MM-DD HH:MM:SS"' % val)
        #Check format of date field
        val1 = val.split()[1].split(':')
        if len(val1) != 3 or len(val1[0]) != 2 or len(val1[1]) != 2 or len(val1[2]) != 2:
            raise Exception('Value specified: "%s". Please use the exact format "YYYY-MM-DD HH:MM:SS"' % val)
    except ValueError, err:
        g_logger.logExit("Error : Invalid --restore-target-time value! \n" \
            "Value specified: \"%s\". Please use the exact format \"YYYY-MM-DD HH:MM:SS\"" % val)
    except Exception, err:
        g_logger.logExit("Error : Invalid --restore-target-time value! \n%s" % str(err))

    #inner function
    def timeStampToInteger(time_stamp):
        """
        Parse timestamp string into an int
        """
        if len(time_stamp.split()) == 1:
            #This means it is a backup-key, just need to remove "_"
            return time_stamp.replace("_", "")
        time_date = time_stamp.split()[0]        
        time_date = time_date.replace("-", "")
        
        time_time = time_stamp.split()[1].split('.')[0]
        time_time = time_time.replace(":", "")
        return time_date + time_time

    local_arckeysdict = dict()
    detail_dict = dict()
    backupDetails = getBackupDetails(command_dict, final_ip_param_dict)
    recTime = int(timeStampToInteger(command_dict["--restore-target-time"]))

    #If backup key specified, it should be a FULL backup!
    if "--backup-key" in command_dict:
        bkpdet = [x for x in backupDetails if x["BackupKey"] == command_dict["--backup-key"]]
        if len(bkpdet) > 0:
            if bkpdet[0]["BackupType"] != "FULL" and bkpdet[0]["BackupType"] != "PITR":
                g_logger.logExit("ERROR: Backup key specified (%s) is not a FULL backup!" \
                    % command_dict["--backup-key"])
            if bkpdet[0]["FileSysVersion"] not in ["Roach FileSystem 7.0"]:
                g_logger.logExit("ERROR: Backup key specified (%s) is not found or invalid for time-based restore!" \
                    % command_dict["--backup-key"])
            if ("--single" in command_dict.keys()):
                startBackupTime = bkpdet[0]["StartTime"]
                g_logger.debug("start backup time is %s" % startBackupTime)
                if (recTime < int(timeStampToInteger(startBackupTime))):
                    g_logger.logExit("Error : Invalid --restore-target-time value, which can not earlier than %s!\n" \
                        % startBackupTime)
                stopBackupTime = bkpdet[0]["EndTime"]
                if stopBackupTime == 'NA':
                    g_logger.debug("[Caution] No stop backup time. Maybe backup is still running or stop action has not been called.")
                    g_logger.debug("[Caution] So continue to recovery.")
                    return
                else:
                    g_logger.debug("stop backup time is %s" % stopBackupTime)
                    if (recTime < int(timeStampToInteger(startBackupTime))):
                        g_logger.logExit("Error : Invalid --restore-target-time value, which can not earlier than %s!\n" \
                            % startBackupTime)
                    elif (recTime > int(timeStampToInteger(stopBackupTime))):
                        g_logger.logExit("Error : Invalid --restore-target-time value, which can not later than %s!\n" \
                            % stopBackupTime)
                    else:
                       return
        else:
            g_logger.logExit("ERROR: Backup key specified (%s) is not found or invalid for time-based restore!" \
                % command_dict["--backup-key"])

    if len(backupDetails) <= 0:
        g_logger.logExit("WARNING: No backup available to proceed...")

    #Find the latest backup key within recovery target time in reverse order
    start_sn = 1
    try:
        while True:
            if start_sn > len(backupDetails):
                #Worst case: found at the start (first backup) of the list!
                start_sn -= 1
                break
            if timeStampToInteger(backupDetails[len(backupDetails) - start_sn]["BarrierTime"]) == "NA":
                start_sn += 1
                continue
            if recTime >= int(timeStampToInteger(backupDetails[len(backupDetails) - start_sn]["BarrierTime"])):
                break
            start_sn += 1
    except ValueError, err:
        g_logger.logExit("FATAL: Invalid backup entry:\n%s" % backupDetails[len(backupDetails) - start_sn])
    start_sn = len(backupDetails) - start_sn
    if str(backupDetails[start_sn]["BarrierTime"]) == 'NA' \
      or recTime < int(timeStampToInteger(backupDetails[start_sn]["BarrierTime"])):
        start_sn = -2 #Time given is even older than 1st backup or invalid 1st backup selected, then make it fail!

    #First time initializations
    ended = False
    loop_sn = start_sn + 1
    #Loop this till we don't find any valid tree ending in a FULL backup!
    while not ended:
        loop_sn -= 1
        if loop_sn < 0:
            g_logger.logExit("Error: No valid backups in the range of --restore-target-time (%s)!" \
                % command_dict["--restore-target-time"])
        nxtBkpKey = backupDetails[loop_sn]["BackupKey"]
        for i in xrange(loop_sn, -1, -1):
            bkpdet = backupDetails[i]
            #Self entry
            if nxtBkpKey != bkpdet["BackupKey"]:
                #Not found my backup-key, keep looping
                continue
            if bkpdet["STATUS"] in ['VALIDATED', 'UNVERIFIED'] and bkpdet["BarrierTime"] not in ['NA'] and \
                bkpdet["FileSysVersion"] in ["Roach FileSystem 7.0"] and \
                bkpdet["BackupType"] not in ['TABLE']:
                try:
                    brrTime = int(timeStampToInteger(bkpdet["BarrierTime"]))
                except ValueError, err:
                    g_logger.logExit("FATAL: Invalid backup entry:\n%s" \
                        % backupDetails[len(backupDetails) - start_sn])
                local_arckeysdict[brrTime] = bkpdet["BackupKey"]
                if doShow:
                    detail_dict[bkpdet["BackupKey"]] = bkpdet
            elif bkpdet['KeyLocation'] == 'AGENT':
                #This tree contains a backup done from another node, select another tree: ended = False
                local_arckeysdict.clear()
                if doShow:
                    detail_dict.clear()
                break
            else:
                #This tree leads to a dead-end, select another tree: ended = False
                local_arckeysdict.clear()
                if doShow:
                    detail_dict.clear()
                break

            #Parent entry
            nxtBkpKey = bkpdet["PriorBackupKey"]
            if str(nxtBkpKey) == 'FULL':
                if "--backup-key" in command_dict and bkpdet["BackupKey"] != command_dict["--backup-key"]:
                    #This tree ends in some other FULL backup key, select another tree: ended = False
                    local_arckeysdict.clear()
                    if doShow:
                        detail_dict.clear()
                elif bkpdet["STATUS"] in ['VALIDATED', 'UNVERIFIED']:
                    #Ended the parse!
                    ended = True
                else:
                    #This tree leads to a dead-end, select another tree: ended = False
                    local_arckeysdict.clear()
                    if doShow:
                        detail_dict.clear()
                break

    #Print out the related keys for "show" output
    if doShow:
        print """
+====+=================+===============+==================+========+===================+===================+===================+=============+=======+================+===========+
|S_NO|   BACKUP KEY    |   BKP TYPE    |      MEDIA       |  NODE  |   BARRIER TIME    |   DELETION TIME   |  VALIDATION TIME  |    STATUS   | TLI   | BKP PRIOR KEY  | BKP LEVEL
+====+=================+===============+==================+========+===================+===================+===================+=============+=======+================+===========+"""
        for item in sorted(detail_dict.items()):
            print "|%4s| %s |%-15s|%-18s| %6s |%-19s|%-19s|%-19s| %-11s |%-7s|%-16s|%-11s|" \
                % (item[1]['S_NO'], str(item[0]), item[1]['BackupType'], \
                item[1]['MediaType'], item[1]['KeyLocation'], item[1]['BarrierTime'][:19], \
                item[1]['DeletionTime'][:19], item[1]['ValidationTime'][:19], \
                item[1]['STATUS'], item[1]['Timeline ID'], \
                item[1]['PriorBackupKey'], item[1]['BackupLevel'])
    return local_arckeysdict


def checkClusterNodeCount(g_opts, backupKey, metaDest):
    """
    function: to check if the cluster topology is compatible for the specified backup key
    """
    cmctlCount = ""
    nodeCount = 0
    backupType = ""

    try:
        l_clusterInfo = dbClusterInfo()
        l_clusterInfo.initFromStaticConfig(g_opts.user)
        cmctlList = l_clusterInfo.getClusterNodeNames()
    except Exception, e:
        try:
            clustFileNameBak = str(e).split('[')[2].split(']')[0] + '_bak'
            l_clusterInfo.initFromStaticConfig(g_opts.user, clustFileNameBak)
            cmctlList = l_clusterInfo.getClusterNodeNames()
        except (IndexError, Exception) as e:
            g_logger.debug("Failed to checkClusterNodeCount, error: %s" % str(e))
            g_logger.logExit("Error: Not able to parse cluster info. Aborting...")

    try:
        cmctlCount = len(cmctlList)
        if os.path.exists(metaDest):
            backupdir = os.path.join(metaDest, "roach", backupKey)
            if not os.path.isdir(backupdir):
                errorString = ErrorCode.GAUSS_534["GAUSS_53404"] % backupKey
                agentErrorReport("roach_%s" % g_opts.action, errorString)
                g_logger.error(errorString)
                return False

            if (g_opts.action == ACTION_BACKUP):
                backupfile = os.path.join(backupdir, "roach_master_prior.metadata")
            else:
                backupfile = os.path.join(backupdir, "roach_master_updated.metadata")
            chkprior = loadMetaFileAsJson(backupfile)
            nodeCount = chkprior['NodeCount']
            backupType = chkprior['BackupType'].upper()

            if backupType == "TABLE":
                if cmctlCount < nodeCount:
                    g_logger.error("Error: Node count of specified backup-key ['%s'] is more than current cluster" % backupKey)
                    return False
            elif backupType != "" and backupType != "TABLE": #Need to match the cluster topology
                if cmctlCount == nodeCount:     #node count should match btwn cluster static and backup metadata list	
                    for count in range(cmctlCount):
                        dataNodeIdList = []
                        dataNodeCount = len(l_clusterInfo.dbNodes[count].datanodes) #cluster static datanode count consider sum of PDN,SDN and DSDN count
                        dataNodePSCount = 0
                        for idx in range(dataNodeCount):     
                            dataNodeIdList.append(l_clusterInfo.dbNodes[count].datanodes[idx].instanceId)
                            if l_clusterInfo.dbNodes[count].datanodes[idx].instanceType == 0 \
                              or l_clusterInfo.dbNodes[count].datanodes[idx].instanceType == 1 :
                                dataNodePSCount +=1 
                        for jdx in range(nodeCount):
                            isNodeMatched = 1 
                            metaDnCount = chkprior['Nodes'][jdx]['DatanodeCount']
                            metaSDnCount = chkprior['Nodes'][jdx]['StandbyCount']
                            if dataNodePSCount != (metaDnCount + metaSDnCount) :
                                isNodeMatched = 0   #if total number of PDN and SDN count mismatch then go for next node
                                continue
                            for kdx in range (0,metaDnCount):   #if PDN id does not match then go for next node
                                if chkprior['Nodes'][jdx]['Datanodes'][kdx]['DatanodeId'] not in dataNodeIdList:
                                    isNodeMatched = 0
                                    break
                            if isNodeMatched == 0:
                                continue
                            for kdx in range (0,metaSDnCount):  #if SDN id does not match then go for next node     
                                if chkprior['Nodes'][jdx]['Standbys'][kdx]['StandbyNodeId'] not in dataNodeIdList:
                                    isNodeMatched = 0
                                    break
                            if isNodeMatched == 0:
                                continue
                            metaDSDnCount = chkprior['Nodes'][jdx]['DummyStandbyCount']
                            for kdx in range (0,metaDSDnCount):  #After matching PDN id and/or SDN id, if DSDN id does not match then topology does not match
                                if chkprior['Nodes'][jdx]['DummyStandbys'][kdx]['StandbyNodeId'] not in dataNodeIdList:
                                    isNodeMatched = 0
                                    break
                            break
                        if isNodeMatched == 0:
                            g_logger.error("Error:DN/CN count of specified backup-key ['%s'] does not match with existing cluster" % backupKey)
                            return False
                else:
                    g_logger.error("Error: Node count of specified backup-key ['%s'] not matching with existing cluster" % backupKey)
                    return False
        else:
            g_logger.error("Error: Specified metadata destination ['%s'] does not exist" % metaDest)
            return False
        return True
    except OSError, oserr:
        g_logger.logExit(str(oserr))

def getParentBkptype(cmd_dict, final_ip_param_dict):
    """
    If incremental, returns the backup type of parent backup key, else 'None'
    """
    if cmd_dict.has_key('--prior-backup-key'):
        bkpKeyList = getParentBackupKey(cmd_dict['--prior-backup-key'], \
            final_ip_param_dict, cmd_dict)
        if len(bkpKeyList) > 0:
            backupDetails = getBackupDetails(cmd_dict, final_ip_param_dict)
            bkpDet = [x for x in backupDetails if x['BackupKey'] == bkpKeyList[-1]]
            if len(bkpDet) == 1:
                return bkpDet[0]["BackupType"]
    return None

def parseMultipleTable(g_opts):
    """
    Parse Multiple Table list
    """

    #Skip the functionality if called for any unexpected operations
    if g_opts.action not in ["backup", "restore"]:
        return []
    if os.path.isfile(g_opts.tablelist):
        file_path = g_opts.tablelist
        try:
            file_tablelist = open(file_path, 'r')
            line = file_tablelist.readlines()
            file_tablelist.close()
            if len(line) == 0:
                GaussLog.exitWithError("Error : No table present in table list file")
            line = map(str.strip, line)
                
            for value in line:
                if len(value) < 128:
                    pass
                else:
                    GaussLog.exitWithError("Error : length of tablename length overflow")

            return line
        except Exception, e:
            file_tablelist.close()
            GaussLog.exitWithError("Error: %s" % str(e))
    else:
        if not g_opts.tablelist:
            GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50001"] % "-table-list")
        GaussLog.exitWithError(ErrorCode.GAUSS_502["GAUSS_50201"] % g_opts.tablelist)

def updateCommand_dict(g_opts, command_dict):
    """
    To remove backup_key from command dict
    """
    #Skip the functionality if called for any unexpected operations
    if g_opts.action not in ["backup", "restore"]:
        return
    if g_opts.backupKey:
        del(command_dict["--backup-key"])

def isClusterBackup(g_opts, command_dict):
    """
    Check backup type is cluster
    Notice: this function can only be used for backup operation,
    because other action maybe has no such keys in command_dict.
    For restore, please use global var: g_isClusterBkp
    """
    if (command_dict.has_key("--tablename") or command_dict.has_key("--table-list")):
        return False
    return True

def checkPrivateTable(g_opts, cn_hosts, tableName):
    """
    Check private table
    """
    if g_opts.dbName != "":
        db = g_opts.dbName
    else:
        db = 'postgres'
    coordPort = getCoHostPort(cn_hosts)
    coordPort = coordPort[1]
    table_name = None
    schemaname = None
    tmp_tablename = tableName

    if (tmp_tablename.find('.')>=0):
        tmp_tablename = tmp_tablename.split(".")
        table_name = tmp_tablename[1].lower()
        schemaname = tmp_tablename[0].lower()
    else:
        schemaname = "public"
        table_name = tableName.lower()

    sqlstring = "select R.rolkind, T.tableowner from pg_roles R, pg_tables T " \
                    "where R.rolname = T.tableowner and T.schemaname=\'%s\' " \
                    "and T.tablename = \'%s\'" % (schemaname, table_name)
    g_logger.debug("Check option indepedent cmd : \n[%s]" % (sqlstring)) 
    (status, output) = ClusterCommand.remoteSQLCommand(sqlstring, g_opts.user, \
        cn_hosts, coordPort, True, db)
    if (status != 0):
        g_logger.logExit("Error : Unable to check private tables '%s.%s'" \
            % (schemaname, table_name))
    try:
        tmp_output = output.split('|')
        rolkind    = tmp_output[0].strip()
        if ('i' == rolkind) :
            g_logger.logExit(ErrorCode.GAUSS_513["GAUSS_51302"] % (schemaname, table_name))
    except Exception, e:
        g_logger.logExit("Error : Unable to check private tables '%s.%s'" \
            % (schemaname, table_name))


def checkFile(filepath, filename):
    """
    check files exist
    """
    tmp_file = os.path.join(filepath, filename)
    tmp_files =  glob.glob(tmp_file)
    if (len(tmp_files) > 0):
        return True
    else:
        return False

def isDirectoryHasFile(dirName, fileName):
    """
    check if fileName exist under dirName
    """
    fullName = os.path.join(dirName, fileName)
    fileList = glob.glob(fullName)
    if (len(fileList) > 0):
        return True
    else:
        return False

def waitForNormal(g_logger, user="", nodeId=0, endTime=None, expected_redistributing="No"):
    """
    Wait the cluster or node become Normal
    """
    checkStatus = 0
    checkResult = ""

    startType = "cluster" if nodeId == 0 else "node"
    if (expected_redistributing == "Yes"):
        g_logger.debug("Waiting for the redistributing status to become yes.")
    else:
        g_logger.debug("Waiting for the %s status to become normal." % startType)

    while True:
        time.sleep(5)

        checkStatus = 0
        checkResult = ""
        (checkStatus, checkResult) = OMCommand.doCheckStaus(user, nodeId, None, expected_redistributing)
        if (checkStatus == 0):
            if (expected_redistributing == "Yes"):
                g_logger.debug("The cluster redistributing status is yes.")
            else:
                g_logger.debug("The %s status is normal." % startType)
            break
        if (endTime is not None and datetime.now() >= endTime):
            g_logger.debug("waitForNormal timeout, output: %s" % checkResult)
            if (expected_redistributing == "Yes"):
                raise Exception("Timeout." + "\n" + ErrorCode.GAUSS_516["GAUSS_51637"] \
                    % ("redistributing status", "Yes"))
            else:
                raise Exception("Timeout." + "\n" + ErrorCode.GAUSS_516["GAUSS_51602"].replace("cluster", startType))

    if (checkStatus != 0):
        g_logger.debug("waitForNormal failed, output: %s" % checkResult)
        raise Exception(ErrorCode.GAUSS_516["GAUSS_51607"] % startType)


def compareVersionFile(g_opts, backupKey):
    """
    compare version string, to prevent restore to different version,
    or incremental backup based on different version.
    only for cluster level backup & restore
    """
    #get current database version info
    cmd = "cat %s/bin/upgrade_version" % DefaultValue.getInstallDir(g_opts.user)
    g_logger.debug("Getting current upgrade_version, cmd: %s" % cmd)
    status, currVersion = commands.getstatusoutput(cmd)
    if status != 0:
        g_logger.logExit("Error: Unable to get current version info, output: %s" % currVersion)
    g_logger.debug("Current upgrade_version is: %s" % currVersion)

    #check if backup key exists
    previousMetaPath = os.path.join(g_opts.metadataDestination, "roach", backupKey)
    if (not os.path.exists(previousMetaPath)):
        g_logger.debug("Fail to find old version meta path: %s" % previousMetaPath)
        errorString = ErrorCode.GAUSS_534["GAUSS_53404"] % backupKey
        agentErrorReport("roach_%s" % g_opts.action, errorString)
        g_logger.logExit(errorString)

    #if version file is not backed up, perhaps previous backup is based on an old version
    previousVerFile = "%s/upgrade_version" % previousMetaPath
    if (not os.path.exists(previousVerFile)):
        g_logger.debug("Fail to find old version file: %s" % previousVerFile)
        g_logger.logExit("Error: Could not fetch version info of previous backup, operation is not allowed!")

    #get database version info that backed up before
    cmd = "cat %s" % previousVerFile
    g_logger.debug("Getting old upgrade_version, cmd: %s" % cmd)
    status, oldVersion = commands.getstatusoutput(cmd)
    if status != 0:
        g_logger.logExit("Error: Unable to get previous version info, output: %s" % oldVersion)
    g_logger.debug("Old upgrade_version is: %s" % oldVersion)

    #only allow restore to or incremental backup based on the same database version
    if (currVersion.split('\n')[:2] == oldVersion.split('\n')[:2]):
        g_logger.debug("Check version match passed!!")
    else:
        g_logger.logExit("Error: Current database version is not the same as previous backup, operation is not allowed!")

def hasStopProcess(g_logger, g_opts, g_clusterInfo, cluster_version):
    """
    check if GaussRoach.py -t stop process is running
    if has stop process or encounter exception, return True
    """
    #check if any controller process of stop is running on any node
    check_cmd = 'source %s && python %s -t precheck --checkObj stop -U %s -l %s' \
        % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, g_opts.user, g_opts.localLog)

    try:
        if not isMiniaturizedDeployment(cluster_version):
            #reset sshTool to give another timeout value, in case of network card is down
            sshTool = SshTool(g_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_PRECHECK)
            (status, output) = sshTool.getSshStatusOutput(check_cmd, g_opts.nodeList)
            if 'Success' in status.values():
                return True
        else:
            (status, output) = commands.getstatusoutput(check_cmd)
            if (0 == status):
                return True
        return False
    except Exception, e:
        g_logger.debug("Error when cmd: [%s] precheck stop process: %s" % (check_cmd, str(e)))
        g_logger.debug("Error: failed to check if stop process is running!")
        return False

def getClusterBackupKey(g_logger, g_opts, g_clusterInfo):
    cmd = 'source %s && python %s -t get_backup_key -U %s -l %s -F' \
          % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, g_opts.user, g_opts.localLog)
    g_logger.debug("Calling GSroachLocal.py to get backup key, cmd=[%s]." % cmd)
    hosts = g_clusterInfo.getClusterNodeNames()
    # reset sshTool to give another timeout value, in case of network card is down
    sshTool = SshTool(hosts, g_logger.logFile, TIMEOUT_PSSH_KILL_ROACH)
    status, output = sshTool.getSshStatusOutput(cmd, hosts, g_opts.sourceFile)
    outputMap = sshTool.parseSshOutput(hosts)
    for hostIp in hosts:
        g_logger.debug("%s:%s" % (hostIp, outputMap[hostIp]))
        if "backupkey=" in outputMap[hostIp]:
            cluster_backup_key = re.findall(r"backupkey=(.+?)backupkey",outputMap[hostIp])[0]
            return cluster_backup_key
    return ""

def updateBackupIniFailures(g_logger, g_opts, updateBackupKey):
    gaussHomeLoc = DefaultValue.getEnv("GAUSSHOME")
    metadata_opt_string = "--metadata-destination '%s'" % g_opts.metadataDestination
    cmd = '%s/bin/gs_roach update --mark-failed --backup-key %s %s' \
        % (gaussHomeLoc, updateBackupKey, metadata_opt_string)
    g_logger.debug("python call gs_roach update in updateForBackupFailures, cmd: %s" % cmd)
    sub = subprocess.Popen([cmd], shell=True, bufsize=-1)
    sub.wait()
    g_logger.debug("success to call gs_roach update in updateForBackupFailures, cmd: %s" % cmd)


def hasPitrBackupProcess(g_logger, g_opts, g_clusterInfo,
                         cluster_version, sshTool):
    """
    check if GaussRoach.py -t backup process is running
    if has backup process, return True
    """
    #check if any controller process is running on any node
    check_cmd = 'source %s && python %s -t precheck ' \
                '--checkObj %s --pitr --metaData %s -U %s -l %s' \
                % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL,
                   g_opts.stopObject, g_opts.metadataDestination,
                   g_opts.user, g_opts.localLog)
    g_logger.debug("Calling GSroachLocal.py to precheck %s python process, cmd=[%s]" % (g_opts.stopObject, check_cmd))

    try:
        if not isMiniaturizedDeployment(cluster_version):
            (status, output) = sshTool.getSshStatusOutput(check_cmd, g_opts.nodeList)
            if 'Success' in status.values():
                    if 'meta data check success' in output:
                        return True
            return False
        else:
            (status, output) = commands.getstatusoutput(check_cmd)
            if (0 == status):
                return True
        return False
    except Exception, e:
        g_logger.debug("Error when precheck %s process: %s" % (g_opts.stopObject, str(e)))
        raise Exception("Error: failed to check "
                        "if %s process is running!" % g_opts.stopObject)

def hasBackupProcess(g_logger, g_opts, g_clusterInfo, cluster_version, sshTool):
    """
    check if GaussRoach.py -t backup process is running
    if has backup process, return True
    """
    #check if any controller process is running on any node
    check_cmd = 'source %s && python %s -t precheck --checkObj %s -U %s -l %s' \
        % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, g_opts.stopObject, g_opts.user, g_opts.localLog)
    g_logger.debug("Calling GSroachLocal.py to precheck %s python process, cmd=[%s]" % (g_opts.stopObject, check_cmd))

    try:
        if not isMiniaturizedDeployment(cluster_version):
            (status, output) = sshTool.getSshStatusOutput(check_cmd, g_opts.nodeList)
            if 'Success' in status.values():
                return True
        else:
            (status, output) = commands.getstatusoutput(check_cmd)
            if (0 == status):
                return True
        return False
    except Exception, e:
        g_logger.debug("Error when precheck %s process: %s" % (g_opts.stopObject, str(e)))
        raise Exception("Error: failed to check if %s process is running!" % (g_opts.stopObject,))

def renameFileOnNodes(g_logger, srcFile, destFile, g_opts, g_clusterInfo):
    """
    check if markFile exists
    if exists, remove it on all nodes
    """
    g_logger.debug("rename %s to %s on all nodes." % (srcFile, destFile))
    cmd = 'mv %s %s' % (srcFile, destFile)

    #use try-catch to handle down network card scenarios
    try:
        #reset sshTool to give another timeout value, in case of network card is down
        sshTool = SshTool(g_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_PRECHECK)
        sshTool.executeCommand(cmd, "rename file", \
            DefaultValue.SUCCESS, g_opts.nodeList)
        g_logger.log("Successfully rename %s to %s on Nodes." % (srcFile, destFile))
    except Exception, e:
        g_logger.debug("Error when renaming %s: %s" % (srcFile, str(e)))
        raise Exception("Error: Failed to rename %s to %s!" % (srcFile, destFile))
    
def removeFileOnNodes(g_logger, srcFile, g_opts, g_clusterInfo):
    """
    check if markFile exists
    if exists, remove it on all nodes
    """
    g_logger.debug("remove %s on all nodes." % (srcFile))
    cmd = 'rm -f %s' % (srcFile)

    #use try-catch to handle down network card scenarios
    try:
        #reset sshTool to give another timeout value, in case of network card is down
        sshTool = SshTool(g_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_PRECHECK)
        sshTool.executeCommand(cmd, "remove mark file", \
            DefaultValue.SUCCESS, g_opts.nodeList)
        g_logger.debug("Successfully remove %s." % (srcFile))
    except Exception, e:
        g_logger.debug("Error when removing %s: %s" % (srcFile, str(e)))
        raise Exception("Error: failed to remove %s on all Nodes!" % (srcFile))

def cleanMetaMediaPath(g_opts, g_logger, BackupKey, g_clusterInfo):
    metaPath = os.path.join(g_opts.metadataDestination, 'roach', BackupKey)
    mediaPath = os.path.join(g_opts.mediaDest, 'roach', BackupKey)
    cmd = "source %s && python %s -t %s --metaPath %s --mediaPath %s -U %s -l %s" \
        % (g_opts.sourceFile, LOCAL_ROACH, ACTION_CLEAN_METAMEDIAPATH, metaPath, mediaPath, g_opts.user, g_opts.logFile)
    g_logger.debug("GaussRoach.py calling LocalRoach.py to clean meta and media files, cmd=[%s]" % (cmd))
    try:
        cluster_version = getClusterVersion(g_opts.user)
        if not isMiniaturizedDeployment(cluster_version):
            sshTool = SshTool(g_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_PRECHECK)
            sshTool.executeCommand(cmd, "Clean resuemBkp meta files")
        else:
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.logExit("Clean esuemBkp meta files failed with output: %s" % (output))
        g_logger.log("Successfully Clean meta and media path files.")
    except Exception, e:
        g_logger.logExit("Failed to Clean resuemBkp meta files, ERROR: %s." % (str(e)))

def distributeDynamicConf(g_logger, g_opts, g_clusterInfo):
    g_logger.log("distribute dynamic config.")
    filePath = "%s/roach/%s/" % (g_opts.metadataDestination, g_opts.backupKey)
    nodeList = " -H ".join(g_clusterInfo.getClusterNodeNames())
    cmdPscp = 'pscp -t 60 -H %s %s/cluster_dynamic_config %s' % (nodeList, filePath, filePath)
    g_logger.debug("distribute dynamic config cmd = %s" % cmdPscp)

    status, output = commands.getstatusoutput(cmdPscp)
    if status != 0:
        g_logger.logExit("Error when distribute dynamic config: %s" % output)
        
    else:
        g_logger.log("Successfully distribute dynamic config.")

def updateBackupInfo(g_logger, g_opts, endTime):
    gaussHome = DefaultValue.getEnv("GAUSSHOME")
    metadata_opt_string = "--metadata-destination '%s'" % g_opts.metadataDestination
    endtime_opt_string = "--pitr-end-time '%s'" % endTime
    cmd = '%s/bin/gs_roach update --pitr --backup-key %s %s %s' \
          % (gaussHome, g_opts.backupKey, metadata_opt_string, endtime_opt_string)
    g_logger.debug("python call gs_roach update in updateBackupInfo, cmd: %s" % cmd)
    sub = subprocess.Popen([cmd], shell=True, bufsize=-1)
    sub.wait()
    g_logger.debug("success to update backup info in updateBackupInfo")

def waitPitrRestart(g_logger, g_opts, g_clusterInfo):
    cmd = 'source %s && python %s -t wait_pitr_restart -U %s -l %s -F' \
          % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, g_opts.user, g_opts.localLog)
    g_logger.debug("Calling GSroachLocal.py to wait pitr restart, cmd=[%s]." % cmd)
    hosts = g_clusterInfo.getClusterNodeNames()
    # reset sshTool to give another timeout value, in case of network card is down
    sshTool = SshTool(hosts, g_logger.logFile, TIMEOUT_PSSH_KILL_ROACH)
    needToWait = 0
    count = 0
    # Wait max 5min to pitr restart is enough, 300 / 5s = 60
    while needToWait < 60:
        count = 0
        # reset sshTool to give another timeout value, in case of network card is down
        status, output = sshTool.getSshStatusOutput(cmd, hosts, g_opts.sourceFile)
        g_logger.debug("Check roach process, retry count:%d, result:%s." % (needToWait, output))
        outputMap = sshTool.parseSshOutput(hosts)
        for hostIp in hosts:
            g_logger.debug("%s:%s" % (hostIp, outputMap[hostIp]))
            if "running" in outputMap[hostIp]:
                count = count + 1
                g_logger.debug("host %s gs_roach pitr process is running." % hostIp)
        if count == len(hosts):
            g_logger.debug("All host %s gs_roach pitr process is running." % hostIp)
            return True
        needToWait = needToWait + 1
        time.sleep(5)
    g_logger.log("Waiting for PITR process timeout.")
    return False

def stopGSroachPitr(g_logger, g_opts, g_clusterInfo):
    g_logger.log("Waiting for PITR process to terminate normally.")
    #use try-catch to handle down network card scenarios
    try:
        cmd = 'source %s && python %s -t stop-pitr -U %s -l %s -F' \
              % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, g_opts.user, g_opts.localLog)
        g_logger.debug("Calling GSroachLocal.py to set stop pitr flag, cmd=[%s]" % cmd)
        hosts = g_clusterInfo.getClusterNodeNames()
        #reset sshTool to give another timeout value, in case of network card is down
        sshTool = SshTool(hosts, g_logger.logFile, TIMEOUT_PSSH_KILL_ROACH)
        g_logger.debug("node list len :%d." % len(hosts))
        status, output = sshTool.getSshStatusOutput(cmd, hosts, g_opts.sourceFile)
        g_logger.debug("set stop pitr flag result:%s." % output)
        outputMap = sshTool.parseSshOutput(hosts)
        for hostIp in hosts:
            g_logger.debug("%s:%s" % (hostIp, outputMap[hostIp]))
            if "success" not in outputMap[hostIp]:
                raise Exception("Error: failed to set stop pitr flag!")

        g_logger.debug("Successfully set stop pitr.")

        cmd = 'source %s && python %s -t wait-pitr -U %s -l %s -F' \
              % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, g_opts.user, g_opts.localLog)
        g_logger.debug("Calling GSroachLocal.py to wait pitr stop, cmd=[%s]." % cmd)
        needToWait = 0
        count = 0
        # Wait max 30min to collect xlog is enough, 1800s / 5s = 360
        while needToWait < 4320:
            # reset sshTool to give another timeout value, in case of network card is down
            count = 0
            status, output = sshTool.getSshStatusOutput(cmd, hosts, g_opts.sourceFile)
            g_logger.debug("Check roach process, retry count:%d, result:%s." % (needToWait, output))
            outputMap = sshTool.parseSshOutput(hosts)
            for hostIp in hosts:
                g_logger.debug("%s:%s" % (hostIp, outputMap[hostIp]))
                if "does not exist" in outputMap[hostIp]:
                    count = count + 1
                    g_logger.debug("host %s gs_roach process has exited." % hostIp)
                else:
                    if needToWait % 120 == 0 and not needToWait == 0:
                        g_logger.log("host %s is processing: %s" % (hostIp, outputMap[hostIp]))
            if count == len(hosts):
                g_logger.debug("All host %s gs_roach process in cluster has exited." % hostIp)
                g_logger.log("Now PITR process has terminated normally.")
                return True
            needToWait = needToWait + 1
            time.sleep(5)
        g_logger.log("Waiting for PITR process timeout.")
        return False
    except Exception, e:
        g_logger.debug("Error when stop pitr: %s" % str(e))
        raise Exception("Error: failed to stop pitr process!")


def pauseResumeBackupProcess(g_logger, g_opts, g_clusterInfo):
    #pause resume
    if g_opts.pauseResumeBackup:
        g_logger.log("Stopping gs_roach process")
        cmd = 'source %s && python %s -t pause -U %s -l %s -F' \
            % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, g_opts.user, g_opts.localLog)
        g_logger.debug("Calling GSroachLocal.py to force stop gs_roach, cmd=[%s]" % cmd)
        #use try-catch to handle down network card scenarios
        try:
            #reset sshTool to give another timeout value, in case of network card is down
            sshTool = SshTool(g_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_KILL_ROACH)
            sshTool.executeCommand(cmd, "stop gs_roach process", \
                DefaultValue.SUCCESS, g_opts.nodeList)
            g_logger.log("Successfully stopped gs_roach process")
        except Exception, e:
            g_logger.debug("Error when stop gs_roach process: %s" % str(e))
            raise Exception("Error: failed to stop gs_roach process!")
        return


def stopGSroachProcess(g_logger, g_opts, g_clusterInfo, cluster_version):
    """
    killall gs_roach processes and kernel side activities 
    """
    #trigger the Stop command to force killall gs_roach
    g_logger.log("Stopping gs_roach process")
    cmd = 'source %s && python %s -t stop -U %s -l %s -F' \
        % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, g_opts.user, g_opts.localLog)
    g_logger.debug("Calling GSroachLocal.py to force stop gs_roach, cmd=[%s]" % cmd)

    #use try-catch to handle down network card scenarios
    try:
        if not isMiniaturizedDeployment(cluster_version):
            #reset sshTool to give another timeout value, in case of network card is down
            sshTool = SshTool(g_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_KILL_ROACH)
            sshTool.executeCommand(cmd, "stop gs_roach process", DefaultValue.SUCCESS, g_opts.nodeList)
        else:
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.debug("Warning: Failed to killall gs_roach when stop backup, output: \n%s." % output)
        g_logger.log("Successfully stopped gs_roach process")
    except Exception, e:
        g_logger.debug("Error when stop gs_roach process: %s" % str(e))
        raise Exception("Error: failed to stop gs_roach process!")

    if (g_opts.stopObject == 'restore'):
        g_logger.debug("Stop restore, no need deal kernel side, return now.")
        return

    #kill the kernel side activities related to roach backup 
    g_logger.log("Stopping the backup process in database")
    cmd = "source %s && python %s -t stop -U %s -l %s" % (g_opts.sourceFile, LOCAL_ROACH, g_opts.user, g_opts.localLog)
    g_logger.debug("Calling LocalRoach.py to trigger pg_stop_backup and pg_disable_delay_ddl, cmd=[%s]" % cmd)

    try:
        if not isMiniaturizedDeployment(cluster_version):
            #reset sshTool to give another timeout value, in case of network card is down
            sshTool = SshTool(g_clusterInfo.getClusterNodeNames(), g_logger.logFile, TIMEOUT_PSSH_PG_STOP)
            sshTool.executeCommand(cmd, "stop backup in database", DefaultValue.SUCCESS, g_opts.nodeList)
        else:
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.debug("Warning: Failed to kill kernel side activities when stop backup, output: \n%s." % output)
        g_logger.log("Successfully stopped the backup process in database")
    except Exception, e:
        g_logger.debug("Error when stop kernel activities: %s" % str(e))
        raise Exception("Error: failed to stop backup process in database!")
    if (g_opts.resumeBkp):
        masterMetaPath = os.path.join(g_opts.metadataDestination, 'resume_backup_master.metadata')
        if (os.path.exists(masterMetaPath)):
            #read context
            try:
                with open(masterMetaPath, 'r') as f:
                    size = f.read(4)
                    jsonBuffSize = struct.unpack("I", size)[0]
                    jsonBuff = f.read(jsonBuffSize)
                    data= json.loads(jsonBuff)
                    if data.has_key("BackupKey"):
                        BackupKey = data["BackupKey"]
                        #clean meta,media
                        cleanMetaMediaPath(g_opts, g_logger, BackupKey, g_clusterInfo)
            except Exception, e:
                g_logger.error("Error: clean resume backup failed, output is [%s]" % str(e))
        #clean tmp files
        postExecCleanup(g_opts, g_logger)

def loadMetaFileAsJson(fileName):
    """
    parse a meta file as json format
    meta file has some integer values end with "UL" which can't be recognized by json,
    so we need to remove "UL" postfix from integer values,
    after that json.loads can be successful
    The meta file is deleted during performBookkeepingInfo in double cluster
    backup and restore parallel, the following operations may fail, so add retry.
    """
    if not os.path.exists(fileName):
        errorString = ErrorCode.GAUSS_502["GAUSS_50201"] % fileName
        # Here we do not know where make error, general error will consider as backup error
        agentErrorReport("roach_backup", errorString)
        g_logger.logExit(errorString)
    retryTimes = 12
    for retryTime in range(retryTimes):
        try:
            with open(fileName, 'r') as filehand:
                lines = filehand.readlines()
                fmtLines = re.sub(r'\"BackupSize\":[0-9]+(UL|U)', '"BackupSize":12345678', lines[0])
                fmtLines = re.sub(r'\"BkpStartTime\":[0-9]+(UL|U)', '"BkpStartTime":12345678', fmtLines)
                fmtLines = re.sub(r'\"DiskUsage\":[0-9]+(UL|U)', '"DiskUsage":12345678', fmtLines)
                fmtLines = re.sub(r'\"DiskSpace\":[0-9]+(UL|U)', '"DiskSpace":12345678', fmtLines)
                fmtLines = re.sub(r'\"StopLsn\":[0-9]+(UL|U)', '"StopLsn":0', fmtLines)
                fmtLines = re.sub(r'\"StartLsn\":[0-9]+(UL|U)', '"StartLsn":0', fmtLines)
                fmtLines = re.sub(r'\"BarrierLsn\":[0-9]+(UL|U)', '"BarrierLsn":0', fmtLines)
                formatedJson = json.loads(fmtLines)
            return formatedJson
        except Exception as e:
            g_logger.debug("Error: load meta file as json failed, output is [%s] for %s times" % (str(e), str(retryTime)))
            if retryTime == retryTimes -1:
                g_logger.logExit("Error: load meta file as json failed, output is [%s]" % str(e))
            else:
                time.sleep(5)

def removeLocalFile(g_logger,localFile):
    if (os.path.exists(localFile)):
        os.remove(localFile)
    else:
        g_logger.debug("Can't remove %s because it does not exist." % localFile)

def deliveryMetaFileToAgents(logger, sshTool, sourceFile, nodeList, pathName, itemName, timeout = TIMEOUT_SCP):
    """
    copy one file or folder from master to the same directory of other agent nodes
    """
    cmd = 'if [ ! -d %s ]; then mkdir -p %s; fi' \
        % (pathName, pathName)
    logger.debug("Creating metadata path, cmd: %s" % cmd)

    (status, output) = sshTool.getSshStatusOutput(cmd, nodeList)
    if 'Failure' in status.values():
        logger.error("Mkdir Failed, path: %s, output: %s" % (pathName, output))
        return 1

    pNodeList = " -H ".join(nodeList)
    cmdPscp = "pscp -t %d -O StrictHostKeyChecking=no -H %s %s/%s %s" \
              % (timeout, pNodeList, pathName, itemName, pathName)
    status, output = commands.getstatusoutput(cmdPscp)
    if status != 0:
        logger.error("Delivery %s/%s failed with output: %s" % (pathName, itemName, output))
        return 1
    return 0

def getInstIdIpDict(oldClusterInfo):
    idIpDict = dict()
    for nodeInfo in oldClusterInfo:
        datanodeInfos = nodeInfo["Datanodes"]
        if (datanodeInfos != None):
            dnIp = nodeInfo['IP']
            for dnInfo in datanodeInfos:
                dnId = dnInfo['DatanodeId']
                idIpDict[dnId] = dnIp
        standbyInfos = nodeInfo["Standbys"]
        if (standbyInfos != None):
            dnIp = nodeInfo['IP']
            for snInfo in standbyInfos:
                snId = snInfo['StandbyNodeId']
                idIpDict[snId] = dnIp
    return idIpDict

def updateDnInPGXC(clusterInfo, idIpDict, cnPort, clusterVersion, g_logger, user, option, userName="", pwd=""):
    for dbNode in clusterInfo.dbNodes:
        if (isSinglePrimaryMultiStandbyDeployment(clusterVersion) == False):
            for dbInstance in dbNode.datanodes:
                if(dbInstance.instanceType == DefaultValue.MASTER_INSTANCE):
                    if (isMiniaturizedDeployment(clusterVersion) == False):
                        standbyInstance = None
                        peerInsts = clusterInfo.getPeerInstance(dbInstance)
                        for i in range(len(peerInsts)):
                            if(peerInsts[i].instanceType == DefaultValue.STANDBY_INSTANCE):
                                standbyInstance = peerInsts[i]
                        if(standbyInstance == None):
                            g_logger.logExit(ErrorCode.GAUSS_516["GAUSS_51620"] % "standby")
                        sql = "update pgxc_node set (node_host, node_host1, node_port, node_port1)=('%s','%s','%s','%s') \
                        where node_name='dn_%s_%s'" % (
                        dbInstance.listenIps[0], standbyInstance.listenIps[0], dbInstance.port, standbyInstance.port, \
                        dbInstance.instanceId, standbyInstance.instanceId)
                    else:
                        sql = "update pgxc_node set (node_host, node_host1, node_port, node_port1)=('%s','%s','%s','%s') \
                        where node_name='dn_%s'" % (
                           dbInstance.listenIps[0], dbInstance.listenIps[0], \
                           dbInstance.port, dbInstance.port, dbInstance.instanceId)

                    g_logger.debug("Update dn node cmd: %s" % sql)
                    status, output = ClusterCommand.execSQLCommand(sql, user, "", cnPort, DefaultValue.DEFAULT_DB_NAME,
                        False, option = option, userName=userName, pwd=pwd)
                    g_logger.log("status: %d, output: %s" % (status, output))
                    if (status != 0 or ClusterCommand.findErrorInSql(output) == True):
                        g_logger.logExit("Update %s info in pgxc node table failed." % dbInstance.instanceId)
        else:
            for dbInstance in dbNode.datanodes:
                instanceIdList = []
                instanceIdStr = ''
                p_listenIp = idIpDict[dbInstance.instanceId]
                peerInsts = clusterInfo.getPeerInstance(dbInstance)
                for peerInst in peerInsts:
                    if (peerInst.instanceType == DefaultValue.MASTER_INSTANCE or peerInst.instanceType == DefaultValue.STANDBY_INSTANCE):
                        instanceIdList.append(peerInst.instanceId)
                if(len(instanceIdList) == 0):
                    g_logger.logExit(ErrorCode.GAUSS_516["GAUSS_51620"] % "standby")

                instanceIdList.append(dbInstance.instanceId)
                instanceIdList.sort()
                instanceIdStr = "_".join(str(i) for i in instanceIdList)
                sql = "update pgxc_node set (node_host, node_host1, node_port, node_port1)=('%s','%s','%s','%s') \
                where node_name='dn_%s' and node_host = '%s'" % (
                      dbInstance.listenIps[0], dbInstance.listenIps[0],
                      dbInstance.port, dbInstance.port,
                      instanceIdStr, p_listenIp)
                g_logger.debug("Update dn node cmd: %s" % sql)
                status, output = ClusterCommand.execSQLCommand(sql, user, "", cnPort, DefaultValue.DEFAULT_DB_NAME,
                    False, option = option, userName=userName, pwd=pwd)
                g_logger.log("status: %d, output: %s" % (status, output))
                if (status != 0 or ClusterCommand.findErrorInSql(output) == True):
                    g_logger.logExit("Update %s info in pgxc node table failed." % dbInstance.instanceId)

def modifyDnInPGXC(clusterInfo, jsonFile, g_logger, user, option):
    hostName = socket.gethostname()
    localNodeInfo = clusterInfo.getDbNodeByName(hostName)
    if(len(localNodeInfo.coordinators) <= 0):
        g_logger.debug("There is no information about CN on the current node.")
        return

    #get the cluster version
    clusterVersion = 0
    clusterVersion = getClusterVersion(user)

    #get primary cluster dic: key is nodeid, value is IP
    infoList = getNodesInfoByUpdatedFile(jsonFile)
    p_clusterInfo_dic = getInstIdIpDict(infoList)

    updateDnInPGXC(clusterInfo, p_clusterInfo_dic, localNodeInfo.coordinators[0].port, clusterVersion, g_logger, user, option)

def copyStaticConfig(logger, clusterInfo):
    sshTool = SshTool(clusterInfo.getClusterNodeNames(), logger.logFile, timeout = 60)
    staticFileBak = "%s/bin/cluster_static_config_bak" % (clusterInfo.appPath)
    staticFile = "%s/bin/cluster_static_config" % (clusterInfo.appPath)
    cmd = "if [ -f %s ] && [ ! -f %s ]; then cp %s %s; fi" \
          % (staticFileBak, staticFile, staticFileBak, staticFile)
    status, output = sshTool.getSshStatusOutput(cmd)
    if (DefaultValue.FAILURE in status.values()):
        logger.logExit("Copy static config file failed with output: %s" % (output))

def checkThreadInCluster(hostIp, gpHome, gaussHome, envFile, threadFlag, clusterUserName, logFile, excludeFlag = 'check-thread', connectTimeout = CONNECTTIMEOUT, retryTimes = RETRYTIMES):
    global g_logger
    g_logger = GaussLog(logFile, "GSroachCommon")
    excludeFlags = "%s, %d " % (excludeFlag, os.getpid())
    cmd = "python %s/script/util/SyncClusterCommon.py -t check-thread --thread-flags '%s' --excluded-thread-flags '%s' \
    --user %s --gauss-home %s --env-file '%s' --log %s --connect-timeout %d --retry-times %d" \
          % (gpHome, threadFlag, excludeFlags, clusterUserName, gaussHome, envFile, logFile, connectTimeout, retryTimes)
    g_logger.debug("check thread cmd is %s" % cmd)
    status, output = retryCommonRemoteCmd(hostIp, cmd, envFile, connectTimeout=connectTimeout, retryTimes=retryTimes)
    if (status == 0):
        if ("syncClusterThreadExistsyncCluster" in output):
            return status, True
    return status, False

def getRemoteCmd(hostIp, env, cmd, connectTimeout = CONNECTTIMEOUT):
    remoteCmd = "pssh -s -H %s -t %d -O StrictHostKeyChecking=no \"if [ %s'a' != 'a' ] && [ -f %s ]; then source %s; else %s; fi; %s\"" \
                 % (hostIp, int(connectTimeout), env, env, env, DEFAULT_SOURCE_CMD, cmd)
    return remoteCmd

def retryCommonRemoteCmd(remoteHostIp, cmd, envFile, connectTimeout = CONNECTTIMEOUT, retryTimes = RETRYTIMES):
    remoteCmd = getRemoteCmd(remoteHostIp, envFile, cmd, connectTimeout)
    g_logger.debug("remoteCmd is %s" % remoteCmd)
    for i in range(1, int(retryTimes+1)):
        status, output = commands.getstatusoutput(remoteCmd)
        if (status != 0):
            if (i < retryTimes):
                time.sleep(1)
            else:
                return 1, output
        else:
            return 0, output



def unsetRoachCron(action, gaussHome, destHostIp, envFile, self, logFile, connectTimeout = CONNECTTIMEOUT):
    global g_logger
    g_logger = GaussLog(logFile, "GSroachCommon")
    g_logger.log("Stop crontab for %s" % action)
    currentTime = int(round(time.time() * 1000))
    cronFileName = "roachCron_%s_%d_%d" % (destHostIp, os.getpid(), currentTime)
    cronFile = "%s/%s" % (gaussHome, cronFileName)
    cronCmd = ""
    cmd = "crontab -l"
    status, output = retryCommonRemoteCmd(destHostIp, cmd, envFile, connectTimeout=connectTimeout)
    if (status != 0):
        g_logger.error("Get crontab failed, stop backup crontab failed.")
        return status
    cronCmd += "crontab -l > %s&& " % cronFile
    cronCmd += "sed -i '/%s -t %s/d' %s; " % (self, action, cronFile)
    cronCmd += "sed -i '{/^#/{/roach_cron_/d}}' %s; " % cronFile
    cronCmd += "sed -i '{/^#/{/roachCron_/d}}' %s; " % cronFile
    cronCmd += "sed -i '{/^#/{/DO NOT EDIT THIS FILE/d}}' %s; " % cronFile
    cronCmd += "sed -i '{/^#/{/Cron version/d}}' %s; " % cronFile
    cronCmd += "cd %s && " % (gaussHome)
    cronCmd += "crontab %s&&" % cronFileName
    cronCmd += "rm -f %s" % cronFile
    status, output = retryCommonRemoteCmd(destHostIp, cronCmd, envFile, connectTimeout=connectTimeout)
    if (status != 0):
        g_logger.error("Remove crontab failed, stop crontab failed.")
        return status
    g_logger.log("Stop crontab successfully.")
    return 0


def cbmRecycle(ignoreError, user, backupKey, clusterInfo, env, cbmRecycleLevel, metaPath, logFile):
    global g_logger
    g_logger = GaussLog(logFile, "GSroachCommon")
    nodeNames = clusterInfo.getClusterNodeNames()
    sshTool = SshTool(nodeNames, logFile, SSH_TOOL_TIMEOUT)
    cluster_version = getClusterVersion(user)
    sourceCmd = "if [ %s'a' != 'a' ] && [ -f %s ]; then source %s; else %s; fi" \
        % (env, env, env, DEFAULT_SOURCE_CMD)

    # recycle cbm files
    cmd = "%s && python %s -t %s --backup-key %s --cbm-recycle-level %s --metaPath %s --env '%s' -U %s -l %s" \
        % (sourceCmd, LOCAL_ROACH, ACTION_CBMRECYCLE, backupKey, cbmRecycleLevel, metaPath, env, user, logFile)
    g_logger.debug("GaussRoach.py calling LocalRoach.py to recycle cbm files, cmd=[%s]" % cmd)
    try:
        if not isMiniaturizedDeployment(cluster_version):
            status, output = sshTool.getSshStatusOutput(cmd, nodeNames, env)
            if ("Failure" in status.values()):
                raise Exception("Recycle cbm files failed with output: %s" % output)
        else:
            status, output = commands.getstatusoutput(cmd)
            if (status != 0):
                raise Exception("Recycle cbm files failed with output: %s" % output)
        g_logger.debug("Successfully recycle cbm files.")
    except Exception as e:
        if (ignoreError is False):
            g_logger.logExit("Failed to recycle cbm files, ERROR: %s." % (str(e)))
        g_logger.debug("Failed to recycle cbm files, ERROR: %s." % (str(e)))


def getDnInstanceNodeName(clusterInfo, dbInst):
    """
    get nodename by instance, this function only adapt to DN instances
    """
    masterInst = None
    standbyInst = None
    dummyStandbyInst = None
    nodeName = ""

    peerInsts = clusterInfo.getPeerInstance(dbInst)
    for i in range(len(peerInsts)):
        if (peerInsts[i].instanceType == MASTER_INSTANCE):
            masterInst = peerInsts[i]
        elif (peerInsts[i].instanceType == STANDBY_INSTANCE):
            standbyInst = peerInsts[i]
        elif (peerInsts[i].instanceType == DUMMY_STANDBY_INSTANCE):
            dummyStandbyInst = peerInsts[i]
    if (dbInst.instanceType == MASTER_INSTANCE):
        masterInst = dbInst
        nodeName = "dn_%d_%d" % (masterInst.instanceId, standbyInst.instanceId)
    elif (dbInst.instanceType == STANDBY_INSTANCE):
        standbyInst = dbInst
        nodeName = "dn_%d_%d" % (masterInst.instanceId, standbyInst.instanceId)
    elif (dbInst.instanceType == DUMMY_STANDBY_INSTANCE):
        dummyStandbyInst = dbInst
        nodeName = "dn_%d_%d" % (masterInst.instanceId, dummyStandbyInst.instanceId)
    return nodeName

def deliveryOneItemToAgents(pathName, itemName, g_opts, NodeList = None):
    """
    copy one file or folder from master to the same directory of other agent nodes
    """
    sendItem = "%s/%s" % (pathName, itemName)
    sendItemBak = "%s/%s.bak" % (pathName, itemName)
    if not os.path.exists(sendItem):
        g_logger.debug("Warning: [%s] does not exits, skipping it." % sendItem)
        return

    if NodeList:
        pNodeList = " -H ".join(NodeList)
    elif g_opts and g_opts.nodeList:
        pNodeList = " -H ".join(g_opts.nodeList)
    else :
        g_logger.debug("Failed to delivery file: %s, Node information does not exist." % itemName)
        return

    if g_opts and g_opts.sourceFile:
        cmdPssh = "source %s && pssh -s -H %s \"mkdir -p %s\"" \
                  % (g_opts.sourceFile, pNodeList, pathName)
    else:
        cmdPssh = "pssh -s -H %s \"mkdir -p %s\"" \
                  % (pNodeList, pathName)

    status, output = commands.getstatusoutput(cmdPssh)
    if status != 0:
        g_logger.debug("failed to makedir %s output: %s" % (pathName, output))

    cmdPscp = "cp %s %s && pscp -H %s %s %s && rm -f %s" % (sendItem, sendItemBak, pNodeList, sendItemBak, sendItem, sendItemBak)
    status, output = commands.getstatusoutput(cmdPscp)
    if status != 0:
        g_logger.debug("Failed to delivery file: %s, output: %s" % (itemName, output))
    else:
        g_logger.debug("Successfully send %s to all nodes" % itemName)

def mark_completion_progress(g_opts):
    """
    mark roach action progress to be 100% after all sub progresses have quit
    """
    # no need to mark completion if roach operation has failed
    if g_roach_op_fail:
        return

    pghost = DefaultValue.getEnv("PGHOST")
    progress_file = os.path.join(pghost, ".roach_task_progress.dat")
    cmd = "echo -n '100.0' > %s" % progress_file
    (status, output) = DefaultValue.retryGetstatusoutput(cmd)
    if status != 0:
        g_logger.debug("Warning: failed to mark completion progress, error: %s" % output)

    deliveryOneItemToAgents(pghost, "/.roach_task_progress.dat", g_opts)

def agentErrorReport(roachAction, errorString):
    #roachAction: roach_stop_backup | roach_backup | roach_restore | roach_delete
    #only report agent error in agent Env
    if not DefaultValue.getEnv("HOST_IP"):
        return
    agentLogPath = DefaultValue.getEnv("AGENTLOGPATH")
    if not agentLogPath or not os.path.exists(agentLogPath):
        return
    logfile = createFilePath(agentLogPath, "agent.log")
    agent_logger = GaussLog(logfile)
    nodeList = None
    l_clusterInfo = dbClusterInfo()
    staticConfig = "%s/bin/cluster_static_config" % (gauss_home_loc)
    staticConfig_bak = "%s/bin/cluster_static_config_bak" % (gauss_home_loc)
    if not os.path.exists(staticConfig):
        if os.path.exists(staticConfig_bak):
            staticConfig = staticConfig_bak
        else:
            agent_logger.error("cluster_static_config does not exist")
    l_clusterInfo.initFromStaticConfigWithoutUser(staticConfig)
    l_sshTool = SshTool(l_clusterInfo.getClusterNodeNames(), agent_logger.logFile, TIMEOUT_PSSH_POSTCLEAN)
    nodeList = l_clusterInfo.getClusterNodeNames()
    DefaultValue.recordTaskResult(errorString, roachAction, nodeList, l_sshTool, agent_logger)

    if (roachAction != "roach_delete"):
        pghost = DefaultValue.getEnv("PGHOST")
        processFile = pghost + "/.roach_task_progress.dat"
        fp = open(processFile,"w")
        fp.write('0.0')
        fp.close()
        deliveryOneItemToAgents(pghost, "/.roach_task_progress.dat", None, nodeList)

