try:
    import sys
    import os
    import commands
    import fcntl
    import string
    import signal
    import time
    import re
    from stat import ST_MODE
    import subprocess
    import json
    import glob
    import struct
    import socket
    from datetime import datetime
    sys.path.append(sys.path[0] + "/../")
    from gspylib.common.ErrorCode import ErrorCode
    from gspylib.common.GaussLog import GaussLog
    from gspylib.common.DbClusterInfo import dbClusterInfo, MASTER_INSTANCE, \
        STANDBY_INSTANCE, DUMMY_STANDBY_INSTANCE
    from gspylib.common.Common import DefaultValue, ClusterCommand
    from gspylib.common.OMCommand import OMCommand
    from gspylib.threads.SshTool import SshTool
    from gspylib.os.gsfile import g_file
    from gspylib.os.gsOSlib import g_OSlib
    from gspylib.common.DbClusterStatus import DbClusterStatus
    from gspylib.common.VersionInfo import VersionInfo
    from util.GSroachCommon import final_ip_param_dict, createFilePath, signalAndExit, sendSignal
except ImportError as err:
    sys.exit('Error: Unable to import module: ' + str(err))

g_logger = None
LOG_PATH = DefaultValue.getEnv("GAUSSLOG")
DEFAULT_LOG_PATH = "%s/roach/controller" % LOG_PATH
LOG_FILE_NAME = "roach-controller.log"
CBM_RECYCLE_LEVEL_0 = 0
CBM_RECYCLE_LEVEL_1 = 1
CBM_RECYCLE_LEVEL_2 = 2


def validateRoachParams(g_opts):
    """
    Validate gs_roach parameters
    """
    global g_logger

    g_opts.logFile = createFilePath(DEFAULT_LOG_PATH, LOG_FILE_NAME)
    g_logger = GaussLog(g_opts.logFile)

    metaDest = ""
    g_logger.debug("start validating roach parameters")
    if not final_ip_param_dict:
        signalAndExit("Error : Host/Ip details are not present, check for arguments or gs_roach.conf file")

    try:
        validateMetadataAndMediaParams(g_opts)
        validateMandatoryOptions(g_opts)
        validateBooleanParam()
        validateDBname()
        validateParamValue()
        validateRange()

        if g_opts.pitr and g_opts.action == "backup":
            validateArchiveParams(g_opts)

        ignoreParams = []
        for key in final_ip_param_dict.keys():
            if g_opts.action in ("validate", "delete"):
                checkValidatedAndDeleteActionParams(g_opts, key)
            if g_opts.action =="restore":
                checkRestoreActionParams(g_opts, key)
            if g_opts.action == "show":
                for tempkey in final_ip_param_dict[key].keys():
                    if tempkey not in ("--metadata-destination", "--backup-key", "--all-backups",
                                       "--logging-level", "--logging-path", "--logging", "--no-display",
                                       "--output-file", "--overwrite", "--related-backup-keys", "--restore-target-time"):
                        if tempkey not in ignoreParams:
                            ignoreParams.append(tempkey)
                checkShowActionParams(key)
            else :
                if '--related-backup-keys' in final_ip_param_dict[key].keys():
                    signalAndExit ("Error: --related-backup-keys option not allowed")

            if g_opts.action == "backup":
                checkBackupActionParams(g_opts, key)

            # Validate other params
            if  '--prior-backup-key' in final_ip_param_dict[key].keys() \
              and not (g_opts.action in ["backup","restore"] or g_opts.origAction in ["backup","restore"]):
                signalAndExit("Error : Prior backup key option only allowed for backup or restore")
            if  ('--media-destination' not in final_ip_param_dict[key].keys() 
              or '--master-port' not in final_ip_param_dict[key].keys()) \
              and not g_opts.action == "show":
                signalAndExit("Error : Master port or media destination not specified")

            for k, v in final_ip_param_dict[key].items():
                if k == '--log-filesize':
                    if int(final_ip_param_dict[key][k]) < 5 or int(final_ip_param_dict[key][k]) > 20:
                        signalAndExit("Error : Invalid value for the --log-filesize, range is [5~20]")
                if k == '--log-filecount':
                    if int(final_ip_param_dict[key][k]) < 5 or int(final_ip_param_dict[key][k]) > 1024:
                        signalAndExit("Error : Invalid value for the --log-filecount, range is [5~1024]")
                if k == '--logging-level':
                    if not (v.lower() in ['error', 'fatal', 'warning', 'info', 'log', 'debug', 'debug2']):
                        signalAndExit("Error : Invalid value for the --logging-level")
                if k == '--enable-logging':
                    if not (v.lower() in ['true', 'false']):
                        signalAndExit("Error : Invalid value for the --enable-logging")
    except Exception, e:
        signalAndExit("Error:!! INVALID ROACH PARAMETERS SPECIFIED!! %s" % str(e))
    g_logger.debug("end validating roach parameters")
    if len(ignoreParams) > 0:
        print "Warning: Ignoring parameters: %s" % str(ignoreParams)

###########################################################################
# validating all the parameters for gs_roach
###########################################################################
def validateBooleanParam():
    """
    validate boolean parameters
    """
    param_map = {"--verbose":"Error : Invalid value for --verbose", \
                 "--logging":"Error : Invalid value for --logging", \
                 "--clean":"Error : Invalid value for --clean", \
                 "--pre-disk-space":"Error : Invalid value for --pre-disk-space", \
                 "--continue":"Error : Invalid value for --continue", \
                 "--create":"Error : Invalid value for --create", \
                 "--logical":"Error : Invalid value for --logical", \
                 "--pitr":"Error : Invalid value for --pitr", \
                 "--resume-backup":"Error : Invalid value for --resume-backup", \
                 "--cluster-migration":"Error : Invalid value for --cluster-migration", \
                 "--single":"Error : Invalid value for --single",
                 "--split-local-disk-storage":"Error : Invalid value for --split-local-disk-storage"}

    param_keys = param_map.keys()
    
    for key in final_ip_param_dict.keys():
        for k, v in final_ip_param_dict[key].items():
            if (k in param_keys):
                if not v == None:
                    signalAndExit(param_map[k])

def validateMetadataAndMediaParams(g_opts):
    """
    Validate --metadata-destination and --media-destination
    """
    for key in final_ip_param_dict.keys():
        if '--metadata-destination' not in final_ip_param_dict[key].keys():
            g_logger.logExit("Error: '--metadata-destination' not provided")
        if '--media-destination' in final_ip_param_dict.keys():
            if len(final_ip_param_dict[key]['--media-destination']) > 1024:
                g_logger.logExit("Error : media destination length overflow")
        if '--metadata-destination' in final_ip_param_dict.keys():
            if len(final_ip_param_dict[key]['--metadata-destination']) > 1024:
                g_logger.logExit("Error : metadata destination length overflow")

def validateArchiveParams(g_opts):
    """
    Validate --archive-media-type and --archive-media-destination and --archive-xlog-buffer-size and --archive-xlog-buffer-path
    """
    for key in final_ip_param_dict.keys():
        if '--archive-media-destination' in final_ip_param_dict[key].keys():
            if len(final_ip_param_dict[key]['--archive-media-destination']) > 1024:
                g_logger.logExit("Error : archive media destination length overflow")
        if '--archive-xlog-buffer-path' in final_ip_param_dict[key].keys():
            if len(final_ip_param_dict[key]['--archive-xlog-buffer-path']) > 1024:
                g_logger.logExit("Error : archive xlog buffer path length overflow")

def validateMandatoryOptions(g_opts):
    """
    Validate mandatory params exist
    """
    for key in final_ip_param_dict.keys():
        if not final_ip_param_dict[key]:
            sendSignal()
            if g_opts.action == 'backup':
                g_logger.logExit("Error :mandatory options media-destination/media-type/master-port not specified")
            else:
                g_logger.logExit("Error :mandatory options media-destination/media-type/master-port/backup-key not specified")

def checkValidatedAndDeleteActionParams(g_opts, key):
    """
    Validate required params for validate and delete operation
    """
    if '--media-destination' not in final_ip_param_dict[key].keys() or \
        '--media-type' not in final_ip_param_dict[key].keys():
        signalAndExit ("Error : Media type or media destination not specified")

    if '--table-list' in final_ip_param_dict[key].keys():
        signalAndExit("Error : Parameter '--table-list' is not allowed for validate/delete.")

    ## Unrecognized parameters for delete
    if g_opts.action == "delete":
        for tempkey in final_ip_param_dict[key].keys():
            if tempkey not in ("--metadata-destination", "--media-destination", "--backup-key",
                               "--master-port", "--media-type", "--cascade", 
                               "--enable-logging", "--logging", "--logging-level", "--logging-path",
                               "--obs-server-ip", "--bucket-name", "--cluster-unique-id", "--username", "-W"):
                g_logger.logExit("Error: Unrecognized option %s.\nTry --help for more information." % tempkey)

def checkRestoreActionParams(l_opts, key):
    """
    Validate required params for restore operation
    """
    if l_opts.validatePreviousBackup:
        signalAndExit("Error : Parameter '%s'  is not allowed for restore." % "--validate-prior-backups")

    if not (l_opts.logical or l_opts.tablelist or l_opts.tableName) and \
            not (l_opts.backupKey or '--restore-target-time' in final_ip_param_dict[key].keys()) :
        signalAndExit("Error : Mandatory parameter, --backup-key or --restore-target-time is required")
    if '--restore-new-cluster' not in final_ip_param_dict[key].keys() \
      and '--restore-configs' in final_ip_param_dict[key].keys():
        signalAndExit("Error : The option --restore-configs is allowed only with --restore-new-cluster option.")

    if '--restore-new-cluster' in final_ip_param_dict[key].keys() \
      and '--restore-configs' not in final_ip_param_dict[key].keys():
        signalAndExit("Error : The option --restore-configs should be specified with --restore-new-cluster option.")

    checkTableRestoreParams(l_opts, key)

    #pitr restore validation
    if l_opts.pitr:
        if not l_opts.restoreTargetTime:
            signalAndExit("Error : Mandatory parameter, --restore-target-time is required for pitr restore")

def checkTableRestoreParams(l_opts, key):
    """
    Validate required params for table restore operation
    """
    #single table restore
    if l_opts.tableName and not l_opts.backupKey:
        signalAndExit("Error : Mandatory parameter, --backup-key is required for single table restore")

    #Logical Multi-table restore
    if l_opts.logical and not l_opts.tablelist:
        signalAndExit("Error : Mandatory parameter, --table-list is required for logical multi-table restore")
    if l_opts.tablelist and not l_opts.logical:
        signalAndExit("Error : Mandatory parameter, --logical is required for logical multi-table restore")
    if l_opts.logical and not l_opts.backupKey:
        signalAndExit("Error : Mandatory parameter, --backup-key is required for logical multi-table restore")
    if l_opts.logical and ('--restore-new-cluster' in final_ip_param_dict[key].keys() \
      or '--restore-target-time' in final_ip_param_dict[key].keys() \
      or '--parallel-restore' in final_ip_param_dict[key].keys()):
        signalAndExit("Error : The options --restore-target-time, --restore-new-cluster and --parallel-restore not allowed for logical multi-table restore")

def checkBackupActionParams(l_opts, key):
    """
    Validate required params for backup operation
    """
    if '--prior-backup-key' in final_ip_param_dict[key].keys() \
      and '--table-list' in final_ip_param_dict[key].keys():
        signalAndExit("Error : Parameter '--table-list' is not allowed for incremental backup.")

    if l_opts.logical and not l_opts.tablelist:
        signalAndExit("Error : Mandatory parameter, --table-list is required for logical multi-table backup")
    if l_opts.tablelist and not l_opts.logical:
        signalAndExit("Error : Mandatory parameter, --logical is required for logical multi-table backup")

    if '--remote-media-destination' in final_ip_param_dict.keys():
        if len(final_ip_param_dict[key]['--remote-media-destination']) > 1024:
            signalAndExit("Error : remote media destination length overflow")
    if '--remote-metadata-destination' in final_ip_param_dict.keys():
        if len(final_ip_param_dict[key]['--remote-metadata-destination']) > 1024:
            signalAndExit("Error : remote metadata destination length overflow")

    if not l_opts.origAction and l_opts.validatePreviousBackup \
      and '--prior-backup-key' not in final_ip_param_dict[key].keys():
        signalAndExit("Error : Parameter '--prior-backup-key' is missing.\n'--validate-prior-backups' can only be given with '--prior-backup-key'.")

    if l_opts.validatePreviousBackup:
        if l_opts.validatePreviousBackup.upper() not in ["FORCE", "OPTIMISTIC"]:
            signalAndExit("Error :Invalid value of '--validate-prior-backups'.")

def checkShowActionParams(key):
    """
    Validate required params for show operation
    """
    finalkeys = final_ip_param_dict[key].keys()
    for k, v in final_ip_param_dict[key].items():
        if "--no-display" in finalkeys and "--output-file" not in finalkeys:
            g_logger.logExit("Error: Parameter '--output-file' is missing.")
        if "--overwrite" in finalkeys and "--output-file" not in finalkeys and "--no-display" in finalkeys:
            g_logger.logExit("Error: Parameter '--output-file' is missing.")
        if k == "--output-file":
            outputfile = final_ip_param_dict[key]["--output-file"]
            outputfile = os.path.realpath(outputfile)
            if os.path.isfile(outputfile):
                if "--overwrite" not in final_ip_param_dict[key].keys() and "--no-display" in final_ip_param_dict[key].keys():
                    g_logger.logExit("Error: Specified output-file ['%s'] already exists.\nProvide different file name or provide ['--overwrite'] option to overwrite." % outputfile)

def checkSpecialCharForDBObject(objectName):
    """
    validate special char for table name, schema name, database name
    """
    #check that ; not there in tablename, dbname, schemaname (for secure forbidden chars)
    unwanted_char = ["|", ";", "&", ">", "<", "`", "\\", "(", ")", "[", "]", "~", "?", "*", "\"", "!", "\n"]
    for i in unwanted_char:
        if i in objectName:
            return False
    return True

def validateParamValue():
    """
    validate for the value
    """
    for key in final_ip_param_dict.keys():
        for k, v in final_ip_param_dict[key].items():
            if k in ['--metadata-destination', '--media-destination', 'archive-xlog-buffer-path', 'archive-xlog-buffer-size']:
                if k == v:
                    signalAndExit("Error : No value specified for the %s" %k)
            if k == '--dbname' and  k == v:
                signalAndExit("Error : No value specified for the dbname")
            if k == '--username':
                if k == v:
                    signalAndExit("Error : No value specified for the %s" %k)

def validateDBname():
    """
    validate the dbname for special characters
    """
    try:
        for key in final_ip_param_dict.keys():
            for k, v in final_ip_param_dict[key].items():
                if k == '--dbname':
                    if not checkSpecialCharForDBObject(v):
                        signalAndExit("Error:Unexpected char in dbname")
    except Exception:
        signalAndExit("Parameter input error, --dbname should be a valid database name")

def checkAgentPort(value):
    """
    validating range of agent port
    """
    if int(value) < 1024 or int(value) > 65535:
        GaussLog.exitWithError("Error : Invalid value for the --agent-port, range is [1024, 65535].")

def checkCpuRelinquishTime(value):
    """
    validating range of cpu relinquish time
    """
    if int(value) < 0 or int(value) > 3600:
        GaussLog.exitWithError("Error : Invalid value for the --cpu-relinquish-time, range is [0, 3600].")

def checkCpuRelinquishSize(value):
    """
    validating range of cpu relinquish size
    """
    if int(value) < 1 or int(value) > 10000:
        GaussLog.exitWithError("Error : Invalid value for the --cpu-relinquish-size, range is [1, 10000].")

def checkFailureRetryCount(value):
    """
    validating range of failure retry count
    """
    if int(value) < 0 or int(value) > 256:
        GaussLog.exitWithError("Error : Invalid value for the --failure-retry-count, range is [0, 256].")

def checkMaxMemoryUsage(value):
    """
    validating range of max memory usage
    """
    if int(value) < 2 or int(value) > 256:
        GaussLog.exitWithError("Error : Invalid value for the --max-memory-usage, range is [2, 256].")

def checkRetryWaitTime(value):
    """
    validating range of retry wait time
    """
    if int(value) < 1 or int(value) > 3600:
        GaussLog.exitWithError("Error : Invalid value for the --retry-wait-time, range is [1, 3600].")

def checkMasterPort(value):
    """
    validating range of master port
    """
    if int(value) < 1024 or int(value) > 65535:
        GaussLog.exitWithError("Error : Invalid value for the --master-port, range is [1024, 65535].")

def checkCompressionLevel(value):
    """
    validating range of compression level
    """
    if int(value) < 0 or int(value) > 9:
        GaussLog.exitWithError("Error : Invalid value for the --compression-level, range is [0, 9].")

def checkCompressionType(value):
    """
    validating range of compression level
    """
    if int(value) < 0 or int(value) > 2:
        GaussLog.exitWithError("Error : Invalid value for the --compression-type, range is [0, 2].")

def checkBufferSize(value):
    """
    validating range of buffer size
    """
    if int(value) < 256 or int(value) > 16384:
        GaussLog.exitWithError("Error : Invalid value for the --buffer-size, range is [256, 16384].")
        
def checkArchiveBufferSize(value):
    if int(value) < 0 or int(value) > 8191:
        GaussLog.exitWithError("Error : Invalid value for the archive-xlog-buffer-size, range is [0, 8191].")

def checkBufferBlockSize(value):
    """
    validating range of buffer block size
    """
    if int(value) < 524288 or int(value) > 268435456:
        GaussLog.exitWithError("Error : Invalid value for the --buffer-block-size, range is [524288, 268435456].")
    if not int(value) % 8192 == 0:
        GaussLog.exitWithError("Error :buffer-block-size value should be multiple of 8192")

def checkFileSplitSize(value):
    """
    validating range of file split size
    """
    if int(value) < 0 or int(value) > 1024:
        GaussLog.exitWithError("Error : Invalid value for the --filesplit-size, range is [0, 1024].")
    if not int(value) % 4 == 0:
        GaussLog.exitWithError("Error :filesplit-size value should be multiple of 4")

def checkParallelProcess(value):
    """
    validating range of parallel process
    """
    if int(value) < 1 or int(value) > 32:
        GaussLog.exitWithError("Error : Invalid value for the --parallel-process, range is [1, 32].")

def checkCbmRecycleLevel(value):
    """
    validating range of cbm recycle level
    """
    if not int(value) in (CBM_RECYCLE_LEVEL_0, CBM_RECYCLE_LEVEL_1, CBM_RECYCLE_LEVEL_2):
        GaussLog.exitWithError("Error : Invalid value for the --cbm-recycle-level, range is [0, 2].")

def checkRestoreBufferThreshold(value):
    """
    validating range of restore buffer threshold
    """
    if int(value) < 1 or int(value) > 100:
        GaussLog.exitWithError("Error : Invalid value for the --restore-buffer-threshold, range is [1, 100].")

def checkReaderCount(value):
    """
    validating range of reader thread count
    """
    if int(value) < 1 or int(value) > 65535:
        GaussLog.exitWithError("Error : Invalid value for the "
            "--reader-thread-count, range is [1, 65535].")

def checkReaderThreadFileCount(value):
    """
    validating range of reader thread file count
    """
    if int(value) < 1 or int(value) > 65535:
        GaussLog.exitWithError("Error : Invalid value for the --reader-thread-file-count, range is [1, 65535].")
        
def checkReaderThreadFileSize(value):
    """
    validating range of reader thread file size
    """
    if int(value) < 1 or int(value) > 65535:
        GaussLog.exitWithError("Error : Invalid value for the --reader-thread-file-size, range is [1, 65535].")

def validateRange():
    """
    validating range
    """
    param_map = {"--agent-port":checkAgentPort, \
                 "--cpu-relinquish-time":checkCpuRelinquishTime, \
                 "--cpu-relinquish-size":checkCpuRelinquishSize, \
                 "--failure-retry-count":checkFailureRetryCount, \
                 "--max-memory-usage":checkMaxMemoryUsage, \
                 "--retry-wait-time":checkRetryWaitTime, \
                 "--master-port":checkMasterPort, \
                 "--compression-level":checkCompressionLevel, \
                 "--compression-type":checkCompressionType, \
                 "--buffer-size":checkBufferSize, \
                 "--archive-xlog-buffer-size":checkArchiveBufferSize,\
                 "--buffer-block-size":checkBufferBlockSize, \
                 "--filesplit-size":checkFileSplitSize, \
                 "--restore-buffer-threshold":checkRestoreBufferThreshold, \
                 "--parallel-process":checkParallelProcess,\
                 "--cbm-recycle-level":checkCbmRecycleLevel,\
                 "--reader-thread-count":checkReaderCount, \
                 "--reader-thread-file-count":checkReaderThreadFileCount, \
                 "--reader-thread-file-size":checkReaderThreadFileSize}
    param_map_keys = param_map.keys()

    for key in final_ip_param_dict.keys():
        for k in final_ip_param_dict[key].keys():
            if k in param_map_keys:
                param_map[k](final_ip_param_dict[key][k])


