#!/usr/bin/env python
# -*- coding:utf-8 -*-
#############################################################################
#Copyright (c): 2020, Huawei Tech. Co., Ltd.
#FileName     : GSroachDelete.py
#Version      : V1.0.0
#Date         : 2020-01-20
#Description  : GSroachDelete.py is the main process for deleting backup data.
#############################################################################

try:
    import time
    import sys
    import os
    import commands
    import subprocess
    import signal
    import copy
    import socket
    from gspylib.common.ErrorCode import ErrorCode
    from ctypes import *
    sys.path.append(sys.path[0] + "/../")
    from gspylib.common.GaussLog import GaussLog
    from util.GSroachCommon import parseRoachConf, \
        makeRaochOptions, gsinitLogger, parseMaster_generated_file, master_ip, \
        master_host, gauss_home_loc, pssh_tout, remove_pattern, \
        pollSubprocess, getRoachOpResult, \
        sigint_handler, validate_logparams, proc_list_dict, backup_key, \
        final_ip_param_dict, proc_id_dict, sendSignal, \
        g_logger, validateGenerateHost, create_roachPath, createLogPath, \
        getMetaDest, postExecCleanup, getBackupDetails, getMaster_ip, \
        check_gs_roach_running, agentErrorReport
    from gspylib.os.gsfile import g_file
    from util.GSroachCommonParameter import validateRoachParams
except ImportError as err:
    sys.exit('Error: Unable to import module: ' + str(err))


#global variables
master_ip = ""
isDiskDst = False
bkp_key_list = list()
local_command_dict = dict()
ip_retCode_dict = dict()


signal.signal(signal.SIGINT, sigint_handler)


def logOptions():
    """
    writing to logfile
    """
    for key in final_ip_param_dict.keys():
        g_logger.debug("options for %s" %key)
        for key, val in final_ip_param_dict[key].items():
            g_logger.debug("%s : %s" %(key.translate(remove_pattern, "--"), val))

def createMasterOption():
    """
    create option for master
    """
    global final_ip_param_dict
    new_value = None
    final_value = ''
    for k in final_ip_param_dict.keys():
        if k != master_ip:
            continue
        for key, val in final_ip_param_dict[k].iteritems():
            if val != None:
                new_value = ''.join("%s %r " %(key, val))
            else:
                new_value = ''.join("%s " %(key))
            final_value += new_value
    return final_value

def deleteBackup(g_opts, command_dict, g_clusterInfo):
    """
    Do backup operation
    """
    g_logger.debug("start backup delete operation...")
    #removing the parameters not relevant to backup
    removeOptions()
    validateGenerateHost(final_ip_param_dict)
    deleteOnNodes(g_opts, command_dict, g_clusterInfo)
    g_logger.debug("End backup delete operation...")

def removeOptions():
    """
    remove unwanted option
    """
    global isDiskDst
    relevant = ['--media-type', '--media-destination', '--backup-key', '--metadata-destination', '--master-port',
                '--split-local-disk-storage', '--obs-server-ip', '--bucket-name', '--cluster-unique-id']
    for key in final_ip_param_dict.keys():
        for option in final_ip_param_dict[key].keys():
            if option not in relevant:
                del final_ip_param_dict[key][option]
        #Check whether is DISK delete
        if final_ip_param_dict[key]['--media-type'].lower() == "disk":
            isDiskDst = True

def getBackupPath(path, key):
    """
    create backup path
    """
    #For blank path, return blank only!
    if not path:
        return path
    if path.endswith('/'):
        backup_fil = '%sroach/%s' %(path, key)
    else:
        backup_fil = '%s/roach/%s' %(path, key)
    return backup_fil

def getMasterPID(master_pid_cmd, master_pid):
    """
    function to fetch master process id
    """
    pid_wait = 0
    while master_pid == '' and pid_wait < 5:
        time.sleep(1)
        master_pid_output = commands.getstatusoutput(master_pid_cmd)
        master_pid = master_pid_output[1]
        pid_wait = pid_wait + 1
    return master_pid

def updateBackupKeyStatus(g_opts, backupKey, status):
    """
    update backups.ini status as 'FAILED'/'DELETED'/'CORRUPTED'
    """
    needUpdate = True
    markOpt = ""

    #If metadata-destination not specified then don't send it,
    #gs_roach will try to resolve it from Env variable!
    metadata_opt_string = " "
    if g_opts.metadataDestination:
        metadata_opt_string = "--metadata-destination '%s'" % (g_opts.metadataDestination)

    #####################################################################
    #                  !! IMPORTANT SECURITY NOTE !!                    #
    # Regarding usage of subprocess.Popen([cmd], shell=True, bufsize=-1)#
    #  1.All the inputs from user are captured as individual options    #
    #  2.Direct inputs from the user are not passed in subprocess to    #
    #    gs_roach.                                                      #
    #  3.All the options are validated for the type as well as range    #
    #  4.The commands to be launched through subprocess are formed in   #
    #    code and passed as string to shell                             #
    #####################################################################

    if ("CORRUPTED" == status):
        markOpt = "--mark-corrupted"
    else:
        if getRoachOpResult():
            needUpdate = False

    if needUpdate:
        cmd = '%s/bin/gs_roach update %s --backup-key %s %s' \
            % (gauss_home_loc, markOpt, backupKey, metadata_opt_string)
        g_logger.debug("python call gs_roach update in deleteOnNodes, cmd: %s" % cmd)
        sub = subprocess.Popen([cmd], shell=True, bufsize=-1)
        sub.wait()

def deleteOnNodes(g_opts, command_dict, g_clusterInfo):
    """
    backup on each nodes
    """
    try:
        res_key = ""
        create_roachPath(master_ip, g_opts)

        for key in final_ip_param_dict.keys():
            if '--backup-key' in final_ip_param_dict[key].keys():
                res_key = final_ip_param_dict[key]['--backup-key']

        option = createMasterOption()
        cmd = 'source %s&&%s/bin/gs_roach --master generate %s' \
            % (g_opts.sourceFile, gauss_home_loc, option)

        g_logger.debug("python call gs_roach generate in deleteOnNodes, cmd: %s" % cmd)
        output = commands.getstatusoutput(cmd)
        if output[0] != 0:
            if not isDiskDst:
                raise Exception("ERROR : generate host details failed, maybe backup key not found.")
            else:
                raise Exception("ERROR : generate host details failed [%s]" %(output[1]))

        validateRoachParams(g_opts)
        master_generated_file = '%s/bin/roach_nodes.done' %(gauss_home_loc)
        cluster_node_details = {}
        if os.path.isfile(master_generated_file):
            tmp_val = parseMaster_generated_file(master_generated_file)
            cluster_node_details = tmp_val[1]
            global backup_key
            backup_key = tmp_val[0]
        else:
            sendSignal()
            raise Exception("Error : Could not fetch cluster node details from metadata, roach_nodes.done file not found")

        # Mark backup status to CORRUPTED before delete
        g_logger.debug("deleteOnNodes: updating backup status to CORRUPTED.")
        updateBackupKeyStatus(g_opts, res_key, "CORRUPTED")

        # Execute delete operation now
        g_logger.debug("Ready for delete operation after generate.")
        if isDiskDst:
            initateDeleteDisk(cluster_node_details, g_opts, command_dict)
        else:
            initateDelete(cluster_node_details, g_opts, command_dict)
    except Exception, e:
        GaussLog.exitWithError(str(e))

    while len(proc_list_dict) > 0:
        time.sleep(1)
        pollSubprocess(proc_list_dict, g_opts)

    if len(ip_retCode_dict) == 0 and len(proc_list_dict) == 0 and not getRoachOpResult():
        print "DELETE OPERATION SUCCESSFUL"

        # Mark backup status to DELETED after delete
        g_logger.debug("deleteOnNodes: updating backup status to DELETED.")
        updateBackupKeyStatus(g_opts, res_key, "DELETED")
    else:
        g_logger.logExit("DELETE OPERATION FAILED")

def makeStoredDataPaths(nodeInfo, backupKey):
    """
    build the real backup path when using the local split disk storage.
    we only get the parent dir path. for example, the stored path is
    '/data/hf/install_cluster1/V1R7C10/data/dn/roachbackup/20190910_164429/dn_6001_6002'
    the parent path is '/data/hf/install_cluster1/V1R7C10/data/dn/roachbackup/20190910_164429'.
    we only delete parent path.
    param: backupPath, it presents the backup media path
    """

    fileString = []
    storedDataPath = "../roachbackup/%s" % backupKey
    #get the CN data path
    for instanceInfo in nodeInfo.coordinators:
        oneSoftLinkPath = os.path.abspath(os.path.join(instanceInfo.datadir, storedDataPath))
        fileString.append(oneSoftLinkPath)
    #get the DN data path
    for instanceInfo in nodeInfo.datanodes:
        oneSoftLinkPath = os.path.abspath(os.path.join(instanceInfo.datadir, storedDataPath))
        fileString.append(oneSoftLinkPath)
    return fileString


# InitDeleteDisk origionally will loop with final_ip_param_dict.keys() and use Popen to delete 
# backup file in each Node. Origionally each node is cluster may have various metadata path. 
# Now we use same metadata path. If roach will allow put metadata in different metadata path in cluster
# or future developer want to trace what file to delete plwase checkout the version of this function before 2020/3/13
def initateDeleteDisk(cluster_node_details, g_opts, command_dict):
    """
    perform disk delete
    """
    g_logger.debug("enter initateDelete.")
    for node_ip, host in cluster_node_details.items():
        makeRaochOptions(g_opts, node_ip, host, command_dict)
    logOptions()
    checkBackupPath(final_ip_param_dict)

    backup_path = getBackupPath\
        (final_ip_param_dict[master_ip]['--media-destination'], \
        final_ip_param_dict[master_ip]['--backup-key'])
    metadata_path = getBackupPath\
        (final_ip_param_dict[master_ip]['--metadata-destination'], \
        final_ip_param_dict[master_ip]['--backup-key'])

    pNodeList = " -H ".join(final_ip_param_dict.keys())
    psshcmd = 'pssh -H %s rm -rf \"%s\" \"%s\"' %(pNodeList, backup_path, metadata_path)
    (status, output) = commands.getstatusoutput(psshcmd)
    if status != 0:
        g_logger.logExit ("delete disk file failed output is : %s", output)

def initateDelete(cluster_node_details, g_opts, command_dict):
    """
    perform delete for NBU/OBS backups
    """

    g_logger.debug("enter initateDeleteNBU/OBS.")
    for node_ip, host in cluster_node_details.items():
        makeRaochOptions(g_opts, node_ip, host, command_dict)
    logOptions()
    option = createMasterOption()
    g_opts.nodeList = final_ip_param_dict.keys()
    for node_ip in final_ip_param_dict.keys():
        if node_ip == master_ip:
            cmd = 'source %s&&%s/bin/gs_roach --master delete --master-ip \"%s\" %s' \
                % (g_opts.sourceFile, gauss_home_loc, master_ip, option)

            #####################################################################
            #                  !! IMPORTANT SECURITY NOTE !!                    #
            # Regarding usage of subprocess.Popen([cmd], shell=True, bufsize=-1)#
            #  1.All the inputs from user are captured as individual options    #
            #  2.Direct inputs from the user are not passed in subprocess to    #
            #    gs_roach.                                                      #
            #  3.All the options are validated for the type as well as range    #
            #  4.The commands to be launched through subprocess are formed in   #
            #    code and passed as string to shell                             #
            #####################################################################

            g_logger.debug("python call gs_roach delete in initateDeleteNBU on master, cmd: %s" % cmd)
            master_proc = subprocess.Popen([cmd], shell=True, bufsize=-1)
            proc_list_dict.update({master_ip:master_proc})
            master_pid_cmd = 'ps -aef | grep %s | grep \"%s/bin/gs_roach --master delete --master-ip %s\" | grep -v grep | awk {\'print $2\'}' \
                % (g_opts.user, gauss_home_loc, master_ip)
            master_pid = ''
            master_pid = getMasterPID(master_pid_cmd, master_pid)
            if master_pid:
                proc_id_dict.update({master_ip:master_pid})
        else:
            cmd = 'source %s&&pssh -t %s -H \"%s\" -P \"source %s&&%s/bin/gs_roach\" --master-ip \\"%s\\" %s "2>&1"' \
                % (g_opts.sourceFile, pssh_tout, node_ip, g_opts.sourceFile, gauss_home_loc, master_ip, option)
            proc_pid = '-1'
            pid_cmd = 'source %s&&pssh -t %s -H \"%s\" -P ps -aef | grep %s | grep \"%s/bin/gs_roach --master-ip %s\" | grep -v grep | awk {\'print $2\'}' \
                % (g_opts.sourceFile, pssh_tout, node_ip, g_opts.user, gauss_home_loc, master_ip)

            #####################################################################
            #                  !! IMPORTANT SECURITY NOTE !!                    #
            # Regarding usage of subprocess.Popen([cmd], shell=True, bufsize=-1)#
            #  1.All the inputs from user are captured as individual options    #
            #  2.Direct inputs from the user are not passed in subprocess to    #
            #    gs_roach.                                                      #
            #  3.All the options are validated for the type as well as range    #
            #  4.The commands to be launched through subprocess are formed in   #
            #    code and passed as string to shell                             #
            #####################################################################

            g_logger.debug("python call gs_roach delete in initateDelete on agent, cmd: %s" % cmd)
            agent_proc = subprocess.Popen([cmd], stdout=subprocess.PIPE, \
                         stderr=subprocess.PIPE, shell=True, bufsize=-1)
            proc_list_dict.update({node_ip:agent_proc})
            proc_for_pid = subprocess.Popen([pid_cmd], \
            stdout = subprocess.PIPE, \
            stderr = subprocess.PIPE, shell=True, bufsize=-1)
            proc_output = proc_for_pid.stdout.readlines()
            if proc_output:
                proc_pid = proc_output[0].strip()
            if proc_pid.isdigit() and (proc_pid != '-1'):
                proc_id_dict.update({node_ip:proc_pid})

def checkBackupPath(final_ip_param_dict):
    """
    check for backup path existence
    """
    for node_ip in final_ip_param_dict.keys():
        if node_ip == master_ip:
            backup_path = getBackupPath\
                (final_ip_param_dict[node_ip]['--media-destination'], \
                final_ip_param_dict[node_ip]['--backup-key'])
            if not os.path.isdir(backup_path):
                g_logger.log("Warning : %s doesn't exist on %s" % (backup_path, node_ip))
        else:
            backup_path = getBackupPath\
                (final_ip_param_dict[node_ip]['--media-destination'], \
                final_ip_param_dict[node_ip]['--backup-key'])
            check_cmd = 'pssh -t %s -H \"%s\" -P "ls %s"' % (pssh_tout, node_ip, backup_path)
            check_result = commands.getstatusoutput(check_cmd)
            if check_result[0] != 0:
                g_logger.log("Warning : %s doesn't exist on %s" % (backup_path, node_ip))

def checkAgentNode(command_dict, final_ip_param_dict):
    """
    check if the current node is agent or master before starting delete operation
    """
    try:
        metaDest = getMetaDest(final_ip_param_dict)
        if '--backup-key' in command_dict.keys():
            bkpKeyPath = os.path.join(metaDest, "roach", command_dict['--backup-key'])
            if os.path.isdir(bkpKeyPath):
                if os.path.isfile(os.path.join(bkpKeyPath, "roach_master.metadata")):
                    g_logger.debug("Current node is the master node.")
                    return True
                else:
                    if (len(bkp_key_list) == 1):
                        raise Exception( "Error: Such failed backup cannot be deleted." )
                    g_logger.log("Warning: Such failed backup cannot be deleted.")
                    return False
            else:
                errorString = ErrorCode.GAUSS_534["GAUSS_53404"] % command_dict['--backup-key']
                errorString = "%s in metadata destination ['%s'] or backup type is Logical Multi-tables where delete is not supported." \
                % (errorString, metaDest)
                agentErrorReport("roach_delete", errorString)
                g_logger.log("Error: %s" % errorString)
                return False
    except OSError, oserr:
        g_logger.logExit(str(oserr))

def getChildKeys(bkp_details, temp_bkp_key):
    """
    function: to collect all backup keys in Depth First Search order
    """
    for item in bkp_details:
        if "PriorBackupKey" in item.keys() and item["PriorBackupKey"] == temp_bkp_key:
            if item["STATUS"] != "DELETED":
                bkp_key_list.append(str(item["BackupKey"]))
            getChildKeys(bkp_details, str(item["BackupKey"]))

def checkDescendantBackup(l_command_dict, bkp_details):
    """
    function: to check if the backup is having any descendants.
    """
    parent_key = l_command_dict['--backup-key']
    bkp_key_list.append(parent_key) 
    getChildKeys(bkp_details, parent_key)
    g_logger.debug("Descendant backup key list: %s" % bkp_key_list)
    if len(bkp_key_list) > 1 and "--cascade" not in l_command_dict.keys():
        g_logger.log("Error: Specified backup is having descendant backups.")
        g_logger.log("Total descendant backups: [%s]" % str(len(bkp_key_list[1:])))
        g_logger.log(", ".join(bkp_key_list[1:]))
        g_logger.logExit("Provide '--cascade' option to delete all descendant backups.")

def changeDeleteParams(g_opts, bkp_details, bkp_key, local_command_dict):
    """
    function: to change the parameters for each backup-key
    """
    local_command_dict["--backup-key"] = bkp_key
    g_opts.backupKey = bkp_key
    for node_ip in final_ip_param_dict.keys():
        for item in bkp_details:
            if item["BackupKey"] == bkp_key:
                final_ip_param_dict[node_ip]["--backup-key"] = str(bkp_key)

                if os.path.realpath(final_ip_param_dict[node_ip]["--media-destination"]) != os.path.realpath(item["MediaDestination"]):
                    g_logger.log("Warning: Provided media destination [%s] is incorrect, using correct media destination [%s]." \
                        % (final_ip_param_dict[node_ip]["--media-destination"], item["MediaDestination"]))                
                local_command_dict["--media-destination"] = str(item["MediaDestination"])
                final_ip_param_dict[node_ip]["--media-destination"] = str(item["MediaDestination"])
                g_opts.mediaDest = str(item["MediaDestination"])

                local_command_dict["--media-type"] = str(item["MediaType"])
                final_ip_param_dict[node_ip]["--media-type"] = str(item["MediaType"])
                g_opts.mediaType = str(item["MediaDestination"])
                if "--cascade" in local_command_dict.keys():
                    del final_ip_param_dict[node_ip]["--cascade"]
                    g_opts.cascade = ""
                    del local_command_dict["--cascade"]


def startDeleteBackup(g_opts, command_dict, g_clusterInfo):
    """
    call for delete backup functionality
    """
    global master_ip
    global local_command_dict

    master_ip = getMaster_ip()
    local_command_dict = copy.deepcopy(command_dict)
    if g_opts.roachConf:
        parseRoachConf(g_opts)
    makeRaochOptions(g_opts, master_ip, master_host, command_dict)
    validate_logparams()

    global g_logger
    g_logger = gsinitLogger(g_opts)
    g_logger.log("Start to delete a backup.")

    check_gs_roach_running(g_opts)

    validateRoachParams(g_opts)
    ## check added to validate if the current node is master node
    bkp_details = getBackupDetails(command_dict, final_ip_param_dict)
    checkDescendantBackup(command_dict, bkp_details)
    total_bkp_keys = len(bkp_key_list)
    count = 1
    for bkp_key in bkp_key_list[::-1]:
        if total_bkp_keys > 1:
            g_logger.log("\nDeleting backup-key: %s\t[%s/%s]" \
                % (bkp_key, str(count), str(total_bkp_keys)))
            count += 1
        changeDeleteParams(g_opts, bkp_details, bkp_key, local_command_dict)
        if not checkAgentNode(local_command_dict, final_ip_param_dict):
            continue
        deleteBackup(g_opts, local_command_dict, g_clusterInfo)
        if getRoachOpResult():
            g_logger.log("Error: Delete operation failed for backup-key: %s" % bkp_key)
    if not isDiskDst:
        postExecCleanup(g_opts, g_logger)
    return getRoachOpResult()
