#!/usr/bin/env python
# -*- coding:utf-8 -*-
#############################################################################
#Copyright (c): 2020, Huawei Tech. Co., Ltd.
#FileName     : GSroachLocal.py
#Version      : V1.0.0
#Date         : 2020-01-20
#Description  : GSroachLocal.py contains the functionality to be executed locally on all the machines.
#############################################################################

try:
    import sys
    import os
    import getopt
    import commands
    import socket
    import time
    import re
    import json
    import shutil
    import StringIO
    import ConfigParser
    import glob
    import subprocess
    import pipes
    from threading import Timer, Thread, current_thread
    sys.path.append(sys.path[0] + "/../")
    from gspylib.common.Common import DefaultValue, ClusterInstanceConfig
    from gspylib.common.DbClusterInfo import dbClusterInfo
    from gspylib.common.GaussLog import GaussLog
    from gspylib.common.ErrorCode import ErrorCode
    from util.GSroachCommon import gauss_home_loc, parseMaster_generated_file
    from gspylib.os.gsfile import g_file
    from multiprocessing.dummy import Pool as ThreadPool
    from util.GSroachCommon import isDirectoryHasFile, checkFile, TIMEOUT_SCP, \
        SCRIPT_ROACH_LOCAL, getNodesInfoByUpdatedFile
    import copy
except ImportError as err:
    sys.exit('Error: Unable to import module: ' + str(err))


#global variables
g_opts = None
g_logger = None
g_clusterInfo = None
g_dbNodeInfo = None
g_remoteClusterInfo = None

#global consts
ACTION_STOP = "stop"
ACTION_STOP_PITR = "stop-pitr"
ACTION_WAIT_PITR = "wait-pitr"
ACTION_WAIT_PITR_RESTART = "wait_pitr_restart"
ACTION_GET_BACKUP_KEY = "get_backup_key"
ACTION_PAUSE = "pause"
ACTION_PRECHECK_PROCESS = "precheck"
ACTION_CONFRESTORE = "confrestore"
ACTION_DELETECONFNEW = "deleteconfnew"
ACTION_COPYNBCNFILES = "copybackupedfiles"
ACTION_COPYNBDNFILES = "copydnbackupedfiles"
ACTION_BACKUPCFUNC = "backupcfuncfiles"
ACTION_BACKUPTSDICT = "backuptsdictfiles"
ACTION_RESTORECFUNC = "restorecfuncfiles"
ACTION_RESTORETSDICT = "restoretsdictfiles"
ACTION_RESTORELIBS = "restorelibfiles"
ACTION_DEALTBLSPC = "dealtblspc"
ACTION_CHECKPREFIXFILE = "checkprefixfile"
ACTION_UPLOAD_META = "uploadmetadata"
ACTION_DOWNLOAD_META = "downloadmetadata"
CN_TYPE = "coordinator"
DN_TYPE = "datanode"
GTM_TYPE = "gtm"
CM_TYPE = "cm"
ROW_FILE_TYPE = 'data'


postgres_keys = None
gtm_keys = ['listen_addresses', 'port', 'log_directory', 'active_host', 'active_port', \
    'local_host', 'local_port', 'alarm_component']
cm_server_keys = ['alarm_component', 'log_dir','enable_transaction_read_only']
cm_agent_keys = ['alarm_component', 'log_dir']
g_confDict = {}

MASTER_INSTANCE = 0
STANDBY_INSTANCE = 1
DUMMY_STANDBY_INSTANCE = 2

CONNECTTIMEOUT = 30
BACKUPFILEMODE = 0o755
LoaclIp = DefaultValue.GetHostIpOrName()

class CmdOptions():
    """
    command-line options
    """
    def __init__(self):
        self.action = ""
        self.user = ""
        self.logFile = ""
        self.mpprcFile = ""
        self.forceStop = False
        self.checkObj = "backup"
        self.needCopyCol = False
        self.instType = ""
        self.primaryInstId = ""
        self.primaryInstIp = ""
        self.primaryInstDataNBPath = ""
        self.primaryInstArcNBPath = ""
        self.primaryInstSrvNBPath = ""
        self.primaryInstOldMediaPath = ""
        self.primaryInstNewMediaPath = ""
        self.standbyInstNewMediaPath = ""
        self.standbyInstNewMetaPath = ""
        self.backupKey = ""
        self.restoreNewCluster = False
        self.oldClusterHostname = ""
        self.remoteMasterIp = ""
        self.clusterUniqueID = 0
        self.bucketName = ""
        self.OBSServerIp = ""
        self.mediaType = ""
        self.metaData = ""
        self.remoteMetaData = ""
        self.sourceFile = ""
        self.resumeRst = False
        self.generateLink = False
        self.masterIp = ""
        self.miniaturized = False
        self.primaryStandbys = False
        self.filePath = ""
        self.pitr = False

def usage():
    """
    GSroachLocal.py will execute the functionality locally on the node
    """
    print(usage.__doc__)

def parseCommandLine():
    """
    Parse command line and save to global variables
    """
    try:
        opts, args = getopt.getopt(
            sys.argv[1:],
            "Ft:U:l:",
            [
                "help", "checkObj=", "primaryInstId=",
                "primaryInstIp=", "primaryInstDataNBPath=",
                "primaryInstArcNBPath=", "primaryInstSrvNBPath=",
                "primaryInstOldMediaPath=", "primaryInstNewMediaPath=",
                "standbyInstNewMediaPath=", "standbyInstNewMetaPath=",
                "backupKey=", "restore-new-cluster", "oldClusterHostname=",
                "resume-restore", "needCopyCol", "parallel-restore",
                "instType=", "remoteMasterIp=", "clusterUniqueID=",
                "mediaType=", "metaData=", "remoteMetaData=",
                "sourceFile=", "generate-link", "masterIp=",
                "miniaturized", "primary-standbys", "filePath=",
                "pitr", "obs-server-ip=", "bucket-name="
            ])

    except (OSError, IOError) as err:
        usage()
        GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50000"] % str(err))

    if len(args) > 0:
        GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50000"] % str(args[0]))

    parameter_bool_map = {"--needCopyCol": False, "-F": False, "--restore-new-cluster": False,
                          "--resume-restore": False, "--parallel-restore": False,
                          "--generate-link": False, "--miniaturized": False,
                          "--primary-standbys": False, "--pitr": False, "--cluster-migration":False}
    parameter_bool_keys = parameter_bool_map.keys()

    parameter_value_map = {"-t": "", "-U": "", "-l": "", "--checkObj": "backup", "--oldClusterHostname": "",
                           "--primaryInstId": "", "--primaryInstIp": "", "--primaryInstDataNBPath": "",
                           "--primaryInstArcNBPath": "", "--primaryInstSrvNBPath": "", "--primaryInstOldMediaPath": "",
                           "--primaryInstNewMediaPath": "", "--standbyInstNewMediaPath": "", "--standbyInstNewMetaPath": "",
                           "--backupKey": "", "--instType": "", "--remoteMasterIp": "", "--clusterUniqueID": "", "--mediaType": "",
                           "--metaData": "", "--remoteMetaData": "", "--sourceFile": "", "--masterIp": "", "--filePath": "",
                           "--bucket-name": "", "--obs-server-ip": ""}
    parameter_value_keys = parameter_value_map.keys()

    global g_opts
    g_opts = CmdOptions()

    for (key, value) in opts:
        if key == "--help":
            usage()
            sys.exit(0)
        elif key in parameter_value_keys:
            parameter_value_map[key] = value
        elif (key in parameter_bool_keys):
            parameter_bool_map[key] = True
            
    g_opts.action = parameter_value_map["-t"]
    g_opts.user = parameter_value_map["-U"]
    g_opts.logFile = parameter_value_map["-l"]
    g_opts.checkObj = parameter_value_map["--checkObj"]
    g_opts.primaryInstId = parameter_value_map["--primaryInstId"]
    g_opts.primaryInstIp = parameter_value_map["--primaryInstIp"]
    g_opts.primaryInstDataNBPath = parameter_value_map["--primaryInstDataNBPath"]
    g_opts.primaryInstArcNBPath = parameter_value_map["--primaryInstArcNBPath"]
    g_opts.primaryInstSrvNBPath = parameter_value_map["--primaryInstSrvNBPath"]
    g_opts.primaryInstOldMediaPath = parameter_value_map["--primaryInstOldMediaPath"]
    g_opts.primaryInstNewMediaPath = parameter_value_map["--primaryInstNewMediaPath"]
    g_opts.standbyInstNewMediaPath = parameter_value_map["--standbyInstNewMediaPath"]
    g_opts.standbyInstNewMetaPath = parameter_value_map["--standbyInstNewMetaPath"]
    g_opts.backupKey = parameter_value_map["--backupKey"]
    g_opts.instType = parameter_value_map["--instType"]
    g_opts.forceStop = parameter_bool_map["-F"]
    g_opts.needCopyCol = parameter_bool_map["--needCopyCol"]
    g_opts.restoreNewCluster = parameter_bool_map["--restore-new-cluster"]
    g_opts.parallelRestore = parameter_bool_map["--parallel-restore"]
    g_opts.oldClusterHostname = parameter_value_map["--oldClusterHostname"]
    g_opts.resumeRst = parameter_bool_map["--resume-restore"]
    g_opts.remoteMasterIp = parameter_value_map["--remoteMasterIp"]
    g_opts.clusterUniqueID = parameter_value_map["--clusterUniqueID"]
    g_opts.bucketName = parameter_value_map["--bucket-name"]
    g_opts.OBSServerIp = parameter_value_map["--obs-server-ip"]
    g_opts.mediaType = parameter_value_map["--mediaType"]
    g_opts.metaData = parameter_value_map["--metaData"]
    g_opts.remoteMetaData = parameter_value_map["--remoteMetaData"]
    g_opts.sourceFile = parameter_value_map["--sourceFile"]
    g_opts.generateLink = parameter_bool_map["--generate-link"]
    g_opts.masterIp = parameter_value_map["--masterIp"]
    g_opts.miniaturized = parameter_bool_map["--miniaturized"]
    g_opts.primaryStandbys = parameter_bool_map["--primary-standbys"]
    g_opts.filePath = parameter_value_map["--filePath"]
    g_opts.pitr = parameter_bool_map["--pitr"]

def checkActionParameter():
    """
    check action parameter
    """
    # check required parameters
    if g_opts.action == "":
        usage()
        GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50001"] % "t" +".")
    elif g_opts.action in [ACTION_STOP, ACTION_WAIT_PITR, ACTION_WAIT_PITR_RESTART, \
            ACTION_GET_BACKUP_KEY, ACTION_STOP_PITR, ACTION_PRECHECK_PROCESS, \
            ACTION_CONFRESTORE, ACTION_DELETECONFNEW, ACTION_COPYNBCNFILES, \
            ACTION_COPYNBDNFILES, ACTION_BACKUPCFUNC, ACTION_RESTORECFUNC, \
            ACTION_RESTORELIBS, ACTION_DEALTBLSPC, ACTION_BACKUPTSDICT, \
            ACTION_RESTORETSDICT, ACTION_CHECKPREFIXFILE, ACTION_UPLOAD_META, \
            ACTION_DOWNLOAD_META,ACTION_PAUSE]:
        pass
    else:
        GaussLog.exitWithError("Invalid action : %s." % g_opts.action)

def checkUserParameter():
    """
    check user parameter
    """
    #check user
    if g_opts.user == "":
        GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50001"] % "U" + ".")
    try:
        DefaultValue.checkUser(g_opts.user, False)
    except (OSError, IOError) as err:
        GaussLog.exitWithError(str(err))

def checkSourceFileParameter():
    """
    check source file parameter
    """
    #check mpprc file path
    g_opts.mpprcFile = DefaultValue.getEnv(DefaultValue.MPPRC_FILE_ENV)
    if g_opts.mpprcFile == None:
        g_opts.mpprcFile = ""
    if g_opts.mpprcFile != "":
        if not os.path.exists(g_opts.mpprcFile):
            GaussLog.exitWithError(ErrorCode.GAUSS_502["GAUSS_50201"] % g_opts.mpprcFile)
        if not os.path.isabs(g_opts.mpprcFile):
            GaussLog.exitWithError(ErrorCode.GAUSS_512["GAUSS_51206"] % g_opts.mpprcFile)

def checkLogFileParameter():
    """
    check log file parameter
    """
    # check log file
    if g_opts.logFile == "":
        g_opts.logFile = DefaultValue.getOMLogPath(DefaultValue.DEFAULT_LOG_FILE, g_opts.user, "")
    if not os.path.isabs(g_opts.logFile):
        GaussLog.exitWithError(ErrorCode.GAUSS_502["GAUSS_50213"] % "log")

def checkParameter():
    """
    Check parameter from command line
    """
    checkActionParameter()
    checkUserParameter()
    checkSourceFileParameter()
    checkLogFileParameter()

def initGlobals():
    """
    init global variables
    """
    global g_logger
    global g_clusterInfo
    global g_dbNodeInfo
    g_logger = GaussLog(g_opts.logFile, g_opts.action)
    g_clusterInfo = dbClusterInfo()
    g_clusterInfo.initFromStaticConfig(g_opts.user)

    hostName = DefaultValue.GetHostIpOrName()
    g_dbNodeInfo = g_clusterInfo.getDbNodeByName(hostName)
    if g_dbNodeInfo is None:
        g_logger.logExit("Failed to obtain local instance information. There is no host named: %s." % hostName)
    #get keysList of postgresql.conf
    getPostgresKeysList()

def getPostgresKeysList():
    global postgres_keys
    postgres_keys = ['listen_addresses', 'local_bind_address', 'port', 'archive_command', 'archive_mode',\
                     'log_directory','audit_directory', 'alarm_component', 'comm_sctp_port', 'comm_control_port',\
                     'pgxc_node_name']
    #get peerinfo of current Datenode
    gtmCount = 0
    count = 0
    for dbnode in g_clusterInfo.dbNodes:
        if (len(dbnode.gtms) != 0):
            if (gtmCount == 0):
                postgres_keys.append('gtm_host')
                postgres_keys.append('gtm_port')
            else:
                gtm_repl_host = 'gtm_host' + str(gtmCount)
                gtm_repl_port = 'gtm_port' + str(gtmCount)
                postgres_keys.append(gtm_repl_host)
                postgres_keys.append(gtm_repl_port)
            gtmCount = gtmCount + 1
            
    for dnInst in g_dbNodeInfo.datanodes:
        peerInsts = g_clusterInfo.getPeerInstance(dnInst)
        for i in range(len(peerInsts)):
            count = i + 1
            repl = 'replconninfo' + str(count)
            postgres_keys.append(repl)
        break
 

def renameFileInDir(dirname, oldname, newname, copyflag = False):
    """
    change name of file in the dir to new name
    """
    g_logger.debug("Renaming file from %s to %s under %s" % (oldname, newname, dirname))
    oldname = os.path.join(dirname, oldname)
    if os.path.exists(oldname):
        newname = os.path.join(dirname, newname)
        if copyflag:
            shutil.copy(oldname, newname)
        else:
            os.rename(oldname, newname)

def getCnConfValue():
    """
    check GUC parameters from current cn postgresql.conf
    """
    global g_confDict
    instanceDict = {}
    #get coordinator values
    for cooInst in g_dbNodeInfo.coordinators:
        #Rename restored conf file and bring in New Cluster file for reading
        #Not needed with fresh design - "clean" will ensure "conf" and "conf.new" are already same!
        #Copy other conf files also to backup
        #Not needed with fresh design - "clean" will ensure "conf" and "conf.new" are already same!
        for item in postgres_keys:
            cmd = "gs_guc check -N %s -Z %s -D %s -c \"%s\"" % (cooInst.hostname, CN_TYPE, cooInst.datadir, item)
            (status, output) = commands.getstatusoutput(cmd)
            if(status != 0):
                g_logger.logExit("Failed to check CN guc parameter. Error: \n%s." % output)
            lines = [x.strip() for x in output.split('\n') if x.find(item + '=') != -1]
            val = lines[0].split('=',1)[1]
            instanceDict[item]=val
        g_confDict[cooInst.instanceId]=instanceDict
        instanceDict = {}

def getAZ():
    AZList = []
    for dbNode in g_clusterInfo.dbNodes:
        AZList.append(dbNode.azName)
    AZStr = ",".join(str(i) for i in set(AZList))
    return AZStr

def getDnConfValue():
    """
    check GUC parameters from current dn postgresql.conf
    """
    global g_confDict
    instanceDict = {}
    postgres_keys.append('synchronous_standby_names')

    if g_opts.primaryStandbys:
        postgres_keys.append('application_name')

    for dnInst in g_dbNodeInfo.datanodes:
        #Rename restored conf file and bring in New Cluster file for reading
        #Not needed with fresh design - "clean" will ensure "conf" and "conf.new" are already same!
        #Copy other conf files also to backup
        #Not needed with fresh design - "clean" will ensure "conf" and "conf.new" are already same!
        azStr = getAZ()
        for item in postgres_keys:
            cmd = "gs_guc check -N %s -Z %s -D %s -c \"%s\"" % (dnInst.hostname, DN_TYPE, dnInst.datadir, item)
            (status, output) = commands.getstatusoutput(cmd)
            if(status != 0):
                g_logger.logExit("Failed to check DN guc parameter. Error: \n%s." % output)
            lines = [x.strip() for x in output.split('\n') if x.find(item + '=') != -1]
            val = lines[0].split('=',1)[1]
            if item == 'synchronous_standby_names':
                if (val.find('ANY') != -1) or (val.find('FIRST') != -1):
                    quorumStandbyType = val.split(' ')[0][1:]
                    syncStandbyNum = int(val.split(' ')[1].split('(')[0])
                    val = '\'' + quorumStandbyType + ' ' + str(syncStandbyNum) + '(' + azStr + ')\''
   
            instanceDict[item]=val
               
        g_confDict[dnInst.instanceId]=instanceDict
        instanceDict = {}

def getGtmConfValue():
    """
    check GUC parameter from current gtm gtm.conf
    """
    global g_confDict
    instanceDict = {}
    for gtmInst in g_dbNodeInfo.gtms:
        #Copy other conf files also to backup
        #Not needed with fresh design - "clean" will ensure "conf" and "conf.new" are already same!
        #Fetching input values from "gtm.conf.new" using a dummy section
        config = StringIO.StringIO()
        config.write('[dummysection]\n')
        config.write(open(os.path.join(gtmInst.datadir, "gtm.conf.new")).read())
        config.seek(0, os.SEEK_SET)
        cp = ConfigParser.ConfigParser()
        cp.readfp(config)
        for item in gtm_keys:
            try:
                val = cp.get('dummysection',item).replace('\t', '#').split('#')[0].strip()
            except ConfigParser.NoOptionError:
                val = 'NULL'
            instanceDict[item]=val
        g_confDict[gtmInst.instanceId]=instanceDict
        instanceDict = {}

def getCmsConfValue():
    """
    check parameter from current cms cm_server.conf
    """
    global g_confDict
    instanceDict = {}
    for cmsInst in g_dbNodeInfo.cmservers:
        #Copy other conf files also to backup
        #Not needed with fresh design - "clean" will ensure "conf" and "conf.new" are already same!
        #Fetching input values from "cm_server.conf.new" using a dummy section
        config = StringIO.StringIO()
        config.write('[dummysection]\n')
        config.write(open(os.path.join(cmsInst.datadir, "cm_server.conf.new")).read())
        config.seek(0, os.SEEK_SET)
        cp = ConfigParser.ConfigParser()
        cp.readfp(config)
        for item in cm_server_keys:
            try:
                val = cp.get('dummysection',item).replace('\t', '#').split('#')[0].strip()
            except ConfigParser.NoOptionError:
                val = 'NULL'
            instanceDict[item]=val
        g_confDict[cmsInst.instanceId]=instanceDict
        instanceDict = {}

def getCmaConfValue():
    """
    check parameter from current cma cm_agent.conf
    """
    global g_confDict
    instanceDict = {}
    for cmsInst in g_dbNodeInfo.cmagents:
        #Copy other conf files also to backup
        #Not needed with fresh design - "clean" will ensure "conf" and "conf.new" are already same!
        #Fetching input values from "cm_agent.conf.new" using a dummy section
        config = StringIO.StringIO()
        config.write('[dummysection]\n')
        config.write(open(os.path.join(cmsInst.datadir, "cm_agent.conf.new")).read())
        config.seek(0, os.SEEK_SET)
        cp = ConfigParser.ConfigParser()
        cp.readfp(config)
        for item in cm_agent_keys:
            try:
                val = cp.get('dummysection',item).replace('\t', '#').split('#')[0].strip()
            except ConfigParser.NoOptionError:
                val = 'NULL'
            instanceDict[item]=val
        g_confDict[cmsInst.instanceId]=instanceDict
        instanceDict = {}

def setCnConfValue():
    """
    set cn val's in restored one
    """
    instanceDict = {}
    for cooInst in g_dbNodeInfo.coordinators:
        #Backup the New Cluster file and bring in the restored file for updating
        renameFileInDir(cooInst.datadir, "postgresql.conf.backup", "postgresql.conf", True)
        #Set the saved value in restored conf file
        instanceDict = g_confDict[cooInst.instanceId]
        for item, val in instanceDict.iteritems():
            if val.find('NULL') == 0:
                #Generate commented item
                cmd = "gs_guc set -N %s -Z %s -D %s -c \"%s\"" % (cooInst.hostname, CN_TYPE, cooInst.datadir, item)
            else:
                cmd = "gs_guc set -N %s -Z %s -D %s -c \"%s = %s\"" % (cooInst.hostname, CN_TYPE, cooInst.datadir, item, val)
            g_logger.debug("Set guc parameter in setCnConfValue, cmd: %s" % cmd)
            (status, output) = DefaultValue.retryGetstatusoutput(cmd, 10)
            if(status != 0):
                if item == 'archive_command' and 'is not empty' in output:
                    #Handling dir exist error
                    fullName = output.split('"')[1]
                    parent = os.path.dirname(fullName)
                    dirName = os.path.basename(fullName)
                    preCmd = "cd %s; rm -rf roach_archive_temp; mv %s roach_archive_temp" % (parent, dirName)
                    postCmd = "cd %s; mv roach_archive_temp %s" % (parent, dirName)
                    cmd = "%s; %s; %s" % (preCmd, cmd, postCmd)
                    g_logger.debug("deleting file in setCnConfValue, cmd: %s" % cmd)
                    (status, output) = commands.getstatusoutput(cmd)
                    if(status != 0):
                        g_logger.logExit("Failed to set coordinator parameter. Error: \n%s." % output)
                else:
                    g_logger.logExit("Failed to set coordinator parameter. Error: \n%s." % output)

def setDnConfValue():
    """
    set dn val's in restored one
    """
    instanceDict = {}
    for dnInst in g_dbNodeInfo.datanodes:
        #Backup the New Cluster file and bring in the restored file for updating
        renameFileInDir(dnInst.datadir, "postgresql.conf.backup", "postgresql.conf", True)
        #Set the saved value in restored conf file
        instanceDict = g_confDict[dnInst.instanceId]
        for item, val in instanceDict.iteritems():
            if (item == 'synchronous_standby_names') and (val.find('*') != -1):
                continue
            if val.find('NULL') == 0:
                #Generate commented item
                cmd = "gs_guc set -N %s -Z %s -D %s -c \"%s\"" % (dnInst.hostname, DN_TYPE, dnInst.datadir, item)
            else:
                cmd = "gs_guc set -N %s -Z %s -D %s -c \"%s = %s\"" % (dnInst.hostname, DN_TYPE, dnInst.datadir, item, val)
            g_logger.debug("Set guc parameter in setDnConfValue, cmd: %s" % cmd)
            (status, output) = DefaultValue.retryGetstatusoutput(cmd, 10)
            if(status != 0):
                if item == 'archive_command' and 'is not empty' in output:
                    #Handling dir exist error
                    fullName = output.split('"')[1]
                    parent = os.path.dirname(fullName)
                    dirName = os.path.basename(fullName)
                    preCmd = "cd %s; rm -rf roach_archive_temp; mv %s roach_archive_temp" % (parent, dirName)
                    postCmd = "cd %s; mv roach_archive_temp %s" % (parent, dirName)
                    cmd = "%s; %s; %s" % (preCmd, cmd, postCmd)
                    g_logger.debug("deleting file in setDnConfValue, cmd: %s" % cmd)
                    (status, output) = commands.getstatusoutput(cmd)
                    if(status != 0):
                        g_logger.logExit("Failed to set datanode parameter. Error: \n%s." % output)
                else:
                    g_logger.logExit("Failed to set datanode parameter. Error: \n%s." % output)

def setGtmConfValue():
    """
    set gtm val's in restored one
    """
    instanceDict = {}
    for gtmInst in g_dbNodeInfo.gtms:
        #Backup the New Cluster file and bring in the restored file for updating
        renameFileInDir(gtmInst.datadir, "gtm.conf.backup", "gtm.conf", True)
        #Set the saved value in restored conf file
        instanceDict = g_confDict[gtmInst.instanceId]
        for item, val in instanceDict.iteritems():
            if val.find('NULL') == 0:
                #Generate commented item
                cmd = "gs_guc set -N %s -Z %s -D %s -c \"%s\"" % (gtmInst.hostname, GTM_TYPE, gtmInst.datadir, item)
            else:
                cmd = "gs_guc set -N %s -Z %s -D %s -c \"%s = %s\"" % (gtmInst.hostname, GTM_TYPE, gtmInst.datadir, item, val)
            g_logger.debug("Set guc parameter in setGtmConfValue, cmd: %s" % cmd)
            (status, output) = DefaultValue.retryGetstatusoutput(cmd, 10)
            if(status != 0):
                g_logger.logExit("Failed to set gtm parameter. Error: \n%s." % output)

def setCmaConfValue():
    """
    set cma val's in restored one
    """
    nonNullItems = {}
    instanceDict = {}
    for cmsInst in g_dbNodeInfo.cmservers:
        #Copy other conf files also to backup
        renameFileInDir(cmsInst.datadir, "cm_server.conf.backup", "cm_server.conf", True)
        #Set the saved value in restored conf file
        instanceDict = g_confDict[cmsInst.instanceId]
        configFile = os.path.join(cmsInst.datadir, "cm_server.conf")
        for item, val in instanceDict.iteritems():
            if val.find('NULL') == 0:
                #Handle commented items
                cmd = "sed -i 's/^.*\(%s.*=.*\)/#\\1/g' %s" % (item, configFile)
                (status, output) = commands.getstatusoutput(cmd)
                if (status != 0):
                    raise Exception(ErrorCode.GAUSS_500["GAUSS_50008"] + " Error: \n%s" % output)
            else:
                nonNullItems[item] = val
        #Handle non-commented items
        g_logger.debug("Set guc parameter using setConfigItem in setCmaConfValue, path: %s" % configFile)
        ClusterInstanceConfig.setConfigItem(DefaultValue.INSTANCE_ROLE_CMSERVER, cmsInst.datadir, configFile, nonNullItems)

def setCmsConfValue():
    """
    set cms val's in restored one
    """
    nonNullItems = {}
    instanceDict = {}
    for cmaInst in g_dbNodeInfo.cmagents:
        #Copy other conf files also to backup
        renameFileInDir(cmaInst.datadir, "cm_agent.conf.backup", "cm_agent.conf", True)
        #Set the saved value in restored conf file
        instanceDict = g_confDict[cmaInst.instanceId]
        configFile = os.path.join(cmaInst.datadir, "cm_agent.conf")
        for item, val in instanceDict.iteritems():
            if val.find('NULL') == 0:
                #Handle commented items
                cmd = "sed -i 's/^.*\(%s.*=.*\)/#\\1/g' %s" % (item, configFile)
                (status, output) = commands.getstatusoutput(cmd)
                if (status != 0):
                    raise Exception(ErrorCode.GAUSS_500["GAUSS_50008"] + " Error: \n%s" % output)
            else:
                nonNullItems[item] = val
        #Handle non-commented items
        g_logger.debug("Set guc parameter using setConfigItem in setCmsConfValue, path: %s" % configFile)
        ClusterInstanceConfig.setConfigItem(DefaultValue.INSTANCE_ROLE_CMAGENT, cmaInst.datadir, configFile, nonNullItems)

def updateConfFile():
    """
    check GUC parameters from old postgresql.conf and set val's in restored one.
    """
    g_logger.debug("Adjusting configuration values after restore-new-cluster.")

    #get coordinator values
    getCnConfValue()
    g_logger.debug("updateConfFile: after getCnConfValue")

    #get datanode values
    getDnConfValue()
    g_logger.debug("updateConfFile: after getDnConfValue")

    #get gtm values
    getGtmConfValue()
    g_logger.debug("updateConfFile: after getGtmConfValue")

    #get cmserver values
    getCmsConfValue()
    g_logger.debug("updateConfFile: after getCmsConfValue")
        
    #get cmagent values
    getCmaConfValue()
    g_logger.debug("updateConfFile: after getCmaConfValue")

    #set coordinator values
    setCnConfValue()
    g_logger.debug("updateConfFile: after setCnConfValue")

    #set datanode values
    setDnConfValue()
    g_logger.debug("updateConfFile: after setDnConfValue")

    #set gtm values
    setGtmConfValue()
    g_logger.debug("updateConfFile: after setGtmConfValue")

    #Set cmaserver values
    setCmaConfValue()
    g_logger.debug("updateConfFile: after setCmaConfValue")

    #Set cmagent values
    setCmsConfValue()
    g_logger.debug("updateConfFile: after setCmsConfValue")

    g_logger.debug("Successfully set configuration values.")

def deleteConfNewBeforeBuild():
    for dnInst in g_dbNodeInfo.datanodes:
        cmd = "find %s -name \"*.conf.new\" | xargs rm -rf" % (dnInst.datadir)
        g_logger.debug("Command is %s" % cmd)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Remove *.new failed. Error: \n%s" % output)
    for cnInst in g_dbNodeInfo.coordinators:
        cmd = "find %s -name \"*.conf.new\" | xargs rm -rf" % (cnInst.datadir)
        g_logger.debug("Command is %s" % cmd)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Remove *.new failed. Error: \n%s" % output)

def fetch_operation(grep_ot):
    """
    this function will fetch the operation to be stopped
    """
    for idx, val in enumerate(grep_ot.split()):
        if val == '-t':
            ind = idx + 1
    operation = grep_ot.split()[ind]
    return operation

def fetch_python_process(grep_ot):
    """
    this function fetch the process id of python GaussRoach
    """
    fetch_id = grep_ot.split()[1]
    return fetch_id

def pre_check_python_process():
    """
    checks if any controller process is running on any of the node
    """
    if "stop" == g_opts.checkObj:
        grep_cmd = "ps ux | grep 'GaussRoach.py' | " \
                   "grep %s | grep 't stop' " \
                   "| grep -v pitr| grep -v grep" \
                   % g_opts.user
    elif "backup" == g_opts.checkObj:
        if g_opts.pitr and not g_opts.metaData == '':
            grep_cmd = "ps ux | grep 'GaussRoach.py' | grep %s " \
                       "| grep 't backup' | grep 'master-port' | " \
                       "grep 'pitr' | grep 'metadata-destination %s ' " \
                       "| grep -v grep" \
                       % (g_opts.user, g_opts.metaData)
        else:
            grep_cmd = "ps ux | grep 'GaussRoach.py' | grep %s " \
                       "| grep 't %s' " \
                       "| grep 'master-port' | grep -v grep" \
                   % (g_opts.user, g_opts.checkObj)
    else:
        grep_cmd = "ps ux | grep 'GaussRoach.py' | grep %s | grep 't %s' " \
                   "| grep 'master-port' | grep -v grep" \
                   % (g_opts.user, g_opts.checkObj)

    g_logger.debug("Pre-checking python %s process..." % g_opts.checkObj)
    grep_st, grep_ot = DefaultValue.retryGetstatusoutput(grep_cmd, 3)
    if grep_st == 0 and grep_ot != '':
        g_logger.debug("controller process of %s is running" % g_opts.checkObj)
        if "backup" == g_opts.checkObj:
            if g_opts.pitr and not g_opts.metaData == '':
                g_logger.log("pitr meta data check success.")
    else:
        g_logger.debug("After precheck %s progress, status: %d, output: %s"
                       % (g_opts.checkObj, grep_st, grep_ot))
        sys.exit(1)


def pause_resume_backup():
    """
    This function will stop the currently running backup process
    """
    g_logger.debug("Stop gs_roach for pause resume backup is called.")

    #Force stop, kill all local gs_roach processes
    g_logger.log("killall gs_roach to pause resume backup.")
    stop_cmd = "ps ux |grep '%s/bin/gs_roach' |grep -v grep |awk '{print $2}' |xargs -r kill -2" % (gauss_home_loc)
    g_logger.debug("before kill gs_roach in GSroachLocal.py, cmd is: %s" % stop_cmd)
    commands.getstatusoutput(stop_cmd)
    g_logger.debug("after kill gs_roach in GSroachLocal.py")
	
	
def stop_pitr():
    stop_cmd = "ps ux |grep '%s/bin/gs_roach' |grep -v grep |grep -v source |awk '{print $2}' |xargs -r kill -34" % gauss_home_loc
    g_logger.debug("before set stop gs_roach pitr in GSroachLocal.py, cmd is: %s" % stop_cmd)
    commands.getstatusoutput(stop_cmd)
    g_logger.debug("after set stop gs_roach pitr in GSroachLocal.py")
    g_logger.log("success set stop roach pitr process.")

def wait_pitr_restart():
    if wait_pitr():
        lock_file = "%s/roach/roach_%s.lock" % (gauss_home_loc, g_opts.user)
        g_logger.debug("check whether gs_roach lock file %s exist or not." % lock_file)
        if not os.path.exists(lock_file):
            g_logger.log("gs_roach pitr does not started.")
            return

        gpHomeLoc = DefaultValue.getEnv("GPHOME")
        cmd = "ps ux |grep '%s/script/GaussRoach.py' |grep 'pitr' |grep ' backup' |grep -v 'grep' |grep -v 'stop' |awk '{print $2}'" % gpHomeLoc
        g_logger.debug("if current node is master, python process must started, cmd is: %s" % cmd)
        (status, output) = commands.getstatusoutput(cmd)
        if status == 0:
            if output == '':
                g_logger.log("gs_roach pitr process %s is running." % output)
                return
        else:
            g_logger.debug("Failed to get gauss roach process.")
            return

        master_generated_file = '%s/bin/roach_nodes.done' % gauss_home_loc
        g_logger.debug("Check roach.done file %s in master node." % master_generated_file)
        if os.path.isfile(master_generated_file):
            g_logger.log("gs_roach pitr process %s is running." % output)
        else:
            g_logger.log("gs_roach pitr does not started.")

def wait_pitr():
    cmd = "ps ux |grep '%s/bin/gs_roach' |grep 'pitr' |grep 'backup'|grep -v grep |grep -v source |awk '{print $2}'" % gauss_home_loc
    g_logger.debug("before query gs_roach pitr process state in GSroachLocal.py, cmd is: %s" % cmd)
    (status, output) = commands.getstatusoutput(cmd)
    g_logger.debug("after query gs_roach pitr process state in GSroachLocal.py, cmd is: %s" % cmd)
    if status == 0:
        if output == '':
            g_logger.debug("gs_roach proc does not exist.")
            return False
        else:
            g_logger.debug("gs_roach pitr process %s is started." % output)
            return True
    return False

def wait_pitr_finish():
    if not wait_pitr():
        gpHomeLoc = DefaultValue.getEnv("GPHOME")
        cmd = "ps ux |grep '%s/script/GaussRoach.py' |grep 'pitr' |grep ' backup' |grep -v 'grep' |grep -v 'stop' |awk '{print $2}'" % gpHomeLoc
        g_logger.debug("before query gauss goach pitr python process state in wait_pitr_finish, cmd is: %s" % cmd)
        (status, output) = commands.getstatusoutput(cmd)
        g_logger.debug("after query gauss goach pitr python process state in wait_pitr_finish, cmd is: %s" % cmd)
        if status == 0:
            if output == '':
                g_logger.log("roach process does not exist.")
                return False
            else:
                g_logger.log("gauss goach pitr python process %s is started." % output)
                return True
    return False

def get_backup_key():
    gpHomeLoc = DefaultValue.getEnv("GPHOME")
    cmd = "ps ux |grep '%s/script/GaussRoach.py' |grep 'pitr' |grep 'backup' |grep -v 'grep' |grep -v 'stop' |awk '{print $2}'" % gpHomeLoc
    g_logger.debug("before query gauss goach pitr python process state in get_backup_key, cmd is: %s" % cmd)
    (status, output) = commands.getstatusoutput(cmd)
    g_logger.debug("after query gauss goach pitr python process state in get_backup_key, status %d output is: %s" % (status, output))
    if status == 0:
        if not output == '':
            gauss_home_location = DefaultValue.getEnv("GAUSSHOME")
            master_generated_file = '%s/bin/roach_nodes.done' % gauss_home_location
            if os.path.isfile(master_generated_file):
                tmp_val = tuple()
                tmp_val = parseMaster_generated_file(master_generated_file)
                backup_key = tmp_val[0]
                g_logger.log("backupkey=%sbackupkey" % backup_key)

def stop_roach():
    """
    This function will stop the currently running backup process
    """
    g_logger.debug("Stop gs_roach is called.")

    #Force stop, kill all local gs_roach processes
    g_logger.log("Force stop and killall gs_roach.")
    stop_cmd = "ps ux |grep '%s/bin/gs_roach' |grep -v grep |awk '{print $2}' |xargs -r kill -9" % (gauss_home_loc)
    g_logger.debug("before kill gs_roach in GSroachLocal.py, cmd is: %s" % stop_cmd)
    commands.getstatusoutput(stop_cmd)
    g_logger.debug("after kill gs_roach in GSroachLocal.py")

def createDirsWithMode(path, mode):
    if (path == ""):
        return
    if (not os.path.exists(path)):
        os.makedirs(path, mode)


def listRchFilesInRemoteInst(host, path):
    global g_opts
    cmd = "ssh -o StrictHostKeyChecking=no -o ConnectTimeout=300 %s@%s 'ls %s/ | sort -k1.6n'" % (g_opts.user, host, path)
    g_logger.debug("ls backup files cmd:%s" % cmd)
    status, output = commands.getstatusoutput(cmd)
    g_logger.debug("ls backup files output:%s" % output)
    return status, output


def listRchFilesInLocalInst(path):
    cmd = "if [ ! -d %s ]; then mkdir -p %s -m 700; fi" % (path, path)
    g_logger.debug("mkdir cmd:%s" % cmd)
    status, output = commands.getstatusoutput(cmd)
    if (status != 0):
        return status, output

    cmd = "ls %s/ | sort -k1.6n" % path
    g_logger.debug("ls backup files cmd:%s" % cmd)
    status, output = commands.getstatusoutput(cmd)
    g_logger.debug("ls backup files output:%s" % output)
    return status, output


def scpNBFilesFromPrimaryInst(hostIp, srcPath, destPath):
    if (srcPath == ""):
        g_logger.debug("srcPath is empty, do not execute scp command.")
        return 0, ""
    cmdPscp = "pscp -t %d -O StrictHostKeyChecking=no -H %s %s/* %s/" \
              % (CONNECTTIMEOUT, LoaclIp, srcPath, destPath)
    cmd = "pssh -t %d -O StrictHostKeyChecking=no -s -H %s \"%s\"" \
          % (CONNECTTIMEOUT, hostIp, cmdPscp)
    g_logger.debug("Scp backup files cmd:%s" % cmd)
    status, output = commands.getstatusoutput(cmd)
    return status, output


def checkLocalFileSize(hostIp, srcPath, destPath):
    #check primary and standby rch file
    try:
        localFileSize = os.path.getsize(destPath)
    except Exception, e:
        g_logger.debug("ERROR: get the size of local file \"%s\" failed, errormessage[%s]." % (destPath, str(e)))
        return 1, str(e)

    #parser remote file size
    cmd =  "pssh -O StrictHostKeyChecking=no -t 300 -O \
           TCPKeepAlive=yes -O ServerAliveInterval=30 -O ServerAliveCountMax=60 -H %s \
           \"ls -l %s | awk '{print \\"  % (hostIp, srcPath) + "\"DoubleCluster" + "\\" + "\"\$5\\" + "\"DoubleCluster" + "\\" + "\"}' 2>&1 \""
    status, output = commands.getstatusoutput(cmd)
    if (status != 0):
        g_logger.debug("Error: Failed to get the size of remote files cmd:%s with status is %s ,\
                        output is %s and localFileSize is %d" % (cmd, str(status), output, localFileSize))
        return status, output

    regex = r'DoubleCluster([\s\S]*)DoubleCluster'
    result = []
    for outputLine in output.split('\n'):
        result = re.findall(regex, outputLine)
        if (len(result) == 1 and result[0] != ''):
            break
    if (len(result) == 0) or (not result[0].isdigit()) or localFileSize != int(result[0]):
        return 1, "Error: Failed to get the size of remote files with cmd is %s, output is %s and localFileSize is %d" %(cmd, output, localFileSize)
    return 0, ""


def scpFileFromPrimaryInst(hostIp, srcPath, destPath):
    if (srcPath == ""):
        g_logger.debug("srcPath is empty, do not execute scp command.")
        return 0, ""
    hostName = socket.gethostname()

    retryTimes = 3
    i = 0
    for i in range(retryTimes):
        cmdPscp = "pscp -t %d -O StrictHostKeyChecking=no -O TCPKeepAlive=yes \
                -O ServerAliveInterval=30 -O ServerAliveCountMax=60 -H %s -r %s %s" % (CONNECTTIMEOUT, hostName, srcPath, destPath)
        cmd = "pssh -t %d -H %s \"%s\"" % (CONNECTTIMEOUT, hostIp, cmdPscp)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.debug("Error: Failed to scp backup files cmd:%s with status is %s ,\
                            output is %s for %d time" % (cmd, str(status), output, i))
            if (i == retryTimes -1):
                return status, output
        else:
            g_logger.debug("Scp backup files successful cmd: [%s]" % (cmd))
            break

    #fsync file
    i = 0
    for i in range(retryTimes):
        try:
            fd = os.open(destPath, os.O_RDONLY)
            os.fsync(fd)
            os.close(fd)
            g_logger.debug("Fsync file %s successful." % (destPath))
            break
        except Exception, e:
            g_logger.debug("ERROR: fsync file \"%s\" failed, errormessage[%s] for %d time" % (destPath, str(e), i))
            if (i == retryTimes -1):
                return 1, str(e)

    #check remote file size
    i = 0
    for i in range(retryTimes):
        status, output = checkLocalFileSize(hostIp, srcPath, destPath)
        if (status != 0):
            g_logger.debug("Error: check lcoal file size failed, with status is %s , output is %s for %d time" \
                           % (str(status), output, i))
            if (i == retryTimes -1):
                return status, output
        else:
            g_logger.debug("Check file size %s successful." % (destPath))
            return status, output

def scpNBFilesToRemote(hostIp, srcPath, destPath):
    timeout = 300
    cmdPscp = "pscp -t %d -O StrictHostKeyChecking=no -H %s %s %s/"\
              % (timeout, LoaclIp, srcPath, destPath)
    cmd = "pssh -t %d -O StrictHostKeyChecking=no -s -H %s \"%s\""\
          % (timeout, hostIp, cmdPscp)
    g_logger.debug("Scp backup files cmd: %s" % cmd)
    status, output = commands.getstatusoutput(cmd)
    g_logger.debug("After scp files to remote")
    return status, output

def removeNBFilesFromPrimaryInst(destPath):
    if (destPath == ""):
        g_logger.debug("destPath is empty, do not execute scp command.")
        return 0

    cmd =  "if [ -d %s ]; then rm -rf %s/*; fi" % (destPath, destPath)
    status, output = commands.getstatusoutput(cmd)
    if (status != 0):
        g_logger.logExit("Remove backup files with status(%d), output(%s)" % (status, output))

def findResumeDelete(IP, path):
    cmd =  "pssh -s -H %s 'find %s -name *resume_delete'" % (IP, path)
    status, output = commands.getstatusoutput(cmd)
    if status == 0 and len(output) > 0:
        return 1
    return 0

def checkNBFiles(primaryDataNBPath, primaryColDataNBPath, primaryArcNBPath, primarySrvNBPath):
    global g_opts

    if (findResumeDelete(g_opts.primaryInstIp, primaryDataNBPath)):
        return 1
    if (findResumeDelete(g_opts.primaryInstIp, primaryColDataNBPath)):
        return 1
    if (findResumeDelete(g_opts.primaryInstIp, primaryArcNBPath)):
        return 1
    if (findResumeDelete(g_opts.primaryInstIp, primarySrvNBPath)):
        return 1
    return 0

def executeNBFilesCopy(primaryDataNBPath, primaryColDataNBPath, primaryArcNBPath, primarySrvNBPath):
    global g_opts

    if g_opts.resumeRst:
        if (checkNBFiles(primaryDataNBPath, primaryColDataNBPath, primaryArcNBPath, primarySrvNBPath)):
            return
    # create dirs before copy backup files
    createDirsWithMode(primaryDataNBPath, BACKUPFILEMODE)
    createDirsWithMode(primaryColDataNBPath, BACKUPFILEMODE)
    createDirsWithMode(primaryArcNBPath, BACKUPFILEMODE)
    createDirsWithMode(primarySrvNBPath, BACKUPFILEMODE)
    # remove files in dirs that was left last time
    removeNBFilesFromPrimaryInst(primaryDataNBPath)
    removeNBFilesFromPrimaryInst(primaryColDataNBPath)
    removeNBFilesFromPrimaryInst(primaryArcNBPath)
    removeNBFilesFromPrimaryInst(primarySrvNBPath)
    # copy backuped data file form primary inst
    status, output = scpNBFilesFromPrimaryInst(g_opts.primaryInstIp, primaryDataNBPath, primaryDataNBPath)
    if (status != 0):
        g_logger.error("Scp primary backup data files failed with output(%s)" % (output))
        return status
    # copy backuped col data file form primary inst
    if (g_opts.needCopyCol):
        status, output = scpNBFilesFromPrimaryInst(g_opts.primaryInstIp, primaryColDataNBPath, primaryColDataNBPath)
        if (status != 0):
            removeNBFilesFromPrimaryInst(primaryDataNBPath)
            g_logger.logExit("Scp primary backup col files failed with output(%s)" % (output))
    # copy backuped xlog file form primary inst
    status, output = scpNBFilesFromPrimaryInst(g_opts.primaryInstIp, primaryArcNBPath, primaryArcNBPath)
    if (status != 0):
        removeNBFilesFromPrimaryInst(primaryDataNBPath)
        removeNBFilesFromPrimaryInst(primaryColDataNBPath)
        g_logger.logExit("Scp primary backup xlog files failed with output(%s)" % (output))
    # copy backuped log file form primary inst
    status, output = scpNBFilesFromPrimaryInst(g_opts.primaryInstIp, primarySrvNBPath, primarySrvNBPath)
    if (status != 0):
        removeNBFilesFromPrimaryInst(primaryDataNBPath)
        removeNBFilesFromPrimaryInst(primaryColDataNBPath)
        removeNBFilesFromPrimaryInst(primaryArcNBPath)
        g_logger.logExit("Scp primary backup log files failed with output(%s)" % (output))

def createLink(linkPath, cnMediaPath):
    try:
        cmd = "if [ ! -d %s ];then ln -s %s %s;fi" % (cnMediaPath, linkPath, cnMediaPath)
        g_logger.debug("Create link command: %s" % cmd)
        commands.getstatusoutput(cmd)
    except Exception, e:
        g_logger.error("Create link %s to %s failed with error: %s"  % (cnMediaPath, linkPath, str(e)))

def copyCnNBFiles():
    global g_opts

    for cooInst in g_dbNodeInfo.coordinators:
        if (cooInst.instanceId != int(g_opts.primaryInstId)):
            primaryInstDataPath = cooInst.datadir
            primaryDataNBPath = g_opts.primaryInstNewMediaPath + g_opts.primaryInstDataNBPath.split(g_opts.primaryInstOldMediaPath)[1]
            if (g_opts.generateLink):
                status, output = generateSoftLink(primaryDataNBPath, primaryInstDataPath)
                if (status != 0):
                    raise Exception("Failed to generate cn soft link, output: %s" %(output))

            masterDataNBPath = primaryDataNBPath + "/data"
            masterColDataNBPath = primaryDataNBPath + "/data_colstore"
            masterArcNBPath = primaryDataNBPath + "/archive"

            dataDiffList = []
            # get diff between remote data dir and local data dir
            dataDiffList = getDiffNBFiles(dataDiffList, g_opts.primaryInstIp, masterDataNBPath, masterDataNBPath)
            g_logger.debug("cn data diff file list: %s" % dataDiffList)
            for remoteRch in dataDiffList:
                #copy backuped data files from masterCN
                if ((remoteRch[-3:] != "tmp") and (remoteRch[:5] == 'file_')):
                    status, output = scpFileFromPrimaryInst(g_opts.primaryInstIp, \
                        masterDataNBPath + "/" + remoteRch, masterDataNBPath + "/" + remoteRch.replace(".resume_delete", ""))
                    if (status != 0):
                        raise Exception("Failed to copy cn data backup files")
                        
            arcDiffList = []
            # get diff between remote arc dir and local arc dir
            arcDiffList = getDiffNBFiles(arcDiffList, g_opts.primaryInstIp, masterArcNBPath, masterArcNBPath)
            g_logger.debug("cn arc diff file list: %s" % arcDiffList)
            for remoteRch in arcDiffList:
                #copy backuped arc files from masterCN
                if ((remoteRch[-3:] != "tmp") and (remoteRch[:5] == 'file_')):
                    status, output = scpFileFromPrimaryInst(g_opts.primaryInstIp, \
                        masterArcNBPath + "/" + remoteRch, masterArcNBPath + "/" + remoteRch.replace(".resume_delete", ""))
                    if (status != 0):
                        raise Exception("Failed to copy cn arc backup files")

            colstoreDiffList = []
            # get diff between remote arc dir and arc data dir
            colstoreDiffList = getDiffNBFiles(colstoreDiffList, g_opts.primaryInstIp, masterColDataNBPath, masterColDataNBPath)
            g_logger.debug("cn colstore diff file list: %s" % colstoreDiffList)
            for remoteRch in colstoreDiffList:
                #copy backuped col_data files from masterCN
                if ((remoteRch[-3:] != "tmp") and (remoteRch[:5] == 'file_')):
                    status, output = scpFileFromPrimaryInst(g_opts.primaryInstIp, 
                        masterColDataNBPath + "/" + remoteRch, masterColDataNBPath + "/" + remoteRch.replace(".resume_delete", ""))
                    if (status != 0):
                        raise Exception("Failed to copy cn col data backup files")

def scpDiffRchFiles(primaryInstIp, masterDataNBPath, fileType):
    dataDiffList = []
    # get diff between remote data dir and local data dir
    dataDiffList = getDiffNBFiles(dataDiffList, g_opts.primaryInstIp, masterDataNBPath, masterDataNBPath)
    g_logger.debug("%s data diff file list: %s" % (fileType, dataDiffList))
    for remoteRch in dataDiffList:
        #copy backuped data files from master
        if ((remoteRch[-3:] != "tmp") and (remoteRch[:5] == 'file_')):
            status, output = scpFileFromPrimaryInst(g_opts.primaryInstIp, \
                masterDataNBPath + "/" + remoteRch, masterDataNBPath + "/" + remoteRch.replace(".resume_delete", ""))
            if (status != 0):
                raise Exception("Failed to copy %s data backup files" %fileType)

def copyGtmNBFiles():
    global g_opts

    for gtmInst in g_dbNodeInfo.gtms:
        if (gtmInst.instanceId != int(g_opts.primaryInstId)):
            masterDataNBPath = g_opts.primaryInstNewMediaPath + g_opts.primaryInstDataNBPath.split(g_opts.primaryInstOldMediaPath)[1]
            scpDiffRchFiles(g_opts.primaryInstIp, masterDataNBPath, GTM_TYPE)

def copyCmNBFiles():
    global g_opts

    for cmsInst in g_dbNodeInfo.cmservers:
        if (cmsInst.instanceId != int(g_opts.primaryInstId)):
            masterDataNBPath = g_opts.primaryInstNewMediaPath + g_opts.primaryInstDataNBPath.split(g_opts.primaryInstOldMediaPath)[1]
            scpDiffRchFiles(g_opts.primaryInstIp, masterDataNBPath, CM_TYPE)

def copyNBFiles():
    global g_opts

    try:
        if (g_opts.instType == CN_TYPE):
            copyCnNBFiles()
        elif (g_opts.instType == GTM_TYPE):
            copyGtmNBFiles()
        elif (g_opts.instType == CM_TYPE):
            copyCmNBFiles()
        else:
            return
    except Exception, e:
        g_logger.logExit("Copy backup files failed: %s" % str(e))

def renameConfFile(standbyDataNBpath, isrenameConf):
    if isrenameConf:
        standbyDataNBpathSrc = os.path.join(standbyDataNBpath, "file_0.rch")
        standbyDataNBpathDest = os.path.join(standbyDataNBpath, "standbyconf.rch")
        cmd = "mv %s %s" %(standbyDataNBpathSrc, standbyDataNBpathDest)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Rename standby conf rch file with status(%d), output(%s)" % (status, output))
    else:
        num_files = 0
        for fn in os.listdir(standbyDataNBpath):
            num_files += 1

        confRchFileName = "file_%d.rch" % (num_files - 1)
        standbyDataNBpathSrc = os.path.join(standbyDataNBpath, "standbyconf.rch")
        standbyDataNBpathDest = os.path.join(standbyDataNBpath, confRchFileName)
        cmd = "mv %s %s" % (standbyDataNBpathSrc, standbyDataNBpathDest)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Rename standby conf rch file with status(%d), output(%s)" % (status, output))

def removeNBFilesFromMasterDN(standbyDataNBPath, standbyColDataNBPath, standbyArcNBPath):
    #clean up standby data files, need remain conf rch file
    standbyNBDataPath_conf1 = os.path.join(standbyDataNBPath, "file_0.rch")
    standbyNBDataPath_conf2 = os.path.join(standbyDataNBPath, "standbyconf.rch")
    num_files = 0
    for fn in os.listdir(standbyDataNBPath):
        num_files += 1
    if (num_files == 1):
        if os.path.exists(standbyNBDataPath_conf2):
            cmd = "mv %s %s" %(standbyNBDataPath_conf2, standbyNBDataPath_conf1)
            status, output = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.logExit("Rename standby DN conf rch file with status(%d), output(%s)" % (status, output))
    else:
         if os.path.exists(standbyNBDataPath_conf2):
             cmd = "rm -rf %s/file_*.rch" % (standbyDataNBPath)
             status, output = commands.getstatusoutput(cmd)
             if (status != 0):
                 g_logger.logExit("Rename standby DN conf rch file with status(%d), output(%s)" % (status, output))
         else:
             standbyNBDataPath_conf3 = os.path.join(standbyDataNBPath, "file_%d.rch" % (num_files - 1))
             cmd = "mv %s %s" %(standbyNBDataPath_conf3, standbyNBDataPath_conf2)
             status, output = commands.getstatusoutput(cmd)
             if (status != 0):
                 g_logger.logExit("Rename standby DN conf rch file with status(%d), output(%s)" % (status, output))
             cmd = "rm -rf %s/file_*.rch" % (standbyDataNBPath)
             status, output = commands.getstatusoutput(cmd)
             if (status != 0):
                 g_logger.logExit("Remove standby DN other rch file with status(%d), output(%s)" % (status, output))

         cmd = "mv %s %s" %(standbyNBDataPath_conf2, standbyNBDataPath_conf1)
         status, output = commands.getstatusoutput(cmd)
         if (status != 0):
             g_logger.logExit("Rename standby DN conf rch file with status(%d), output(%s)" % (status, output))

    #clean up standby arcLog  files
    if os.path.exists(standbyArcNBPath):
        cmd = "rm -rf %s" % (standbyArcNBPath)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Remove standby DN arcLog backup files with status(%d), output(%s)" % (status, output))

    #clean up standby data_colstore  files
    if os.path.exists(standbyColDataNBPath):
        cmd = "rm -rf %s" % (standbyColDataNBPath)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Remove standby DN data_colstore backup files with status(%d), output(%s)" % (status, output))

def removeAllStandbysNBFiles(standbyDataNBPath, standbyColDataNBPath, standbyArcNBPath):
    #clean up standby arcLog  files
    if os.path.exists(standbyDataNBPath):
        cmd = "rm -rf %s" % (standbyDataNBPath)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Remove standby DN data backup files with status(%d), output(%s)" % (status, output))
    #clean up standby arcLog  files
    if os.path.exists(standbyArcNBPath):
        cmd = "rm -rf %s" % (standbyArcNBPath)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Remove standby DN arcLog backup files with status(%d), output(%s)" % (status, output))

    #clean up standby data_colstore  files
    if os.path.exists(standbyColDataNBPath):
        cmd = "rm -rf %s" % (standbyColDataNBPath)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Remove standby DN data_colstore backup files with status(%d), output(%s)" % (status, output))

def mvResumeFileToRch(path):
    #mv resume_delete to rch file
    if os.path.exists(path):
        cmd = "ls %s/*.resume_delete " % (path)
        status, output = commands.getstatusoutput(cmd)
        if status != 0:
            raise Exception("ls resume_delete file failed. status(%d), output(%s)" % (status, output))
        for name in output:
            rchname = name[0:len(name) - len(".resume_delete")]
            cmd = "mv %s %s" % (name, rchname)
            status, output = commands.getstatusoutput(cmd)
            if (status != 0):
                raise Exception("mv resume_delete file failed. status(%d), output(%s)" % (status, output))

def getNodesInfoByUpdatedFile(jsonFile):
    fp = open(jsonFile, 'r')
    infostr = fp.read()
    infostr = re.sub(r'(\d+)UL','\g<1>',infostr)
    fp.close()
    infodict = json.loads(infostr)
    infoList = infodict['Nodes']
    return infoList

def getStandbyNBPaths(standbyInstId):
    updatedJson = os.path.join(g_opts.standbyInstNewMetaPath, 'roach', g_opts.backupKey, 'roach_master_updated.metadata')
    oldInfoList = getNodesInfoByUpdatedFile(updatedJson)

    standbyInstDataNBPath = ""
    standbyColDataNBPath = ""
    standbyArcNBPath = ""
    primaryDataNBPath = ""
    standbyMediaPath = ""
    for nodeInfo in oldInfoList:
        if (g_dbNodeInfo.sshIps[0] == nodeInfo['IP']):
            for i in range(0, int(nodeInfo['StandbyCount'])):
                standbyInfo = nodeInfo['Standbys'][i]
                if (standbyInfo != None and standbyInfo['StandbyNodeId'] == standbyInstId):
                    standbyInstDataNBPath = standbyInfo['StandbyConfNBPath']
                    standbyInstArcNBPath = standbyInfo['ArchiveLogNBPath']
                    standbyMediaPath = nodeInfo['BackupLocation']
                    primaryInstDataNBPath = standbyInfo['PrimaryDataNBPath']
                    primaryIp = standbyInfo['PrimaryIP']
                    break
            break

    standbyDataNBPath = g_opts.standbyInstNewMediaPath + standbyInstDataNBPath.split(standbyMediaPath)[1]
    standbyColDataNBPath = standbyDataNBPath + "_colstore"
    standbyArcNBPath = g_opts.standbyInstNewMediaPath + standbyInstArcNBPath.split(standbyMediaPath)[1]
    primaryDataNBPath = g_opts.standbyInstNewMediaPath + primaryInstDataNBPath.split(standbyMediaPath)[1]

    return standbyDataNBPath, standbyColDataNBPath, standbyArcNBPath,  primaryDataNBPath, primaryIp

def checkMasterInstBkp(primaryIP, masterNBPath, fileType):
    resp = subprocess.call( ['pssh -s -H', primaryIP, 'test -e ' + pipes.quote(masterNBPath) + r"/%s/file_0.rch.resume_delete" % fileType] )
    if(resp == 0):
        return 1
    else:
        return 0

def copyDNNBFilesParallel():
    global g_opts

    standbyInstId = []
    updatedJson = os.path.join(g_opts.standbyInstNewMetaPath, 'roach', g_opts.backupKey, 'roach_master_updated.metadata')
    oldInfoList = getNodesInfoByUpdatedFile(updatedJson)

    for nodeInfo in oldInfoList:
        if (g_dbNodeInfo.sshIps[0] == nodeInfo['IP']):
            for i in range(0, int(nodeInfo['StandbyCount'])):
                standbyInfo = nodeInfo['Standbys'][i]
                standbyInstIdTemp = standbyInfo['StandbyNodeId']
                standbyInstId.append(standbyInstIdTemp)
            break

    if (len(standbyInstId) == 0):
        return
    try:
        pool = ThreadPool(min(DefaultValue.getCpuSet(), len(standbyInstId)))
        pool.map(copyDNNBFiles, standbyInstId)
        pool.close()
        pool.join()
    except Exception, e:
        g_logger.logExit(str(e))

def checkAllMasterInstBkp(standbyInstId):
    global g_opts
    if not g_opts.restoreNewCluster:
        standbyDataNBPath, standbyColDataNBPath, standbyArcNBPath, primaryInstDataNBPath, masterInstIp = getStandbyNBPaths(standbyInstId)
        standbyNBPath = standbyDataNBPath[0:len(standbyDataNBPath) - 4]
        masterNBPath = primaryInstDataNBPath[0:len(primaryInstDataNBPath) - 4]
        status = checkMasterInstBkp(masterInstIp, masterNBPath, ROW_FILE_TYPE)
        if (status == 1):
            return 1
    return 0

def getDiffNBFiles(diffList, hostIp, localPath, remotePath):
    status, output = listRchFilesInRemoteInst(hostIp, remotePath)
    if (status != 0):
        if ("No such file or directory" in output):
            return
        raise Exception("Failed to ls remote host %s dn or cn %s backup files" % (hostIp, remotePath))
    elif ("No such file or directory" in output):
        return diffList

    remoteList = output.split('\n')
    g_logger.debug("remote file list: %s" % remoteList)

    status, output = listRchFilesInLocalInst(localPath)
    if (status != 0):
        raise Exception("Failed to ls local host dn or cn %s data backup files" % localPath)

    if (output == ''):
        diffList.extend(remoteList)
        return diffList

    localList = output.split('\n')
    g_logger.debug("local file list: %s" % localList)

    localRchFile = localList[-1]
    matched = 0

    for remoteRchFile in remoteList:
        if matched:
            diffList.append(remoteRchFile)
            g_logger.debug("append %s to diff file list because of matched." % remoteRchFile)
        if (localRchFile.replace(".resume_delete", "") == remoteRchFile.replace(".resume_delete", "")):
            matched = 1
            if localRchFile[-3:] == "rch":
                cmd = "rm -rf %s/%s" % (localPath, localRchFile)
                status, output = commands.getstatusoutput(cmd)
                g_logger.debug("Remove local DN or CN last rch file,cmd: %s" % (cmd))
                if (status != 0):
                    g_logger.logExit("Remove local DN or CN last rch file with status(%d), output(%s)" % (status, output))
                diffList.append(remoteRchFile)
                g_logger.debug("append %s to diff file list because of last rch file." % remoteRchFile)
    return diffList

def generateSoftLink(path, dataPath):
    dataPathCopy = copy.deepcopy(dataPath)
    backupPath = os.path.join(os.path.dirname(dataPath), "roachbackup", g_opts.backupKey, os.path.basename(dataPathCopy))
    cmd = "if [ ! -d %s ]; then mkdir -p %s -m 700; fi" % (backupPath, backupPath)
    g_logger.debug("mkdir roachbackup fold cmd:%s" % cmd)
    status, output = commands.getstatusoutput(cmd)
    if (status != 0):
        return status, output

    dirPath = os.path.dirname(copy.deepcopy(path))
    cmd = "if [ ! -d %s ]; then mkdir -p %s -m 700; fi" % (dirPath, dirPath)
    g_logger.debug("mkdir roachbackup fold cmd:%s" % cmd)
    status, output = commands.getstatusoutput(cmd)
    if (status != 0):
        return status, output

    cmd = "if [ ! -d %s ]; then ln -s %s %s; fi" %(path, backupPath, path)
    status, output = commands.getstatusoutput(cmd)
    g_logger.debug("create soft link cmd:%s" % cmd)
    if (status != 0):
        g_logger.debug("create soft link failed cmd:%s with output:%s" % (cmd, output))
    return status, output

def copyDNNBFiles(standbyInstId):
    global g_opts
    if (g_opts.resumeRst):
        status = checkAllMasterInstBkp(standbyInstId)
        if (status == 1):
            return
    if g_opts.restoreNewCluster:
        for dbInst in g_dbNodeInfo.datanodes:
            if (int(dbInst.instanceId) == int(standbyInstId)):
                standbyInst = dbInst
                standbyInstDataPath = standbyInst.datadir
                peerInsts = g_clusterInfo.getPeerInstance(dbInst)
                masterInst = None
                for i in range(len(peerInsts)):
                    if (peerInsts[i].instanceType == MASTER_INSTANCE):
                        masterInst = peerInsts[i]
                        standbyDataNBPath, standbyColDataNBPath, standbyArcNBPath, primaryInstDataNBPath, primaryIP = getStandbyNBPaths(standbyInstId)

                        standbyNBPath = standbyDataNBPath[0:len(standbyDataNBPath) - 4]
                        masterNBPath = primaryInstDataNBPath[0:len(primaryInstDataNBPath) - 4]
                        
                        masterDataNBPath = masterNBPath + "/data"
                        masterColDataNBPath = masterNBPath + "/data_colstore"
                        masterArcNBPath = masterNBPath + "/archive"

                        if (g_opts.generateLink):
                            status, output = generateSoftLink(os.path.dirname(standbyDataNBPath), standbyInstDataPath)
                            if (status != 0):
                                raise Exception("Failed to generate soft link, output: %s" %(output))

                        dataDiffList = []
                        # get diff between remote data dir and local data dir
                        dataDiffList = getDiffNBFiles(dataDiffList, masterInst.listenIps[0], standbyDataNBPath, masterDataNBPath)
                        g_logger.debug("diff file list: %s" % dataDiffList)
                        for remoteRch in dataDiffList:
                            #copy backuped data files from masterDN
                            if ((remoteRch[-3:] != "tmp") and (remoteRch[:5] == 'file_')):
                                status, output = scpFileFromPrimaryInst(masterInst.listenIps[0], \
                                    masterDataNBPath + "/" + remoteRch, standbyDataNBPath + "/" + remoteRch.replace(".resume_delete", ""))
                                if (status != 0):
                                    raise Exception("Failed to copy dn data backup files")
                                    
                        arcDiffList = []
                        # get diff between remote arc dir and local arc dir
                        arcDiffList = getDiffNBFiles(arcDiffList, masterInst.listenIps[0], standbyArcNBPath, masterArcNBPath)
                        g_logger.debug("diff file list: %s" % arcDiffList)
                        for remoteRch in arcDiffList:
                            #copy backuped arc files from masterDN
                            if ((remoteRch[-3:] != "tmp") and (remoteRch[:5] == 'file_')):
                                status, output = scpFileFromPrimaryInst(masterInst.listenIps[0], \
                                    masterArcNBPath + "/" + remoteRch, standbyArcNBPath + "/" + remoteRch.replace(".resume_delete", ""))
                                if (status != 0):
                                    raise Exception("Failed to copy dn arc backup files")

                        colstoreDiffList = []
                        # get diff between remote arc dir and arc data dir
                        colstoreDiffList = getDiffNBFiles(colstoreDiffList, masterInst.listenIps[0], standbyColDataNBPath, masterColDataNBPath)
                        g_logger.debug("diff file list: %s" % colstoreDiffList)
                        for remoteRch in colstoreDiffList:
                            #copy backuped col_data files from masterDN
                            if ((remoteRch[-3:] != "tmp") and (remoteRch[:5] == 'file_')):
                                status, output = scpFileFromPrimaryInst(masterInst.listenIps[0], 
                                    masterColDataNBPath + "/" + remoteRch, standbyColDataNBPath + "/" + remoteRch.replace(".resume_delete", ""))
                                if (status != 0):
                                    raise Exception("Failed to copy dn col data backup files")
                break
    else:
            standbyDataNBPath, standbyColDataNBPath, standbyArcNBPath, primaryInstDataNBPath, masterInstIp = getStandbyNBPaths(standbyInstId)

            standbyNBPath = standbyDataNBPath[0:len(standbyDataNBPath) - 4]
            masterNBPath = primaryInstDataNBPath[0:len(primaryInstDataNBPath) - 4]
            status = checkMasterInstBkp(masterInstIp, masterNBPath, ROW_FILE_TYPE)
            if (status):
                return

            # remove files in dirs that was left last time
            removeNBFilesFromMasterDN(standbyDataNBPath, standbyColDataNBPath, standbyArcNBPath)

            renameConfFile(standbyDataNBPath, True)
            #copy backuped data files from masterDN
            status, output = scpNBFilesFromPrimaryInst(masterInstIp, masterNBPath, standbyNBPath)
            if (status != 0):
                raise Exception("Failed to copy dn backup files")
            renameConfFile(standbyDataNBPath, False)

    return

def parseStandbyClusterConf(g_opts):
    """
    Parse cluster config file
    """
    global g_remoteClusterInfo

    try:
        g_logger.debug("Parsing the configuration file.")

        #in case of GaussRoach.py -t clean then GaussRoach.py -t restore, there is no static config file
        gaussHome = DefaultValue.getEnv("GAUSSHOME")
        standbyStaticConfig = "%s/bin/cluster_static_config_remote" % gaussHome
        g_remoteClusterInfo = dbClusterInfo()
        g_remoteClusterInfo.initFromStaticConfig(g_opts.user, standbyStaticConfig)
    except Exception, e:
        g_logger.logExit(str(e))
    g_logger.debug("Successfully parsed the configuration file.")

def createBackupLibDir(g_opts, dirpath):
    """
    create backup dir for libs
    """
    create_cmd = 'source %s && if [ ! -e %s ]; then mkdir -p %s -m 700; fi' \
        % (g_opts.sourceFile, dirpath, dirpath)
    (status, output) = commands.getstatusoutput(create_cmd)
    g_logger.debug("Create dir commond when backup libs: %s" % create_cmd)
    if status != 0:
        raise Exception("Error: Unable to create path \"%s\" when backup libs! %s" % (dirpath, output))

def backupNodeLibs(sourceFile, filetype, srcpath, destpath):
    """
    backup libs
    """
    if (not os.path.exists(srcpath)):
        g_logger.logExit("Error: source path \"%s\" does not exist when backup libs" % srcpath)
    if (not os.path.exists(destpath)):
        g_logger.logExit("Error: destination path \"%s\" does not exist when backup libs" % destpath)

    backupCmd = "source %s && cp -rf %s/%s %s" % (sourceFile, srcpath, filetype, destpath)
    g_logger.debug("Libs backup command: %s" % backupCmd)
    (status, output) = commands.getstatusoutput(backupCmd)
    if status != 0:
        raise Exception("Error: backup lib file \"%s\" failed when backup libs! %s" % (filetype, output))

def getClusterNodeIps(ClusterInfo):
    """
    Get node IP in cluster
    """
    return [dbNode.sshIps[0] for dbNode in ClusterInfo.dbNodes]

def createRemoteDir(remoteIp, remoteDir):
    """
    create meta dir for backup lib files
    """
    timeout = 60
    cmd = 'pssh -s -H %s -t %d -O StrictHostKeyChecking=no \" if [ ! -d %s ]; then mkdir -p %s -m 700; fi \"' \
        % (remoteIp, timeout, remoteDir, remoteDir)
    g_logger.debug("Creating metadata path, cmd: %s" % cmd)

    (status, output) = commands.getstatusoutput(cmd)
    if(status != 0):
        raise Exception("Failed to create metadata path. Error: \n%s." % output)

def copyCnCFunc(coorInstIP):
    srcpath_so = "%s/lib/postgresql/pg_plugin/cn_*" \
        % DefaultValue.getInstallDir(g_opts.user)
    backupdir = os.path.join(g_opts.remoteMetaData, "roach", \
        g_opts.backupKey, "binaries", "pg_plugin")

    createRemoteDir(coorInstIP, backupdir)

    (status, output) = scpNBFilesToRemote(coorInstIP, srcpath_so, backupdir)
    if (status != 0):
        raise Exception("Failed to copy cn lib files to remote!")

def copyCnCFuncFiles():
    """
    Copy master cn lib file to remote cluster metadata
    """
    coorInstIdMin = 0
    coorInstIdHostName = ""
    coorInstIP = []
    for dbnode in g_clusterInfo.dbNodes:
        for cooInst in dbnode.coordinators:
            coorInstIdTemp = cooInst.instanceId
            if (coorInstIdMin == 0 or coorInstIdTemp < coorInstIdMin):
                coorInstIdMin = cooInst.instanceId
                coorInstIdHostName = dbnode.name

    g_logger.debug("Find the Min Cn instanceId is %s, hostName is %s" % (coorInstIdMin, coorInstIdHostName))
    hostName = socket.gethostname()
    g_logger.debug("Local hostName is %s" % hostName)
    if (hostName == coorInstIdHostName):
        for dbnode in g_remoteClusterInfo.dbNodes:
            for cooInst in dbnode.coordinators:
                coorInstIP.append(cooInst.listenIps[0])

        g_logger.debug("Copy the cn %s to remote cluster" % coorInstIdMin)
        if (len(coorInstIP) > 0):
            try:
                pool = ThreadPool(min(DefaultValue.getCpuSet(), len(coorInstIP)))
                pool.map(copyCnCFunc, coorInstIP)
                pool.close()
                pool.join()
            except Exception, e:
                raise Exception("Failed to copy cn lib files, output is %s" % str(e))
    g_logger.debug("Successfully copy the cn lib file ")

def copyDnCFunc(dnNeedCopyInst):
    dnInstId = dnNeedCopyInst[0]
    dnInstIdIP = dnNeedCopyInst[1]

    srcpath_so = "%s/lib/postgresql/pg_plugin/dn_%s*" \
        % (DefaultValue.getInstallDir(g_opts.user), dnInstId)
    backupdir = os.path.join(g_opts.remoteMetaData, "roach", \
        g_opts.backupKey, "binaries", "pg_plugin")

    createRemoteDir(dnInstIdIP, backupdir)

    (status, output) = scpNBFilesToRemote(dnInstIdIP, srcpath_so, backupdir)
    if (status != 0):
        raise Exception("Failed to copy dn lib files to remote!")

def copyDnCFuncFiles():
    """
    Copy master dn lib file to remote cluster metadata
    """
    dnNeedCopyInstList = []
    hostName = socket.gethostname()
    g_dbNodeInfo = g_clusterInfo.getDbNodeByName(hostName)

    for dnInst in g_dbNodeInfo.datanodes:
        instanceIdList = []
        instanceIdList.append(dnInst.instanceId)
        peerInsts = g_clusterInfo.getPeerInstance(dnInst)
        for peerInst in peerInsts:
            if (peerInst.instanceType == MASTER_INSTANCE or peerInst.instanceType == STANDBY_INSTANCE):
                instanceIdList.append(peerInst.instanceId)
        instanceIdList.sort()
        instanceName = "_".join(str(i) for i in instanceIdList)

        dnInstIdTemp = dnInst.instanceId
        dnInstTypeTemp = dnInst.instanceType
        for dbRemoteNode in g_remoteClusterInfo.dbNodes:
            for dnRemoteInst in dbRemoteNode.datanodes:
                if (dnInstIdTemp == dnRemoteInst.instanceId and dnInstTypeTemp == dnRemoteInst.instanceType):
                    if (dnInstTypeTemp == MASTER_INSTANCE or dnInstTypeTemp == STANDBY_INSTANCE):
                        dnNeedCopyInst = []
                        dnNeedCopyInst.append(instanceName)
                        dnNeedCopyInst.append(dnRemoteInst.listenIps[0])
                        dnNeedCopyInstList.append(dnNeedCopyInst)

    g_logger.debug("Copy the dn to remote cluster")
    if (len(dnNeedCopyInstList) > 0):
        try:
            pool = ThreadPool(min(DefaultValue.getCpuSet(), len(dnNeedCopyInstList)))
            pool.map(copyDnCFunc, dnNeedCopyInstList)
            pool.close()
            pool.join()
        except Exception, e:
            g_logger.error("Backup dn lib files for remote cluster failed: %s" % str(e))
            raise Exception("Failed to backup dn lib files")
    g_logger.debug("Successfully copied the dn lib files")

def backupCFuncFiles():
    """
    backup libs for C-UDF
    """
    g_logger.debug("Start backup libs for cluster backup.")

    #backup .so for C-Function
    srcpath_so = "%s/lib/postgresql/pg_plugin/" % DefaultValue.getInstallDir(g_opts.user)
    soType = "*"

    if (g_opts.remoteMasterIp == ""):

        backupdir = os.path.join(g_opts.metaData, \
            "roach", g_opts.backupKey, "binaries")

        destpath_so = "%s/pg_plugin" % backupdir
        if (isDirectoryHasFile(srcpath_so, soType)):
            createBackupLibDir(g_opts, destpath_so)
            backupNodeLibs(g_opts.sourceFile, soType, srcpath_so, destpath_so)
    else:
        #double cluster
        if (isDirectoryHasFile(srcpath_so, soType)):
            parseStandbyClusterConf(g_opts)
            copyCnCFuncFiles()
            copyDnCFuncFiles()

    g_logger.debug("End backup local node libs for cluster backup.")

def copyCnTsDict(funcParaList):
    if (len(funcParaList) != 3):
        raise Exception("Parameter : %s in wrong format!" % (funcParaList))

    hostIp = funcParaList[0]
    srcpath_so = "%s/cn_*" % (funcParaList[1])
    backupdir = funcParaList[2]

    fileList = glob.glob(srcpath_so)
    if (len(fileList) > 0):
        createRemoteDir(hostIp, backupdir)
        (status, output) = scpNBFilesToRemote(hostIp, srcpath_so, backupdir)
        if (status != 0):
            raise Exception("Failed to copy cn ts dict files to remote, ERROR: %s!" % (output))

def copyCnTsDictFiles(srcPath, destPath):
    """
    Copy master cn lib file to remote cluster metadata
    """
    coorInstIdMin = 0
    coorInstIdHostName = ""

    paraList = []
    for dbnode in g_clusterInfo.dbNodes:
        for cooInst in dbnode.coordinators:
            coorInstIdTemp = cooInst.instanceId
            if (coorInstIdMin == 0 or coorInstIdTemp < coorInstIdMin):
                coorInstIdMin = cooInst.instanceId
                coorInstIdHostName = dbnode.name

    g_logger.debug("Find the Min Cn instanceId is %s, hostName is %s" % (coorInstIdMin, coorInstIdHostName))
    hostName = socket.gethostname()
    g_logger.debug("Local hostName is %s" % hostName)
    if (hostName == coorInstIdHostName):
        for dbnode in g_remoteClusterInfo.dbNodes:
            for cooInst in dbnode.coordinators:
                funcParaList = []
                funcParaList.append(cooInst.listenIps[0])
                funcParaList.append(srcPath)
                funcParaList.append(destPath)
                paraList.append(funcParaList)

        g_logger.debug("Copy the cn %s to remote cluster" % coorInstIdMin)
        if (len(paraList) > 0):
            try:
                pool = ThreadPool(min(DefaultValue.getCpuSet(), len(paraList)))
                pool.map(copyCnTsDict, paraList)
                pool.close()
                pool.join()
            except Exception, e:
                raise Exception("Failed to copy cn ts dict files, output is %s" % str(e))
    g_logger.debug("Successfully copy the cn ts dict files ")

def copyDnTsDict(dnNeedCopyInst):
    if (len(dnNeedCopyInst) != 4):
        raise Exception("Parameter : %s in wrong format!" % (dnNeedCopyInst))

    hostIp = dnNeedCopyInst[0]
    instanceName = dnNeedCopyInst[1]
    srcPath = dnNeedCopyInst[2]
    destPath = dnNeedCopyInst[3]

    srcpath_so = "%s/dn_%s*" % (srcPath, instanceName)

    fileList = glob.glob(srcpath_so)
    if (len(fileList) > 0):
        createRemoteDir(hostIp, destPath)
        (status, output) = scpNBFilesToRemote(hostIp, srcpath_so, destPath)
        if (status != 0):
            raise Exception("Failed to copy dn ts dict files to remote, ERROR: %s!" % (output))

def copyDnTsDictFiles(srcPath, destPath):
    """
    Copy master dn lib file to remote cluster metadata
    """
    dnNeedCopyInstList = []
    hostName = socket.gethostname()
    g_dbNodeInfo = g_clusterInfo.getDbNodeByName(hostName)

    for dnInst in g_dbNodeInfo.datanodes:
        instanceIdList = []
        instanceIdList.append(dnInst.instanceId)
        peerInsts = g_clusterInfo.getPeerInstance(dnInst)
        for peerInst in peerInsts:
            if (peerInst.instanceType == MASTER_INSTANCE or peerInst.instanceType == STANDBY_INSTANCE):
                instanceIdList.append(peerInst.instanceId)
        instanceIdList.sort()
        instanceName = "_".join(str(i) for i in instanceIdList)

        dnInstIdTemp = dnInst.instanceId
        dnInstTypeTemp = dnInst.instanceType
        for dbRemoteNode in g_remoteClusterInfo.dbNodes:
            for dnRemoteInst in dbRemoteNode.datanodes:
                if (dnInstIdTemp == dnRemoteInst.instanceId and dnInstTypeTemp == dnRemoteInst.instanceType):
                    if (dnInstTypeTemp == MASTER_INSTANCE or dnInstTypeTemp == STANDBY_INSTANCE):
                        dnNeedCopyInst = []
                        dnNeedCopyInst.append(dnRemoteInst.listenIps[0])
                        dnNeedCopyInst.append(instanceName)
                        dnNeedCopyInst.append(srcPath)
                        dnNeedCopyInst.append(destPath)
                        dnNeedCopyInstList.append(dnNeedCopyInst)

    g_logger.debug("Copy the dn ts dict files to remote cluster")
    if (len(dnNeedCopyInstList) > 0):
        try:
            pool = ThreadPool(min(DefaultValue.getCpuSet(), len(dnNeedCopyInstList)))
            pool.map(copyDnTsDict, dnNeedCopyInstList)
            pool.close()
            pool.join()
        except Exception, e:
            g_logger.error("Backup ts dict files for remote cluster failed: %s" % str(e))
            raise Exception("Failed to backup dn ts dict files")
    g_logger.debug("Successfully copied the dn ts dict files")

def backupTsDictFiles():
    """
    backup libs for C-UDF
    """
    g_logger.debug("Start backup ts dict for cluster backup.")

    #backup .so for TS-DICT
    srcpath_so = "%s/share/postgresql/tsearch_data/" % DefaultValue.getInstallDir(g_opts.user)
    soType = "*"

    if (g_opts.remoteMasterIp == ""):

        backupdir = os.path.join(g_opts.metaData, \
            "roach", g_opts.backupKey, "binaries")

        destpath_so = "%s/share/postgresql/tsearch_data" % backupdir
        if (isDirectoryHasFile(srcpath_so, soType)):
            createBackupLibDir(g_opts, destpath_so)
            backupNodeLibs(g_opts.sourceFile, soType, srcpath_so, destpath_so)
    else:
        #double cluster
        backupdir = os.path.join(g_opts.remoteMetaData, "roach", g_opts.backupKey, "binaries")
        destpath_so = "%s/share/postgresql/tsearch_data" % backupdir

        if (isDirectoryHasFile(srcpath_so, soType)):
            parseStandbyClusterConf(g_opts)
            copyCnTsDictFiles(srcpath_so, destpath_so)
            copyDnTsDictFiles(srcpath_so, destpath_so)

    g_logger.debug("End backup local node libs for cluster backup.")

def isDirectoryHasFile(dirName, fileName):
    """
    check if fileName exist under dirName
    """
    fullName = os.path.join(dirName, fileName)
    fileList = glob.glob(fullName)
    if (len(fileList) > 0):
        return True
    else:
        return False

def restoreNodeLibs(srcpath, destpath):
    """
    restore lib files
    """
    if (not os.path.exists(srcpath)):
        g_logger.debug("Source path \"%s\" does not exist when restore libs" % srcpath)
        return
    if (not os.path.exists(destpath)):
        g_logger.logExit("Error: destination path \"%s\" does not exist when restore libs" % destpath)

    #check if src path has some files
    if (isDirectoryHasFile(srcpath, "*")):
        restoreCmd = "cp -rf %s/* %s" % (srcpath, destpath)
        g_logger.debug("Libs restore command: %s" % restoreCmd)
        (status, output) = commands.getstatusoutput(restoreCmd)
        if status != 0:
            g_logger.logExit("Error: restore lib files failed! msg: %s" % output)
    else:
        g_logger.debug("Directory has no such files")

def restoreCFuncFiles():
    """
    restore libs for C-UDF
    """
    g_logger.debug("Start restore c function.")
    restoredir = os.path.join(g_opts.metaData, \
        "roach", g_opts.backupKey, "binaries")

    #restore .so for C-Function
    srcpath_so = "%s/pg_plugin" % restoredir
    destpath_so = "%s/lib/postgresql/pg_plugin" % DefaultValue.getInstallDir(g_opts.user)
    g_logger.debug("Restore dir is %s, dest path is %s" % (srcpath_so, destpath_so))
    restoreNodeLibs(srcpath_so, destpath_so)
    g_logger.debug("End restore c function for cluster backup.")

def restoreTsDictFiles():
    """
    restore libs for TS_DICT
    """
    g_logger.debug("Start restore ts dict.")

    restoredir = os.path.join(g_opts.metaData, \
        "roach", g_opts.backupKey, "binaries")

    #restore .so for C-Function
    srcpath_so = "%s/share/postgresql/tsearch_data" % restoredir
    destpath_so = "%s/share/postgresql/tsearch_data" % DefaultValue.getInstallDir(g_opts.user)
    g_logger.debug("Restore dir is %s, dest path is %s" % (srcpath_so, destpath_so))
    restoreNodeLibs(srcpath_so, destpath_so)
    g_logger.debug("End restore ts dict.")

def dealTblspc():
    try:
        tblspcFlag = "PG_9.2_201611171_cn_"
        hostName = socket.gethostname()
        g_dbNodeInfo = g_clusterInfo.getDbNodeByName(hostName)
        for cnInst in g_dbNodeInfo.coordinators:
            tblspcPath = os.path.join(cnInst.datadir, 'pg_tblspc')
            if not os.path.exists(tblspcPath):
                g_logger.debug("Warining: The table space path %s is not exist." % (tblspcPath))
                return
            parentDirList = os.listdir(tblspcPath)
            for parentDirName in parentDirList:
                parentDirPath = os.path.join(tblspcPath, parentDirName)
                dirList = os.listdir(parentDirPath)
                for dirName in dirList:
                    repPath = tblspcFlag + str(cnInst.instanceId)
                    if (dirName.startswith(repPath)):
                        continue
                    if (dirName.startswith(tblspcFlag)):
                        srcPath = os.path.join(parentDirPath, dirName)
                        destPath = srcPath.replace(dirName, repPath)
                        cmd = "rm -rf %s" % (destPath)
                        g_logger.debug("Remove %s" % destPath)
                        status, output = commands.getstatusoutput(cmd)
                        if (status != 0):
                            g_logger.logExit("Remove %s failed with output(%s)" % (destPath, output))
                        g_logger.debug("rename %s to %s" % (srcPath, destPath))
                        os.rename(srcPath, destPath)
    except Exception, e:
        g_logger.logExit("Deal tableSpace failed, ERROR: %s" % (str(e)))

def restoreLibs(ip, libtype, srcpath, destpath, timeout = TIMEOUT_SCP):
    """
    restore libs
    """
    if (not os.path.exists(destpath)):
        g_logger.logExit("Error: destination path \"%s\" does not exist when restore libs" % (destpath))

    if g_opts.miniaturized:
        restorecmd = 'source %s && cp -rp %s/%s %s' \
            % (g_opts.sourceFile, srcpath, libtype, destpath)
    else:
        cmdPscp = "pscp -t %d -O StrictHostKeyChecking=no -H %s %s/%s %s" \
                  % (timeout, LoaclIp, srcpath, libtype, destpath)
        restorecmd = 'source %s && pssh -t %d -O StrictHostKeyChecking=no -s -H %s \"%s\"' \
            % (g_opts.sourceFile, timeout, ip, cmdPscp)
    g_logger.debug("libs restore cmd: %s" % (restorecmd))
    (status, output) = commands.getstatusoutput(restorecmd)
    if status != 0:
        g_logger.logExit("Error: restore file \"%s\" failed when restore libs! %s" \
            % (libtype, output))

def restoreLibFilesLocally():
    """
    restore libs for postgis and javaudf
    """

    g_logger.debug("start restore libs for cluster backup.")

    restoredir = os.path.join(g_opts.metaData, \
        "roach", g_opts.backupKey, "binaries")

    #restoreLibPostgis
    srcpath_lib = "%s/lib" % (restoredir)
    destpath_lib = "%s/lib" % (DefaultValue.getInstallDir(g_opts.user))
    libList = ["libgeos_c.so.*", "libproj.so.*", "libjson-c.so.*", \
        "libgeos-*.*.*so", "libgcc_s.so.*", "libstdc++.so.*"]
    for lib in libList:
        if (checkRemoteFile(srcpath_lib, lib)):
            restoreLibs(g_opts.masterIp, lib, srcpath_lib, destpath_lib)

    #restorePostgis
    srcpath_postgis  = "%s/postgresql" % (restoredir)
    destpath_postgis = "%s/postgresql" % (destpath_lib)
    postgisLibs = "postgis-*.*.so"
    if (checkRemoteFile(srcpath_postgis, postgisLibs)):
        restoreLibs(g_opts.masterIp, postgisLibs, srcpath_postgis, destpath_postgis)

    #restoreJavaudf
    srcpath_javaudf  = "%s/java" % (restoredir)
    destpath_javaudf = "%s/java" % (destpath_postgis)
    javaudfLibs = "*"
    #only check pljava.jar file
    pljavaLib = "pljava.jar"
    if (checkRemoteFile(srcpath_javaudf, pljavaLib)):
        restoreLibs(g_opts.masterIp, javaudfLibs, srcpath_javaudf, destpath_javaudf)

    #restoreshare
    srcpath_share = "%s/postgresql/extension" % (restoredir)
    destpath_share = "%s/share/postgresql/extension/" % (DefaultValue.getInstallDir(g_opts.user))
    shareTypeList = ["postgis--*.*.*.sql", "postgis.control"]
    for shareType in shareTypeList:
        if (checkRemoteFile(srcpath_share, shareType)):
            restoreLibs(g_opts.masterIp, shareType, srcpath_share, destpath_share)

    #restorebin
    srcpath_bin = "%s/bin" % (restoredir)
    destpath_bin = "%s/bin" % (DefaultValue.getInstallDir(g_opts.user))
    binTypeList = ["pgsql2shp", "shp2pgsql", "gs_tde_keys.cipher"]
    for binType in binTypeList:
        if (checkRemoteFile(srcpath_bin, binType)):
            restoreLibs(g_opts.masterIp, binType, srcpath_bin, destpath_bin)

    g_logger.debug("end restore libs for cluster backup.")

    g_logger.debug("Start restore secbox for cluster backup.")
    srcpath = "%s/secbox" % (restoredir)
    destpath = "%s/secbox" % (DefaultValue.getInstallDir(g_opts.user))
    secboxConf = "secbox.conf"
    if (checkRemoteFile(srcpath, secboxConf)):
        restoreLibs(g_opts.masterIp, secboxConf, srcpath, destpath)

    g_logger.debug("end restore secbox for cluster backup.")

    #restart om_monitor to avoid so mismatch due to above substitution
    #we only kill om_monitor process here and we will wait for it to restart at cluster start step
    g_logger.debug("restart monitor.")
    gaussHome = DefaultValue.getEnv("GAUSSHOME")
    cmd = "(ps ux | grep '\<%s\>' |  grep '%s' | grep -v grep | awk '{print $2}' | xargs -r kill -9 )" % ("om_monitor", gaussHome)
    g_logger.debug("Command to kill om_monitor process: %s" % cmd)
    retryTimes = 3
    for rtime in range(retryTimes):
        (status, output) = commands.getstatusoutput(cmd)
        if status == 0:
            g_logger.debug("Successfully killed om_monitor.")
            break
        else:
            g_logger.debug("Warning: failed to kill om_monitor in %s time, %s." % (rtime, output))
            time.sleep(5)

def checkPrefixFileExist():
    tmp_files =  glob.glob(g_opts.filePath)
    if (len(tmp_files) > 0):
        g_logger.log("REMOTE_FILE_EXIST")
    else:
        g_logger.log("REMOTE_FILE_NOT_EXIST")

def checkRemoteFile(fileDir, fileName):
    filePath = os.path.join(fileDir, fileName)
    cmd = "source %s && python %s -t %s --filePath '%s' -U %s -l %s" \
          % (g_opts.sourceFile, SCRIPT_ROACH_LOCAL, ACTION_CHECKPREFIXFILE, filePath, g_opts.user, g_opts.logFile)
    if not g_opts.miniaturized:
        remoteCmd = "pssh -s -H %s -t %d -O StrictHostKeyChecking=no \"%s\"" \
                    % (g_opts.masterIp, TIMEOUT_SCP, cmd)
    else:
        remoteCmd = cmd

    g_logger.debug("cmd to check remote file: %s" % remoteCmd)
    status, output = commands.getstatusoutput(remoteCmd)
    if (status != 0):
        g_logger.logExit("Check %s exist in %s failed with status: %d, output: %s" \
                         % (filePath, g_opts.masterIp, status, output))
    if ("REMOTE_FILE_EXIST" in output):
        return True
    if ("REMOTE_FILE_NOT_EXIST" in output):
        return False
    g_logger.logExit("Check %s exist in %s return wrong output: %s" \
                     % (filePath, g_opts.masterIp, output))

def upload_metadata():
    metaDataPath = g_opts.metaData
    metaDataPath = metaDataPath.rstrip("/")
    tarDirName = os.path.dirname(metaDataPath)
    tarBaseName = os.path.basename(metaDataPath)
    tarFileName = "%s/metadata.tar.gz" % tarDirName
    cmd = "tar -czf %s -C %s %s" % (tarFileName, tarDirName, tarBaseName)
    (status, output) = commands.getstatusoutput(cmd)
    if (status != 0):
        g_logger.logExit("Failed to tar backup file. Error: \n%s." % output)
    g_logger.debug("Successfully tar file %s" % tarFileName)
    if g_opts.mediaType.upper() == "NBU":
        cmd = 'source %s&&%s/bin/gs_roach uploadmeta --media-destination \'%s\' --metadata-destination \'%s\' --media-type \'NBU\' --backup-key \'%s\'' \
            % (g_opts.sourceFile, gauss_home_loc, g_opts.remoteMetaData, g_opts.metaData, g_opts.backupKey)
    elif g_opts.mediaType.upper() == "OBS":
        cmd = 'source %s&&%s/bin/gs_roach uploadmeta --metadata-destination \'%s\' --media-type \'OBS\'' \
        ' --backup-key \'%s\' --obs-server-ip \'%s\' --bucket-name \'%s\' --cluster-unique-id \'%s\'' \
            % (g_opts.sourceFile, gauss_home_loc, g_opts.metaData, g_opts.backupKey, 
            g_opts.OBSServerIp, g_opts.bucketName, g_opts.clusterUniqueID)

    g_logger.debug("Uploading metadata.")
    (status, output) = commands.getstatusoutput(cmd)
    if(status != 0):
        g_logger.logExit("Execute upload metadata Failed: %s" % cmd)
    cmd = "rm -rf %s" % (tarFileName)
    if(os.path.isfile(tarFileName) == True):
        output = commands.getstatusoutput(cmd)
    g_logger.debug("Upload metadata finished.")

def getHostNameForRestore():
    """
    Get hostname when restoring from OBS/NBU.
    """
    if len(g_opts.oldClusterHostname) > 0:
        return g_opts.oldClusterHostname
    clusterStaticFilePath = os.path.join(g_opts.metaData, 'bak_cluster_static_config')
    old_clusterInfo = dbClusterInfo()
    old_clusterInfo.initFromStaticConfig(g_opts.user, clusterStaticFilePath)
    for dbNode in old_clusterInfo.dbNodes:
        if g_dbNodeInfo.id == dbNode.id:
            hostname = dbNode.name
    return hostname

def download_metadata():
    metaDataPath = g_opts.metaData
    metaDataPath = metaDataPath.rstrip('/')
    tarDirName = os.path.dirname(metaDataPath)
    tarBaseName = os.path.basename(metaDataPath)
    tarFileName = "%s/metadata.tar.gz" % tarDirName
    if not os.path.exists(g_opts.metaData):
        g_logger.debug("Making dir:[%s]." % g_opts.metaData)
        os.makedirs(g_opts.metaData)
    hostname = getHostNameForRestore()
    #Usig local metada if the path is not empty
    judge_path = os.path.join(g_opts.metaData, "roach", g_opts.backupKey)
    if not g_opts.restoreNewCluster and os.path.exists(judge_path) and len(os.listdir(judge_path)) > 0:
        g_logger.debug("[%s] exists and is not empty!" % judge_path)
        return
    if g_opts.mediaType.upper() == "NBU":
        g_logger.debug("Downloading metadata from NBU.")
        cmd = 'source %s&&%s/bin/gs_roach downloadmeta --media-destination \'%s\' --metadata-destination \'%s\'' \
            '--media-type \'NBU\' --backup-key \'%s\' --old-cluster-hostname \'%s\'' \
            % (g_opts.sourceFile, gauss_home_loc, g_opts.remoteMetaData,
               g_opts.metaData, g_opts.backupKey, hostname)
    elif g_opts.mediaType.upper() == "OBS":
        cmd = 'source %s&&%s/bin/gs_roach downloadmeta --metadata-destination \'%s\' --media-type \'OBS\'' \
            ' --backup-key \'%s\' --obs-server-ip \'%s\' --bucket-name \'%s\' --cluster-unique-id \'%s\' --old-cluster-hostname \'%s\'' \
            % (g_opts.sourceFile, gauss_home_loc, g_opts.metaData, g_opts.backupKey, g_opts.OBSServerIp, 
            g_opts.bucketName, g_opts.clusterUniqueID, hostname)

    # in new cluster or new node, we need $GAUSSHOME/roach for lock files
    gaussHome = DefaultValue.getEnv("GAUSSHOME")
    tmpFilePath = os.path.join(gaussHome, "roach")
    mkdirCmd = 'source %s && if [ ! -d %s ]; then mkdir -p %s -m 700; fi' \
        % (g_opts.sourceFile, tmpFilePath, tmpFilePath)
    g_logger.debug("Creating $GAUSSHOME/roach path when restore new cluster, cmd: %s" % mkdirCmd)
    (status, output) = commands.getstatusoutput(mkdirCmd)
    if(status != 0):
        g_logger.debug("Warning: Create $GAUSSHOME/roach path failed, cmd is %s, status = %d, output is %s" % (mkdirCmd, status, output))

    if g_opts.restoreNewCluster:
        cmd += " --restore-new-cluster"
    g_logger.log("Downloading metadata from netbackup %s." % cmd)
    master_proc = subprocess.call([cmd], shell = True, bufsize = -1)
    if(os.path.isfile(tarFileName) == False):
        g_logger.debug("Download metadata :%s not exits." % tarBaseName)
        return
    cmd = "tar -zxPf %s -C %s --strip-components 1" % (tarFileName, g_opts.metaData)
    g_logger.debug("decompressing metadata: %s" % cmd)
    master_proc = subprocess.call([cmd], shell = True, bufsize = -1)
    cmd = "rm -rf %s" % (tarFileName)
    g_logger.debug("Cleaning tmp files: %s" % cmd)
    master_proc = subprocess.call([cmd], shell = True, bufsize = -1)

if __name__ == '__main__':
    parseCommandLine()
    checkParameter()
    initGlobals()
    if g_opts.action == ACTION_PRECHECK_PROCESS:
        pre_check_python_process()
    elif g_opts.action == ACTION_STOP:
        stop_roach()
    elif g_opts.action == ACTION_STOP_PITR:
        stop_pitr()
    elif g_opts.action == ACTION_WAIT_PITR:
        wait_pitr_finish()
    elif g_opts.action == ACTION_WAIT_PITR_RESTART:
        wait_pitr_restart()
    elif g_opts.action == ACTION_GET_BACKUP_KEY:
        get_backup_key()
    elif g_opts.action == ACTION_PAUSE:
        pause_resume_backup()
    elif g_opts.action == ACTION_CONFRESTORE:
        updateConfFile()
    elif g_opts.action == ACTION_DELETECONFNEW:
        deleteConfNewBeforeBuild()
    elif g_opts.action == ACTION_COPYNBCNFILES:
        copyNBFiles()
    elif g_opts.action == ACTION_COPYNBDNFILES:
        copyDNNBFilesParallel()
    elif g_opts.action == ACTION_BACKUPCFUNC:
        backupCFuncFiles()
    elif g_opts.action == ACTION_BACKUPTSDICT:
        backupTsDictFiles()
    elif g_opts.action == ACTION_RESTORECFUNC:
        restoreCFuncFiles()
    elif g_opts.action == ACTION_RESTORETSDICT:
        restoreTsDictFiles()
    elif g_opts.action == ACTION_RESTORELIBS:
        restoreLibFilesLocally()
    elif g_opts.action == ACTION_DEALTBLSPC:
        dealTblspc()
    elif g_opts.action == ACTION_CHECKPREFIXFILE:
        checkPrefixFileExist()
    elif g_opts.action == ACTION_UPLOAD_META:
        upload_metadata()
    elif g_opts.action == ACTION_DOWNLOAD_META:
        download_metadata()
    else:
        g_logger.logExit("Incorrect parameter. Unknown action: %s." % g_opts.action)

    sys.exit(0)
