#!/usr/bin/env python
# -*- coding:utf-8 -*-
#############################################################################
#Copyright (c): 2020, Huawei Tech. Co., Ltd.
#FileName     : GSroachRestore.py
#Version      : V1.0.0
#Date         : 2020-01-20
#Description  : GSroachRestore.py is a utility for data restore.
#############################################################################

try:
    import time
    import sys
    import os
    import commands
    import subprocess
    import signal
    import shutil
    import json
    import re
    import copy

    sys.path.append(sys.path[0] + "/../")
    from gspylib.common.GaussLog import GaussLog
    from util.GSroachCommon import sigint_handler, \
        final_ip_param_dict, getRoachOpResult, makeRaochOptions, \
        master_host, isClusterBackup, remove_pattern, reform_tablename, \
        isMiniaturizedDeployment, checkFile, check_gs_roach_running, \
        create_roachPath, mark_completion_progress, \
        gauss_home_loc, proc_id_dict, parseMaster_generated_file, \
        sendSignal, pollSubprocess, create_command, createLogPath, pssh_tout, \
        getMaster_ip, parseRoachConf, validate_logparams, gsinitLogger, \
        master_ip, getClusterVersion, LOCAL_ROACH, getBackupDetails, \
        gsinitLogger, getMetaDest, getArchiveBackupKeys, checkClusterNodeCount, \
        postExecCleanup, getRoachOpResult, compareVersionFile, \
        SCRIPT_ROACH_LOCAL, loadMetaFileAsJson, hasStopProcess, \
        getNodesInfoByUpdatedFile, deliveryMetaFileToAgents, ETCD_TERM_FILE, \
        isSinglePrimaryMultiStandbyDeployment, copyStaticConfig, deliveryOneItemToAgents, \
        agentErrorReport
    from gspylib.common.DbClusterInfo import dbClusterInfo
    from gspylib.common.Common import DefaultValue, ClusterCommand
    from gspylib.common.OMCommand import OMCommand
    from gspylib.threads.SshTool import SshTool
    from gspylib.common.ErrorCode import ErrorCode
    from gspylib.os.gsfile import g_file
    import fcntl
    import csv
    from util.GSroachCommonParameter import validateRoachParams
except ImportError, e:
    sys.exit('Error: Unable to import module: ' + str(e))

#global variables
userProfile = "~/.bashrc"
osProfile = "/etc/profile"
currPath = os.path.dirname(os.path.realpath(__file__))
outFile = ''
g_mediatype = ""
g_mediadest = ""
g_backupkey = ""
master_ip = ""
bkp_key = ""

CN_TYPE = "coordinator"
DN_TYPE = "datanode"
GTM_TYPE = "gtm"
CM_TYPE = "cm"

#Warning: this var cannot be replaced by isClusterBackup()
g_isClusterBkp = False #Cleanup on Restore failure to be done only for Cluster Restore
cluster_version = 0
backup_key = None
g_logger = None
g_dbClusterInfo = None
bkpkeylist = None
isIncremental = None
noBackupIni = None
ACTION_RENAME_BACKUPLABEL= "rename_backup_label"
attributeIndex = {
    'BackupKey': 0, 'BackupType': 1, 'PreBkpKey': 2, 'StartLsn': 3, 'BarrierLsn': 4, 'StartTime': 5, 'BarrierTime': 6,
    'EndTime': 7,  'ApplyStatus': 8, 'ApplyPid': 9, 'ApplyTime': 10, 'MediaType': 11, 'Version': 12, 'DeleteStatus': 13,
    'StartRestoreTime': 14, 'EndRestoreTime': 15, 'RemainedRestore': 16, 'RestoreFaildCount': 17, 'BackupState': 18
}


signal.signal(signal.SIGINT, sigint_handler)

#set guc and write state
def renameBackupLabel(g_opts, g_logger):
    if (g_opts.doubleCluster):
        cmd = "source %s && python %s -t %s --double-cluster -U %s -l %s" \
            % (g_opts.sourceFile, LOCAL_ROACH, ACTION_RENAME_BACKUPLABEL, g_opts.user, g_opts.logFile)
    elif (g_opts.parallelRestore):
        cmd = "source %s && python %s -t %s --parallel-restore -U %s -l %s" \
            % (g_opts.sourceFile, LOCAL_ROACH, ACTION_RENAME_BACKUPLABEL, g_opts.user, g_opts.logFile)
    else:
        cmd = "source %s && python %s -t %s -U %s -l %s" \
            % (g_opts.sourceFile, LOCAL_ROACH, ACTION_RENAME_BACKUPLABEL, g_opts.user, g_opts.logFile)
    g_logger.debug("GaussRoach.py calling LocalRoach.py to rename bakcup_label.roach, cmd=[%s]" % (cmd))
    try:
        g_clusterInfo = dbClusterInfo()
        g_clusterInfo.initFromStaticConfig(g_opts.user)
        if not isMiniaturizedDeployment(cluster_version):
            sshTool = SshTool(g_clusterInfo.getClusterNodeNames(), g_logger.logFile, timeout = 60)
            sshTool.executeCommand(cmd, "rename bakcup_label.roach")
        else:
            status, output = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.logExit("Rename backup_label.roach failed with output: %s" % output)
        g_logger.debug("Successfully rename bakcup_label.roach.")
    except Exception, e:
        g_logger.logExit("Failed to rename bakcup_label.roach, output: %s " %  str(e))

def dealCnTableSpace(g_opts, sshTool):
    if g_opts.tableName == '' and g_opts.tablelist == '' and g_opts.mediaType.lower() == 'disk' and g_opts.doubleCluster:
        pyPath = os.path.dirname(currPath)
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t dealtblspc -U %s -l %s" \
              % (g_opts.sourceFile, pyPath, g_opts.user, g_opts.localLog)
        g_logger.debug("Deal CN table space cmd: %s" % cmd)
        if not isMiniaturizedDeployment(cluster_version):
            (status, output) = sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
            g_logger.debug("Deal CN table space status.values() is %s" % str(status.values()))
            g_logger.debug("Deal CN table space output is %s" % output)
            if 'Failure' in status.values():
                raise Exception("Deal CN table space failed!")
        else:
            status, output = commands.getstatusoutput(cmd)
            if (status != 0):
                raise Exception("Deal CN table space failed with output: %s!" % (output))
        g_logger.debug("Deal CN table space successfully.")

def downloadMetaForNewCluster(g_opts, other_node_list):
    '''
    Download the metadata backed up by the client, 
    analyze the correspondence between the old and new clusters from it, 
    and then download the metadata of the corresponding nodes again,
    after the download is complete, perform the normal new cluster recovery process.
    '''
    g_logger.debug("Downloading old cluster metadata.")
    if not os.path.exists (os.path.join(g_opts.metadataDestination, "bak_cluster_static_config")):
        if g_opts.action == "restore":
            backupKey = g_opts.backupKey
        elif ((g_opts.action == "backup") and (g_opts.priorBkpKey != "")):
            backupKey = g_opts.priorBkpKey
        if g_opts.mediaType.upper() == "NBU":
            cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t" \
                " downloadmetadata --backupKey %s --metaData '%s' --mediaType 'NBU' --remoteMetaData '%s' --sourceFile '%s' -U %s" \
                " --restore-new-cluster --oldClusterHostname %s" \
                % (g_opts.sourceFile, os.path.dirname(currPath), backupKey, g_opts.metadataDestination,
                g_opts.mediaDest, g_opts.sourceFile, g_opts.user, g_opts.oldClusterHostname)
        elif g_opts.mediaType.upper() == "OBS":
            cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t downloadmetadata" \
                " --backupKey %s --mediaType 'OBS' --metaData '%s' --sourceFile '%s' -U %s --restore-new-cluster" \
                " --oldClusterHostname %s --obs-server-ip \'%s\' --bucket-name \'%s\' --clusterUniqueID \'%s\'" \
                % (g_opts.sourceFile, os.path.dirname(currPath), backupKey, g_opts.metadataDestination, 
                g_opts.sourceFile, g_opts.user, "cluster_static_config", g_opts.OBSAddres, g_opts.bucketName, g_opts.clusterUniqueID)
        g_logger.debug("download metadata for new cluster command : %s" % cmd)
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Download metadata for new cluster ERROR: %s" % output)

    nodes = " -H ".join(other_node_list)

    psshCmd = "pssh -H %s \"if [ ! -d %s ]; then mkdir -p %s -m 700; fi\"" % (nodes, g_opts.metadataDestination, g_opts.metadataDestination)
    status, output = commands.getstatusoutput(psshCmd)
    if (status != 0):
        g_logger.logExit("Create metadata path in new cluster failed : %s" % output)

    pscpCmd = "pscp -H %s %s/bak_cluster_static_config %s/bak_cluster_static_config" \
        % (nodes, g_opts.metadataDestination, g_opts.metadataDestination)

    g_logger.debug("pscpCmd of copy cluster conf is : %s" % pscpCmd)
    status, output = commands.getstatusoutput(pscpCmd)
    if (status != 0):
        g_logger.logExit("copy clusterconf in new cluster failed : %s" % output)

    if not os.path.exists(g_opts.metadataDestination) or len(os.listdir(g_opts.metadataDestination)) == 0:
       g_logger.log("Downloading metadata in all nodes failed.")
       g_logger.logExit("[GAUSS-50201] %s does not exist." % os.path.join(g_opts.metadataDestination, "bak_cluster_static_config"))

def download_metadata_in_all_node(g_opts):
    global g_logger
    g_logger = gsinitLogger(g_opts)
    if g_opts.action == "restore":
        backupKey = g_opts.backupKey
    elif ((g_opts.action == "backup") and (g_opts.priorBkpKey != "")):
        backupKey = g_opts.priorBkpKey
    l_clusterInfo = dbClusterInfo()
    l_clusterInfo.initFromStaticConfig(g_opts.user)
    l_sshTool = SshTool(l_clusterInfo.getClusterNodeNames(), timeout = 60 * 10)
    # check metadata in every node, if exits, use local metadata
    metadataFile = os.path.join(g_opts.metadataDestination, "roach", backupKey)
    checkCmd = "if [ -d '%s' ]; then echo 'GetFile'; else echo 'NoThisFile'; fi" % metadataFile
    (status, output) = l_sshTool.getSshStatusOutput(checkCmd, g_opts.nodeList)
    g_logger.debug("Cmd of check metadata in every node: %s" % checkCmd)
    g_logger.debug("The output is %s" % output)
    if (not "Failure" in status.values()) and (not "NoThisFile" in output):
        g_logger.debug("Using local metadata")
        return
    g_logger.log("Downloading metadata in all nodes.")   
    other_node_list = l_clusterInfo.getClusterNodeNames()
    other_node_list.remove((l_clusterInfo.getLocalNode()).name)
    downloadMetaForNewCluster(g_opts, other_node_list)
    if g_opts.mediaType.upper() == "NBU":
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t" \
            " downloadmetadata --backupKey %s --metaData '%s' --mediaType 'NBU' --remoteMetaData '%s' --sourceFile '%s' -U %s" \
            % (g_opts.sourceFile, os.path.dirname(currPath), backupKey, g_opts.metadataDestination,
            g_opts.mediaDest, g_opts.sourceFile, g_opts.user)
    elif g_opts.mediaType.upper() == "OBS":
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t downloadmetadata" \
            " --backupKey %s --mediaType 'OBS' --metaData '%s' --sourceFile '%s' -U %s" \
            " --obs-server-ip \'%s\' --bucket-name \'%s\' --clusterUniqueID \'%s\'" \
            % (g_opts.sourceFile, os.path.dirname(currPath), backupKey, g_opts.metadataDestination, 
            g_opts.sourceFile, g_opts.user, g_opts.OBSAddres, g_opts.bucketName, g_opts.clusterUniqueID)

    if g_opts.restoreNewCluster:
        cmd += " --restore-new-cluster"
    if isMiniaturizedDeployment(getClusterVersion(g_opts.user)):
        cmd += " --miniaturized"
    g_logger.debug("Downloading metadata in all nodes: cmd %s" % cmd)
    (status, output) = l_sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
    g_logger.debug("The status.values() is %s" % str(status.values()))
    g_logger.debug("The output is %s" % output)
    if 'Failure' in status.values():
        g_logger.logExit("SshTools Execute cmd failed: %s" % cmd)
    if not os.path.exists(g_opts.metadataDestination):
        g_logger.logExit("Failed to download metadata, please check if the path has permission.")

    if not "not empty" in output:
        g_logger.log("Downloading metadata in all nodes finished.")

def doRestore(g_opts, command_dict, sshTool, table_list = []):
    """
    do restore operation
    """
    global bkp_key
    global g_isClusterBkp
    bkp_key = g_opts.backupKey

    g_logger.debug("start restore operation")
        
    if g_opts.logical:
        #Schema name additional handling
        restoreOnNodes(g_opts, command_dict, bkp_key)
        if getRoachOpResult():
            g_logger.debug("Restore failed...")
            return
        else:
            final_ip_param_dict.clear()
            makeRaochOptions(g_opts, master_ip, master_host, command_dict)
            g_logger.log("Logical Mult-table Restore SUCCESSFUL!")
    else:
        restoreOnNodes(g_opts, command_dict)
        if getRoachOpResult():
            g_logger.debug("Restore failed...")
            return

        #rename backup_label.roach to backup_label for cluster
        if g_opts.tableName == '' and g_opts.tablelist == '' and (not g_opts.clusterMigration):
            renameBackupLabel(g_opts, g_logger)
            #restore postgis and javaudf libs
            g_logger.debug("This is a cluster restore operation, restore lib files.")
            l_ClusterInfo = dbClusterInfo()
            l_ClusterInfo.initFromStaticConfig(g_opts.user)
            if (g_opts.priorBkpKey == ""):
                restoreLibFiles(g_opts, backup_key, sshTool, master_ip)
            dealRestoreCFunc(g_opts, backup_key, sshTool)
            dealRestoreTsDict(g_opts, backup_key, sshTool)
            dealCnTableSpace(g_opts, sshTool)
            dealRestoreEtcdTermFile(g_opts, backup_key)

    g_logger.debug("End restore operation")

def logOptions():
    """
    writing to logfile
    """
    for key in final_ip_param_dict.keys():
        g_logger.debug("options for %s" % key)
        for key, val in final_ip_param_dict[key].items():
            if key == "-W":
                g_logger.debug("%s : %s" %(key.translate(remove_pattern, "--"), "***"))
            else:
                g_logger.debug("%s : %s" % (key.translate(remove_pattern, "--"), val))

#In case master metadata file is present in only in old-format
#path, this will silently copy it to the new-format path!
def adjustForCompatibility(dictKey):
    """
    check and adjust compatibility for metadata-destination
    """
    #global dictionary
    global final_ip_param_dict

    try:
        metafile = 'roach_master.metadata'
        meta_dst = final_ip_param_dict[dictKey].get('--metadata-destination')
        bkup_key = final_ip_param_dict[dictKey].get('--backup-key')
        new_name = os.path.join(os.path.join(os.path.join(meta_dst, 'roach'), bkup_key), metafile)
        old_name = os.path.join(os.path.join(meta_dst, bkup_key), metafile)
        if not os.path.exists(new_name) and os.path.exists(old_name):
            if os.path.exists(os.path.dirname(new_name)):
                shutil.copy2(old_name, os.path.dirname(new_name))
            else:
                shutil.copytree(os.path.dirname(old_name), os.path.dirname(new_name))
    except Exception:
        #Note : If any exception comes, fail silently!
        pass

def createMasterOption(g_opts):
    """
    create option for master
    """
    #global dictionary
    global final_ip_param_dict
    new_value = None
    final_value = ''
    for k in final_ip_param_dict.keys():
        for key, val in final_ip_param_dict[k].iteritems():
            if key == '--pitr':
                g_logger.debug("pitr is opened.")

            if key == '--parallel-restore':
                g_logger.debug("parallel restore is opened.")
        
            if key == '--restore-target-time':
                g_logger.debug("restore target time is %s." % val)
                if not g_opts.pitr:
                    continue
            
            if val != None:
                if key == '--tablename':
                    val = reform_tablename(val)
                    #If tablename and/or schemaname starting with
                    #"digit", need to protect them with quotes...
                    if '.' in val:
                        lval = val.split('.')
                    else:
                        lval = [val]
                    tname = ''
                    for elem in lval:
                        if elem.strip()[0].isdigit() or '$' in elem:
                            #Add quoting...
                            tname += '\'"%s"\'.' % elem.strip()
                        else:
                            tname += '%s.' % elem.strip()
                    tname = tname[:-1]
                    new_value = ''.join("%s %s " %(key, tname))
                elif key == '--restore-target-time':
                    new_value = ''.join("%s \'\"%s\"\' " %(key, val)) 
                else:
                    new_value = ''.join("%s %r " %(key, val))
            else:
                new_value = ''.join("%s " %(key))
            if key == '--metadata-destination':
                #Fixing possible compatibility break for Metadata destination
                adjustForCompatibility(k)
            final_value += new_value
    if g_opts.logical:
        final_value += "--backup-type logical-mtable "

    if isMiniaturizedDeployment(cluster_version):
        final_value += ''.join("--miniaturized-deployment ")

    return final_value

def restoreLibFiles(g_opts, backupkey, sshTool, master_ip):
    pyPath = os.path.dirname(currPath)
    g_logger.debug("Restoring other lib files.")

    if (g_opts.mediaType.upper() == "OBS"):
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
        restorelibfiles --backupKey %s --metaData '%s' --clusterUniqueID %s --mediaType %s \
        --masterIp %s --sourceFile '%s' -U %s -l %s  " \
        % (g_opts.sourceFile, pyPath, backupkey, g_opts.metadataDestination, str(g_opts.clusterUniqueID), \
        g_opts.mediaType, master_ip, g_opts.sourceFile, g_opts.user, g_opts.localLog)
    else:
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
        restorelibfiles --backupKey %s --metaData '%s' --mediaType %s --masterIp %s --sourceFile '%s' \
        -U %s -l %s  " \
        % (g_opts.sourceFile, pyPath, backupkey, g_opts.metadataDestination, g_opts.mediaType, \
           master_ip, g_opts.sourceFile, g_opts.user, g_opts.localLog)

    g_logger.debug("cmd to restore libs: %s" % cmd)
    if not isMiniaturizedDeployment(cluster_version):
        (status, output) = sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
        g_logger.debug("The status.values() is %s" % str(status.values()))
        g_logger.debug("The output is %s" % output)
        if 'Failure' in status.values():
            g_logger.logExit("Restore lib backup files failed!")
    else:
        cmd += " --miniaturized"
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Restore lib files failed with output: %s" % (output))
    g_logger.debug("Restore lib backup files successfully.")

def dealRestoreCFunc(g_opts, backupkey, g_sshTool):
    pyPath = os.path.dirname(currPath)
    g_logger.debug("Restoring up C function so file.")

    if (g_opts.mediaType.upper() == "OBS"):
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
        restorecfuncfiles --backupKey %s --metaData '%s' --clusterUniqueID %s --mediaType %s \
        --sourceFile '%s' -U %s -l %s  " \
        % (g_opts.sourceFile, pyPath, backupkey, g_opts.metadataDestination, str(g_opts.clusterUniqueID), \
        g_opts.mediaType, g_opts.sourceFile, g_opts.user, g_opts.localLog)
    else:
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
        restorecfuncfiles --backupKey %s --metaData '%s' --sourceFile '%s' \
        -U %s -l %s  " \
        % (g_opts.sourceFile, pyPath, backupkey, g_opts.metadataDestination, g_opts.sourceFile, \
        g_opts.user, g_opts.localLog)

    if not isMiniaturizedDeployment(cluster_version):
        (status, output) = g_sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
        g_logger.debug("The status.values() is %s" % str(status.values()))
        g_logger.debug("The output is %s" % output)
        if 'Failure' in status.values():
            g_logger.logExit("Restore CFunc files failed with output: %s" % (output))
    else:
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Restore CFunc files with output: %s" % (output))
    g_logger.debug("Restore CFunc files successfully.")

def dealRestoreTsDict(g_opts, backupkey, g_sshTool):
    pyPath = os.path.dirname(currPath)
    g_logger.debug("Restoring up ts dict file.")

    if (g_opts.mediaType.upper() == "OBS"):
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
        restoretsdictfiles --backupKey %s --metaData '%s' --clusterUniqueID %s --mediaType %s \
        --sourceFile '%s' -U %s -l %s  " \
        % (g_opts.sourceFile, pyPath, backupkey, g_opts.metadataDestination, str(g_opts.clusterUniqueID), \
        g_opts.mediaType, g_opts.sourceFile, g_opts.user, g_opts.localLog)
    else:
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
        restoretsdictfiles --backupKey %s --metaData '%s' --sourceFile '%s' \
        -U %s -l %s  " \
        % (g_opts.sourceFile, pyPath, backupkey, g_opts.metadataDestination, g_opts.sourceFile, \
        g_opts.user, g_opts.localLog)

    if not isMiniaturizedDeployment(cluster_version):
        (status, output) = g_sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
        g_logger.debug("The status.values() is %s" % str(status.values()))
        g_logger.debug("The output is %s" % output)
        if 'Failure' in status.values():
            g_logger.logExit("Restore ts dic files failed with output: %s" % (output))
    else:
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Restore ts dic files with output: %s" % (output))
    g_logger.debug("Restore ts dic files successfully.")

def dealRestoreEtcdTermFile(g_opts, backupkey):
    #just return if no etcd is deployed or it is master-standby cluster
    if (DefaultValue.hasEtcd(g_dbClusterInfo) == False or g_dbClusterInfo.isMasterStandbyCluster()):
        g_logger.debug("No need to restore etcd term file due to no etcd installed or master-standby cluster.")
        return

    g_logger.debug("Restoring up etcd term file.")

    filepath = os.path.join(g_opts.metadataDestination, \
        "roach", backupkey)
    etcdTermFile = os.path.join(filepath, ETCD_TERM_FILE)

    if (not os.path.isfile(etcdTermFile)):
        g_logger.logExit(ErrorCode.GAUSS_534["GAUSS_53400"] + \
                         "etcd term file does not exist, expected file path is %s" % etcdTermFile)

    destpath = os.path.join(DefaultValue.getEnv("GAUSSHOME"), 'bin')
    cmd = "source %s && cp -frp %s %s" % (g_opts.sourceFile, etcdTermFile, destpath)
    g_logger.debug("restore etcd term file command: %s" % (cmd))
    status, output = commands.getstatusoutput(cmd)
    if status != 0:
        g_logger.logExit(ErrorCode.GAUSS_534["GAUSS_53400"] + \
                         "failed to restore etcd term file, output: %s" % output)

    g_logger.debug("Restore etcd term file successfully.")

def restoreOnNodes(g_opts, command_dict, bkp_key = None):
    """
    restore on each nodes
    """

    g_logger.debug("Checking flock of python in restoreOnNodes")
    check_gs_roach_running(g_opts)
    try:
        proc_list_dict = dict()
        validateRoachParams(g_opts)

        global g_mediatype
        global g_mediadest
        global g_backupkey
        if bkp_key:
            g_mediadest = g_opts.mediaDest
            g_mediatype = g_opts.mediaType
            g_backupkey = bkp_key

        if not command_dict.has_key("--restore-new-cluster") and not noBackupIni:
            for node_ip in final_ip_param_dict.keys():
                final_ip_param_dict[node_ip]["--media-destination"] = g_mediadest
                final_ip_param_dict[node_ip]["--media-type"] = g_mediatype
                final_ip_param_dict[node_ip]["--backup-key"] = g_backupkey

        option = createMasterOption(g_opts)
        create_roachPath(master_ip, g_opts)
        for key in final_ip_param_dict.keys():
            if ('--tablename' in final_ip_param_dict[key].keys()) \
              or ('--logical' in final_ip_param_dict[key].keys()):

                if (bkp_key == None):
                    cmd = 'source %s&&%s/bin/gs_roach --master restore '\
                          '--master-ip \"%s\" %s' %(g_opts.sourceFile, \
                        gauss_home_loc, master_ip, option)
                else:
                    cmd = 'source %s&&%s/bin/gs_roach --master restore '\
                          '--backup-key \"%s\" --master-ip \"%s\" %s' \
                          %(g_opts.sourceFile, gauss_home_loc, bkp_key, \
                            master_ip, option)
                table_restore = True
            else:
                cmd = 'source %s&&%s/bin/gs_roach --master restore '\
                      '--master-ip \"%s\" %s' \
                      %(g_opts.sourceFile, gauss_home_loc, master_ip, option)

        # removing old intermediate file before master creates same
        master_generated_file = '%s/bin/roach_nodes.done' % (gauss_home_loc)
        removeFile(master_generated_file)

        #####################################################################
        #                  !! IMPORTANT SECURITY NOTE !!                    #
        # Regarding usage of subprocess.Popen([cmd], shell=True, bufsize=-1)#
        #  1.All the inputs from user are captured as individual options    #
        #  2.Direct inputs from the user are not passed in subprocess to    #
        #    gs_roach.                                                      #
        #  3.All the options are validated for the type as well as range    #
        #  4.The commands to be launched through subprocess are formed in   #
        #    code and passed as string to shell                             #
        #####################################################################

        g_logger.debug("python call gs_roach restore in restoreOnNodes, cmd: %s" % cmd)
        master_proc = subprocess.Popen([cmd], shell=True, bufsize=-1)
        proc_list_dict.update({master_ip:master_proc})
        master_pid_cmd = 'ps -aef | grep %s | grep \"%s/bin/gs_roach --master restore --master-ip %s\" | grep -v grep | awk {\'print $2\'}' \
            %(g_opts.user, gauss_home_loc, master_ip)
        # wait time to get the master process id
        master_pid = ''
        master_pid = getMasterPID(master_pid_cmd, master_pid)
        if master_pid:
            proc_id_dict.update({master_ip:master_pid})
        cluster_node_details = ""
        tmp_val = 0
        while not os.path.isfile(master_generated_file) \
          and tmp_val < g_opts.metadataFileWaittime and master_proc.poll() == None:
            time.sleep(1)
            tmp_val = tmp_val + 1
        if os.path.isfile(master_generated_file):
            tmp_val = tuple()
            tmp_val = parseMaster_generated_file(master_generated_file)
            cluster_node_details = tmp_val[1]
            global backup_key
            backup_key = tmp_val[0]
        else:
            raise Exception("Error : Could not fetch cluster node details from metadata, roach_nodes.done file not found")

        #start agents
        startAgents(cluster_node_details, g_opts, command_dict, proc_list_dict)

        # make sure we do not miss concurrent stop backup operation
        if hasStopProcess(g_logger, g_opts, g_dbClusterInfo, cluster_version):
            g_logger.log("Warning: Send singal to kill gs_roach due to user cancel or another operation is stopping!")
            sendSignal()

    except Exception, e:
        g_logger.debug("Error: Some exception happened in restoreOnNodes")
        sendSignal()
        GaussLog.exitWithError(str(e))
    count = 0
    pghost = DefaultValue.getEnv("PGHOST")
    while len(proc_list_dict) > 0:
        time.sleep(1)
        count += 1
        if count % 5 == 0 :
            deliveryOneItemToAgents(pghost, "/.roach_task_progress.dat", g_opts)
        pollSubprocess(proc_list_dict, g_opts)

    mark_completion_progress(g_opts)

def getMasterPID(master_pid_cmd, master_pid):
    """
    function to fetch master process id
    """
    pid_wait = 0
    while master_pid == '' and pid_wait < 5:
        time.sleep(1)
        master_pid_output = commands.getstatusoutput(master_pid_cmd)
        master_pid = master_pid_output[1]
        pid_wait = pid_wait + 1
    return master_pid


def check_mastreIP(master_ip, cluster_node_details):
    """
    check for multiple ip associated with master ip
    """
    if ' ' in master_ip:
        mast_ips = master_ip.split(' ')
        for ip in mast_ips:
            if ip in cluster_node_details.keys():
                cluster_node_details.pop(ip)

def startAgents(cluster_node_details, g_opts, command_dict, proc_list_dict):
    """
    start restore on agents
    """
    global g_mediatype
    global g_mediadest
    global g_backupkey

    check_mastreIP(master_ip, cluster_node_details)
    for node_ip, host in cluster_node_details.items():
        create_roachPath(node_ip, g_opts)
        makeRaochOptions(g_opts, node_ip, host, command_dict)
    validateRoachParams(g_opts)
    logOptions()

    if g_opts.tablelist:
        g_mediadest = g_opts.mediaDest
        g_mediatype = g_opts.mediaType
        g_backupkey = bkp_key

    ##removing --tablename from final_ip_param_dict
    for node_ip in final_ip_param_dict.keys():
        if node_ip != master_ip:
            if '--tablename' in final_ip_param_dict[node_ip].keys():
                del final_ip_param_dict[node_ip]['--tablename']

            if isIncremental:
                final_ip_param_dict[node_ip].update({"--incremental": None})
            ## adding media-destination and media-type
            if not command_dict.has_key("--restore-new-cluster") and not noBackupIni:
                final_ip_param_dict[node_ip]["--media-destination"] = g_mediadest
                final_ip_param_dict[node_ip]["--media-type"] = g_mediatype
                final_ip_param_dict[node_ip]["--backup-key"] = g_backupkey

    ## creating deepcopy of the final ip param dictionary
    tempFinalDict = copy.deepcopy(final_ip_param_dict)

    create_command(g_opts)
    for node_ip, option in final_ip_param_dict.items():
        if node_ip != master_ip:
            proc_pid = '-1'
            c_log = createLogPath(g_opts.loggingPath)
            cmd = 'source %s&&pssh -t %s -H \"%s\" -P \"%s&& source %s&&%s/bin/gs_roach\" --master-ip \\"%s\\" %s "2>&1"' \
                % (g_opts.sourceFile, pssh_tout, node_ip, c_log, g_opts.sourceFile, \
                gauss_home_loc, master_ip, option)
            pid_cmd = 'source %s&&pssh -t %s -H \"%s\" -P ps -aef | grep %s | grep \"%s/bin/gs_roach --master-ip %s\" | grep -v grep | awk {\'print $2\'}' \
                % (g_opts.sourceFile, pssh_tout, node_ip, g_opts.user, gauss_home_loc, master_ip)

            #####################################################################
            #                  !! IMPORTANT SECURITY NOTE !!                    #
            # Regarding usage of subprocess.Popen([cmd], shell=True, bufsize=-1)#
            #  1.All the inputs from user are captured as individual options    #
            #  2.Direct inputs from the user are not passed in subprocess to    #
            #    gs_roach.                                                      #
            #  3.All the options are validated for the type as well as range    #
            #  4.The commands to be launched through subprocess are formed in   #
            #    code and passed as string to shell                             #
            #####################################################################

            g_logger.debug("python call gs_roach restore in startAgents, cmd: %s" % cmd)
            agent_proc = subprocess.Popen([cmd], stdout=subprocess.PIPE, \
                         stderr=subprocess.PIPE, shell=True, bufsize=-1)
            proc_list_dict.update({node_ip:agent_proc})
            proc_for_pid = subprocess.Popen([pid_cmd], \
                           stdout=subprocess.PIPE, \
                           stderr=subprocess.PIPE, shell=True, bufsize=-1)
            proc_output = proc_for_pid.stdout.readlines()
            if proc_output:
                proc_pid = proc_output[0].strip()
            if proc_pid.isdigit() and (proc_pid != '-1'):
                proc_id_dict.update({node_ip:proc_pid})
    return proc_list_dict

def removeFile(master_generated_file):
    """
    remove intermediate file
    """
    cmd = 'rm -f %s' % (master_generated_file)
    g_logger.debug("deleting file, cmd: %s" % cmd)
    if os.path.isfile(master_generated_file):
        g_logger.debug("deleting file again, cmd: %s" % cmd)
        output = commands.getstatusoutput(cmd)
        del_file_status = output[0]
        if del_file_status != 0:
            g_logger.log("Warning :while deleting %s" % master_generated_file)

def validateGenerateParams(g_opts):
    """
    check params for generateXML command
    """
    global outFile
    for key in final_ip_param_dict.keys():
        #Check mandatory params present
        if not final_ip_param_dict[key].has_key('--media-destination') or \
           not final_ip_param_dict[key].has_key('--metadata-destination') or \
           not final_ip_param_dict[key].has_key('--master-port') or \
           not final_ip_param_dict[key].has_key('--backup-topology') or \
           not final_ip_param_dict[key].has_key('--backup-key'):
            g_logger.logExit("Error: Mandatory options media-destination/metadata-destination/master-port/backup-topology/backup-key not specified.\n GenXML operation failed!!")
        if not (final_ip_param_dict[key])['--backup-topology'].lower() == 'xml':
            g_logger.logExit('Error: Only XML backup-topology type supported.\n GenXML operation failed!!')

        #Get actual-canonical path for backup key and check it
        g_opts.mediaDest = os.path.realpath(g_opts.mediaDest)
        bkpPath = os.path.join(g_opts.mediaDest, 'roach/' + g_opts.backupKey)
        if not os.access(bkpPath, os.R_OK):
            g_logger.logExit('Error: The dir "'+ bkpPath + '" does not exist.\n GenXML operation failed!!')

        #Check output path
        outFile = os.path.realpath(g_opts.outputFile)
        outDir = os.path.dirname(outFile)
        g_opts.outputFile = outDir
        final_ip_param_dict[key]['--output-file'] = outDir
        if not os.access(outDir, os.W_OK | os.X_OK):
            g_logger.logExit('Error: Failed to write "' + (outDir) + '".\n GenXML operation failed!!')
        if os.path.isfile(outFile) and g_opts.generateForce == False:
            g_logger.logExit("Error: --output-file already exists, provide '--generate-force' to overwrite.")

        #If XML already available in MetaDest then just copy it!
        metaDest = final_ip_param_dict[key]['--metadata-destination']
        g_opts.metadataDestination = metaDest #Save the value in object
        global backup_key
        backup_key = final_ip_param_dict[key]['--backup-key']
        media_type = final_ip_param_dict[key]['--media-type']
        g_opts.mediaType = media_type

        outMetaFile = os.path.join(metaDest, "roach", backup_key, \
            'clusterTopology_%s.xml' % (backup_key))
        if os.access(outMetaFile, os.R_OK):
            try:
                shutil.copy(outMetaFile, outFile)
            except (shutil.Error, IOError), err:
                if 'are the same file' in str(err):
                    g_logger.debug("File '%s' already present!" % outMetaFile)
                else:
                    #Since it failed here, we'll try complete GENERATE ahead...
                    g_logger.debug("XML copy to metadata destination failed!\n%s" % str(err))
                    return False
            g_logger.log("GenXML operation successful!\nOutput: %s" % (outFile))
            return True
        else:
            return False

def generateXMLTopology(g_opts, command_dict):
    """
    call for Topology XML generation
    """
    global outFile
    global master_ip
    global backup_key

    master_ip = getMaster_ip()
    bFailed = True
    bNeedToCopyToMeta = False
    #If media-type not specified, add as 'DISK'
    if '--media-type' not in command_dict.keys():
        command_dict['--media-type'] = 'DISK'
    if command_dict['--media-type'].upper() != "DISK" and not 'backup' in [g_opts.action, g_opts.origAction]:
        print "Error: Invalid media-type ['%s'] for generate operation" % command_dict['--media-type']
        sys.exit(1)

    #If automatically triggered from backup, then add needed params and finish!
    if 'backup' in [g_opts.action, g_opts.origAction]:
        if not g_opts.tableName and not g_opts.mediaType.lower() == 'nbu':
            clustStaticFile = os.path.join(gauss_home_loc, "bin", 'cluster_static_config')
            metaDest = g_opts.metadataDestination #Get the val used during backup!
            backup_key = command_dict["--backup-key"]
            outFile = os.path.join(metaDest, "roach", backup_key, 'clusterTopology_%s.xml' % (backup_key))
            clusterInfo = dbClusterInfo()
            clusterInfo.generateXMLFromStaticConfigFile(g_opts.user, clustStaticFile, outFile)
        return not bFailed

    #If output-file not specified, use default value in PWD
    if not command_dict.has_key('--output-file'):
        bkpKeyStr = command_dict.get('--backup-key')
        if not bkpKeyStr:
            bkpKeyStr = ''
        else:
            bkpKeyStr = '_' + bkpKeyStr
        command_dict['--output-file'] = './clusterTopology%s.xml' % (bkpKeyStr)
        g_opts.outputFile = command_dict['--output-file']

    #Generate CLI options and check them...
    if g_opts.roachConf:
        parseRoachConf(g_opts)
    makeRaochOptions(g_opts, master_ip, master_host, command_dict)
    validate_logparams()

    global g_logger
    g_logger = gsinitLogger(g_opts)
    g_logger.debug("GenXML operation start for BkpKey '%s'..." % command_dict.get('--backup-key'))
    if validateGenerateParams(g_opts):
        #Operation already completed, can exit as success!
        return not bFailed
    else:
        #XML not found in MetaDest, so need to copy generated XML there too!
        bNeedToCopyToMeta = True

    #Execute and wait for gs_roach 'generate'
    option = createMasterOption(g_opts)
    cmd = "%s/bin/gs_roach --master generate --master-ip %s %s" % (gauss_home_loc, master_ip, option)
    g_logger.debug("python call gs_roach generate in generateXMLTopology, cmd: %s" % cmd)
    status, output = commands.getstatusoutput(cmd)
    if status != 0:
        for line in output.split('\n'):
            g_logger.log(line)
        g_logger.log("GenXML operation failed!")
        return bFailed

    #Use the generated cluster static file to generate Topology XML
    clustStaticFile = os.path.join(os.path.dirname(outFile), 'cluster_static_config_' + g_opts.backupKey)
    if not os.access(clustStaticFile, os.R_OK):
        g_logger.logExit('Error: Failed to read "' + (clustStaticFile) + '".\n GenXML operation failed!!')
    clusterInfo = dbClusterInfo()
    clusterInfo.generateXMLFromStaticConfigFile(g_opts.user, clustStaticFile, outFile)

    #If Cluster XML not found in MetaDest, then silently copy it there too!
    if bNeedToCopyToMeta:
        metaDest = g_opts.metadataDestination #Get the saved value
        backup_key = final_ip_param_dict[master_ip]['--backup-key']
        media_type = final_ip_param_dict[master_ip]["--media-type"]
        outMetaFile = os.path.join(metaDest, "roach", backup_key, 'clusterTopology_%s.xml' % (backup_key))
        try:
            shutil.copy(outFile, outMetaFile)
        except (shutil.Error, IOError), err:
            #This is a best-effort basis copy - no errors to be handled!
            g_logger.debug("Silent copy of XML to metadata destination failed!\n%s" % str(err))
    g_logger.log("GenXML operation successful!\nOutput: %s" % (outFile))
    return not bFailed

def getBkpDetails(metaDest, backupKey, command_dict):
    """
    function: to get the parent backup key
    input: media destination, backup key
    output: list of backup keys
    """
    g_logger.debug("Reading backups.ini to get backup details before restore.")

    bkpdetails = ""
    backupfile = os.path.join(metaDest, "backups.ini")
    retryTimes = 12
    for retryTime in range(retryTimes):
        try:
            if os.path.isfile(backupfile):
                chkprior = loadMetaFileAsJson(backupfile)
                if chkprior.has_key("BackupDetails"):
                    bkpdetails = chkprior["BackupDetails"]
                    for bkpdict in bkpdetails:
                        if bkpdict.has_key("BackupKey") and bkpdict["BackupKey"] == backupKey:
                            return bkpdict
                    #if it couldnot find the key then throw error
                    errorString = ErrorCode.GAUSS_534["GAUSS_53404"] % backupKey
                    agentErrorReport("roach_restore", errorString)
                    raise Exception(errorString)
            else:
                raise Exception("Error: Specified metadata-destination [%s] is corrupted, restore not possible!" % (metaDest))
        except Exception as e:
            g_logger.debug("Error: get backup details from backups.ini failed, output is [%s] for %s times" % (str(e), str(retryTime)))
            if retryTime == retryTimes -1:
                g_logger.logExit("Error: get backup details from backups.ini failed, output is [%s]" % (str(e)))
            else:
                time.sleep(5)

def getParentBkpKey(metaDest, command_dict, skipDependentBackup):
    """
    function: to get the parent backup key
    input: media destination, command_dict
    output: list of backup keys
    """
    global g_isClusterBkp
    backupKey = command_dict["--backup-key"]
    bkpdetails = ""

    #inner function
    def searchBackupKey(bkpkey, skipDependentBackup):
        """
        function: recursive fn to find the backup key in the json
        input: backup key
        output: list of backup keys
        """
        global g_isClusterBkp
        for bkpdict in bkpdetails:
            if bkpdict.has_key("BackupKey") and bkpdict["BackupKey"] == bkpkey:
                ## Check if metadata-destination exist for
                ## the current backup key. Checked only for media type "DISK".
                if bkpdict["MediaType"] == "DISK" and bkpdict["MediaDestination"]:
                    metapath = os.path.join(metaDest, "roach", bkpkey)
                    if not os.path.isdir(metapath):
                        errorString = ErrorCode.GAUSS_534["GAUSS_53404"] % bkpkey
                        errorString = "%s in metadata-destination [%s]" % (errorString, metaDest)
                        agentErrorReport("roach_restore", errorString)
                        g_logger.logExit(errorString)
                bkpkeylist.append(str(bkpkey))
                if (skipDependentBackup == True):
                    return bkpkeylist
                if bkpdict.has_key("PriorBackupKey") and bkpdict["PriorBackupKey"][:4] == "FULL":
                    if bkpdict["PriorBackupKey"] == "FULL":
                        #This restore is for a Cluster backup, set the global flag
                        g_isClusterBkp = True
                    return bkpkeylist
                else:
                    searchBackupKey(bkpdict["PriorBackupKey"], skipDependentBackup)

    # reading backups.ini in restore operation
    backupfile = os.path.join(metaDest, "backups.ini")
    retryTimes = 12
    for retryTime in range(retryTimes):
        try:
            if os.path.isfile(backupfile):
                chkprior = loadMetaFileAsJson(backupfile)
                if chkprior.has_key("BackupDetails"):
                    bkpdetails = chkprior["BackupDetails"]
                    keyExists = False
                    for bkpdict in bkpdetails:
                        if bkpdict.has_key("BackupKey") and bkpdict["BackupKey"] == backupKey:
                            keyExists = True
                            if bkpdict["BackupType"] == "INCREMENTAL":
                                searchBackupKey(backupKey, skipDependentBackup)
                            else:
                                if bkpdict["PriorBackupKey"] == "FULL":
                                    #This restore is for a Cluster backup, set the global flag
                                    g_isClusterBkp = True
                                bkpkeylist.append(str(backupKey))
                            break
                    if not keyExists:
                        errorString = ErrorCode.GAUSS_534["GAUSS_53404"] % backupKey
                        errorString = "%s or backup type is logical backup." % errorString
                        agentErrorReport("roach_restore", errorString)
                        g_logger.logExit(errorString)
                        break
                    break
            else:
                raise Exception("Error: Specified metadata-destination [%s] is corrupted, restore not possible!" % (metaDest))
        except Exception as e:
            g_logger.debug("Error: search backupKey from backups.ini failed, output is [%s] for %s times" % (str(e), str(retryTime)))
            if retryTime == retryTimes -1:
                g_logger.logExit("Error: search backupKey from backups.ini failed, output is [%s]" % (str(e)))
            else:
                time.sleep(5)

def resetFinalIPParam(g_opts, master_ip, master_host, command_dict):
    """
    reform the final_ip_param_dict
    """
    global final_ip_param_dict
    final_ip_param_dict.clear()
    if g_opts.roachConf:
        parseRoachConf(g_opts)
    makeRaochOptions(g_opts, master_ip, master_host, command_dict)

def formCommandDict(backupkey, final_ip_param_dict, org_backupkey):
    """
    form the final_ip_param_dict
    """
    global g_mediatype
    global g_mediadest
    global g_backupkey

    metaDest = getMetaDest(final_ip_param_dict)
    for node_ip in final_ip_param_dict.keys():
        media_type = final_ip_param_dict[node_ip]["--media-type"]

        inifile = os.path.join(metaDest, "backups.ini")

        inidata = loadMetaFileAsJson(inifile)
        backuplist = inidata["BackupDetails"]
        for bkpitem in backuplist:
            if bkpitem.get("BackupKey") and bkpitem.get("BackupType")\
            and bkpitem["BackupKey"] == backupkey:
                if final_ip_param_dict[node_ip]["--media-type"].upper() not in ["NBU", "DISK", "OBS", "EISOO"]:
                    g_logger.logExit("Error: Invalid media type. Media type should be NBU, DISK, EISOO or OBS.")
                metaMediaType = str(bkpitem["MediaType"]).strip()
                if backupkey == org_backupkey:
                    metaMediaType = metaMediaType.replace(' with CONTRIB','') #If the type from Meta is with Contrib then ignore that part
                    if final_ip_param_dict[node_ip]["--media-type"].upper().strip() != metaMediaType.upper():
                        g_logger.log("Warning: Provided media type [%s] is incorrect, using correct media type [%s]." \
                            % (final_ip_param_dict[node_ip]["--media-type"], metaMediaType))
                    final_ip_param_dict[node_ip]["--media-type"] = metaMediaType

                    if os.path.realpath(final_ip_param_dict[node_ip]["--media-destination"]) != os.path.realpath(bkpitem["MediaDestination"]):
                        g_logger.log("Warning: Provided media destination [%s] is incorrect, using correct media destination [%s]." \
                            % (final_ip_param_dict[node_ip]["--media-destination"], bkpitem["MediaDestination"]))
                final_ip_param_dict[node_ip]["--media-destination"] = str(bkpitem["MediaDestination"])
                g_mediatype = metaMediaType
                g_mediadest = str(bkpitem["MediaDestination"])
                g_backupkey = str(backupkey)


def killAliveProc(procFileName, g_clusterInfo, g_sshTool, g_logger):
    """
    function: We must make sure that all process about gaussdb cluster have been stopped. Not including om_monitor.
    input : procFileName
    output: NA
    """ 
    try:
        failedNodes = []
        nodeNameLists = g_clusterInfo.getClusterNodeNames()
        #the command for killing all process
        if not isMiniaturizedDeployment(cluster_version):
            cmd_check_kill = "proc_pid_list=\`ps ux | grep %s | grep -v grep | awk '{print \$2}'\`" % procFileName
            cmd_check_kill += " && (if [ X\\\"\$proc_pid_list\\\" != X\\\"\\\" ]; then echo \\\"\$proc_pid_list\\\" | xargs kill -9 ; fi)"
            #use sshTool to kill process in all nodes
            g_logger.debug("killAliveProc: kill all process in cluster, cmd: [%s]" % cmd_check_kill)
            (status, output) = g_sshTool.getSshStatusOutput(cmd_check_kill, nodeNameLists)
            #get the node which not be killed
            for node in nodeNameLists:
                if(status[node] != DefaultValue.SUCCESS):
                    failedNodes.append(node)
            #kill process in nodes again
            if(len(failedNodes)):
                time.sleep(1)
                g_logger.debug("killAliveProc: try again to kill process in failed nodes, cmd: [%s]" % cmd_check_kill)
                (status, output) = g_sshTool.getSshStatusOutput(cmd_check_kill, failedNodes)
                for node in failedNodes:
                    #if still fail, throw error
                    if(status[node] != DefaultValue.SUCCESS):
                        g_logger.debug("killAliveProc: got exception when kill process: [%s]" % output)
                        raise Exception(output)
        else:
            cmd_check_kill = "proc_pid_list=`ps ux | grep %s | grep -v grep | awk '{print $2}'`" % procFileName
            cmd_check_kill += " && (if [ X\"$proc_pid_list\" != X\"\" ]; then echo \"$proc_pid_list\" | xargs kill -9 ; fi)"
            g_logger.debug("killAliveProc: single cluster kill all process in cluster, cmd: [%s]" % cmd_check_kill)
            (status, output) = commands.getstatusoutput(cmd_check_kill)
            if(status != 0):
                time.sleep(1)
                g_logger.debug("killAliveProc: single cluster try again to kill process in failed nodes, cmd: [%s]" % cmd_check_kill)
                (status, output) = commands.getstatusoutput(cmd_check_kill)
                if(status != 0):
                    g_logger.debug("killAliveProc: single cluster got exception when kill process: [%s]" % output)
                    raise Exception(output)
    except Exception, e:
        g_logger.debug("Got exception in killAliveProc: [%s]" % str(e))
        raise Exception(str(e))

def checkExpectedConfFiles(g_opts, g_logger, sshTool):
    g_logger.debug("Check necessary config files before clean and restore.")
    cmd = "source %s && python %s -t %s -U %s -l %s" \
        % (g_opts.sourceFile, LOCAL_ROACH, 'checkconf', g_opts.user, g_opts.logFile)
    g_logger.debug("Cmd for check config files is: %s" % cmd)

    try:
        if not isMiniaturizedDeployment(cluster_version):
            sshTool.executeCommand(cmd, "check config files", DefaultValue.SUCCESS, g_opts.nodeList)
        else:
            status, output = commands.getstatusoutput(cmd)
            if (status != 0):
                g_logger.logExit("Check expected config files failed with output: %s" % (output))
    except Exception, e:
        g_logger.debug("Got exception in checkExpectedConfFiles: [%s]" % str(e))
        g_logger.logExit("Error: Some necessary config files donot exist before clean or restore!")
    g_logger.debug("Leave necessary config files checking.")

def doClean(g_opts, g_logger):
    """
    clean data before restore
    """
    g_logger.log("Cleaning cluster.")

    try:
        global cluster_version
        cluster_version = getClusterVersion(g_opts.user)
        ClusterCommand.CopyClusterStatic()
        g_logger.log("Parsing the configuration file.")
        l_clusterInfo = dbClusterInfo()
        l_clusterInfo.initFromStaticConfig(g_opts.user)
        l_sshTool = SshTool(l_clusterInfo.getClusterNodeNames(), g_logger.logFile, timeout = 3600)
    except Exception, e:
        g_logger.debug("Got exception when get cluster info in doClean, result: %s" % str(e))
        #Cluster clean may have been done so will try to generate info from
        #BAK file! Sample exception string:
        #'[GAUSS-50201] : The static configuration file \
        #[/abc/gaussdb/bin/cluster_static_config] of \
        #designated user [abc] does not exist.'
        try:
            clustFileNameBak = str(e).split('[')[2].split(']')[0] + '_bak'
            l_clusterInfo.initFromStaticConfig(g_opts.user, clustFileNameBak)
        except (IndexError, Exception) as e:
            #Not possible to get cluster static file name. Abort!
            g_logger.debug("Catch exception again when read cluster info in doClean, result: %s" % str(e))
            g_logger.logExit('Error: Not able to parse cluster info. Aborting...')

    copyStaticConfig(g_logger, l_clusterInfo)

    if (g_opts.action == 'clean'):
        checkExpectedConfFiles(g_opts, g_logger, l_sshTool)

    #stop cluster
    g_logger.log("Stopping cluster.")
    static_config = "%s/bin/cluster_static_config" % l_clusterInfo.appPath
    cm_ctl_file = "%s/bin/cm_ctl" % l_clusterInfo.appPath
    if not os.path.exists(static_config) or not os.path.exists(cm_ctl_file):
        g_logger.debug("Failed to stop cluster, app path: %s" % l_clusterInfo.appPath)
        g_logger.logExit(ErrorCode.GAUSS_516["GAUSS_51612"] % ("0", "the cluster"))

    cmd = ClusterCommand.getStopCmd(g_opts.user, 0, "i")
    g_logger.debug("Cmd to stop cluster during clean is: %s" % cmd)
    (status, output) = commands.getstatusoutput(cmd)
    if status != 0:
        g_logger.debug("Failed to stop cluster, cmd: %s" % cmd)
        g_logger.logExit(ErrorCode.GAUSS_516["GAUSS_51612"] % ("0", "the cluster") \
            + " Error:\n%s." % output)
    g_logger.log("Successfully stopped cluster.")

    try:
        # check and kill all processes about
        # clean cm_agent,cm_server,gs_gtm,gaussdb(CN/DN) and etcd.
        g_logger.debug("Checking and killing all cluster processes except om_monitor.")

        #local mode, kill all process in local node
        cmAgent_file = "%s/bin/cm_agent" % l_clusterInfo.appPath
        cmServer_file = "%s/bin/cm_server" % l_clusterInfo.appPath
        gaussdb_file = "%s/bin/gaussdb" % l_clusterInfo.appPath
        gs_gtm_file = "%s/bin/gs_gtm" % l_clusterInfo.appPath
        etcd_file = "%s/bin/etcd" % l_clusterInfo.appPath
        for progname in [cmAgent_file, cmServer_file, gaussdb_file, gs_gtm_file, etcd_file]:
            killAliveProc(progname, l_clusterInfo, l_sshTool, g_logger)
        g_logger.debug("Successfully checked and killed all cluster processes.") 
    except (IndexError, Exception) as e:
        #Not possible to get cluster static file name. Abort!
        g_logger.debug("Catch exception when killAliveProc in doClean, result: %s" % str(e))
        g_logger.logExit('Error: Failed to check and kill processes. Aborting...')

    #clean data before restore
    g_logger.log("Cleaning data before restoration.")
    if g_opts.restoreNewCluster:
        cmd = "source %s && python %s -t %s -U %s -l %s -p" \
            % (g_opts.sourceFile, LOCAL_ROACH, 'clean', g_opts.user, g_opts.logFile)
    else:
        cmd = "source %s && python %s -t %s -U %s -l %s" \
            % (g_opts.sourceFile, LOCAL_ROACH, 'clean', g_opts.user, g_opts.logFile)
    if g_opts.cleanLog:
        cmd += " -d "
    if g_opts.resumeRst:
        cmd += " --mediaPath %s" % g_opts.mediaDest
    if g_opts.parallelRestore:
        cmd += " --parallel-restore "
    g_logger.debug("doClean: clean cluster cmd is: %s" % cmd)
    if isMiniaturizedDeployment(cluster_version):
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit(ErrorCode.GAUSS_534["GAUSS_53401"] % "cluster." + "Error:\n%s." % output)
    else:
        l_sshTool.executeCommand(cmd, "configure ENV parameter", DefaultValue.SUCCESS, g_opts.nodeList)
    g_logger.log("Successfully cleaned data before restoration.")
    g_logger.log("Successfully cleaned cluster.")

def dealCnBackupFiles(g_opts, sshTool, oldClusterInfo):
    cmd  = ""
    pyPath = os.path.dirname(currPath)
    for nodeInfo in oldClusterInfo:
        coordInfo = nodeInfo['Coordinator']
        if (coordInfo != None):
            primaryFlag = coordInfo['IsPrimaryCn']
            if (primaryFlag == True):
                primaryCnId = coordInfo['CoordId']
                primaryCnIp = nodeInfo['IP']
                primaryCnDataNBPath = coordInfo['DataNBPath']
                pos = primaryCnDataNBPath.rfind("/data")
                primaryCnNBPath = primaryCnDataNBPath[:pos]
                primaryCnOldMediaPath = nodeInfo['BackupLocation']
                cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
                copybackupedfiles --primaryInstId '%s' --primaryInstIp '%s' --primaryInstDataNBPath '%s' \
                --primaryInstOldMediaPath '%s' --primaryInstNewMediaPath '%s' --needCopyCol --instType '%s' \
                -U %s -l %s" \
                      % (g_opts.sourceFile, pyPath, str(primaryCnId), primaryCnIp, primaryCnNBPath, \
                         primaryCnOldMediaPath, g_opts.mediaDest, CN_TYPE, g_opts.user, g_opts.localLog)
                if (g_opts.resumeRst):
                    cmd += " --resume-restore"
                if (g_opts.clusterMigration == True and g_opts.skipDependentBackup == False):
                    cmd += " --generate-link"
                break
    if (cmd == ""):
        raise Exception("Read restore metadata failed when copy primary cn backup files.")

    g_logger.debug("Copy CN backup file cmd: %s" % cmd)
    if not isMiniaturizedDeployment(cluster_version):
        (status, output) = sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
        g_logger.debug("Copy CN status.values() is %s" % str(status.values()))
        g_logger.debug("Copy CN output is %s" % output)
        if 'Failure' in status.values():
            raise Exception("Copy CN backup files failed!")
    else:
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            raise Exception("Copy CN backup files failed with output: %s!" % (output))
    g_logger.log("Copy cn backup files successfully.")

def dealGtmBackupFiles(g_opts, sshTool, oldClusterInfo):
    cmd  = ""
    pyPath = os.path.dirname(currPath)
    for nodeInfo in oldClusterInfo:
        gtmInfo = nodeInfo['Gtm']
        if (gtmInfo != None):
            primaryFlag = gtmInfo['IsPrimaryGtm']
            if (primaryFlag == True):
                primaryGtmId = gtmInfo['GtmId']
                primaryGtmIp = nodeInfo['IP']
                primaryGtmDataNBPath = gtmInfo['DataNBPath']
                primaryGtmOldMediaPath = nodeInfo['BackupLocation']
                cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
                copybackupedfiles --primaryInstId '%s' --primaryInstIp '%s' --primaryInstDataNBPath '%s' \
                --primaryInstOldMediaPath '%s' --primaryInstNewMediaPath '%s' --instType '%s' -U %s -l %s" \
                      % (g_opts.sourceFile, pyPath, str(primaryGtmId), primaryGtmIp, primaryGtmDataNBPath, \
                         primaryGtmOldMediaPath, g_opts.mediaDest, GTM_TYPE, g_opts.user, g_opts.localLog)
                if (g_opts.resumeRst):
                    cmd += " --resume-restore"
                break
    if (cmd == ""):
        raise Exception("Read restore metadata failed when copy primary gtm backup files.")

    g_logger.debug("Copy gtm backup file cmd: %s" % cmd)
    if not isMiniaturizedDeployment(cluster_version):
        (status, output) = sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
        g_logger.debug("Copy GTM status.values() is %s" % str(status.values()))
        g_logger.debug("Copy GTM output is %s" % output)
        if 'Failure' in status.values():
            raise Exception("Copy GTM backup files failed!")
    else:
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            raise Exception("Copy GTM backup files failed with output: %s" % (output))
    g_logger.log("Copy GTM backup files successfully.")

def dealCmBackupFiles(g_opts, sshTool, oldClusterInfo):
    cmd  = ""
    pyPath = os.path.dirname(currPath)
    for nodeInfo in oldClusterInfo:
        cmInfo = nodeInfo['CmMeta']
        if (cmInfo != None):
            primaryFlag = cmInfo['IsPrimaryCms']
            if (primaryFlag == True):
                primaryCmId = cmInfo['CmServerId']
                primaryCmIp = nodeInfo['IP']
                primaryCmDataNBPath = cmInfo['CmNBLocalPath']
                primaryGtmOldMediaPath = nodeInfo['BackupLocation']
                cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
                copybackupedfiles --primaryInstId '%s' --primaryInstIp '%s' --primaryInstDataNBPath '%s' \
                --primaryInstOldMediaPath '%s' --primaryInstNewMediaPath '%s' --instType %s -U %s -l %s" \
                      % (g_opts.sourceFile, pyPath, str(primaryCmId), primaryCmIp, primaryCmDataNBPath, \
                         primaryGtmOldMediaPath, g_opts.mediaDest, CM_TYPE, g_opts.user, g_opts.localLog)
                if (g_opts.resumeRst):
                    cmd += " --resume-restore"
                break
    if (cmd == ""):
        g_logger.logExit("Read restore metadata failed when copy primary cm backup files.")

    g_logger.debug("Copy CM backup file cmd: %s" % cmd)
    if not isMiniaturizedDeployment(cluster_version):
        (status, output) = sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
        g_logger.debug("Copy CM status.values() is %s" % str(status.values()))
        g_logger.debug("Copy CM output is %s" % output)
        if 'Failure' in status.values():
            raise Exception("Copy CM backup files failed!")
    else:
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            raise Exception("Copy CM backup files failed with output: %s" % (output))
    g_logger.log("Copy CM backup files successfully.")

def dealStandbyBackupFiles(g_opts, sshTool, key):
    pyPath = os.path.dirname(currPath)

    if g_opts.restoreNewCluster:
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
        copydnbackupedfiles --restore-new-cluster --backupKey %s  --standbyInstNewMediaPath '%s' \
        --standbyInstNewMetaPath '%s' -U %s -l %s  " \
              % (g_opts.sourceFile, pyPath, key, g_opts.mediaDest, g_opts.metadataDestination, \
                 g_opts.user, g_opts.localLog)
        g_logger.debug("g_opts.clusterMigration is %s" % g_opts.clusterMigration)
        if g_opts.clusterMigration:
            cmd = cmd + '--generate-link'
    else:
        cmd = "source %s && export PYTHONPATH=%s:$PYTHONPATH && python $GPHOME/script/util/GSroachLocal.py -t \
        copydnbackupedfiles --backupKey %s --standbyInstNewMediaPath '%s' --standbyInstNewMetaPath '%s' \
        -U %s -l %s  " \
              % (g_opts.sourceFile, pyPath, key, g_opts.mediaDest,  g_opts.metadataDestination, \
                 g_opts.user, g_opts.localLog)
    if (g_opts.resumeRst):
        cmd += " --resume-restore"
    g_logger.debug("Copy SN cmd: %s" % cmd)
    if not isMiniaturizedDeployment(cluster_version):
        (status, output) = sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
        g_logger.debug("The status.values() is %s" % str(status.values()))
        g_logger.debug("The output is %s" % output)
        if 'Failure' in status.values():
            g_logger.logExit("Copy SN backup files failed!")
    else:
        status, output = commands.getstatusoutput(cmd)
        if (status != 0):
            g_logger.logExit("Copy SN backup files failed with output: %s" % (output))
    g_logger.log("Copy SN backup files successfully.")

def prepareBackupFiles(g_opts, g_sshTool, bkpkeylist, unfinishedBkp):
    try:
        if (g_opts.copyCnGtmCmsFiles):
            for key in bkpkeylist:
                updatedJson = os.path.join(g_opts.metadataDestination, 'roach', key, 'roach_master_updated.metadata')
                oldInfoList = getNodesInfoByUpdatedFile(updatedJson)
                dealCnBackupFiles(g_opts, g_sshTool, oldInfoList)
                if (unfinishedBkp != key):
                    dealGtmBackupFiles(g_opts, g_sshTool, oldInfoList)
                    dealCmBackupFiles(g_opts, g_sshTool, oldInfoList)
    except Exception, e:
        g_logger.logExit("Prepare backup files failed with %s" % str(e))

def hasSkipCopyGtmCm(g_opts, key):
    csvPath = os.path.join(g_opts.metadataDestination, 'backuprestoreinfo.csv')
    with open(csvPath, 'r') as f:
        fcntl.flock(f.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
        csv_reader = csv.reader(f)
        backupState = ''

        for row in csv_reader:
            if row[attributeIndex['BackupKey']] == key:
                backupState = row[attributeIndex['BackupState']]
                if backupState == 'BACKUPING':
                    return True
                elif backupState == 'BACKUPED':
                    return False
                else:
                    g_logger.logExit("Error: BackupState is FAILED!")

def getUnfinishedBkp(g_opts, bkpkeylist):
    keyCount = 0
    unfinishedKey = ''
    for key in bkpkeylist:
        if hasSkipCopyGtmCm(g_opts, key):
            unfinishedKey = key
            keyCount= keyCount + 1
    if keyCount > 1:
        g_logger.logExit("Error: BackupState is unnormal!")
    else:
        return unfinishedKey
def prepareStandbyBackupFiles(g_opts, g_sshTool, bkpkeylist):
    for key in bkpkeylist:
        if (g_opts.restoreNewCluster):
            if not isMiniaturizedDeployment(cluster_version):
                metaPath = os.path.join(g_opts.metadataDestination, "roach", key)
                status = deliveryMetaFileToAgents(g_logger, g_sshTool, g_opts.sourceFile, g_opts.nodeList, metaPath, "roach_master_updated.metadata")
                if (status != 0):
                    g_logger.logExit("Failed to delivery metadate file, prepare backup files for SN failed.")
            dealStandbyBackupFiles(g_opts, g_sshTool, key)
        else:
            appPath = DefaultValue.getEnv("GAUSSHOME")
            static_config = "%s/bin/cluster_static_config" % appPath
            static_config_bak = "%s/bin/cluster_static_config_bak" % appPath
            cpconfigfile = False
            if os.path.exists(static_config_bak) and not os.path.exists(static_config):
                cmd = "cp %s %s" % (static_config_bak, static_config)
                if not isMiniaturizedDeployment(cluster_version):
                    (status, output) = g_sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
                    if 'Failure' in status.values():
                        g_logger.logExit("Copy cluster_static_config file failed with output: %s!" % (output))
                else:
                    status, output = commands.getstatusoutput(cmd)
                    if (status != 0):
                        g_logger.logExit("Copy cluster_static_config file failed with output: %s!" % (output))
                cpconfigfile = True
            dealStandbyBackupFiles(g_opts, g_sshTool, key)
            if cpconfigfile:
                cmd = "rm %s" % (static_config)
                if not isMiniaturizedDeployment(cluster_version):
                    g_sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
                else:
                    commands.getstatusoutput(cmd)


def getClusterInfo(g_opts):
    gaussHome = DefaultValue.getEnv("GAUSSHOME")
    staticConfig = "%s/bin/cluster_static_config" % gaussHome
    staticConfig_bak = "%s/bin/cluster_static_config_bak" % gaussHome
    if (not os.path.exists(staticConfig)):
        g_logger.debug("Warning: static config file not found, use its bak file!")
        staticConfig = staticConfig_bak
    clusterInfo = dbClusterInfo()
    clusterInfo.initFromStaticConfig(g_opts.user, staticConfig)
    return clusterInfo

def HasTableNameBackedup(command_dict):
    if "--tablename" not in command_dict:
        return True

    database_name = "postgres"
    schema_name = "public"
    table_name = command_dict["--tablename"]
    bkDetail = getBkpDetails(command_dict["--metadata-destination"], command_dict["--backup-key"], command_dict)

    if len(command_dict["--tablename"].split(".")) == 2:
        schema_name = command_dict["--tablename"].split(".")[0]
        table_name = command_dict["--tablename"].split(".")[1]

    if "--dbname" in command_dict.keys():
        database_name = command_dict["--dbname"]

    g_logger.debug("bkDetail['DBName']: %s, dbname: %s." % (bkDetail['DBName'], database_name))
    g_logger.debug("bkDetail['SchemaName']: %s, schema_name: %s." % (bkDetail['SchemaName'], schema_name))
    g_logger.debug("bkDetail['TableName']: %s, table_name: %s." % (bkDetail['TableName'], table_name))

    if bkDetail['DBName'] != database_name:
        return False
    if bkDetail['SchemaName'] != schema_name.lower():
        return False
    if bkDetail['TableName'] != table_name.lower():
        return False

    return True


def updateBackupIniForRestoreSuccess(g_logger, g_opts):
    gaussHomeLoc = DefaultValue.getEnv("GAUSSHOME")
    metadata_opt_string = "--metadata-destination '%s'" % g_opts.metadataDestination
    cmd = '%s/bin/gs_roach update --mark-restored --backup-key %s %s' \
        % (gaussHomeLoc, g_opts.backupKey, metadata_opt_string)
    g_logger.debug("python call gs_roach update in updateBackupIniForRestoreSuccess, cmd: %s" % cmd)
    sub = subprocess.Popen([cmd], shell=True, bufsize=-1)
    sub.wait()
    g_logger.debug("success to call gs_roach update in updateBackupIniForRestoreSuccess, cmd: %s" % cmd)


def startRestore(g_opts, command_dict, g_sshTool, table_list = []):
    """
    call for restore functionality
    """
    global master_ip
    final_ip_param_dict.clear()
    master_ip = getMaster_ip()
    if g_opts.roachConf:
        parseRoachConf(g_opts)
    makeRaochOptions(g_opts, master_ip, master_host, command_dict)
    validate_logparams()

    global g_logger
    g_logger = gsinitLogger(g_opts)

    #get cluster version used for miniaturized deployment 
    global cluster_version
    cluster_version = getClusterVersion(g_opts.user)

    validateRoachParams(g_opts)
    if '--prior-backup-key' in command_dict.keys():
        g_logger.logExit("Error: Invalid Parameter, '--prior-backup-key'")

    #parallel restore only support OBS
    if "--parallel-restore" in command_dict.keys() and not command_dict["--media-type"].upper() == "OBS":
        g_logger.logExit("Error: parallel-restore is supported only for OBS")

    global g_dbClusterInfo
    g_dbClusterInfo = getClusterInfo(g_opts)

    #dictionary to store the key(i.e. time stamp as integer) and value(i.e. backup key)
    #so that we can restore according to the time stamp i.e. in reverse chronological order
    global bkpkeylist
    global g_isClusterBkp
    global isIncremental
    isIncremental = False
    bkpkeylist = list()
    arckeysdict = dict()

    if "--backup-key" in command_dict:
        org_backupkey = command_dict["--backup-key"]
    else:
        #actual values should come from --restore-target-time processing
        org_backupkey = ''
    metaDest = getMetaDest(final_ip_param_dict)
    if not metaDest:
        g_logger.logExit("Error: '--metadata-destination' not provided")

    
    if not HasTableNameBackedup(command_dict):
        g_logger.logExit("Error: Table backup Failed, please check if db/schema/table name is the same as that backed up in the backup-key.")
         
    #restore operation if backups.ini does not exist
    global noBackupIni
    noBackupIni = False

    #make list of backup keys for time-based restore
    #Notice: g_isClusterBkp is set here, before this if-else, it has no correct value
    if "--restore-target-time" in command_dict.keys():
        if "--tablename" in command_dict.keys() or "--logical" in command_dict.keys():
            g_logger.logExit("Error: --restore-target-time can not be used in logical restore")
        #This restore is always for a Cluster backup, set the global flag
        g_isClusterBkp = True
        if g_opts.single:
            bkpkeylist.append(org_backupkey)
            getArchiveBackupKeys(command_dict)
        else:
            arckeysdict = getArchiveBackupKeys(command_dict)
            for item in sorted(arckeysdict.items(),reverse=True):
                bkpkeylist.append(str(item[1]))
        if not org_backupkey:
            #in case not directly specified, need to set from --restore-target-time processing
            org_backupkey = bkpkeylist[0]
    elif not g_opts.logical:
        getParentBkpKey(metaDest, command_dict, g_opts.skipDependentBackup)

    #for cluster level restore, check version are the same,
    #if version is different from that when backup, then restore is not allowed.
    #use org_backupkey because maybe cmd has no backup key, but has --restore-target-time
    if g_isClusterBkp:
        g_logger.debug("startRestore: this is a cluster level restore, "
            "need check version is matched, backup key is: %s" % org_backupkey)
        compareVersionFile(g_opts, org_backupkey)

    #check for NBU and OBS if backups.ini not exist
    if os.path.isdir(metaDest) and command_dict["--media-type"].upper() == "NBU":
        if not os.path.isfile(os.path.join(metaDest, "backups.ini")):
            g_logger.log("Warning: backups.ini does not exist in the specified metadata destination ['%s']. \nAssuming provided backup key ['%s'] is of a full backup." \
                % (metaDest, command_dict["--backup-key"]))
            noBackupIni = True
            #clean before cluster restore
            if not g_opts.tableName and g_opts.clean == True and len(g_opts.tableName) == 0:
                doClean(g_opts, g_logger)
            doRestore(g_opts, command_dict, g_sshTool)
            postExecCleanup(g_opts, g_logger)
            g_logger.debug("Finish startRestore for NBU while backups.ini not exist.")
            return getRoachOpResult()
    if (os.path.isdir(metaDest) and command_dict["--media-type"].upper() == "OBS"):
        if not os.path.isfile(os.path.join(metaDest, "backups.ini")):
            g_logger.log("Warning: backups.ini does not exist in the specified metadata destination ['%s']. \nAssuming provided backup key ['%s'] is of a full backup." \
                % (metaDest, command_dict["--backup-key"]))
            noBackupIni = True
            #clean before cluster restore
            if not g_opts.tableName and g_opts.clean == True and len(g_opts.tableName) == 0:
                doClean(g_opts, g_logger)
            doRestore(g_opts, command_dict, g_sshTool)
            postExecCleanup(g_opts, g_logger)
            g_logger.debug("Finish startRestore for OBS while backups.ini not exist.")
            return getRoachOpResult()

    bkpkeylist.reverse()
    #get the backup type of the full backup key
    if not g_opts.logical:
        bkpDetail = getBkpDetails(metaDest, bkpkeylist[0], command_dict)

    #check if restore-new-cluster is done only for Cluster backups
    if "--restore-new-cluster" in command_dict.keys() and bkpDetail["BackupType"].upper() not in ["FULL", "INCREMENTAL", "PITR"] :
        g_logger.logExit("Error: restore-new-cluster is supported only for restore of cluster backup")

    if ("--restore-new-cluster" in command_dict.keys()) :
        checkExpectedConfFiles(g_opts, g_logger, g_sshTool)

    #clean before cluster restore
    #Notice: g_isClusterBkp cannot be replaced by isClusterBackup()
    if g_opts.clean == True and g_isClusterBkp:
        g_logger.debug("Clean cluster before restore because of --clean")

        if hasStopProcess(g_logger, g_opts, g_dbClusterInfo, cluster_version):
            g_logger.log("Stop restore due to stop restore command.")
            return True
        doClean(g_opts, g_logger)

    #copy the user provided backup key as target backup key
    if g_opts.backupKey != '':
        g_opts.targetBackupKey = g_opts.backupKey
    else:
        g_opts.targetBackupKey = bkpkeylist[0]
    command_dict["--target-backup-key"] = g_opts.targetBackupKey
    #get unfinishedBkp
    unfinishedBkp = ''
    if g_opts.clusterMigration:
        unfinishedBkp = getUnfinishedBkp(g_opts, bkpkeylist)
        g_logger.debug("unfinishedBkp is %s" %unfinishedBkp)
    #copy backup files from primary instance
    if g_opts.tableName == '' and g_opts.tablelist == '' and g_opts.mediaType.lower() == 'disk' and g_opts.doubleCluster:
        if hasStopProcess(g_logger, g_opts, g_dbClusterInfo, cluster_version):
            g_logger.log("Stop restore due to stop restore command.")
            return True
        prepareBackupFiles(g_opts, g_sshTool, bkpkeylist, unfinishedBkp)

        if hasStopProcess(g_logger, g_opts, g_dbClusterInfo, cluster_version):
            g_logger.log("Stop restore due to stop restore command.")
            return True
        prepareStandbyBackupFiles(g_opts, g_sshTool, bkpkeylist)

    for item in range(len(bkpkeylist)):
        #calling function to modify values for various types of backup keys
        final_ip_param_dict.clear()
        makeRaochOptions(g_opts, master_ip, master_host, command_dict)
        if not command_dict.has_key("--restore-new-cluster"):
            formCommandDict(bkpkeylist[item], final_ip_param_dict, org_backupkey)

        if item != 0:
            print "--------------------------------------------------------------------------------"
            print "Restore operation in-progress for 'INCREMENTAL BACKUP' key[%s/%s]: %s" \
                % (str(item + 1), str(len(bkpkeylist)), bkpkeylist[item])
            resetFinalIPParam(g_opts, master_ip, master_host, command_dict)
        elif len(bkpkeylist) > 1:
            print "\nRestore operation in-progress for 'FULL BACKUP' key[%s/%s]: %s" \
                % (str(item + 1), str(len(bkpkeylist)), bkpkeylist[item])

        for key in final_ip_param_dict.keys():
            if item == 0:
                final_ip_param_dict[key].update({"--backup-key": str(bkpkeylist[item])})
                if (g_opts.skipDependentBackup == True and bkpDetail["BackupType"].upper() == "INCREMENTAL"):
                    isIncremental = True
                    final_ip_param_dict[key].update({"--incremental": None})
                if (unfinishedBkp == bkpkeylist[item]):
                    final_ip_param_dict[key].update({"--skip-restore-gtmcm": None})
            else:
                isIncremental = True
                g_opts.priorBkpKey = str(bkpkeylist[item -1])
                command_dict["--prior-backup-key"] = g_opts.priorBkpKey
                final_ip_param_dict[key].update({"--incremental": None})
                final_ip_param_dict[key].update({"--backup-key": str(bkpkeylist[item])})

        if (unfinishedBkp == bkpkeylist[item]):
            command_dict["--skip-restore-gtmcm"] = None
        doRestore(g_opts, command_dict, g_sshTool)
        if getRoachOpResult():
            g_logger.log("Error: Restore operation failed for backup key: %s" % bkpkeylist[item])
            if item != 0 and g_isClusterBkp:
                g_logger.debug("Clean cluster when cluster restore failed...")
                doClean(g_opts, g_logger)
            postExecCleanup(g_opts, g_logger)
            return getRoachOpResult()
        if not isIncremental and g_isClusterBkp:
            updateBackupIniForRestoreSuccess(g_logger, g_opts)

    if command_dict.has_key("--restore-new-cluster") and g_opts.restoreConfigs == True:
        #do conf files merge-update on all nodes
        g_logger.debug("After doRestore, will restore guc params because of --restore-new-cluster with --restore-configs")
        pyPath = os.path.dirname(currPath)
        g_logger.log("Performing config files updation...")
        check_cmd = 'source %s && export PYTHONPATH=%s:$PYTHONPATH && python %s -t confrestore -U %s -l %s' \
            % (g_opts.sourceFile, pyPath, SCRIPT_ROACH_LOCAL, g_opts.user, g_opts.localLog)
        if isSinglePrimaryMultiStandbyDeployment(cluster_version):
            check_cmd += ' --primary-standbys'
        g_logger.debug("after restore to new cluster, confrestore cmd: %s" % check_cmd)
        if not isMiniaturizedDeployment(cluster_version):
            (status, output) = g_sshTool.getSshStatusOutput(check_cmd, g_opts.nodeList)
            g_logger.debug("The status.values() is %s" % str(status.values()))
            g_logger.debug("The output is %s" % output)
            if 'Failure' in status.values():
                g_logger.logExit("Updating the config file values failed!")
            else:
                pass
        else:
            commands.getstatusoutput(check_cmd)

    if g_opts.logical:
        g_logger.debug("Call doRestore for logical mtable restore...")
        doRestore(g_opts, command_dict, table_list)
    postExecCleanup(g_opts, g_logger)

    return getRoachOpResult()

