try:
    import sys
    import os
    import getopt
    import commands
    import pwd
    import time
    import string
    import signal
    import subprocess
    import multiprocessing
    import json
    import getpass
    import re
    from threading import Timer
    from datetime import datetime, timedelta
    
    sys.path.append(sys.path[0] + "/../")
    from gspylib.common.DbClusterInfo import dbClusterInfo
    from gspylib.common.GaussLog import GaussLog
    from gspylib.threads.SshTool import SshTool
    from gspylib.common.Common import DefaultValue, ClusterCommand
    from gspylib.os.gsOSlib import g_OSlib, VersionInfo
    from gspylib.os.gsfile import g_file
    from gspylib.common.OMCommand import OMCommand
    from gspylib.common.ErrorCode import ErrorCode

    import util.GSroachCommon
except ImportError, e:
    sys.exit('Error:\nUnable to import module: ' + str(e))

ACTION_CONFIG = "config"

g_logger = None
LOG_PATH = DefaultValue.getEnv("GAUSSLOG")
DEFAULT_LOG_PATH = "%s/roach/controller" % LOG_PATH
LOG_FILE_NAME = "roach-controller.log"


def createFilePath(path, file_val):
    """
    Function to create file path
    """
    if path.endswith('/'):
        file_path = '%s%s' %(path, file_val)
    else:
        file_path = '%s/%s' %(path, file_val)
    return file_path

def isOldBackupsIni(filePath, g_logger):
    """
    check if backups.ini is from old version.
    means that R8C00/R7C10 and earlier versions which use 32 bit LSN
    """
    keyWords = ["StartLsn_xlogid", "StartLsn_xrecoff", \
        "StopLsn_xlogid", "StopLsn_xrecoff", \
        "BarrierLsn_xlogid", "BarrierLsn_xrecoff"]

    try:
        with open(filePath, 'r') as filehand:
            lines = filehand.readlines()
        if (len(lines) == 0):
            g_logger.debug("Meta file %s is empty!" % filePath)
            return False
        for key in keyWords:
            if (lines[0].find(key) != -1):
                g_logger.debug("Find key [%s] in old backups.ini" % key)
                return True
        return False
    except OSError as e:
        g_logger.debug("Warning: Some failure happened when "
                       "validate backups.ini")
        g_logger.debug("Exception in checking if backups.ini old is: %s"
                       % str(e))
        return False
    except Exception as e:
        g_logger.debug("Warning: Maybe backups.ini is invalid, please check, "
                       "msg: %s" % str(e))
        return False


def checkIp(ip):
    if re.match(r"^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$", ip):
        return True
    return False


def cleanTmpDir(g_logger):
    """
    clean temp files under $PGHOST
    """
    #clean tmp dir
    cmd = "ls %s/lockdb_*| xargs rm -fr" % (DefaultValue.getTmpDirFromEnv())
    g_logger.debug("deleting file in cleanTmpDir, cmd: %s" % cmd)
    (status, output) = commands.getstatusoutput(cmd)
    if(status != 0):
        g_logger.debug(ErrorCode.GAUSS_534["GAUSS_53401"] % "temp files under directory '$PGHOST'." + \
                       "Error: \n%s." % output)


def isInstancePrimary(query_result):
    result_list=query_result.split("\n")
    for item in result_list:
        if ('local_role' in item):
            local_role = item.split(":")
            role = local_role[1].strip()
            if(role == "Primary"):
                return True
    return False


def retryExecuteSQL(cmd, retryTime = 3, sleepTime = 1):
    """
    retry more times when execute a command
    @param cmd: command  going to be execute
    @param retryTime: default retry 3 times after execution failure
    @param sleepTime: default sleep 1 second then start retry 
    """
    for i in range(retryTime):
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0 or findError(output) == True):
            time.sleep(sleepTime)
        else:
            break
    return status, output


def findError(output):
    """
    check error string in sql output
    """
    ERROR_MSG_FLAG = "(ERROR|FATAL|PANIC)"
    ERROR_PATTERN = "^%s:.*" % ERROR_MSG_FLAG
    pattern = re.compile(ERROR_PATTERN)

    for line in output.split("\n"):
        line = line.strip()
        result = pattern.match(line)
        if (result is not None):
            g_logger.debug("This line contain errors:\n%s." % line)
            return True
    return False


def cleanFiles(filePath):
    cmd = "rm -fr %s" % (filePath)
    (status, output) = commands.getstatusoutput(cmd)
    g_logger.debug("Cleaning %s File, cmd=[%s]  result: %s." % (filePath, cmd, output))
    if (status != 0):
        raise Exception(ErrorCode.GAUSS_502["GAUSS_50207"] % "roachbackup" + " Error: \n%s." % output)


def setGucParameter(g_opts, sshTool, g_logger, gucParameter, gucValue, gucType):
    cluster_version = util.GSroachCommon.getClusterVersion(g_opts.user)

    cmd = "source %s && python %s -t %s --guc-parameter %s --guc-value %s --guc-type %s -U %s -l %s" \
        % (g_opts.sourceFile, util.GSroachCommon.LOCAL_ROACH, ACTION_CONFIG, \
           gucParameter, gucValue, gucType, g_opts.user, g_opts.logFile)
    g_logger.debug("Calling LocalRoach.py to set %s %s, cmd=%s." % (gucParameter, gucValue, cmd))
    if not util.GSroachCommon.isMiniaturizedDeployment(cluster_version):
        (status, output) = sshTool.getSshStatusOutput(cmd, g_opts.nodeList)
        if 'Failure' in status.values():
            g_logger.logExit("Set %s %s failed with output: %s" % (gucParameter, gucValue, output))
    else:
        status, output = DefaultValue.retryGetstatusoutput(cmd, 60)
        if (status != 0):
            g_logger.logExit("Set %s %s failed with output: %s" % (gucParameter, gucValue, output))
    g_logger.debug("Successfully set %s %s." % (gucParameter, gucValue))


def initLogger():
    global g_logger
    g_logger = GaussLog(createFilePath(DEFAULT_LOG_PATH, LOG_FILE_NAME))

if __name__ == '__main__':
    """
    main function
    """
    initLogger()

