#!/usr/bin/env python
#-*- coding:utf-8 -*-
#############################################################################
#Copyright (c): 2012-2019, Huawei Tech. Co., Ltd.
#FileName     : ParameterParsecheck.py
#Version      : Gauss200 
#Date         : 2016-09-06
#Description  : ParameterParsecheck.py is a utility to get Parameter information and check it.
#############################################################################
try:
    import os
    import getopt
    import sys
    sys.path.append(sys.path[0] + "/../../")
    from gspylib.common.ErrorCode import ErrorCode
    from gspylib.common.Common import DefaultValue
    from gspylib.common.GaussLog import GaussLog
    from gspylib.os.gsfile import g_file
    from gspylib.common.VersionInfo import VersionInfo
except ImportError, e:
    sys.exit("[GAUSS-52200] : Unable to import module: %s." % str(e))
    
PARAMETER_VALUEDICT = {}
PARAMETER_KEYLIST = []
ParameterDict = {}
Itemstr = []
skipItems = []
user_passwd = []
EnvParams = []
DbInitParam = []
GtmInitParam = []
CooGucParam = []
DataGucParam = []
CmServerGucParam = []
NODE_NAME = []

#Add parameter: the logic cluster name
PARA_CHECK_LIST = ["-t", "-h", "-m", "--mode",
                   "-i", "-j", "-U", "-u", "-G", "-g", "--alarm-type", "-n", "-g",
                   "-N", "--autostart", "--time-out", "--alarm-component", 
                   "--parallel-jobs", '--redis-mode', "--ring-num", "--virtual-ip",
                   "--nodeName", "--name", "--failure-limit"]
PATH_CHEKC_LIST = ["-M", "-o", "-f", "-X", "-P", "-s", "-R", "-Q", "--position", "-B",
                   "--backupdir", "--sep-env-file", "-l", "--logpath", "--backup-dir",
                   "--priority-tables","--exclude-tables"]
VALUE_CHECK_LIST = ["|",";","&","$","<",">","`","\\","'","\"","{","}","(",")","[","]","~","*","?","!","\n"]

#append ':' after short options if it required parameter
#append '=' after long options if it required parameter
# no child branch
gs_preinstall          = ["-?","--help","-V","--version","-U:","-G:","-L","-M:",
                          "--skip-os-set","-X:","--alarm-type=","--alarm-server-addr=","--agent-mode",
                          "--env-var=","--sep-env-file=","--skip-hostname-set","-l:","--non-interactive"]

gs_install             = ["-?","--help","-V","--version","--dws-mode", "-X:", "-l:", "--call-thread=",
                          "--gsinit-parameter=", "--gtminit-parameter=",
                          "--cn-guc=","--dn-guc=","--time-out=","--alarm-component=","--autostart=","--product="]
gs_uninstall           = ["-?","--help","-V","--version","-l:","-L","-X:","--delete-data",
                          "--cn-guc=","--dn-guc=","--cms-guc=","--time-out=","--alarm-component=","--autostart=","--product="]
gs_postuninstall       = ["-?","--help","-V","--version","--delete-user", "--delete-group", "--clean-gphome", 
                          "--delete-virtual-IP","-U:","-X:","-l:", "-h:", "-L"]
gs_check               = ["-?", "--help", "-V", "--version", "-e:", "-i:", "-U:", "-o:", "-l:", "-L", "--hosts=",
                          "--format=", "--cid=", "--disk-threshold=", "--time-out=", "--routing=", "--skip-items=",
                          "--ShrinkNodes=", "--nodegroup-name=", "--skip-root-items", "--set"]

gs_shrink              = ["-?","--help","-V","--version","--nodegroup-name=","--parallel-jobs=","-l:","-h:","-f:",
                          "--express", "--time-out=", "--rollback","--checkdisk","--io-priority=", "--redis-mode="]
gs_sshexkey            = ["-?","--help","-V","--version","--password=", "-f:", "-W:", "--skip-hostname-set","-l:"]
gs_backup              = ["-?","--help","-V","--version","--backup-dir=", "--parameter", 
                          "--binary", "--all","-l:","-h:","-t:","-X:"]
gs_collector           = ["-?","--help","-V","--version","--begin-time=", "--end-time=", 
                          "--keyword=","--speed-limit=","-h:","-f:","-o:","-l:","-C:"]
gs_checkperf           = ["-?","--help","-V","--version","--detail","-o:","-i:","-l:","-U:"]
gs_ssh                 = ["-?","--help","-V","--version","-c:"]
gs_checkos             = ["-?","--help","-V","--version","-h:","-f:","-o:","-i:","--detail",
                          "-l:","-X:"]

#gs_replace child branch
gs_replace_start       = ["-t:","-?","--help","-V","--version","-h:","--time-out=", "--dws-mode", "-l:", "--force" ,"--call-thread="]
gs_replace_install     = ["-t:","-?","--help","-V","--version", "--dws-mode", "-h:", "-l:" ,"--call-thread="]
gs_replace_config      = ["-t:","-?","--help","-V","--version", "--dws-mode", "-h:", "-l:", "--force" ,"--call-thread=", "--ecs"]
gs_replace_WarmStandby = ["-t:","-?","--help","-V","--version","-X:", "-h:", "--dws-mode", "-l:" ,"--call-thread="]

#gs_om child branch
gs_om_start            = ["-t:","-?","--help","-V","--version","-h:","-I:","--time-out=","--az=","-l:" ,"--call-thread="]
gs_om_stop             = ["-t:","-?","--help","-V","--version","-h:","-I:","-m:","--az=","-l:","--mode=" ,"--call-thread="]
gs_om_restart          = ["-t:","-?","--help","-V","--version","--nodegroup-name=","-l:","--time-out="]
gs_om_status           = ["-t:","-?","--help","-V","--version","-h:","-o:","--detail","-l:", "--call-thread="]
gs_om_show_disk        = ["-t:","-?","--help","-V","--version"]
gs_om_show_disk_and_log = ["-t:","-?","--help","-V","--version"]
gs_om_show_cluster_disk = ["-t:","-?","--help","-V","--version"]
gs_om_show_alarm_log = ["-t:","-?","--help","-V","--version"]
gs_om_switch           = ["-t:","-?","--help","-V","--version","-h:","-I:","-D:","--reset","--time-out=","--az=","-l:","--call-thread="]
gs_om_startetcd        = ["-t:","-?","--help","-V","--version","-l:"]
gs_om_managecn         = ["-t:","-?","--help","-V","--version","-m:","-X:","-l:","--mode="]
gs_om_changeip         = ["-t:","-?","--help","-V","--version","-X:","--rollback","-l:"]
gs_om_generateconf     = ["-t:","-?","--help","-V","--version","-X:","--distribute","-l:"]
gs_om_formatcu         = ["-t:","-?","--help","-V","--version","-j:","-l:"]
gs_om_virtualip        = ["-t:","-?","--help","-V","--version","-m:","-X:","-L","-l:","--mode=","--virtual-ip="]
gs_om_cert             = ["-t:","-?","--help","-V","--version","-L","-l:","--cert-file=","--rollback"]
gs_om_kerberos = ["-t:", "-?", "--help", "-V", "--version", "-r", "-m:", "-U:", "-X:", "-l:", "--krb-server=", "--krb-client",
                  "--krb-standby="]
gs_om_tablespace       = ["-t:","-?","--help","-V","--version","-h:","-l:"]
gs_om_estimate_upgrade_addCN        = ["-t:","-?","--help","-V","--version","-m:","-l:","--mode="]
gs_om_estimate_install_expand       = ["-t:","-?","--help","-V","--version","-m:","-l:","--mode=","-X:"]
gs_om_estimate_replace_warm_standby = ["-t:","-?","--help","-V","--version","-m:","-l:","--mode=","-h:"]
gs_om_ec               = ["-t:","-?","--help","-V","--version","-m:","--target=","--key-files","--all","--force","-N:","--type=","-U:","-L"]
gs_om_javaUDF          = ["-t:", "-?", "--help", "-V", "--version", "-m:", "-l:", "--mode=", "-s:", "-d:"]
gs_om_postgis          = ["-t:", "-?", "--help", "-V", "--version", "-m:", "-l:", "--mode="]
gs_om_dailyAlarm       = ["-t:", "-?", "--help", "-V", "--version", "-U:", "-l:"]
gs_om_express = ["-t:", "-?", "--help", "-V", "--version", "-m:", "--mode=", "-h:", "--detail"]
gs_om_compare_cluster = ["-t:", "-?", "--help", "-V", "--version", "--remote-host=", "--remote-env-file=", "-l:"]
gs_om_license = ["-t:", "-?", "--help", "-m:", "--mode=", "-f:", "--features=", "-r", "--rollback"]
gs_om_isolate_stop        = ["-t:","-?","--help","-V","--version","-l:","-h:"]
gs_om_isolate_tostandby   = ["-t:","-?","--help","-V","--version","-l:","-h:"]
gs_om_isolate_restore     = ["-t:","-?","--help","-V","--version","-l:","-h:"]
gs_om_killsession = ["-t:", "-?", "--help", "-V",
                     "--version", "-l:", "--close-hba", "--restore-hba", "--public-cloud"]
gs_om_slow_node           = ["-t:","-?","--help","-V","--version","-l:","-m:","--interval=","--check-count=","--threshold=","--wait-count="]
gs_om_hotpatch            = ["-t:", "-?", "--help", "-V", "--version", "-n:", "-c:"]
gs_om_perf_job            = ["-t:", "-?", "--help", "-V", "--version", "-m:", "--mode="]
gs_hotpatch_load          = ["-t:", "-?", "--help", "-V", "--version", "-n:"]
gs_hotpatch_unload          = ["-t:", "-?", "--help", "-V", "--version", "-n:"]
gs_hotpatch_active          = ["-t:", "-?", "--help", "-V", "--version", "-n:"]
gs_hotpatch_deactive          = ["-t:", "-?", "--help", "-V", "--version", "-n:"]
gs_hotpatch_info          = ["-t:", "-?", "--help", "-V", "--version", "-n:"]
gs_hotpatch_list          = ["-t:", "-?", "--help", "-V", "--version", "-n:"]
#gs_expand child branch
gs_expand_dilatation   = ["-t:", "-?", "--help", "-V", "--version", "-X:", "-l:",
                          "--dws-mode",  "--parallel-jobs=", "--time-out=", "--call-thread=",
                          "--nodegroup-name=", "--addto-elastic-group",
                          "--dilatation-mode=", "--express"]
gs_expand_redistribute = ["-t:",
                          "-?",
                          "--help",
                          "-V",
                          "--version",
                          "--vacuum-full",
                          "--fast-redis",
                          "--parallel-jobs=",
                          "--build-redistb",
                          "--dws-mode",
                          "--redis-mode=",
                          "--time-out=",
                          "-l:",
                          "--priority-tables=",
                          "--failure-limit=",
                          "--exclude-tables=",
                           "--call-thread="]
gs_expand_crashclean   = ["-t:","-?","--help","-V","--version","-X:","--dws-mode","-l:", "--express"]
# for resource control in data redistribution
gs_expand_resourcectl  = ["-t:","-?","--help","-V","--version","--resource-level="]
gs_expand_nodegroup_migrate = ["-t:","-?","--help","-V","--version","--resource-level="]
gs_expand_progress  = ["-t:","-?","--help","-V","--version","-X:"]

#parameter list for sub-action of gs_resize
gs_resize = ["-t:","-?","--help","-V","--version","-X:","-l:","--parallel-jobs=","--rollback"]
gs_resize_clean = ["-t:","-?","--help","-V","--version","-X:","-l:"]
gs_resize_progress = ["-t:","-?","--help","-V","--version","-X:","-l:"]

#gs_upgradectl child branch
#AP and TP are same
gs_upgradectl_chose_strategy = ["-t:","-?","--help","-V","--version","-l:"]
#auto-upgrade parameter lists
gs_upgradectl_auto_upgrade = ["-t:","-?","--help","-V","--version","-l:","-X:", "--call-thread=",
                              "-M:","--product=", "--inplace", "-g:", "-h:", "--continue", "--begin=", "--end="]
#auto-rollback parameter lists
gs_upgradectl_auto_rollback = ["-t:","-?","--help","-V","--version","-l:","-X:", "--call-thread=", "--force"]
#commit-upgrade parameter lists
gs_upgradectl_commit = ["-t:","-?","--help","-V","--version","-l:","-X:" ,"--call-thread="]

#gs_lcctl child branch
#create the logic cluster
gs_lcctl_create = ["-t:","-?","--help","-V","--version","-l:","-h:","-f:","--name=",
                   "--upgrade","--high-perform","--non-interactive"]
#rollback the logic cluster
gs_lcctl_rollback = ["-t:","-?","--help","-V","--version","-l:","--name="]
#add nodes to a logic cluster
gs_lcctl_add = ["-t:","-?","--help","-V","--version","-l:","-h:","-f:","--name="]
#display message for logic cluster
gs_lcctl_display = ["-t:","-?","--help","-V","--version","--lcname-only"]
#delete the logic cluster
gs_lcctl_delete = ["-t:","-?","--help","-V","--version","--name=","--elastic-group","-h:","-f:","-l:"]
gs_lcctl_progress  = ["-t:","-?","--help","-V","--version","--name=","--option="]

ParameterDict = {"preinstall"     : gs_preinstall,
                 "install"        : gs_install, 
                 "uninstall"      : gs_uninstall, 
                 "replaceinstall" : gs_replace_install,
                 "replaceconfig"  : gs_replace_config,
                 "replacestart"   : gs_replace_start,
                 "WarmStandby"    : gs_replace_WarmStandby, 
                 "checkos"        : gs_checkos, 
                 "dilatation"     : gs_expand_dilatation,
                 "redistribute"   : gs_expand_redistribute,
                 "crashclean"     : gs_expand_crashclean,
                 "resourcectl"    : gs_expand_resourcectl,
                 "nodegroup_migrate": gs_expand_nodegroup_migrate,
                 "progress"       : gs_expand_progress,
                 "checkperf"      : gs_checkperf,
                 "check"          : gs_check,
                 "shrink"         : gs_shrink, 
                 "auto_upgrade"   : gs_upgradectl_auto_upgrade,
                 "chose_strategy" : gs_upgradectl_chose_strategy,
                 "commit_upgrade" : gs_upgradectl_commit,
                 "auto_rollback"  : gs_upgradectl_auto_rollback,
                 "start"          : gs_om_start,
                 "stop"           : gs_om_stop, 
                 "restart"        : gs_om_restart,
                 "status"         : gs_om_status,
                 "show_disk"      : gs_om_show_disk,
                 "show_disk_and_log"      : gs_om_show_disk_and_log,
                 "show_cluster_disk" : gs_om_show_cluster_disk,
                 "show_alarm_log" : gs_om_show_alarm_log,
                 "startetcd"      : gs_om_startetcd,
                 "managecn"       : gs_om_managecn, 
                 "changeip"       : gs_om_changeip,
                 "generateconf"   : gs_om_generateconf,
                 "formatcu"       : gs_om_formatcu, 
                 "switch"         : gs_om_switch,
                 "virtualip"      : gs_om_virtualip,
                 "cert"           : gs_om_cert,
                 "tablespace"     : gs_om_tablespace,
                 "kerberos"       : gs_om_kerberos,
                 "ec"             : gs_om_ec,
                 "javaUDF"        : gs_om_javaUDF,
                 "postgis"        : gs_om_postgis,
                 "dailyAlarm"     : gs_om_dailyAlarm,
                 "compare-cluster": gs_om_compare_cluster,
                 "hotpatch"       : gs_om_hotpatch,
                 "perf_job"       : gs_om_perf_job,
                 "sshexkey"       : gs_sshexkey,
                 "backup"         : gs_backup, 
                 "collector"      : gs_collector, 
                 "ssh"            : gs_ssh, 
                 "postuninstall"  : gs_postuninstall,
                 "lcctlcreate"    : gs_lcctl_create,
                 "lcctlrollback"  : gs_lcctl_rollback,
                 "lcctladd"       : gs_lcctl_add,
                 "lcctldisplay"   : gs_lcctl_display,
                 "lcctldelete"   : gs_lcctl_delete,
                 "lcctlprogress"   : gs_lcctl_progress,
                 "estimate_upgrade_addCN"         : gs_om_estimate_upgrade_addCN,
                 "estimate_install_expand"        : gs_om_estimate_install_expand,
                 "estimate_replace_warm_standby"  : gs_om_estimate_replace_warm_standby,
                 "isolate_stop"                   : gs_om_isolate_stop,
                 "isolate_tostandby"              : gs_om_isolate_tostandby,
                 "isolate_restore"                : gs_om_isolate_restore,
                 "killsession"                    : gs_om_killsession,
                 "slow_node"                      : gs_om_slow_node,
                 "cr_resize"       : gs_resize,
                 "cr_clean"        : gs_resize_clean,
                 "cr_progress"     : gs_resize_progress,
                 "express" : gs_om_express,
                 "license" : gs_om_license,
                 "load" : gs_hotpatch_load,
                 "unload": gs_hotpatch_unload,
                 "active": gs_hotpatch_active,
                 "deactive": gs_hotpatch_deactive,
                 "info": gs_hotpatch_info,
                 "list": gs_hotpatch_list
                 } 

#List of scripts with the -t parameter
#add gs_lcctl
special_list       = [ "gs_om", 
                       "replace", 
                       "backup", 
                       "upgradectl", 
                       "expand", 
                       "clusterresize",
                       "shrink", 
                       "lcctl",
                       "hotpatch"
                      ]

estimate_list     = ["upgrade", "install", "expand", "addCN", "replace", "warm-standby"]
#The -t parameter list
action_om         = [ "start", "stop", "status", "show_disk", "show_disk_and_log", "show_cluster_disk", "show_alarm_log", "startetcd", "managecn", "switch",
                      "changeip", "generateconf", "formatcu", "estimate", "virtualip", 
                       "cert", "tablespace", "kerberos", "ec", "javaUDF", "postgis",
                       "dailyAlarm", "express", "compare-cluster", "restart", "license",
                       "isolate_stop", "isolate_tostandby", "isolate_restore", "killsession", "slow_node",
                       "hotpatch", "perf_job"
                     ]
action_replace    = ["install", "config", "start", "warm-standby"]
action_upgradectl = ["chose-strategy", "auto-upgrade", "auto-rollback", "commit-upgrade"]
# add "resourcectl" for resource control in data redistribution
action_expand     = ["dilatation", "redistribute", "crashclean", "resourcectl", "managedn", "progress", "nodegroup_migrate"]
action_resize = ["resize", "clean", "progress"]
#add gs_vcctl for a virtual cluster
action_lcctl       = ["create", "rollback", "add", "display", "delete", "progress"]

class Parameter():
    '''
    get Parameter information and check it. 
    '''
    def __init__(self):
        '''
        '''
        self.action = ""
        self.mode = ""
        self.helpflag = False
        self.versionflag = False

    def ParseParameterValue(self, module):
        """
        function: parse the parameter value
        input : parameter_list
        output: options
        """
        # get the parameter list
        (shortParameter, longParameter) = self.getParseParameter(module)

        try:
            paraList = sys.argv[1:]
            for paraInfo in paraList:
                if (paraInfo.startswith('--')):
                    isFlag = False
                    for longPara in longParameter:
                        if (paraInfo[2:].startswith(longPara.strip("="))):
                            isFlag = True
                    if (not isFlag):
                        GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50000"] % paraInfo)
            #check delete parameter -h and -f, if specified lcname, not required -h or -f.
            check_delete_name = False
            for check_i in sys.argv[1:]:
                if ("--name" in check_i):
                    check_delete_name = True
                    break
            if (module == "lcctl" and "delete" in sys.argv[1:] and check_delete_name and ('-h' in sys.argv[1:] or '-f' in sys.argv[1:])):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50002"] % 'h or -f')
            (opts, args) = getopt.getopt(sys.argv[1:], shortParameter, longParameter)
        except Exception, e:
            s1 = str(e).split(" ")
            option = s1[1]
            if ("requires argument" in str(e)):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % \
                         option[1:] + " Error:\n%s" % str(e))
            elif ("not recognized" in str(e)):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50000"] % option)
            elif ("not a unique prefix" in str(e)):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50006"] % option)
            else:
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50000"] % str(e))

        if (len(args) > 0):
            GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50000"] % str(args[0]))

        return opts

    def moveCrypto(self, module):
        """
        function: Parse the parameter
        input : parameter_list
        output: PARAMETER_VALUEDICT
        """
        if (module in ("preinstall", "sshexkey")):
            DefaultValue.doConfigForParamiko()

    def printVersionInfo(self):
        """
        """
        if (self.versionflag):
            print ("%s %s" % (sys.argv[0].split("/")[-1], VersionInfo.COMMON_VERSION))
            sys.exit(0)

    def ParameterCommandLine(self, module):
        """
        function: Parse the parameter
        input : parameter_list
        output: PARAMETER_VALUEDICT
        """
        # copy crypto
        self.moveCrypto(module)

        #Determines whether help and version information is output
        self.helpflag, self.versionflag = self.getHelpAndVersionStatus()
        if (self.helpflag):
            PARAMETER_VALUEDICT['helpFlag'] = self.helpflag
            return PARAMETER_VALUEDICT

        # print version information
        self.printVersionInfo()

        #Special handling of the -t parameter
        self.getActionParameterValue(module)

        # get the parameter list
        opts = self.ParseParameterValue(module)

        parameterNeedValue = { "-t":"action",
                               "-c":"cmd",
                               "-m":"Mode",
                               "--mode":"Mode",
                               # hotpatch name
                               "-n": "patch_name",
                               "-d": "destPath",
                               "-s": "sourcePath",
                               "-j":"jobs",
                               "-U":"user",
                               "-G":"group",
                               "-I":"instance_name",
                               "-e": "scenes",
                               "-C": "configFile",
                               "--format": "format",
                               "--cid": "cid",
                               "--routing": "routing",
                               "--ShrinkNodes": "shrinkNodes",
                               "--az":"az_name", 
                               "--root-passwd":"rootPasswd",
                               "--alarm-type":"warningType",
                               "--alarm-server-addr":"warningserverip",
                               "--autostart":"autostart",
                               "--time-out":"time_out","":"",
                               "--alarm-component":"alarm_component",
                               "--SSD-fault-time":"SSDFaultTime",
                               "--begin-time":"begintime",
                               "--end-time":"endtime",
                               "--keyword":"keyword",
                               "--redis-mode":"redismode",
                               "--failure-limit":"failure_limit",
                               "--virtual-ip":"virtual-ip",
                               "--master":"master",
                               "--standby":"standby",
                               "--disk-threshold":"disk-threshold",
                               "--target":"target",
                               "--name":"name",
                               "-N":"DSN",
                               "--type":"type",
                               "--remote-host": "remote_host",
                               "--remote-env-file": "remote_mpprcfile",
                               "--dilatation-mode":"dilatation_mode",
                               "--nodegroup-name": "nodegroup_name",
                               "--speed-limit": "speedLimit",
                               # add "resourcectl" for resource control in data redistribution
                               "--resource-level":"resource_level",
                               "-p":"port",
                               "--dn-port":"dn-port",
                               "--dn-ip":"dn-ip",
                               # Adapt 200 and 300 versions
                               "--product":"productVersion",
                               "--interval": "interval",
                               "--threshold": "threshold",
                               "--check-count": "check_count",
                               "--wait-count" : "wait_count",
                               "--option":"option",
                               "--dbname":"dbname",
                               "--dbuser":"dbuser",
                               "--krb-server": "krb-server",
                               "--krb-standby": "krb-standby",
                               "--begin": "begin",
                               "--end": "end",
                               # Add call process thread
                               "--call-thread": "call_thread"}
        parameterNeedValue_keys = parameterNeedValue.keys()

        parameterIsBool = { "-L":"localMode",
                            "--set": "set",
                            "--skip-root-items": "skipRootItems",
                            "--non-interactive":"preMode",
                            "--skip-os-set":"skipOSSet",
                            "--skip-hostname-set":"skipHostnameSet",
                            "--reset":"reset",
                            "--parameter":"isParameter",
                            "--binary":"isBinary",
                            "--delete-data":"cleanInstance",
                            "--delete-user":"delete-user",
                            "--delete-group":"delete-group",
                            "--delete-virtual-IP":"delete-virtual-IP",
                            "--dws-mode":"dws-mode",
                            "--detail":"show_detail",
                            "--detail-all":"show_detail_all",
                            "--rollback":"rollback",
                            "--vacuum-full":"enable_vacuum",
                            "--fast-redis":"enable_fast",
                            "--distribute":"distribute",
                            "--build-redistb":"buildTable",
                            "--key-files":"key-files",
                            "--all":"all",
                            "--upgrade":"upgrade",
                            "--lcname-only":"lcname-only",
                            "--high-perform":"high-perform",
                            "--elastic-group":"elastic-group",
                            "--addto-elastic-group":"isAddElasticGroup",
                            "--express" : "express",
                            "--checkdisk" : "checkdisk",
                            "--inplace" : "inplace",
                            "--continue" : "continue",
                            "--force" : "force",
                            "--ecs" : "ecs",
                            "--agent-mode": "agentMode",
                            "--krb-server": "krb-server",
                            "--krb-client": "krb-client",
                            "--close-hba": "closehba",
                            "--restore-hba": "restorehba",
                            "--public-cloud": "publiccloud"
                          }
        parameterIsBool_keys = parameterIsBool.keys()

        #Parameter assignment and return
        for (key, value) in opts:
            if (key in parameterNeedValue_keys):
                PARAMETER_VALUEDICT[parameterNeedValue[key]] = value
            elif (key in parameterIsBool_keys):
                PARAMETER_VALUEDICT[parameterIsBool[key]] = True
            elif(key == "-h"):
                #Only obtain the last value of hostname
                del NODE_NAME[:]
                for node in value.strip().split(","):
                    if(node != None and node != "" and (node not in NODE_NAME)):
                        NODE_NAME.append(node.strip())
            elif(key == "-W" or key == "--password"):
                user_passwd.append(value)
            elif(key == "-D"):
                PARAMETER_VALUEDICT['dataDir'] = os.path.normpath(value)
            elif(key == "-M"):
                PARAMETER_VALUEDICT['cgroupMountDir'] = os.path.realpath(value.strip())
            elif(key == "-o"):
                PARAMETER_VALUEDICT['outFile'] = os.path.realpath(value)
                if(module not in ["collector", "check"]):
                    self.createOutputDir(os.path.realpath(value))
            elif(key == "-i"):
                for item in value.strip().split(","):
                    if(item != None and item != "" and (item not in Itemstr)):
                        Itemstr.append(item)
            elif(key == "--skip-items"):
                for item in value.strip().split(","):
                    if(item != None and item != "" and (item not in skipItems)):
                        skipItems.append(item)
            elif self.action != "license" and (key == "-f" or key == "--hosts"):
                hostFile = self.checkPath(key, value)
                PARAMETER_VALUEDICT['hostfile'] = os.path.realpath(hostFile)
            elif(key == "-X"):
                if (module != "uninstall"):
                    xmlFile = self.checkPath(key, value)
                    PARAMETER_VALUEDICT['confFile'] = os.path.realpath(xmlFile)
                else:
                    xmlFile = str(value)
                    PARAMETER_VALUEDICT['confFile'] = os.path.realpath(xmlFile)
            elif(key == "--env-var"):
                EnvParams.append(value)
            elif(key == "--sep-env-file"):
                PARAMETER_VALUEDICT['mpprcFile'] = os.path.realpath(value)
            elif (key == "--gsinit-parameter"):
                DbInitParam.append(value)
            elif (key == "--gtminit-parameter"):
                GtmInitParam.append(value)
            elif (key == "--cn-guc"):
                CooGucParam.append(value)
            elif (key == "--dn-guc"):
                DataGucParam.append(value)
            elif (key == "--cms-guc"):
                CmServerGucParam.append(value)
            elif(key == "-l"):
                PARAMETER_VALUEDICT['logFile'] = os.path.realpath(value)
            elif(key == "--backup-dir"):
                PARAMETER_VALUEDICT['backupDir'] = os.path.realpath(value.strip())
            elif(key =="--all"):
                PARAMETER_VALUEDICT['isParameter'] = True
                PARAMETER_VALUEDICT['isBinary'] = True
            elif (key == "--parallel-jobs"):
                paralleljobs = self.checkParamternum(key , value)
                PARAMETER_VALUEDICT['paralleljobs'] = paralleljobs
            elif (key == "-g"):
                nodesNum = self.checkParamternum(key, value)
                PARAMETER_VALUEDICT['nodesNum'] = nodesNum
            elif key == "--begin":
                begin = self.checkParamternum(key, value)
                PARAMETER_VALUEDICT['begin'] = begin
            elif key == "--end":
                end = self.checkParamternum(key, value)
                PARAMETER_VALUEDICT['end'] = end
            elif(key == "--ring-num"):
                ringNumbers = self.checkParamternum(key ,value)
                PARAMETER_VALUEDICT['ringNumbers'] = ringNumbers
            elif (key == "--cert-file"):
                PARAMETER_VALUEDICT['cert-file'] = os.path.realpath(value.strip())
            elif(key == "--priority-tables"):
                PARAMETER_VALUEDICT['priority-tables'] = os.path.realpath(value.strip())
            elif key == "--role":
                PARAMETER_VALUEDICT['role'] = value.strip()
            elif(key == "--exclude-tables"):
                PARAMETER_VALUEDICT['exclude-tables'] = os.path.realpath(value.strip())

            #Only check / symbol for gs_lcct.
            if key in ("--name", "--nodegroup-name"):
                self.checkLcGroupName(key, value)
            Parameter.checkParaVaild(key, value)

        parameterIsList = { "passwords":user_passwd,
                            "envparams":EnvParams,
                            "dbInitParams":DbInitParam,
                            "gtmInitParams":GtmInitParam,
                            "cooGucParams":CooGucParam,
                            "dataGucParams":DataGucParam,
                            "cmServerGucParams":CmServerGucParam,
                            "itemstr":Itemstr,
                            "skipItems": skipItems,
                            "nodename":NODE_NAME}
        parameterlenkeys = parameterIsList.keys()
        for key in parameterlenkeys:
            if (len(parameterIsList[key]) > 0):
                PARAMETER_VALUEDICT[key] = parameterIsList[key]
        return PARAMETER_VALUEDICT
     
    @staticmethod
    def checkParaVaild(para, value):
        """
        function: check para vaild
        input : NA
        output: NA
        """
        for role in VALUE_CHECK_LIST:
            if PARA_CHECK_LIST.__contains__(para):
                if value.find(role) >= 0:
                    GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50011"] % \
                             (para, value) + " Invaild value: %s." % role) 
            if PATH_CHEKC_LIST.__contains__(para):
                if os.path.realpath(value).find(role) >= 0:
                    GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50011"] % \
                             (para, value) + " Invaild value: %s." % role) 
    def checkLcGroupName(self, lcPara, lcGroupName):
        """
        function: Check if the virtual cluster name is legal.
        input : lcGroupName
        output: NA
        """
        import re
        PATTERN = "^[a-zA-Z0-9_]{1,63}$"
        pattern = re.compile(PATTERN)
        result = pattern.match(lcGroupName)
        if (result is None):
            raise Exception(ErrorCode.GAUSS_500["GAUSS_50004"] % lcPara[1:] + " The name of the logical cluster does not exceed 63 characters and can only contain letters, numbers, and underscores.")
        if (lcGroupName in ["group_version1","group_version2","group_version3","elastic_group"]):
            raise Exception(ErrorCode.GAUSS_500["GAUSS_50004"] % lcPara[1:] + " The name of the logical cluster cannot be 'group_version1' or 'group_version2' or 'group_version3' or 'elastic_group'.")

    def getHelpAndVersionStatus(self):
        """
        function: get help and version information status
        input : NA
        output: helpflag, versionflag
        """
        helpflag = False
        versionflag = False
        for parameter in sys.argv[1:]:
            if (parameter =="-?" or parameter == "--help"):
                helpflag = True
            if(parameter == "-V" or parameter == "--version"):
                versionflag = True
        return helpflag, versionflag

    def getActionParameterValue(self, module):
        """
        function: get the action value
        input : parameter_list
        output: NA
        """
        actions = []
        getMode = False
        if (module in special_list):
            if (sys.argv[1:] == []):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50014"] % module)
            elif('-t' in sys.argv[1:] and module == "shrink"):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50002"] % "t"+".")

            if (sys.argv[1:][-1] == "-t"):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % \
                         "t" + " option -t requires argument.")

            for n, value in enumerate(sys.argv[1:]):
                if (sys.argv[1:][n-1] == "-t"):
                    actions.append(value)
                    if (len(actions) != 1):
                        GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50006"] % actions[0])
                    self.action = value

            if self.action == "" and module != "shrink":
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50001"] % "t" + ".")

            if ((module == "gsom" and not self.action in action_om) or
                (module == "replace" and not self.action in action_replace) or
                (module == "upgradectl" and not self.action in action_upgradectl) or
                (module == "expand" and not self.action in action_expand) or
                (module == "clusterresize" and not self.action in action_resize) or
                (module == "lcctl" and not self.action in action_lcctl)):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "t")
        if (self.action == "estimate"):
            if (sys.argv[1:][-1] == "-m"):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % \
                         "m" + " option -m requires argument.")
            if (sys.argv[1:][-1] == "--mode"):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % \
                         "-mode" + " option --mode requires argument.")

            for n, value in enumerate(sys.argv[1:]):
                if (sys.argv[1:][n-1] == "-m" or sys.argv[1:][n-1] == "--mode"):
                    self.mode = value
                if (value.startswith("--mode=")):
                    self.mode = value[len("--mode="):]
                    getMode = True
                
            if (self.mode == ""):
                if getMode:
                    GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % \
                             "-mode" + " option --mode requires argument.")
                else:
                    GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50001"] % "m or --mode" + ".")
                            
            if (self.mode not in estimate_list):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "m or --mode")

    def createOutputDir(self, path):
        """
        function: create output directory
        input : path
        output: NA
        """
        try:
            DefaultValue.checkOutputFile(path)
        except Exception, e:
            GaussLog.exitWithError(str(e))
        dirName = os.path.dirname(os.path.realpath(path))
        if (not os.path.isdir(dirName)):
            try:
                os.makedirs(dirName, 0750)
            except Exception, e:
                GaussLog.exitWithError(ErrorCode.GAUSS_502["GAUSS_50206"] % \
                         ("outputfile[%s]" % path) + "Error:\n%s" % str(e))

    def checkParamternum(self, key, value):
        """
        function: Check some number parameters
        input : key, value
        output: numvalue
        """
        try:
            numvalue  = int(value)
            if (numvalue <= 0):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % \
                         key[1:] + " Parameter '%s' must be greater than or equal to 1." % key)
        except Exception as e:
            GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50003"] % \
                     (key[1:], "integer") + " Error:\n%s" % str(e))
            
        return numvalue  
    
    def checkPath(self, key, value):
        """
        function: Check some path parameters
        input : key, value
        output: path
        """
        #Check that the path parameter is a file
        try:
            if (not value):
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % key[1:] +
                                       "Error:\noption %s requires argument" % key)
            path = str(value)
            g_file.checkFilePermission(path, True)
            return path
        except Exception as e:
            GaussLog.exitWithError(str(e))

    def getParseParameter(self, module):
        """
        function: get parse parameters
        input : parameter_list
        output: shortPara,longPara
        """
        
        shortPara = ""
        longPara  = []
        var = "--"
        
        ParameterList = ""
        if (module == "replace"):
            parameter_map = {"warm-standby":"WarmStandby",
                             "install":"replaceinstall",
                             "config":"replaceconfig",
                             "start":"replacestart"}
            if (self.action in parameter_map.keys()):
                ParameterList = ParameterDict.get(parameter_map[self.action])
            else:
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "t")

        elif(module == "expand"):
            if (self.action in action_expand):
                ParameterList = ParameterDict.get(self.action)
            else:
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "t")

        elif(module == "clusterresize"):
            action_map = { "resize"      : "cr_resize",
                           "clean"       : "cr_clean",
                           "progress"    : "cr_progress"
                         }
            if (self.action in action_map.keys()):
                ParameterList = ParameterDict.get(action_map[self.action])
            else:
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "t")

        elif(module == "upgradectl"):
            if (self.action == "chose-strategy"):
                ParameterList = ParameterDict.get("chose_strategy")
            elif(self.action == "auto-rollback"):
                ParameterList = ParameterDict.get("auto_rollback")
            elif(self.action == "auto-upgrade"):
                ParameterList = ParameterDict.get("auto_upgrade")
            elif(self.action == "commit-upgrade"):
                ParameterList = ParameterDict.get("commit_upgrade")
            else:
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "t")

        elif(module == "gs_om"):
            if(self.action == "estimate"):
                parameter_map = {"upgrade":"estimate_upgrade_addCN",
                                 "addCN":"estimate_upgrade_addCN",
                                 "install":"estimate_install_expand",
                                 "expand":"estimate_install_expand",
                                 "replace":"estimate_replace_warm_standby",
                                 "warm-standby":"estimate_replace_warm_standby"}
                if (self.mode in parameter_map.keys()):
                    ParameterList = ParameterDict.get(parameter_map[self.mode])
                else:
                    GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "m")
            elif(self.action in action_om):
                ParameterList = ParameterDict.get(self.action)
            else:
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "t")

        elif (module == "lcctl"):
            parameter_map = {"create":"lcctlcreate",
                             "add":"lcctladd",
                             "rollback":"lcctlrollback",
                             "display":"lcctldisplay",
                             "delete":"lcctldelete",
                             "progress":"lcctlprogress"}
            if (self.action in parameter_map.keys()):
                ParameterList = ParameterDict.get(parameter_map[self.action])
            else:
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "t")
        elif module == "hotpatch":
            if self.action in DefaultValue.HOTPATCH_ACTION_LIST:
                ParameterList = ParameterDict.get(self.action)
            else:
                GaussLog.exitWithError(ErrorCode.GAUSS_500["GAUSS_50004"] % "t")
        else:
            ParameterList = ParameterDict.get(module)
            
        for para in ParameterList:
            if var in para:
                varlong = para.strip("--")
                longPara.append(varlong)
            else:
                varshort = para.strip("-")
                shortPara += varshort

        return shortPara,longPara

