#!/usr/bin/env python
#-*- coding:utf-8 -*-
#############################################################################
#Copyright (c): 2012-2019, Huawei Tech. Co., Ltd.
#FileName     : CM.py 
#Version      : Gauss200 
#Date         : 2018-04-04
#Description  : CM.py is a class of CM part of Gauss200.
#############################################################################
try:
    import sys
    import os
    import commands
    import time
    import datetime
    import ConfigParser
    from datetime import datetime, timedelta
    
    sys.path.append(sys.path[0] + "/../../../../")
    from gspylib.common.ErrorCode import ErrorCode
    from gspylib.common.Common import DefaultValue
    from gspylib.common.DbClusterStatus import DbClusterStatus
    from gspylib.os.gsfile import g_file
    from gspylib.os.gsOSlib import g_OSlib
    from gspylib.component.CM.CM import CM
    from gspylib.common.DbClusterInfo import dbClusterInfo
except ImportError, e:
    sys.exit("[GAUSS-52200] : Unable to import module: %s." % str(e))


class CM_OLAP(CM):
    '''
    The class is used to define cluster manager component for olap database.
    '''
    # Don't need DEFAULT_TIMEOUT = 300, user cm_ctl default timeout
    DEFAULT_TIMEOUT = 0
    # Don't need DEFAULT_RESTART_NODEGROUP_TIMEOUT = 1800, user cm_ctl default timeout
    DEFAULT_RESTART_NODEGROUP_TIMEOUT = 0

    def __init__(self):
        '''
        Constructor
        '''
        super(CM_OLAP, self).__init__()

    def initInstance(self):
        """
        function : install a single cm component
        input  : NA
        output : NA
        """
        if (self.instInfo.datadir == ""):
            raise Exception("Data directory of instance is invalid.")
        
        if(not os.path.exists(self.instInfo.datadir)):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50201"] % ("data directory [%s]" % self.instInfo.datadir))

        toolspath = DefaultValue.getEnv(DefaultValue.TOOL_PATH_ENV) 
        DefaultValue.checkPathVaild(toolspath)
        cmd = " "
        if self.dwsMode:
            image_path = DefaultValue.DWS_IMAGE_PATH
            if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_CMSERVER):
                #decompress the package
                packagePath = "%s/cm_server.tar.gz" % image_path
                g_file.decompressFiles(packagePath, self.instInfo.datadir)
                #set parameter of cm_server.conf
                path = "%s/cm_server.conf" % self.instInfo.datadir
                read_only = "enable_transaction_read_only"
                timeout = "coordinator_heartbeat_timeout"
                ha_timeout = "cmserver_ha_heartbeat_timeout"
                para_dict = {read_only:"on",timeout:"0",ha_timeout:"30"}
                self.setGucConfig(para_dict)
            elif (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_CMAGENT):
                #decompress the package
                packagePath = "%s/cm_agent.tar.gz" % image_path
                g_file.decompressFiles(packagePath, self.instInfo.datadir)
                #get path of cm_agent.conf
                path = "%s/cm_agent.conf" % self.instInfo.datadir
                w_List = [""]
                g_file.writeFile(path, w_List)
                #set parameter of cm_agent.conf
                log = "enable_log_compress"
                security_mode = "security_mode"
                para_dict = {log:"on",security_mode:"on"}
                self.setGucConfig(para_dict)
        else:
            if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_CMSERVER):
                cmd += "%s/gs_initcm -Z cm_server -D %s" % (self.binPath, self.instInfo.datadir)
            elif (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_CMAGENT):
                cmd += "%s/gs_initcm -Z cm_agent -D %s" % (self.binPath, self.instInfo.datadir)

            self.logger.debug("Command for initializing instance: %s" % cmd)
            (status, output) = DefaultValue.retryGetstatusoutput(cmd)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51615"] + " Command:%s. Error:\n%s" % (cmd , output))
    
    def uninstall(self):
        """
        function: uninstall the cm component
        input : NA
        output: NA
        """
        datadir = self.instInfo.datadir
        if (os.path.exists(datadir)):
            g_file.removeDirectory(datadir)

    def startCluster(self, user, nodeId = 0, timeout = DEFAULT_TIMEOUT, isSwitchOver = True, isSingle = False, cluster_normal_status = None, isSinglePrimaryMultiStandbyCluster = False, azName = "", datadir=""):
        """
        function:Start cluster or node
        input:String,int,int
        output:NA
        """
        startType = "cluster"
        if (nodeId > 0):
            startType = "node"
        if (datadir != ""):
            startType = "instance"
        if(azName != ""):
            startType = azName
        endTime = datetime.now() + timedelta(seconds=timeout)
        #The output starts the screen-printing information of the group
        self.logger.log("Starting %s." % startType)
        self.logger.log("======================================================================")
        #Call cm_ctl to start the cluster
        cmd = self.getStartCmd(user, nodeId, timeout=0, datadir=datadir, azName=azName)
        (status, output) = commands.getstatusoutput(cmd)
        #The output prompts when the failure to start
        if (status != 0):
            self.logger.error(ErrorCode.GAUSS_516["GAUSS_51607"] % startType + " Error: \n%s" % output)
            self.logger.log("The cluster may continue to start in the background.")
            self.logger.log("If you want to see the cluster status, please try command gs_om -t status.")
            self.logger.log("If you want to stop the cluster, please try command gs_om -t stop.")
            raise Exception(ErrorCode.GAUSS_516["GAUSS_51607"] % startType + " Error: \n%s" % output)

        if(isSingle):
            self.logger.log("Successfully started cluster. Waiting for cluster status to become Normal.")
            self.logger.log("======================================================================")
        elif (isSinglePrimaryMultiStandbyCluster):
            if (azName):
                self.logger.log("Successfully started %s." % startType)
                self.logger.log("======================================================================")
                self.logger.log("End start %s." % startType)
                return True
            else:
                self.logger.log("Successfully started single primary multi standby. Wait for standby instance.")
                self.logger.log("======================================================================")
        #Output the startup instance success information
        elif (nodeId == 0):
            self.logger.log("Successfully started primary instance. Wait for standby instance.")
            self.logger.log("======================================================================")
        else:
            self.logger.log("Successfully started %s." % startType)
            self.logger.log("======================================================================")
            self.logger.log("End start %s." % startType)
            return True

        dotCount = 0
        startStatus = 0
        startResult = ""
        # 1 -> failed
        # 0 -> success
        issuccessed = False
        #Wait for the cluster to start completely
        while True:
            #A point is output every 5 seconds
            time.sleep(5)
            sys.stdout.write(".")
            dotCount += 1
            #A line break is output per minute
            if (dotCount >= 12):
                dotCount = 0
                sys.stdout.write("\n")
    
            startStatus = 0
            startResult = ""
            #The cluster status is checked every 5 seconds
            (startStatus, startResult) = self.doCheckStaus(nodeId, cluster_normal_status)
            if (startStatus == 0):
                #Output successful start information
                if (dotCount != 0): sys.stdout.write("\n")
                self.logger.log("Successfully started %s." % startType)
                issuccessed = True
                break
            #The output prompts when the timeout does not start successfully
            if (endTime is not None and datetime.now() >= endTime):
                if (dotCount != 0): sys.stdout.write("\n")
                self.logger.log("Failed to start %s " % startType + " in (%s)s." % timeout)
                self.logger.log("It will continue to start in the background.")
                self.logger.log("If you want to see the cluster status, please try command gs_om -t status.")
                self.logger.log("If you want to stop the cluster, please try command gs_om -t stop.")
                break
        self.logger.log("======================================================================")
        self.logger.log(startResult)
        if (issuccessed and isSwitchOver and not isSingle):
            #Perform the switch reset operation
            cmd = self.getResetSwitchOverCmd(user, timeout)
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                self.logger.log("Failed to reset switchover the cluster. Command: \"%s\".\nOutput: \"%s\"." % (cmd, output))
        return issuccessed

    def stopCluster(self,  user, nodeId=0, stopMode="", timeout=0, datadir="", azName = ""):
        stopType = "cluster"
        #Specifies the stop node
        #Gets the specified node id
        if (nodeId > 0):
            stopType = "node"
        if (datadir != ""):
            stopType = "instance"
        if (azName != ""):
            stopType = azName
        #Perform a stop operation
        self.logger.log("Stopping %s." % stopType)
        self.logger.log("=========================================")
        timeout = timeout if timeout != 0 else CM_OLAP.DEFAULT_TIMEOUT
        cmd = self.getStopCmd(user, nodeId, stopMode, timeout = 0, datadir = datadir, azName = azName)
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0 and stopType == "cluster"):
            self.logger.log("Failed to stop %s." % stopType + " Try to stop it forcibly." + " Error:\n%s" % output)
            cmd = self.getStopCmd(user, nodeId, "i", timeout, datadir, azName)
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                self.logger.debug(output)
                self.logger.log("Failed to stop %s forcibly." % stopType)
            else:
                self.logger.log("Successfully stopped %s forcibly." % stopType)
        elif(status != 0):
            self.logger.log(output)
            raise Exception(ErrorCode.GAUSS_516["GAUSS_51610"] % stopType)
        else:
            self.logger.log("Successfully stopped %s." % stopType)
    
        self.logger.log("=========================================")
        self.logger.log("End stop %s." % stopType)

    def restartNodeGroup(self, nodegroupName, timeout = 0):
        """
        function: restart node group with logical cluster
        input:
          nodegroupName: string, The name if the nodegroup to restart
        output:
          result: bool, The sign of execution result
        """
        cmd = self.getRestartNodegroup(nodegroupName, timeout)
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_516["GAUSS_51645"] % "logical cluster [%s]" % nodegroupName +
                            "Error:\n%s" % output)
        else:
            return True

    def queryClusterStatus(self, nodeId=0, outFile="", showDetail=True, isFormat=False):
        """
        function: query cluster status
        input : int,string,boolean
        output: cluster status
        """
        
        if (isFormat):
            cmd = self.getQueryStatusCmd(0, outFile, True, True)
        else:
            #query and save status into a file
            statusFile = "%s/gauss_check_status_%d.dat" % (DefaultValue.getTmpDirFromEnv(), os.getpid())
            DefaultValue.cleanTmpFile(statusFile)
            cmd = self.getQueryStatusCmd(0, statusFile)
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0 and not isFormat):
            fp = open(statusFile, 'r')
            output = fp.read()
            fp.close()
            DefaultValue.cleanTmpFile(statusFile)
            if (output.find("cm_ctl: can't connect to cm_server.") >= 0):
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51640"])
            raise Exception(ErrorCode.GAUSS_516["GAUSS_51601"] % "cluster"  + " Error: \n%s" % output)
        if (isFormat):
            return output
        #check cluster status
        clusterStatus = DbClusterStatus()
        clusterStatus.initFromFile(statusFile)
        DefaultValue.cleanTmpFile(statusFile)
        return clusterStatus
    
    def switchOver(self, user, nodeId, datadir):
        """
        function: Switch instances on a node to standby
        input : string, int, string
        output: NA
        """
        cmd = self.getSwitchOverCmd(user, nodeId, datadir)
        self.logger.debug("Switch over command: %s" % cmd)
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_530["GAUSS_53007"] % "instances" +" Error: \n%s" % output)
    
    def querySwitchOver(self, user):
        """
        function:Query switchover
        input: string user
        output:string switchoverInfo
        """
        cmd = self.getQuerySwitchOverCmd(user)
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s" % output)
        return output
    
    def resetSwitchOver(self, user, timeout = DEFAULT_TIMEOUT):
        """
        function:Reset switchover
        input: string user, int timeout
        output: string output
        """
        cmd = self.getResetSwitchOverCmd(user, timeout)
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s" %  output)
        return output
    
    def setMonitor(self, user):
        """"
        function: Set om monitor cron
        input : NA
        output: NA
        """
        self.logger.log("Set CRON.")
        appPath = DefaultValue.getInstallDir(user)
        agent_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../../../../../../om_agent.conf")
        scene = ""
        if os.path.isfile(agent_file):
            fp = ConfigParser.ConfigParser()
            fp.read(agent_file)
            scene = fp.get("PUBLIC_CLOUD_CONFIGURATION", 'scene')

        log_path = DefaultValue.getOMLogPath(DefaultValue.OM_MONITOR_DIR_FILE, user, appPath, "")
        mpprcFile = DefaultValue.getEnv(DefaultValue.MPPRC_FILE_ENV)
        if scene != DefaultValue.PUBLIC_CLOUD:
            cronFile = "%s/gauss_cron_%d" % (DefaultValue.getTmpDirFromEnv(), os.getpid())
            #get all content by crontab command
            (status, output) = g_OSlib.getAllCrontab()
            if(status == 0):
                #overwrit cronFile, make it empty.
                g_file.createFile(cronFile, True)
                content_CronFile = [output]
                if (output != ""):
                    g_file.writeFile(cronFile, content_CronFile)
                    g_file.deleteLine(cronFile,"\\/bin\\/om_monitor")
            elif(status != 256):#status==256 means this user has no cron
                raise Exception(ErrorCode.GAUSS_508["GAUSS_50803"] + " Error: \n%s" % output)

            if (mpprcFile != "" and mpprcFile != None):
                cronContent = "*/1 * * * * source /etc/profile;(if [ -f ~/.profile ];then source ~/.profile;fi);source ~/.bashrc;source %s;nohup %s/bin/om_monitor -L %s >>/dev/null 2>&1 &" % (
                    mpprcFile, appPath, log_path)
                content_CronFile = [cronContent]
            else:
                cronContent = "*/1 * * * * source /etc/profile;(if [ -f ~/.profile ];then source ~/.profile;fi);source ~/.bashrc;nohup %s/bin/om_monitor -L %s >>/dev/null 2>&1 &" % (
                    appPath, log_path)
                content_CronFile = [cronContent]

            # Set cron for clear corefiles
            gphome = DefaultValue.getEnvironmentParameterValue("GPHOME", "")
            defaultXML = "%s/cluster_default_agent.xml" % gphome
            if os.path.exists(defaultXML):
                corefile = dbClusterInfo.readClustercorePath(defaultXML)
                if not os.path.exists(corefile):
                    cronContentCoreclear = "* */1 * * * diskUse=$(df %s -h | grep -v 'Use' | awk '{print $5}');fileNum=$(expr $(ls %s | wc -l) - 5);" % (corefile, corefile)
                    cronContentCoreclear += "(if [ ${diskUse%%%%%%} -gt %s -a $fileNum -gt 5 ];then ls -tr | head -$fileNum | xargs -i -n$fileNum rm -rf {};fi) >>/dev/null 2>&1 &" % DefaultValue.CORE_PATH_DISK_THRESHOLD
                    content_CronFile.append(cronContentCoreclear)
            g_file.writeFile(cronFile, content_CronFile)
            g_OSlib.execCrontab(cronFile)
            g_file.removeFile(cronFile)

        if(mpprcFile != "" and mpprcFile != None):
            root_path = "/etc/profile"
            g_OSlib.source(root_path)
            if_Path = "~/.profile"
            if (os.path.exists(if_Path)):
                g_OSlib.source(if_Path)
            user_Path = "~/.bashrc"
            g_OSlib.source(user_Path)
            g_OSlib.source(mpprcFile)
        else:
            root_path = "/etc/profile"
            g_OSlib.source(root_path)
            if_Path = "~/.profile"
            if (os.path.exists(if_Path)):
                g_OSlib.source(if_Path)
            user_Path = "~/.bashrc"
            g_OSlib.source(user_Path)

        if scene != DefaultValue.PUBLIC_CLOUD:
            cmd = "nohup %s/bin/om_monitor -L %s >>/dev/null 2>&1 &" % (appPath, log_path)
            self.logger.debug("Command for start om_monitor: %s" % cmd)
            status = os.system(cmd)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51607"] % "om_monitor")
        else:
            if not DefaultValue.checkProcess("om_monitor"):
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51607"] % "om_monitor")

    def cleanMonitor(self):
        """
        function: clean om_monitor process and delete cron
        input : NA
        output: NA
        """
        self.logger.log("Deleting monitor.")
        try:
            # Remove cron
            crontabFile = "%s/gauss_crontab_file_%d" % (DefaultValue.getTmpDirFromEnv(), os.getpid())
            #get all content by crontab command
            (status, output) = g_OSlib.getAllCrontab()
            #overwrit crontabFile, make it empty.
            g_file.createFile(crontabFile, True)
            content_CronTabFile = [output]
            g_file.writeFile(crontabFile, content_CronTabFile)
            g_file.deleteLine(crontabFile, "\\/bin\\/om_monitor")
            g_OSlib.execCrontab(crontabFile)
            g_file.removeFile(crontabFile)

            # clean om_monitor,cm_agent,cm_server process
            for progname in ["om_monitor", "cm_agent", "cm_server"]:
                g_OSlib.killallProcess(self.user, progname)

        except Exception, e:
            raise Exception(str(e))
        self.logger.log("Successfully deleted OMMonitor.")
    
    def setGucConfig(self, paraDict={}, setMode='set'):
        """
        function: set cm guc config file (cm_server.conf/cm_agent.conf)
        input : NA
        output: NA
        """
        userProfile = DefaultValue.getMpprcFile()
        g_OSlib.source(userProfile)
        if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_CMSERVER):
            config_file = os.path.join(self.instInfo.datadir, "cm_server.conf")
        else:
            config_file = os.path.join(self.instInfo.datadir, "cm_agent.conf")

        for key in paraDict.keys():
            value = paraDict[key].replace("/", "\/").replace("'", "\\'")
            cmd = "sed -i 's/\\t/    /g' %s && \
                   if [ 0 -eq `grep '%s' %s | grep -v '^[ ]*#' | wc -l` ]; \
                   then \
                       echo \"%s = %s\" >> %s; \
                   else \
                     if [ 0 -ne `grep '%s' %s | grep '#' | wc -l` ]; then \
                       sed -i \"s/^[ \\t]*%s.*=.*\\([ ]*#.*\\)/%s = %s \\1/g\" %s; \
                     else \
                       sed -i \"s/^[ \\t]*%s.*=.*\\([ ]*\\)/%s = %s \\1/g\" %s; \
                         fi \
                   fi" % \
                (config_file,
                 key, config_file,
                 key, paraDict[key], config_file,
                 key, config_file,
                 key, key, value,config_file,
                 key, key, value, config_file)
            self.logger.debug("Command for setting cm parameter: %s" % cmd)
            (status, output) = DefaultValue.retryGetstatusoutput(cmd)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_500["GAUSS_50007"] % "GUC" + " Error: \n%s" % str(output))
        
    def getGucConfig(self, paraList):
        """
        function: get cm guc parameter value from file (cm_server.conf/cm_agent.conf)
        input : NA
        output: NA
        """
        if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_CMSERVER):
            gucFile = "%s/cm_server.conf" % self.instInfo.datadir
        else:
            gucFile = "%s/cm_agent.conf" % self.instInfo.datadir
        paraDict = {}
        for paraName in paraList:
            (status, output) = g_OSlib.getGrepValue("-E", "^[ \\t]*%s ", gucFile)
            outputList = output.strip().split("\n")
            for line in outputList:
                if (line.find("=") < 0):
                    continue
                valueLine = line.split("=")[1]
                paraValue = valueLine.split("#")[0].strip()
                paraDict[paraName] = paraValue
        return paraDict

    def getCMSDict(self, user, configItemType = None, alarm_component = None):
        """
        function: Get CMServer configuration
        input : NA
        output: NA
        """
        tmpCMSDict = {}
        tmpCMSDict["log_dir"] = "%s/cm/cm_server" % DefaultValue.getUserLogDirWithUser(user)
        if (configItemType == "ConfigInstance"):
            tmpCMSDict["alarm_component"] = "%s" % alarm_component
        return tmpCMSDict

    def getAgentDict(self, user, configItemType = None, alarm_component = None):
        """
        function: Get CMAgent configuration
        input : NA
        output: NA
        """ 
        tmpAgentDict = {}
        tmpAgentDict["log_dir"] = "%s/cm/cm_agent" % DefaultValue.getUserLogDirWithUser(user)
        if (configItemType == "ConfigInstance"):
            tmpAgentDict["alarm_component"] = "%s" % alarm_component
            tmpAgentDict["unix_socket_directory"] = "'%s'" % DefaultValue.getTmpDirFromEnv()
        return tmpAgentDict

    def configInstance(self, user, configItemType = None, alarm_component = None, cmsConfig=None):
        """
        function: Get CMAgent configuration
        input : user, configItemType, alarm_component
        output: NA
        """
        if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_CMSERVER):
            tmpDict = self.getCMSDict(user, configItemType, alarm_component)
            tmpDict.update(cmsConfig)
        else:
            tmpDict = self.getAgentDict(user, configItemType, alarm_component)
        self.setGucConfig(tmpDict)

    def doCheckStaus(self, nodeId, cluster_normal_status = None, expected_redistributing = ""):
        """
        function: Check cluster status
        input : user, nodeId, cluster_normal_status, expected_redistributing
        output: status, output
        """
        statusFile = "%s/gauss_check_status_%d.dat" % (DefaultValue.getTmpDirFromEnv(), os.getpid())
        DefaultValue.cleanTmpFile(statusFile)
        cmd = self.getQueryStatusCmd(0, statusFile, showDetail=True, isFormat=False)
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0):
            DefaultValue.cleanTmpFile(statusFile)
            return (status, output)
        
        clusterStatus = DbClusterStatus()
        clusterStatus.initFromFile(statusFile)
        DefaultValue.cleanTmpFile(statusFile)
        
        status = 0
        output = ""
        statusRep = None
        if (nodeId > 0):
            nodeStatus = clusterStatus.getDbNodeStatusById(nodeId)
            if (nodeStatus is None):
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51619"] % nodeId)
            
            status = 0 if nodeStatus.isNodeHealthy() else 1
            statusRep = nodeStatus.getNodeStatusReport()
        else:
            status = 0 if clusterStatus.isAllHealthy(cluster_normal_status) \
                and (clusterStatus.redistributing == expected_redistributing or expected_redistributing == "") else 1
            statusRep = clusterStatus.getClusterStatusReport()
            output += "cluster_state      : %s\n" % clusterStatus.clusterStatus
            output += "redistributing     : %s\n" % clusterStatus.redistributing
            output += "node_count         : %d\n" % statusRep.nodeCount

        output += "Coordinator State\n"
        output += "    normal         : %d\n" % statusRep.cooNormal
        output += "    abnormal       : %d\n" % statusRep.cooAbnormal
        output += "GTM State\n"
        output += "    primary        : %d\n" % statusRep.gtmPrimary
        output += "    standby        : %d\n" % statusRep.gtmStandby
        output += "    abnormal       : %d\n" % statusRep.gtmAbnormal
        output += "    down           : %d\n" % statusRep.gtmDown
        output += "Datanode State\n"
        output += "    primary        : %d\n" % statusRep.dnPrimary
        output += "    standby        : %d\n" % statusRep.dnStandby
        output += "    secondary      : %d\n" % statusRep.dnDummy
        output += "    building       : %d\n" % statusRep.dnBuild
        output += "    abnormal       : %d\n" % statusRep.dnAbnormal
        output += "    down           : %d\n" % statusRep.dnDown
        
        return (status, output)
    
    def getStartCmd(self, user, nodeId=0, timeout = DEFAULT_TIMEOUT, datadir = "", azName = ""):
        """
        function : Start all cluster or a node
        input : String, int, String, String
        output : String
        """
        userProfile = DefaultValue.getMpprcFile()
        cmd = "source %s ; cm_ctl start" % userProfile
        # check node id
        if (nodeId > 0):
            cmd += " -n %d" % nodeId
        # check data directory
        if (datadir != ""):
            cmd += " -D %s" % datadir
        # check timeout
        if (timeout > 0):
            cmd += " -t %d" % timeout
        # azName
        if (azName != ""):
            cmd += " -z%s" % azName

        return cmd

    def getStopCmd(self, user, nodeId=0, stopMode="", timeout = DEFAULT_TIMEOUT, datadir="", azName = ""):
        """
        function : Stop all cluster or a node
        iinput : String,int,String,String
        output : String
        """
        userProfile = DefaultValue.getMpprcFile()
        cmd = "source %s ; cm_ctl stop" % userProfile
        # check node id
        if (nodeId > 0):
            cmd += " -n %d" % nodeId
        # check data directory
        if (datadir != ""):
            cmd += " -D %s" % datadir
        # check stop mode
        if (stopMode != ""):
            cmd += " -m %s" % stopMode
        # check timeout
        if (timeout > 0):
            cmd += " -t %d" % timeout
        # azName
        if (azName != ""):
            cmd += " -z%s" % azName

        return cmd

    def getRestartNodegroup(self, nodegroupName, timeout = DEFAULT_RESTART_NODEGROUP_TIMEOUT):
        """
        function: get the command of restart node group by group name
        input:
          nodegroupName: string, The name of the nodegroup to restart
        output:
          cmd: string, The command to restart
        """
        userProfile = DefaultValue.getMpprcFile()
        cmd = "source %s ; cm_ctl restart -L %s" % (userProfile, nodegroupName)
        return cmd

    @staticmethod
    def getSwitchOverCmd(user, nodeId, datadir):
        """
        function : Get the command of switching over standby instance
        input : String,int,String
        output : String
        """
        userProfile = DefaultValue.getMpprcFile()
        cmd = "source %s ; cm_ctl switchover -n %d -D %s" % (userProfile, nodeId, datadir)
        # build shell command
        if (user != "") and (os.getuid() == 0):
            cmd = "su - %s -c 'source %s;%s'" % (user, userProfile, cmd)

        return cmd

    @staticmethod
    def getQuerySwitchOverCmd(user):
        """
        function : Query Switch over
        input : String
        output : String
        """
        userProfile = DefaultValue.getMpprcFile()
        cmd = "source %s ; cm_ctl query -v -C -s -i -d" % userProfile
        # build shell command
        if (user != "") and (os.getuid() == 0):
            cmd = "su - %s -c 'source %s;%s'" % (user, userProfile, cmd)

        return cmd

    @staticmethod
    def getResetSwitchOverCmd(user, timeout):
        """
        function : Reset Switch over
        input : String,String
        output : String
        """
        userProfile = DefaultValue.getMpprcFile()
        cmd = "source %s ; cm_ctl switchover -a -t %d" % (userProfile, timeout)
        # build shell command
        if (user != "") and (os.getuid() == 0):
            cmd = "su - %s -c 'source %s;%s'" % (user, userProfile, cmd)

        return cmd
    
    @staticmethod
    def getQueryStatusCmd(nodeId=0, outFile="", showDetail=True, isFormat=True):
        """
        function : Get the command of querying status of cluster or node
        input : String
        output : String
        """
        userProfile = DefaultValue.getMpprcFile()
        cmd = "source %s ; cm_ctl query" % userProfile
        # check node id
        if (nodeId > 0):
            cmd += " -n %d" % nodeId
        # check -v
        if (showDetail):
            cmd += " -v" 
        # status format
        if (isFormat):
            cmd += " -C -i -d"
        # check out put file
        if (outFile != ""):
            cmd += " > %s" % outFile

        return cmd
