#!/usr/bin/env python
#-*- coding:utf-8 -*-
try:
    import sys
    import os
    import commands
    import time
    sys.path.append(sys.path[0] + "/../../../../../lib/netifaces/")
    import netifaces

    sys.path.append(sys.path[0] + "/../../../../")
    from gspylib.common.ErrorCode import ErrorCode
    from gspylib.component.Kernal.Kernal import Kernel
    from gspylib.os.gsplatform import g_Platform
    from gspylib.os.gsfile import g_file
    from gspylib.os.gsOSlib import g_OSlib
    from gspylib.common.Common import DefaultValue, ClusterInstanceConfig
    from gspylib.common.DbClusterInfo import dbClusterInfo
    from gspylib.threads.parallelTool import parallelTool

except ImportError, e:
    sys.exit("[GAUSS-52200] : Unable to import module: %s." % str(e))

METHOD_TRUST = "trust"
METHOD_SHA = "sha256"
MAX_PARA_NUMBER = 1000
RETRY_COUNT = 3
 
class CN_OLAP(Kernel):
    '''
    The class is used to define base component.
    '''
    def __init__(self):
        '''
        Constructor
        '''
        super(CN_OLAP, self).__init__()

    def getCnGUCDict(self):
        """
        function : get init CN install guc parameter
        input : String,String,String,int
        output : String
        """
        tmpDict = {}
        if(self.dwsMode == True):
            tmpDict["pgxc_node_name"] = "'cn_%s'" % self.instInfo.instanceId
        tmpDict["ssl"] = "on"
        tmpDict["ssl_cert_file"] = "'server.crt'"
        tmpDict["ssl_key_file"] = "'server.key'"
        tmpDict["ssl_ca_file"] = "'cacert.pem'"
        return tmpDict
    
    def initInstance(self):
        """
        function: 
            init cn instance
        input:string:dbInitParams
        output: 
        """
        if(not os.path.exists(self.instInfo.datadir)):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50201"] % ("data directory [%s]" % self.instInfo.datadir))

        # [scene] : get pwd by key_path in public_cloud:huaweiyun
        user_name = ""
        for paras in self.initParas:
            if paras.find("-U") != -1:
                user_name = paras.split(" ")[-1].strip()
                break
        key_path = os.path.join(DefaultValue.PUBLIC_CLOUD_TOKEN_FILE, user_name)
        if not os.path.isdir(key_path):
            key_path = self.binPath

        if(self.dwsMode):
            image_path = DefaultValue.DWS_IMAGE_PATH
            # decompress package to files
            packageName = "%s/coordinator.tar.gz" % image_path
            g_file.decompressFiles(packageName, self.instInfo.datadir)
            
        else:
            cmd = "%s/gs_initdb --locale=C -D %s --nodename=cn_%d -H %s -C %s %s" % (self.binPath,
                                                                                     self.instInfo.datadir,
                                                                                     self.instInfo.instanceId,
                                                                                     self.instInfo.listenIps[0],
                                                                                     key_path,
                                                                                     " ".join(self.initParas))
            self.logger.debug("Command for initializing CN instance: %s" % DefaultValue.mock_password(cmd))
            (status, output) = DefaultValue.retryGetstatusoutput(cmd)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51615"] + " Command:%s. Error:\n%s" % (
                    DefaultValue.mock_password(cmd), output))
        # set GUC parameter
        cnGucParas = self.getCnGUCDict()
        self.setGucConfig(cnGucParas)
        self.copyAndModCertFiles()
    
    def getInstanceNodeName(self):
        """
        function: Get Instance Node Name
        input : NA
        output: instance node name
        """
        return "cn_%d" % self.instInfo.instanceId

    """
    Desc: 
        do something for GUC 
    """
    def setPghbaConfig(self, clusterAllIpList):
        """
        """
        principal = None
        if (DefaultValue.checkKerberos(DefaultValue.getMpprcFile())):
            (status, output) = g_OSlib.getGrepValue("-Er", "^default_realm", os.path.join(os.path.dirname(DefaultValue.getMpprcFile()), DefaultValue.FI_KRB_CONF))
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50222"] % "krb5.conf" + "Error:\n%s" % output)
            principal = output.split("=")[1].strip()

        # build ip string list
        # Every 1000 records merged into one
        i = 0
        GUCParasStr = ""
        GUCParasStrList = []
        for ipAddress in clusterAllIpList:
            i += 1
            # Set the initial user and initial database access permissions 
            if (principal == None):
                GUCParasStr += "-h \"host    all    all    %s/32    %s\" " % (ipAddress, METHOD_TRUST)
            else:
                GUCParasStr += "-h \"host    all    all    %s/32    gss    include_realm=1    krb_realm=%s\" " % (ipAddress, principal)
            if (i % MAX_PARA_NUMBER == 0):
                GUCParasStrList.append(GUCParasStr)
                i = 0
                GUCParasStr = ""

        if (self.dwsMode == True):
            GUCParasStr += "-h \"host    all    all    0.0.0.0/0 %s\" " % (METHOD_SHA)
            
        if (GUCParasStr != ""):
            GUCParasStrList.append(GUCParasStr)

        for parasStr in GUCParasStrList:
            self.doGUCConfig("set", parasStr, True)

    def setSecretFree(self, mode):
        """
        function: Secret-free settings
        input : NA
        output: NA
        """
        #Secret-free settings 
        if (self.dwsMode == True):
            self.logger.debug("Begin to set secret free.")
            GUCParasStr = " -h \"host    all    all    %s\"" % (mode)
            self.doGUCConfig("set", GUCParasStr, False)
            self.logger.debug("Set secret free succeeded.")
    """
    Desc: 
        GUC parameter
    """        
    def getCNDict(self, user, configItemType = None):
        """
        function: Get CN configuration
        input : NA
        output: NA
        """
        tmpCNDict = {}
        tmpCNDict["listen_addresses"] = "'localhost,%s'" % ",".join(self.instInfo.listenIps)
        tmpCNDict["local_bind_address"] =  "'%s'" % self.instInfo.listenIps[0]
        tmpCNDict["port"] = str(self.instInfo.port)
        tmpCNDict["pooler_port"] = str(self.instInfo.port+1)

        if (configItemType == "ConfigInstance"):
            tmpCNDict["cstore_buffers"] = "32MB"
            tmpCNDict["max_connections"] = "200"
            tmpCNDict["shared_buffers"] = "512MB"
            tmpCNDict["work_mem"] = "64MB"
            tmpCNDict["maintenance_work_mem"] = "128MB"
            tmpCNDict["data_replicate_buffer_size"] = "128MB"
        if(self.clusterType == DefaultValue.CLUSTER_TYPE_SINGLE_PRIMARY_MULTI_STANDBY):
            tmpCNDict["enable_data_replicate"] = "off"
            tmpCNDict["replication_type"] = "1"
        if(self.clusterType == DefaultValue.CLUSTER_TYPE_SINGLE):
            tmpCNDict["replication_type"] = "2"
            
        if (configItemType != "ChangeIPUtility"):
            tmpCNDict["pooler_port"] = str(self.instInfo.haPort)
            tmpCNDict["log_directory"] = "'%s/pg_log/cn_%d'" % (DefaultValue.getUserLogDirWithUser(user), self.instInfo.instanceId)
            tmpCNDict["audit_directory"] = "'%s/pg_audit/cn_%d'" % (DefaultValue.getUserLogDirWithUser(user), self.instInfo.instanceId)

        if (configItemType != "ConfigInstance" and configItemType != "ChangeIPUtility"):    
            tmpCNDict["pgxc_node_name"] = "'cn_%d'" % self.instInfo.instanceId

        if (len(self.instInfo.ssdDir) != 0 and configItemType != "ChangeIPUtility"):
            tmpCNDict["ssd_cache_dir"] = "'%s'" % (self.instInfo.ssdDir)
            tmpCNDict["enable_adio_function"] = "on"
            tmpCNDict["enable_cstore_ssd_cache"] = "on"

        return tmpCNDict

    def configInstance(self, user, cooConfig, gtmList, configItemType = None, alarm_component = None, gucXml = (False,"")):
        """
        """
        commonDict = self.setCommonItems()
        self.setGucConfig(commonDict)

        gtmDict = ClusterInstanceConfig.setGtmInfo(gtmList)
        self.setGucConfig(gtmDict)
        tmpCNDict = self.getCNDict(user, configItemType)

        self.logger.debug("Check if tmp_guc file exists.")
        tmpGucFile = ""
        tmpGucPath = DefaultValue.getTmpDirFromEnv(user)
        tmpGucFile = "%s/tmp_guc" % tmpGucPath
        if (os.path.exists(tmpGucFile)):
            self.logger.debug("Check if dict is not null.")
            dynamicDict = {}
            dynamicDict = DefaultValue.dynamicGuc(user, self.logger, "cn", tmpGucFile, gucXml)
            if gucXml[0]:
                dynamicDict["thread_pool_attr"] = "'%s'" % dynamicDict["thread_pool_attr"]
                dynamicDict["log_line_prefix"] = "'%s'" % dynamicDict["log_line_prefix"]
            if (len(dynamicDict) != 0):
                tmpCNDict.update(dynamicDict)
            else:
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50219"] % "guc_list.conf")
        tmpCNDict.update(cooConfig)
        tmpCNDict["alarm_component"] = "'%s'" % alarm_component
        self.setGucConfig(tmpCNDict)

    """
    Desc: 
        execute SQL by execSQLCommand
    """            
    def getGucConfig(self, paraList, user, port, database="postgres", host="", option=""):
        """
        """
        dictList = {}
        parasStr = ""
        for para in paraList:
            parasStr += "\'%s\'," % para
        if (parasStr == ""):
            return
        parasStr = parasStr[:-1]
        sqlQuery = "SELECT name,setting FROM pg_catalog.pg_settings WHERE name in (%s)" % parasStr
        (status, output) = self.execSQLCommand(sqlQuery, user, host, port, database, option)
        if (0 != status or "" == output):
            raise Exception(str(output))
        for line in output.split('\n'):
            key = line.split('|')[0]
            value = line.split('|')[1]
            if (value.isdigit()):
                value = int(value)
            dictList[key] = value
        return dictList

    def createNewNode(self, nodeName, instType):
        """
        function: Create new datanode node info
        input : nodeName, instType(C/D)
        output: NA
        """
        self.logger.log("Creating new instance.")
        try:
            createSql = "set application_name='OM';CREATE NODE %s WITH (type='%s', host='%s', port=%s);" % (nodeName, instType, self.instInfo.listenIps[0], self.instInfo.port)
            createSql += "SELECT pgxc_pool_reload();"   
            self.logger.debug("Command for creating new DN node: %s" % createSql)
            self.setSecretFree(METHOD_TRUST)
            (status, output) = self.execSQLCommand(createSql, self.user, "", self.instInfo.port)
            self.setSecretFree(METHOD_SHA)
            self.logger.debug("Create new node result: %s." % output)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % createSql + " Error:\n%s" % output)
        except Exception, e:
            raise Exception(str(e))

    def dropNode(self, nodeName):
        """
        function: Drop record from pgxc_node of dn
        input : inst, nodename
        output: NA
        """
        self.logger.log("Dropping XC node.")
        try:
            dropSql = ""
            selectSql = ""

            # make sure nodename exists
            selectSql = "SELECT count(*) FROM pg_catalog.pgxc_node WHERE node_name = '%s'" % nodeName
            self.logger.debug("Check record from pgxc_node command: %s" % selectSql)
            self.setSecretFree(METHOD_TRUST)
            (status, output) = self.execSQLCommand(selectSql, self.user, "", self.instInfo.port)
            self.logger.debug("Create new node result: %s." % output)
            if (status != 0):
                self.setSecretFree(METHOD_SHA)
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % selectSql + " Error:\n%s" % output)

            # make drop SQL
            if (int(output.strip()) == 1):
                dropSql += "set application_name='OM';DROP NODE IF EXISTS %s with (%s);" % (nodeName, "cn_%s" % self.instInfo.instanceId)
            if (dropSql == ""):
                self.logger.log("There is no node needs to been dropped.")
                return
            dropSql += "SELECT pgxc_pool_reload();"

            self.logger.debug("Drop record from pgxc_node command: %s." % dropSql)
            (status, output) = self.execSQLCommand(dropSql, self.user, "", self.instInfo.port)
            self.setSecretFree(METHOD_SHA)
            self.logger.debug("Drop record from pgxc_node result: %s." % output)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % dropSql + " Error:\n%s" % output)
        except Exception, e:
            raise Exception(str(e))
        
        self.logger.log("Successfully dropped XC node.")

    def checkNodegroupSet(self, user):
        """
        function: Check if the "default_storage_nodegroup" parameter value is the same as the nodegroup of in_redistribution='n'
        output: if True, they are same, else not same.
        """
        self.logger.debug("Checking if the nodegroup parameter settings to meet the requirements.")
        try:
            # get default_storage_nodegroup value
            dictList = {}
            paraList = ['default_storage_nodegroup']
            dictList = self.getGucConfig(paraList, user, self.instInfo.port)
        
            # get the nodegroup that is in_redistribution='n' and is_installation = TRUE.
            obtainOldGroupNameSql = "SELECT group_name FROM pg_catalog.pgxc_group WHERE in_redistribution='y' and is_installation = FALSE;"
            self.logger.debug("Command for getting old node group: %s" % obtainOldGroupNameSql)
            (status, output) = self.execSQLCommand(obtainOldGroupNameSql, user, "", self.instInfo.port)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % obtainOldGroupNameSql + " Error:\n%s" % output)

            #Determine whether the two values are equal. Only two values are not equal, in order to make redistribution.
            if(dictList['default_storage_nodegroup'].strip() == output.strip()):
                return True
            else:
                return False
        except Exception, e:
            raise Exception(str(e))
  
        self.logger.debug("Successfully checked if the nodegroup parameter settings to meet the requirements.")
            
    def createNewNodeGroup(self, user, group_name, DNNameStr, CNNameStr, isResize = False):
        """
        function: Create new node group
        input : NA
        output: NA
        """
        try:
            OBTAIN_OLD_GROUP_SQL = "SELECT group_name FROM pg_catalog.pgxc_group WHERE in_redistribution='n' and is_installation = TRUE;"
            (status, output) = self.execSQLCommand(OBTAIN_OLD_GROUP_SQL, user, "", self.instInfo.port)
            if(status != 0 or output.strip() == ""):
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50219"] % "old group" + " Error:\n%s" % str(output))
            old_group_name  = output.strip()

            CREATE_GROUP_SQL = "CREATE NODE GROUP \"%s\" WITH(%s); "
            if(isResize):
		START_TRANSACTION_SQL = "SET xc_maintenance_mode = on; START TRANSACTION; "
                UPDATE_SQL = "EXECUTE DIRECT ON(%s) 'UPDATE pg_catalog.pgxc_group SET in_redistribution = ''y'' WHERE group_name = ''%s'''; \
EXECUTE DIRECT ON(%s) 'UPDATE pg_catalog.pgxc_group SET in_redistribution = ''t'' WHERE group_name = ''%s'''; \
EXECUTE DIRECT ON(%s) 'update pgxc_group set group_acl= concat(''{'', concat(current_user,''=UCp/'', current_user, '',=UCp/'',current_user), ''}'')::aclitem[] WHERE group_name = ''%s'''; "
            else:
		START_TRANSACTION_SQL = "SET xc_maintenance_mode = on; SET enable_cluster_resize = on; START TRANSACTION; "
                UPDATE_SQL = "EXECUTE DIRECT ON(%s) 'UPDATE pg_catalog.pgxc_group SET is_installation = FALSE , in_redistribution = ''y'',group_kind=''n'' WHERE group_name = ''%s'''; \
EXECUTE DIRECT ON(%s) 'UPDATE pg_catalog.pgxc_group SET is_installation = TRUE , in_redistribution = ''n'',group_kind=''i'' WHERE group_name = ''%s'''; \
EXECUTE DIRECT ON(%s) 'update pgxc_group set group_acl= concat(''{'', concat(current_user,''=UCp/'', current_user, '',=UCp/'',current_user), ''}'')::aclitem[] WHERE group_name = ''%s'''; "
            END_TRANSACTION_SQL = "COMMIT; RESET xc_maintenance_mode; "

            create_sql = CREATE_GROUP_SQL % (group_name, DNNameStr)
            update_sql = ""
            for cnname in CNNameStr.split(','):
                update_sql += UPDATE_SQL % (cnname, old_group_name, cnname, group_name, cnname, group_name)
            sql = "%s%s%s%s" % (START_TRANSACTION_SQL, create_sql, update_sql, END_TRANSACTION_SQL)
            self.logger.debug("Sql cmmand for create pgxc_group: %s" % sql)
            #create new group
            (status, output) = self.execSQLCommand(sql, user, "", self.instInfo.port)
            if(status != 0):
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % sql + " Error:\n%s" % str(output))
        except Exception, e:
            raise Exception(str(e))
        self.logger.log("Successfully created new node group.")

    def deleteOldNodeGroup(self, user):
        """
        function: Drop old node group
        input : NA
        output: NA
        """
        START_TRANSACTION_SQL = "SET xc_maintenance_mode = on; START TRANSACTION; "
        DELETE_GROUP_SQL = "DROP NODE GROUP %s;"
        END_TRANSACTION_SQL = "COMMIT; RESET xc_maintenance_mode; "

        self.logger.log("Deleting old node group.")

        obtainOldGroupNameSql = "SELECT group_name FROM pg_catalog.pgxc_group WHERE in_redistribution='y' and is_installation = FALSE;"
        self.logger.debug("Command for getting old node group: %s" % obtainOldGroupNameSql)
        (status, output) = self.execSQLCommand(obtainOldGroupNameSql, user, "", self.instInfo.port)
        if(status != 0):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50219"] % "old group" + " Error:\n%s" % str(output))
        if(output.strip() == ""):
            self.logger.debug("The nodegroup to delete is empty. The current operation may be performing re-entry.")
            return
        oldGroupName  = output.strip()

        #sql command: drop old node group
        deleteSql = DELETE_GROUP_SQL % oldGroupName

        executeSql = "%s%s%s" % (START_TRANSACTION_SQL, deleteSql, END_TRANSACTION_SQL)
        self.logger.debug("Sql command for deleting old node group: %s" % executeSql)
        #delete old node group
        (status, output) = self.execSQLCommand(executeSql, user, "", self.instInfo.port)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % executeSql + " Error: \n%s" % str(output))

        self.logger.log("Successfully deleted old node group.")

    def getIsAlterUserValue(self):
        """
        isAlterUser, The value isAlterUser of True indicates that the password must be updated when performing an increase in CN, gs_expand and gs_replace
        """
        return True
        
    def dumpCooSchema(self):
        """
        function: Dump schema from coordinator
        input : NA
        output: NA
        """
        self.logger.log("Dumping CN schema.")

        tmpsh = "tmp.sh"
        fp = None
        try:
            # get tmp dir 
            tmpDir = DefaultValue.getTmpDirFromEnv()

            # before do gs_dumpall, clean the dump file first no matter it exist or not.
            g_file.removeFile("'%s'*" % self.schemaCoordinatorFile[:-4], "shell")

            # only need the objects define, ignore the data.
            # using all global paremeter and --parallel-jobs
            dumpallcmd = self.getDUMPALLCommand()
            options = "--include-nodes --dump-nodes --include-templatedb --include-buckets --include-alter-table --dump-wrm --non-lock-table"
            cmd = "%s -p %d -s %s --file='%s' --parallel-jobs %s " % (dumpallcmd, self.instInfo.port, options, self.schemaCoordinatorFile, parallelTool.getCpuCount())

            # write command into tmpsh file
            self.logger.debug("Command for dumpping CN: %s" % cmd)
            currentTime = time.strftime("%Y-%m-%d_%H_%M_%S")
            pid = os.getpid()
            shfile = "dumpcn_%s_%s.sh" % (pid, currentTime)
            tmpsh = os.path.join(tmpDir, shfile)
            fp = open(tmpsh, "w")
            fp.write(cmd)
            fp.flush()
            fp.close()
            g_file.changeMode("shell", DefaultValue.KEY_FILE_MODE, tmpsh)

            sshcmd = "%s %s" % (g_Platform.getShellCmd(), tmpsh)
            (status, output) = commands.getstatusoutput(sshcmd)
            if((len(tmpsh) > 0) and os.path.isfile(tmpsh)):
                os.remove(tmpsh)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51616"] % "CN" + " Error:\n%s" % output)

            # add alter node SQL command into dump file
            try:
                context = ["set application_name='OM';ALTER NODE cn_%d WITH (HOST = '%s', HOST1 = '%s');" % (self.instInfo.instanceId, self.instInfo.listenIps[0], self.instInfo.listenIps[0])]
                g_file.writeFile(self.schemaCoordinatorFile, context)
            except Exception, e:
                g_file.removeFile("'%s'*" % self.schemaCoordinatorFile[:-4], "shell")
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50223"] % self.schemaCoordinatorFile + " Command:%s. Error:\n%s" % (cmd, output))

            # SQL Data cleaning 
            self.modifySchemaFile("coordinator", self.schemaCoordinatorFile, self.getIsAlterUserValue())

            # fix the dump file
            # every 1000 records, submitted once
            self.fixDumpFile(self.schemaCoordinatorFile, self.instInfo.port)

            # dump data for pg_job and pg_job_proc
            g_file.removeFile("'%s'" % self.coordinatorJobDataFile, "shell")
            dumpcmd = self.getDUMPCommand()
            options = "-t pg_catalog.pg_job -t pg_catalog.pg_job_proc"
            cmd = "%s -p %d %s -a --file='%s' %s" % (dumpcmd, self.instInfo.port, DefaultValue.DEFAULT_DB_NAME, self.coordinatorJobDataFile, options)

            self.logger.debug("Command for dumpping CN Job data: %s" % cmd)
            currentTime = time.strftime("%Y-%m-%d_%H_%M_%S")
            pid = os.getpid()
            shfile = "dumpcn_%s_%s.sh" % (pid, currentTime)
            tmpsh = os.path.join(tmpDir, shfile)
            g_file.createFileInSafeMode(tmpsh)
            fp = open(tmpsh, "w")
            fp.write(cmd)
            fp.flush()
            fp.close()
            g_file.changeMode("shell", DefaultValue.KEY_FILE_MODE, tmpsh)

            sshcmd = "%s %s" % (g_Platform.getShellCmd(), tmpsh)
            (status, output) = commands.getstatusoutput(sshcmd)
            if (status != 0 and output.find("No matching tables were found") < 0):
                g_file.removeFile("'%s'*" % self.schemaCoordinatorFile[:-4], "shell")
                g_file.removeFile("'%s'" % self.coordinatorJobDataFile, "shell")
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51616"] % "CN" + " Error:\n%s" % output)

            #dump statistics data
            self.dumpStatistics(self.instInfo.port)

            if((len(tmpsh) > 0) and os.path.isfile(tmpsh)):
                os.remove(tmpsh)
        except Exception, e:
            if(fp):
                fp.close()
            if((len(tmpsh) > 0) and os.path.isfile(tmpsh)):
                os.remove(tmpsh)
                raise Exception(str(e))

        self.logger.log("Successfully dumpped CN schema.")
        
    def getDumpTableFiles(self):
        """
        function: get default group tables info files
        input : NA
        output: NA
        """
        #delete files before create files
        g_file.removeFile("'%s'*" % self.dumpTableFile[:-4], "shell")

        # get the table info from node group under every database
        databases = self.getAllDatabase()
        selectSQL = "SELECT nspname, relname FROM pg_class, pgxc_class, pg_namespace, pgxc_group "\
                    "WHERE pg_class.oid = pgxc_class.pcrelid "\
                    "AND pg_class.relnamespace = pg_namespace.oid "\
                    "AND pgxc_class.pgroup = pgxc_group.group_name "\
                    "AND pgxc_group.is_installation = false;"

        dumpFile = ""
        for database in databases:
            dumpTableFilePre = self.dumpTableFile[:-4]
            dumpFile = "%s_%s.dat" % (dumpTableFilePre, database)
            (status, output) = self.execSQLCommand(selectSQL, self.user, "", self.instInfo.port)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % selectSQL + " Error:\n%s" % output)
            tableList = output.split("\n")
            for table in tableList:
                table = table.strip()
                if (table == ""):
                    g_file.writeFile(dumpFile, [''])
                else:
                    column = (table).split('|')
                    nspName = (column[0]).strip()
                    tableName = (column[1]).strip()
                    tableInfo = nspName + '.' + tableName +'\n'
                    g_file.writeFile(dumpFile, [tableInfo])
                    
        tmpDir = DefaultValue.getTmpDirFromEnv()
        cmd = g_Platform.getRemoteCopyCmd("'%s'*" % self.dumpTableFile[:-4], tmpDir, self.instInfo.hostname, False, "file")        
        (status, output) = commands.getstatusoutput(cmd)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error:\n%s" % output)
        self.logger.log("Successfully copied DN tables from first CN.")

    """
    Desc: 
        Under the AP branch, we don't need to the uninstall/postcheck for every componet. 
    """
    def fixPermission(self):
        pass

    def upgrade(self):
        pass

    def createPath(self):
        pass

    def postCheck(self):
        pass
 
