#!/usr/bin/env python
#-*- coding:utf-8 -*-
try:
    import sys
    import os
    sys.path.append(sys.path[0] + "/../../../../")
    from gspylib.common.ErrorCode import ErrorCode
    from gspylib.common.Common import DefaultValue, ClusterInstanceConfig
    from gspylib.component.Kernal.Kernal import Kernel
    from gspylib.common.DbClusterInfo import dbClusterInfo
    from gspylib.os.gsfile import g_file
    from gspylib.os.gsOSlib import g_OSlib
except ImportError, e:
    sys.exit("[GAUSS-52200] : Unable to import module: %s." % str(e))

METHOD_TRUST = "trust"
METHOD_SHA = "sha256"
MAX_PARA_NUMBER = 1000
INSTANCE_TYPE_UNDEFINED = -1
MASTER_INSTANCE = 0
STANDBY_INSTANCE = 1
DUMMY_STANDBY_INSTANCE = 2
 
class DN_OLAP(Kernel):
    '''
    The class is used to define base component.
    '''
    def __init__(self):
        '''
        Constructor
        '''
        super(DN_OLAP, self).__init__()
        

    def getDnGUCDict(self):
        """
        function : get init DN install guc parameter
        input : String,String,String,int
        output : String
        """
        tmpDict = {}
        tmpDict["ssl"] = "off"
        tmpDict["ssl_cert_file"] = "'server.crt'"
        tmpDict["ssl_key_file"] = "'server.key'"
        tmpDict["ssl_ca_file"] = "'cacert.pem'"
        return tmpDict

    def initInstance(self):
        """
        function: 
            init DN instance
        input:string:NA
        output: 
        """
        if(not os.path.exists(self.instInfo.datadir)):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50201"] % ("data directory [%s]" % self.instInfo.datadir))

        nodename = self.getInstanceNodeName()
        # [scene] : get pwd by key_path in public_cloud:huaweiyun
        user_name = ""
        for paras in self.initParas:
            if paras.find("-U") != -1:
                user_name = paras.split(" ")[-1].strip()
                break
        key_path = os.path.join(DefaultValue.PUBLIC_CLOUD_TOKEN_FILE, user_name)
        if not os.path.isdir(key_path):
            key_path = self.binPath

        #If xlogdir is set in xmlfile, an independent xlog path will be created.
        xlog_dir = ""
        if(self.instInfo.xlogdir != ''):
            xlog_dir = "-X %s" % self.instInfo.xlogdir

        if(self.dwsMode):
            image_path = DefaultValue.DWS_IMAGE_PATH
            # decompress package to files
            packageName = "%s/datanode.tar.gz" % image_path
            g_file.decompressFiles(packageName, self.instInfo.datadir)
            # set GUC parameter
            tmpDict = {}
            tmpDict["pgxc_node_name"] = "'%s'" % nodename
            self.setGucConfig(tmpDict)
        else:
            cmd = "%s/gs_initdb --locale=C -D %s %s --nodename=%s -C %s %s" % (self.binPath,
                                                                               self.instInfo.datadir,
                                                                               xlog_dir,
                                                                               nodename,
                                                                               key_path,
                                                                               " ".join(self.initParas))
            self.logger.debug("Command for initializing DN instance: %s" % DefaultValue.mock_password(cmd))
            (status, output) = DefaultValue.retryGetstatusoutput(cmd)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_516["GAUSS_51615"] + " Command:%s. Error:\n%s" % (
                    DefaultValue.mock_password(cmd), output))
        dnGucParas = self.getDnGUCDict()
        self.setGucConfig(dnGucParas)
        self.copyAndModCertFiles()
    
    def getInstanceNodeName(self):
        """
        function: Get Instance Node Name
        input : NA
        output: instance node name
        """
        user = g_OSlib.getUserInfo()["name"]
        clusterInfo = dbClusterInfo()
        clusterInfo.initFromStaticConfig(user)
        peerInsts = clusterInfo.getPeerInstance(self.instInfo)
        if(clusterInfo.isMasterStandbyCluster()):
            nodename = ClusterInstanceConfig.setReplConninfo(self.instInfo, peerInsts, clusterInfo)[3]
        elif(clusterInfo.isSinglePrimaryMultiStandbyCluster() or
             clusterInfo.isSingleInstCluster()):
            nodename = ClusterInstanceConfig.setReplConninfoForSinglePrimaryMultiStandbyCluster(self.instInfo, peerInsts, clusterInfo)[1]
        else:
            nodename = "dn_%d" % self.instInfo.instanceId

        return nodename


    def getDNDict(self, user, configItemType = None, peerInsts = [], azNames = [], clusterInfo = None):
        """
        function: Get DN configuration
        input : NA
        output: NA
        """
        tmpDNDict = {}
        tmpDNDict["listen_addresses"] = "'%s'" % ",".join(self.instInfo.listenIps)
        tmpDNDict["local_bind_address"] = "'%s'" % self.instInfo.listenIps[0]
        tmpDNDict["port"] = self.instInfo.port
        tmpDNDict["pooler_port"] = self.instInfo.port+1
        
        if (configItemType == "ConfigInstance"):
            tmpDNDict["cstore_buffers"] = "1GB"
            tmpDNDict["max_connections"] = "3000"
            tmpDNDict["shared_buffers"] = "1GB"
            tmpDNDict["work_mem"] = "64MB"
            tmpDNDict["maintenance_work_mem"] = "128MB"
            tmpDNDict["data_replicate_buffer_size"] = "128MB"
        if(self.clusterType == DefaultValue.CLUSTER_TYPE_SINGLE_PRIMARY_MULTI_STANDBY or
           self.clusterType == DefaultValue.CLUSTER_TYPE_SINGLE_INST):
            tmpDNDict["enable_data_replicate"] = "off"
            tmpDNDict["replication_type"] = "1"
            tmpDNDict["max_replication_slots"] = "%d" % len(peerInsts)
            tmpDNDict["max_wal_senders"] = "8"
            tmpDNDict["application_name"] = "'dn_%s'" % self.instInfo.instanceId
            tmpDict= ClusterInstanceConfig.getAzname(azNames, peerInsts, clusterInfo)
            tmpDNDict.update(tmpDict)

            if(self.clusterType == DefaultValue.CLUSTER_TYPE_SINGLE_INST):
                if(len(peerInsts) == 1):
                    tmpDNDict["most_available_sync"] = "on"
        if(self.clusterType == DefaultValue.CLUSTER_TYPE_SINGLE):
            tmpDNDict["replication_type"] = "2"

        if (configItemType != "ChangeIPUtility"):
            tmpDNDict["log_directory"] = "'%s/pg_log/dn_%d'" % (DefaultValue.getUserLogDirWithUser(user), self.instInfo.instanceId)
            tmpDNDict["audit_directory"] = "'%s/pg_audit/dn_%d'" % (DefaultValue.getUserLogDirWithUser(user), self.instInfo.instanceId)

        if (len(self.instInfo.ssdDir) != 0 and configItemType != "ChangeIPUtility"):
            tmpDNDict["ssd_cache_dir"] = "'%s'" % (self.instInfo.ssdDir)
            tmpDNDict["enable_adio_function"] = "on"
            tmpDNDict["enable_cstore_ssd_cache"] = "on"
        return tmpDNDict
        
    def getPrivateGucParamList(self):
        """
        function : Get the private guc parameter list.
        input : NA
        output
        """
        # only used by dummy standby instance
        #     max_connections value is 100
        #     memorypool_enable value is false
        #     shared_buffers value is 32MB
        #     bulk_write_ring_size value is 32MB
        #     max_prepared_transactions value is 10
        #     cstore_buffers value is 16MB
        #     autovacuum_max_workers value is 0
        #     max_pool_size value is 50
        #     wal_buffers value is -1

        # add the parameter content to the dictionary list
        priavetGucParamDict = {}
        priavetGucParamDict["max_connections"] = "100"
        priavetGucParamDict["memorypool_enable"] = "false"
        priavetGucParamDict["shared_buffers"] = "32MB"
        priavetGucParamDict["bulk_write_ring_size"] = "32MB"
        priavetGucParamDict["max_prepared_transactions"] = "10"
        priavetGucParamDict["cstore_buffers"] = "16MB"
        priavetGucParamDict["autovacuum_max_workers"] = "0"
        priavetGucParamDict["max_pool_size"] = "50"
        priavetGucParamDict["wal_buffers"] = "-1"
        priavetGucParamDict["max_locks_per_transaction"] = "64"
        priavetGucParamDict["sysadmin_reserved_connections"]= "3"
        priavetGucParamDict["max_wal_senders"] = "4"
        return priavetGucParamDict

    def modifyDummpyStandbyConfigItem(self):
        """
        function: Modify the parameter at dummyStandby instance. It only be used by DN instance.
        input : Inst, configFile
        output: NA
        """
        # only modify config item for dummpy standby instance
        if(self.instInfo.instanceType != DefaultValue.DUMMY_STANDBY_INSTANCE):
            return
        tmpDNDict = self.getPrivateGucParamList()
        self.setGucConfig(tmpDNDict)

    def setPrimaryStandyConnInfo(self, peerInsts):
        """
        function: Modify replconninfo for datanode
        input : peerInsts
        output: NA
        """
        connInfo1 = None
        connInfo2 = None
        dummyStandbyInst = None
        nodename = None
        user = g_OSlib.getUserInfo()["name"]
        clusterInfo = dbClusterInfo()
        clusterInfo.initFromStaticConfig(user)        
        if(self.clusterType == DefaultValue.CLUSTER_TYPE_SINGLE_PRIMARY_MULTI_STANDBY or
           self.clusterType == DefaultValue.CLUSTER_TYPE_SINGLE_INST):
            (connInfo1, nodename) = ClusterInstanceConfig.setReplConninfoForSinglePrimaryMultiStandbyCluster(self.instInfo, peerInsts, clusterInfo)
            for i in range(len(connInfo1)):
                connInfo = "replconninfo" + "%d" % (i+1)
                tmpDict1 = {}
                tmpDict1[connInfo] = "'%s'" % connInfo1[i]
                self.setGucConfig(tmpDict1)
        else:
            (connInfo1, connInfo2 , dummyStandbyInst, nodename) = ClusterInstanceConfig.setReplConninfo(self.instInfo, peerInsts, clusterInfo)
            connInfo = "replconninfo1"
            tmpDict1 = {}
            tmpDict1[connInfo] = "'%s'" % connInfo1
            self.setGucConfig(tmpDict1)

        if(dummyStandbyInst != None):
            tmpDict2 = {}
            tmpDict2["replconninfo2"] = "'%s'" % connInfo2
            self.setGucConfig(tmpDict2)

    def configInstance(self, user, dataConfig, gtmList, peerInsts, configItemType = None, alarm_component = None, azNames = [], gucXml = (False,""), clusterInfo = None):
        """
        peerInsts : peerInsts is empty means that it is a single cluster.
        """
        tmpDNDict = self.getDNDict(user, configItemType, peerInsts, azNames, clusterInfo)

        commonDict = self.setCommonItems()
        self.setGucConfig(commonDict)

        if(clusterInfo is None or (clusterInfo is not None and not clusterInfo.isSingleInstCluster())):
            gtmDict = ClusterInstanceConfig.setGtmInfo(gtmList)
            self.setGucConfig(gtmDict)

        self.logger.debug("Check if tmp_guc file exists.")
        tmpGucFile = ""
        tmpGucPath = DefaultValue.getTmpDirFromEnv(user)
        tmpGucFile = "%s/tmp_guc" % tmpGucPath
        if (os.path.exists(tmpGucFile)):
            dynamicDict = {}
            dynamicDict = DefaultValue.dynamicGuc(user, self.logger, "dn", tmpGucFile, gucXml)
            if gucXml[0]:
                dynamicDict["log_line_prefix"] = "'%s'" % dynamicDict["log_line_prefix"]
                dynamicDict["thread_pool_attr"] = "'%s'" % dynamicDict["thread_pool_attr"]
            if (len(dynamicDict) != 0):
                self.logger.debug("set dynamic guc parameters for dn instances.")
                if (self.instInfo.instanceType  == DefaultValue.DUMMY_STANDBY_INSTANCE):
                    self.logger.debug("remove max_process_memory if current datanode is dummy one.")
                    dummydynamicDict = dynamicDict
                    dummydynamicDict.pop("max_process_memory")
                    tmpDNDict.update(dummydynamicDict)
                else:
                    tmpDNDict.update(dynamicDict)
            else:
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50219"] % "guc_list.conf")
        tmpDNDict.update(dataConfig)
        tmpDNDict["alarm_component"] = "'%s'" % alarm_component
        self.setGucConfig(tmpDNDict)

        if(len(peerInsts)):
            self.setPrimaryStandyConnInfo(peerInsts)
        else:
            tmpDict1 = {}
            tmpDict1["synchronous_commit"] = "off"
            tmpDict1["enable_stream_replication"] = "off"
            self.setGucConfig(tmpDict1)

        self.modifyDummpyStandbyConfigItem()

    def setPghbaConfig(self, clusterAllIpList):
        """
        """
        principal = None
        if (DefaultValue.checkKerberos(DefaultValue.getMpprcFile())):
            
            (status, output) = g_OSlib.getGrepValue("-Er", "^default_realm", os.path.join(os.path.dirname(DefaultValue.getMpprcFile()), DefaultValue.FI_KRB_CONF))
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50222"] % "krb5.conf" + "Error:\n%s" % output)
            principal = output.split("=")[1].strip()

        # build ip string list
        # Every 1000 records merged into one
        i = 0
        GUCParasStr = ""
        GUCParasStrList = []
        for ipAddress in clusterAllIpList:
            i += 1
            # Set the initial user and initial database access permissions 
            if (principal == None):
                GUCParasStr += "-h \"host    all    all    %s/32    %s\" " % (ipAddress, METHOD_TRUST)
            else:
                GUCParasStr += "-h \"host    all    all    %s/32    gss    include_realm=1    krb_realm=%s\" " % (ipAddress, principal)
            if (i % MAX_PARA_NUMBER == 0):
                GUCParasStrList.append(GUCParasStr)
                i = 0
                GUCParasStr = ""

        if (GUCParasStr != ""):
            GUCParasStrList.append(GUCParasStr)

        for parasStr in GUCParasStrList:
            self.doGUCConfig("set", parasStr, True)

    """
    Desc: 
        Under the AP branch, we don't need to the uninstall/postcheck for every componet. 
    """     
    def fixPermission(self):
        pass

    def upgrade(self):
        pass

    def createPath(self):
        pass

    def postCheck(self):
        pass
 
