#!/usr/bin/env python
#-*- coding:utf-8 -*-
try:
    import sys
    import os
    import commands
    import grp, pwd
    import base64
    import re
    import ConfigParser
    import time
    import subprocess

    sys.path.append(sys.path[0] + "/../../../")
    from gspylib.common.ErrorCode import ErrorCode
    from gspylib.component.BaseComponent import BaseComponent
    from gspylib.os.gsfile import g_file
    from gspylib.common.Common import DefaultValue, ClusterCommand
    from gspylib.threads.parallelTool import parallelTool, CommandThread
    from gspylib.os.gsfile import g_file, g_Platform
except ImportError, e:
    sys.exit("[GAUSS-52200] : Unable to import module: %s." % str(e))

RETRY_COUNT = 3
MAX_PARA_NUMBER = 1000

class Kernel(BaseComponent):
    '''
    The class is used to define base component.
    '''
    def __init__(self):
        """
        """
        super(Kernel, self).__init__()
        # init paramter schemaCoordinatorFile, schemaJobFile and schemaDatanodeFile
        tmpDir = DefaultValue.getTmpDirFromEnv()
        self.schemaCoordinatorFile = "%s/%s" % (tmpDir, DefaultValue.SCHEMA_COORDINATOR)
        self.coordinatorJobDataFile = "%s/%s" % (tmpDir, DefaultValue.COORDINATOR_JOB_DATA)
        self.schemaDatanodeFile = "%s/%s" % (tmpDir, DefaultValue.SCHEMA_DATANODE)
        self.dumpTableFile = "%s/%s" % (tmpDir, DefaultValue.DUMP_TABLES_DATANODE)
        self.dumpOutputFile = "%s/%s" % (tmpDir, DefaultValue.DUMP_Output_DATANODE)
        self.coordinatorStatisticsDataFile = "%s/%s" % (tmpDir, DefaultValue.COORDINATOR_STAT_DATA)

    """
    Desc: 
        start/stop/query single instance 
    """
    def start(self, nodeType = ""):
        """
        """
        instype = "restoremode"
        if (nodeType == "coordinator"):
            instype = "coordinator"
        elif (nodeType == "datanode"):
            instype = "datanode"

        cmd = "%s/gs_ctl start -D %s -Z %s " % (self.binPath, self.instInfo.datadir, instype)
        (status, output) = commands.getstatusoutput(cmd)
        if(status != 0):
            raise Exception(ErrorCode.GAUSS_516["GAUSS_51607"] % "instance" + " Error: \n%s." % output)

    def stop(self, nodeType = "", stopMode = ""):
        """
        """
        instType = "restoremode"
        if (nodeType == "coordinator"):
            instType = "coordinator"
        elif (nodeType == "datanode"):
            instType = "datanode"
            
        cmd = "%s/gs_ctl stop -D %s -Z %s " % (self.binPath, self.instInfo.datadir, instType)
        # check stop mode
        if (stopMode != ""):
            cmd += " -m %s" % stopMode
        (status, output) = commands.getstatusoutput(cmd)
        if(status != 0):
            raise Exception(ErrorCode.GAUSS_516["GAUSS_51610"] % "instance" + " Error: \n%s." % output)

    def query(self):
        """
        """
        cmd = "%s/gs_ctl query -D %s" % (self.binPath, self.instInfo.datadir)
        (status, output) = commands.getstatusoutput(cmd)
        if(status != 0):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)
        return (status, output)

    def build(self, buidMode = "full", standByBuildTimeout = 300):
        """
        """
        cmd = "%s/gs_ctl build -D %s -M standby -Z datanode -b %s -r %d " % (self.binPath, self.instInfo.datadir, buidMode, standByBuildTimeout)
        (status, output) = commands.getstatusoutput(cmd)
        if(status != 0):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)

    def queryBuild(self):
        """
        """
        cmd = "%s/gs_ctl querybuild -D %s" % (self.binPath, self.instInfo.datadir)
        (status, output) = commands.getstatusoutput(cmd)
        if(status != 0):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)

    """
    Desc: 
        Under the AP branch, the installation package of each component is not distinguished. 
        After checking, unzip the public installation package and complete the installation. 
    """
    def install(self, nodeName = "", dbInitParams = ""):
        """
        """
        pass

    def getInstanceTblspcDirs(self, nodeName = ""):
        """
        function: Get instance dirs 
        input : NA
        output: NA
        """
        tbsDirList = []

        if (not os.path.exists("%s/pg_tblspc" % self.instInfo.datadir)):
            self.logger.debug("%s/pg_tblspc does not exists." % self.instInfo.datadir)
            return tbsDirList

        fileList = os.listdir("%s/pg_tblspc" % self.instInfo.datadir)
        if (len(fileList)):
            for filename in fileList:
                if(os.path.islink("%s/pg_tblspc/%s" % (self.instInfo.datadir, filename))):
                    linkDir = os.path.realpath("%s/pg_tblspc/%s" % (self.instInfo.datadir, filename))
                    if(os.path.isdir(linkDir)):
                        tblspcDir = "%s/%s_%s" % (linkDir, DefaultValue.TABLESPACE_VERSION_DIRECTORY, nodeName)
                        self.logger.debug("Table space directories is %s." % tblspcDir)
                        tbsDirList.append(tblspcDir)
                    else:
                        self.logger.debug("%s is not link directory." % linkDir)
                else:
                    self.logger.debug("%s is not a link file." % filename)
        else:
            self.logger.debug("%s/pg_tblspc is empty." % self.instInfo.datadir)

        return tbsDirList

    def getLockFiles(self):
        """
        function: Get lock files 
        input : NA
        output: NA
        """
        fileList = []
        # the static file must be exists
        tmpDir = os.path.realpath(DefaultValue.getTmpDirFromEnv())

        pgsql = ".s.PGSQL.%d" % self.instInfo.port
        pgsqlLock = ".s.PGSQL.%d.lock" % self.instInfo.port
        fileList.append(os.path.join(tmpDir, pgsql))
        fileList.append(os.path.join(tmpDir, pgsqlLock))
        return fileList
    
    def removeSocketFile(self, fileName):
        """
        """
        g_file.removeFile(fileName, "shell")

    def removeTbsDir(self, tbsDir):
        """
        """
        g_file.removeDirectory(tbsDir)

    def cleanDir(self, instDir):
        """
        function: Clean the dirs
        input : NA
        output: NA
        """
        if(not os.path.exists(instDir)):
            return 
        
        dataDir = []
        dataDir = os.listdir(instDir)
        if(os.getuid() == 0):
            pglDir = '%s/pg_location' % instDir
            isPglDirEmpty = False
            if(os.path.exists(pglDir) and len(os.listdir(pglDir)) == 0):
                isPglDirEmpty = True
            if(len(dataDir) == 0 or isPglDirEmpty):
                g_file.cleanDirectoryContent(instDir)
        else:
            for info in dataDir:
                if(str(info) == "pg_location"):
                    resultMount = []
                    resultFile = []
                    resultDir = []
                    pglDir = '%s/pg_location' % instDir

                    #delete all files in the mount point
                    cmd = "%s | %s '%s' | %s '{printf $3}'" % (g_Platform.getMountCmd(), g_Platform.getGrepCmd(), pglDir, g_Platform.getAwkCmd())
                    (status, outputMount) = commands.getstatusoutput(cmd)
                    if(status != 0):
                        raise Exception(ErrorCode.GAUSS_502["GAUSS_50207"] % instDir + " Error:\n%s" % str(outputMount))
                    else:
                        if(len(outputMount) > 0):
                            resultMount = str(outputMount).split()
                            for infoMount in resultMount:
                                g_file.cleanDirectoryContent(infoMount)
                        else:
                            g_file.cleanDirectoryContent(instDir)
                            continue

                    #delete file in the pg_location directory
                    if (not os.path.exists(pglDir)):
                        continue
                    cmd = "cd '%s'" % pglDir
                    (status, output) = commands.getstatusoutput(cmd)
                    if(status != 0):
                        raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)

                    outputFile = g_file.findFile(".", "f", "type")
                    if(len(outputFile) > 0):
                        for infoFile in outputFile:
                            tmpinfoFile = pglDir + infoFile[1:]
                            for infoMount in resultMount:
                                if(tmpinfoFile.find(infoMount) < 0 and infoMount.find(tmpinfoFile) < 0):
                                    realFile = "'%s/%s'" % (pglDir, infoFile)
                                    g_file.removeFile(realFile, "shell")

                    #delete directory in the pg_location directory 
                    cmd = "if [ -d '%s' ]; then cd '%s' && find -type d; fi" % (pglDir, pglDir)
                    (status, outputDir) = commands.getstatusoutput(cmd)
                    if(status != 0):
                        raise Exception(ErrorCode.GAUSS_502["GAUSS_50207"] % instDir + " Error:\n%s" % str(outputDir))
                    else:
                        resultDir = g_file.findFile(".", "d", "type")
                        resultDir.remove(".")
                        if(len(resultDir) > 0):
                            for infoDir in resultDir:
                                tmpinfoDir = pglDir + infoDir[1:]
                                for infoMount in resultMount:
                                    if(tmpinfoDir.find(infoMount) < 0 and infoMount.find(tmpinfoDir) < 0):
                                        realPath = "'%s/%s'" % (pglDir, infoDir)
                                        g_file.removeDirectory(realPath)
            cmd = "if [ -d '%s' ];then cd '%s' && find . ! -name 'pg_location' ! -name '..' ! -name '.' -print0 |xargs -r -0 -n100 rm -rf; fi" % (instDir, instDir)
            (status, output) = commands.getstatusoutput(cmd)
            if(status != 0):
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50207"] % instDir + " Error:\n%s" % str(output))
                    

    def uninstall(self, instNodeName):
        """
        function: Clean node instances.
                  1.get the data dirs, tablespaces, soketfiles
                  2.use theard delete the dirs or files
        input : NA
        output: NA
        """
        self.logger.log("Cleaning instance.")

        # tablespace data directory
        tbsDirList = self.getInstanceTblspcDirs(instNodeName)

        # sockete file
        socketFiles = self.getLockFiles()

        # clean tablespace dir
        if (len(tbsDirList) != 0):
            try:
                self.logger.debug("Deleting instances tablespace directories.")
                for tbsDir in tbsDirList:
                    self.removeTbsDir(tbsDir)
            except Exception, e:
                raise Exception(str(e))
            self.logger.log("Successfully cleaned instance tablespace.")

        if (len(self.instInfo.datadir) != 0):
            try:
                self.logger.debug("Deleting instances directories.")
                self.cleanDir(self.instInfo.datadir)
            except Exception, e:
                raise Exception(str(e))
            self.logger.log("Successfully cleaned instances.")

        if (len(self.instInfo.xlogdir) != 0):
            try:
                self.logger.debug("Deleting instances xlog directories.")
                self.cleanDir(self.instInfo.xlogdir)
            except Exception, e:
                raise Exception(str(e))
            self.logger.log("Successfully cleaned instances.")

        if (len(socketFiles) != 0):
            try:
                self.logger.debug("Deleting socket files.")
                for socketFile in socketFiles:
                    self.removeSocketFile(socketFile)
            except Exception, e:
                raise Exception(str(e))
            self.logger.log("Successfully cleaned socket files.")

    def copySSLCertFiles(self, srcdir, destdir):
        """
        function : copy permission cert files to instance datadir
        input : NA
        output : NA
        """
        g_file.cpFile("%s/server.crt" % srcdir, "%s/" % destdir)
        g_file.cpFile("%s/server.key" % srcdir, "%s/" % destdir)
        g_file.cpFile("%s/cacert.pem" % srcdir, "%s/" % destdir)
        g_file.cpFile("%s/server.key.cipher" % srcdir, "%s/" % destdir)
        g_file.cpFile("%s/server.key.rand" % srcdir, "%s/" % destdir)

    def copyAndModCertFiles(self):
        """
        function : copy and chage permission cert files
        input : NA
        output : NA
        """
        huaweiyun = DefaultValue.isHuaweiyunScene()
        if huaweiyun:
            try:
                ssl_srcdir = os.path.realpath("/tmp/tmptoken/sslUser")
                #cp user ssl_crt to instance datadir
                self.copySSLCertFiles(ssl_srcdir, self.instInfo.datadir)
                self.logger.debug("Successfully copy user SSLCertFiles from %s to %s" % (ssl_srcdir, self.instInfo.datadir))
                # change mode
                g_file.changeMode(DefaultValue.KEY_FILE_MODE, "%s/server.crt" % self.instInfo.datadir)
                g_file.changeMode(DefaultValue.KEY_FILE_MODE, "%s/server.key" % self.instInfo.datadir)
                g_file.changeMode(DefaultValue.KEY_FILE_MODE, "%s/cacert.pem" % self.instInfo.datadir)
                g_file.changeMode(DefaultValue.KEY_FILE_MODE, "%s/server.key.cipher" % self.instInfo.datadir)
                g_file.changeMode(DefaultValue.KEY_FILE_MODE, "%s/server.key.rand" % self.instInfo.datadir)
            except Exception as e:
                raise Exception(str(e))

    def setCommonItems(self):
        """
        function: set common items
        input : tmpDir
        output: tempCommonDict
        """
        tempCommonDict = {}
        tmpDir = DefaultValue.getTmpDirFromEnv()
        tempCommonDict["unix_socket_directory"] = "'%s'" % tmpDir
        tempCommonDict["unix_socket_permissions"] = "0700"
        tempCommonDict["log_file_mode"] = "0600"
        tempCommonDict["max_coordinators"] = "1024"
        tempCommonDict["max_datanodes"] = "4096"
        tempCommonDict["enable_nestloop"] = "off"
        tempCommonDict["enable_mergejoin"] = "off"
        tempCommonDict["explain_perf_mode"] = "pretty"
        tempCommonDict["log_line_prefix"] = "'%m %c %d %p %a %x %n %e '"
        tempCommonDict["max_active_statements"] = "-1"

        return tempCommonDict  
        
    def doGUCConfig(self, action, GUCParasStr, isHab = False):
        """
        """
        # check instance data directory
        if (self.instInfo.datadir == "" or not os.path.exists(self.instInfo.datadir)):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50219"] % ("data directory of the instance[%s]" % str(self.instInfo)))

        if (GUCParasStr == ""):
            return

        # check conf file
        if (isHab == True):
            configFile = "%s/pg_hba.conf" % self.instInfo.datadir
        else:
            configFile = "%s/postgresql.conf" % self.instInfo.datadir
        if(not os.path.exists(configFile)):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50201"] % configFile)

        if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_COODINATOR):
            instype = "coordinator"
        else:
            instype = "datanode"
        cmd = "%s/gs_guc %s -Z %s -D %s %s " % (self.binPath, action, instype, self.instInfo.datadir, GUCParasStr)
        (status, output) = DefaultValue.retryGetstatusoutput(cmd, 3, 3)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_500["GAUSS_50007"] % "GUC" + " Command: %s. Error:\n%s" % (cmd , output))

    def setGucConfig(self, paraDict = {}, setMode = 'set'):
        """
        """
        i = 0
        GUCParasStr = ""
        GUCParasStrList = []
        for paras in paraDict:
            i += 1
            value = str(paraDict[paras])
            if value.find('\'') < 0:
                value = "'%s'" % value
            GUCParasStr += " -c \"%s=%s\" " % (paras, value)
            if (i % MAX_PARA_NUMBER == 0):
                GUCParasStrList.append(GUCParasStr)
                i = 0
                GUCParasStr = ""
        if (GUCParasStr != ""):
            GUCParasStrList.append(GUCParasStr)

        for parasStr in GUCParasStrList:
            self.doGUCConfig(setMode, parasStr, False)

    def removeIpInfoOnPghbaConfig(self, ipAddressList):
        """
        """
        i = 0
        GUCParasStr = ""
        GUCParasStrList = []
        for ipAddress in ipAddressList:
            i += 1
            GUCParasStr += " -h \"host    all    all    %s/32\"" % (ipAddress)
            if (i % MAX_PARA_NUMBER == 0):
                GUCParasStrList.append(GUCParasStr)
                i = 0
                GUCParasStr = ""
        if (GUCParasStr != ""):
            GUCParasStrList.append(GUCParasStr)

        for parasStr in GUCParasStrList:
            self.doGUCConfig("set", parasStr, True)

    """
    Desc: 
        get binary file execute command 
    """
    def getDbAdminPwd(self):
        """
        """
        tmptoken = DefaultValue.TMP_TOKEN_FILE
        dbAdminPwd = ""
        if (os.path.exists(tmptoken)):
            fp = open(tmptoken,"r")
            lines = fp.readlines()
            if len(lines) == 1:
                dbAdminPwd = base64.b64decode(lines[0])
            fp.close()
        return dbAdminPwd

    def getSQLCommand(self, port, database = DefaultValue.DEFAULT_DB_NAME, gsqlBin = "gsql"):
        """
        function : get SQL command
        input : port, database
        output : cmd
        """
        dbAdminPwd = self.getDbAdminPwd()
        host = DefaultValue.GetHostIpOrName()
        if (dbAdminPwd == ""):
            agent_conf = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../../../../../om_agent.conf")
            scene = ""
            if os.path.exists(agent_conf):
                fp = ConfigParser.ConfigParser()
                fp.read(agent_conf)
                if fp.has_option("PUBLIC_CLOUD_CONFIGURATION", 'scene'):
                    scene = fp.get("PUBLIC_CLOUD_CONFIGURATION", 'scene')
            if scene == "huaweiyun":
                cmd = DefaultValue.SQL_EXEC_COMMAND_WITHOUT_HOST_WITHOUT_USER % ("%s/gsql" % (self.binPath), str(int(port) + 1), database) + ClusterCommand.get_passwd()
            else:
                cmd = DefaultValue.SQL_EXEC_COMMAND_WITHOUT_HOST_WITHOUT_USER % ("%s/gsql" % (self.binPath), str(int(port) + 1), database)
        else:
            cmd = DefaultValue.SQL_EXEC_COMMAND_WITHOUT_HOST_WITH_USER % (gsqlBin, str(int(port) + 1), database, DefaultValue.RUBY, dbAdminPwd)

        return cmd

    def getDUMPCommand(self):
        """
        function : get dump command
        input : NA
        output : cmd
        """
        dbAdminPwd = self.getDbAdminPwd()  
        if (dbAdminPwd == ""):
            cmd = "%s/gs_dump" % (self.binPath)
        else:
            cmd = "gs_dump -U %s -W %s" % (DefaultValue.RUBY, dbAdminPwd)
        return cmd  

    def getDUMPALLCommand(self):
        """
        function : get dumpall command
        input : NA
        output : cmd
        """
        dbAdminPwd = self.getDbAdminPwd()
        if (dbAdminPwd == ""):
            cmd = "%s/gs_dumpall" % (self.binPath)
        else:
            cmd = "gs_dumpall -U %s -W %s" % (DefaultValue.RUBY, dbAdminPwd)
        return cmd  

    """
    Desc: 
        base execute function
    """
    def findErrorInSqlFile(self, sqlFile, output):
        """
        function : Find error in the sql file
        """
        GSQL_BIN_FILE = "gsql"
        # init flag
        ERROR_MSG_FLAG = "(ERROR|FATAL|PANIC)"
        GSQL_ERROR_PATTERN = "^%s:%s:(\d*): %s:.*" % (GSQL_BIN_FILE, sqlFile, ERROR_MSG_FLAG)
        pattern = re.compile(GSQL_ERROR_PATTERN)
        for line in output.split("\n"):
            line = line.strip()
            result = pattern.match(line)
            if (result is not None):
                return True
        return False

    def findErrorInSqlQuery(self, output):
        """
        function : Find error in sql
        """
        # init flag
        ERROR_MSG_FLAG = "(ERROR|FATAL|PANIC)"
        ERROR_PATTERN = "^%s:.*" % ERROR_MSG_FLAG
        pattern = re.compile(ERROR_PATTERN)

        for line in output.split("\n"):
            line = line.strip()
            result = pattern.match(line)
            if (result is not None):
                return True
        return False

    def execSQLFile(self, sqlFile, user, host, port, database = "postgres", option = ""):
        """
        function : Execute sql file
        input : String,String,String,int
        output : String
        """
        database = database.replace('$', '\$')
        currentTime = time.strftime("%Y-%m-%d_%H%M%S")
        pid = os.getpid()
        # init SQL result file
        queryResultFile = os.path.join(DefaultValue.getTmpDirFromEnv(user), "gaussdb_result.sql_%s_%s_%s" %(str(port), str(currentTime), str(pid)))
        if(os.path.exists(queryResultFile)):
            DefaultValue.cleanFile(queryResultFile)

        # init hostPara
        userProfile = DefaultValue.getMpprcFile()
        hostPara = ("-h %s" % host) if host != "" else "" 
        # build shell command
        # if the user is root, switch the user to execute
        gsql_cmd = self.getSQLCommand(port, database)
        if(os.getuid() == 0):
            cmd = "su - %s -c \"source %s; %s %s -f '%s' --output '%s' -t -A -X %s\" \
            " % (user, userProfile, gsql_cmd, hostPara, sqlFile, queryResultFile, option)
        else:
            cmd = "%s %s -f '%s' --output '%s' -t -A -X %s" % (gsql_cmd, hostPara, sqlFile, queryResultFile, option)
        (status, output) = commands.getstatusoutput(cmd)
        if (self.findErrorInSqlFile(sqlFile, output) == True):
            status = 1
        if (status != 0):
            DefaultValue.cleanFile(queryResultFile)
            return (status, output)

        #read the content of query result file.
        try:
            fp = None
            fp = open(queryResultFile, 'r')
            rowList = fp.readlines()
            if (fp):
                fp.close()
        except Exception, e:
            if (fp):
                fp.close()
            DefaultValue.cleanFile(queryResultFile)
            return (1, str(e))

        if(os.path.exists(queryResultFile)):
            DefaultValue.cleanFile(queryResultFile)

        return (0, "".join(rowList)[:-1])

    def execSQLCommand(self, sql, user, host, port, database = "postgres", option = ""):
        """
        function : Execute sql command
        input : String,String,String,int
        output : String
        """
        database = database.replace('$', '\$')
        currentTime = time.strftime("%Y-%m-%d_%H%M%S")
        pid = os.getpid()
        # init SQL query file
        sqlFile = os.path.join(DefaultValue.getTmpDirFromEnv(user), "gaussdb_query.sql_%s_%s_%s" %(str(port), str(currentTime), str(pid)))
        # create an empty sql query file
        g_file.createFile(sqlFile)
        g_file.changeMode(DefaultValue.KEY_FILE_MODE, sqlFile)

        if(self.dwsMode == True):
            sql = "set cgroup_name='Rush';" + sql
        # witer the SQL command into sql query file
        try:
            fp = None
            g_file.createFileInSafeMode(sqlFile)
            fp = open(sqlFile, 'w')
            fp.writelines(sql)
            if(fp):
                fp.close()
        except Exception, e:
            if(fp):
                fp.close()
            DefaultValue.cleanFile(sqlFile)
            return (1, str(e))
        try:
            (status, output) = self.execSQLFile(sqlFile, user, host, port, database, option)
            DefaultValue.cleanFile(sqlFile)
            return (status, output)          
        except Exception, e:
            DefaultValue.cleanFile(sqlFile)
            return (1, str(e))

    def killProcess(self, signaltype, pidList):
        """
        function: kill process
        input:  process flag
        output: NA
        """
        killCmd = g_Platform.getKillProcessCmd(signaltype, " ".join(pidList))
        self.logger.debug("Command for killing: %s" % killCmd)
        (status, output) = commands.getstatusoutput(killCmd)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % killCmd + "\nOutput:%s" % output)

    """
    Desc: 
        comm function for CN/DN
    """    
    def getAllDatabase(self):
        """
        function: Get all the databases in the cluster
        output: databaseList
        """
        self.logger.log("Getting all the databases.")
        try: 
            databaseList = []        
            # get databases from cluster
            selectSql = "SELECT datname FROM pg_catalog.pg_database WHERE datallowconn ORDER BY 1"
            self.logger.debug("Get databases from cluster command: %s." % selectSql)
            (status, output) = self.execSQLCommand(selectSql, self.user, "", self.instInfo.port)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % selectSql + " Error: \n%s" % str(output))
            self.logger.debug("Get databases from cluster result: %s." % output)
            #split the output string with '\n'
            databaseList = output.split("\n")
            return databaseList
        except Exception, e:
            raise Exception(str(e))

    def cleanConnection(self, sql):
        """
        Clean the connection with CN nodes
        """       
        # Clean connection for "template1" database
        self.logger.debug("Get the connection cleaned command: %s" % sql)
        # Try three times
        for count in range(0, RETRY_COUNT):
            # Exectue clean command
            (status, output) = self.execSQLCommand(sql, self.user, "", self.instInfo.port)
            if(status != 0):
                # Check 3 times after execution failure
                if (count < RETRY_COUNT - 1):
                    # Default sleep 1 second then start retry
                    time.sleep(1)
                    continue
                else:
                    # If all failed, add failed info
                    self.logger.debug("Warning: Sql execute failed [%s]. Output:%s" % (sql, output))
            else:
                self.logger.debug("Get the connection cleaned result: %s" % output)
                break

    def getClientEncoding(self, dbname, port):
        """
        function: get client encoding
        input : dbname, port
        output: client_encoding
        """
        self.logger.log("Getting client encoding of database %s." % dbname)
        try:
            showSql = "SHOW client_encoding;"
            self.logger.debug("Get client_encoding command: %s." % showSql)
            (status, output) = self.execSQLCommand(showSql, self.user, "", port)
            self.logger.debug("Get client_encoding result: %s." % output)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % showSql + " Error: \n%s" % str(output))
            #split the output string with '\n'
            client_encoding = output.split("\n")[0].strip()
            return client_encoding
        except Exception, e:
            raise Exception(str(e))
        
    def dumpStatistics(self, port):
        """
        function: dump statistics from normal coordinator
        input: NA
        output: NA
        """
        if(os.path.exists(self.coordinatorStatisticsDataFile)):
            os.remove(self.coordinatorStatisticsDataFile)

        restore_scheam_name = "restore_statistic_scheam_%d" % os.getpid()
        databases = self.getAllDatabase()
        g_file.createFileInSafeMode(self.coordinatorStatisticsDataFile)
        fp = open(self.coordinatorStatisticsDataFile, "w")
        fp.write("\n\n--\n-- STATISTICS\n--\n")

        for dbname in databases:
            encoding = self.getClientEncoding(dbname, port)

            fp.write("\n\n--\n-- RELATION STATS\n--\n")
            fp.write("\\connect %s\n\n" % dbname)

            fp.write("SET enable_hadoop_env = off;\n\n\n")
            fp.write("SET xc_maintenance_mode = on;\n\n\n")

            fp.write("DROP SCHEMA IF EXISTS %s CASCADE;\n" \
                     "CREATE SCHEMA %s;\n" \
                     "CREATE TABLE %s.pg_class_stat AS \n" \
                     "  SELECT relname, nspname, relpages, reltuples, relallvisible \n" \
                     "  FROM pg_class c JOIN pg_namespace n ON(c.relnamespace = n.oid AND n.nspname NOT IN ('pg_toast', 'pg_catalog', 'information_schema'))\n" \
                     "  WHERE false;\n" \
                     "\n" \
                     "CREATE TABLE %s.pg_statistic_stat AS \n" \
                     "  SELECT relname, rn.nspname as relnspname, staattnum, starelkind, t.typname as typname, tn.nspname as typnspname, stainherit, stanullfrac,\n"\
                     "    stawidth, stadistinct, stakind1, stakind2, stakind3, stakind4, stakind5, staop1, staop2, staop3, staop4, staop5,\n"\
                     "    stanumbers1::text, stanumbers2::text, stanumbers3::text, stanumbers4::text, stanumbers5::text,\n"\
                     "    stavalues1::text, stavalues2::text, stavalues3::text, stavalues4::text, stavalues5::text,\n"\
                     "    t.typelem as typelem, s.stadndistinct as stadndistinct\n"\
                     "  FROM pg_statistic s JOIN pg_class c ON(c.oid = s.starelid AND c.relnamespace <>11) \n"\
                     "    JOIN pg_attribute a ON (a.attrelid = s.starelid AND a.attnum = s.staattnum) \n"\
                     "    JOIN pg_namespace rn ON(rn.oid = c.relnamespace AND rn.nspname NOT IN('pg_toast', 'pg_catalog', 'information_schema'))\n"\
                     "    JOIN pg_type t ON (t.oid = a.atttypid)\n"\
                     "    JOIN pg_namespace tn ON(tn.oid = t.typnamespace)\n"\
                     "  WHERE false;\n" \
                     "\n" \
                     "CREATE TABLE %s.pg_statistic_stat_ext AS \n" \
                     "  SELECT relname, rn.nspname as relnspname, stakey, starelkind, stainherit, stanullfrac,\n"\
                     "    stawidth, stadistinct, stadndistinct, stakind1, stakind2, stakind3, stakind4, stakind5, staop1, staop2, staop3, staop4, staop5,\n"\
                     "    stanumbers1::text, stanumbers2::text, stanumbers3::text, stanumbers4::text, stanumbers5::text,\n"\
                     "    stavalues1::text, stavalues2::text, stavalues3::text, stavalues4::text, stavalues5::text\n"\
                     "  FROM pg_statistic_ext s JOIN pg_class c ON(c.oid = s.starelid AND c.relnamespace <>11) \n"\
                     "    JOIN pg_namespace rn ON(rn.oid = c.relnamespace AND rn.nspname NOT IN('pg_toast', 'pg_catalog', 'information_schema'))\n"\
                     "  WHERE false;\n\n" % (restore_scheam_name, restore_scheam_name, restore_scheam_name, restore_scheam_name, restore_scheam_name))

            fp.write("set client_encoding = '%s'; \nCOPY %s.pg_class_stat FROM STDIN WITH ENCODING '%s';\n" \
                            % (encoding, restore_scheam_name, encoding))
            fp.flush()

            sql = "set client_encoding = '%s'; " \
                  "COPY (SELECT relname, nspname, relpages, reltuples, relallvisible " \
                  "FROM pg_class c JOIN pg_namespace n ON(c.relnamespace = n.oid " \
                  "AND n.nspname NOT IN ('pg_toast', 'pg_catalog', 'information_schema')))" \
                  "TO STDOUT WITH ENCODING '%s';" % (encoding, encoding)
            self.logger.debug("SQL for copying pg_class data: %s." % sql)
            (status, output) = self.execSQLCommand(sql, self.user, "", port, dbname, option="--quiet")
            if(status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % sql + " Error:\n%s" % output)

            if(len(output)):
                fp.writelines(output)
                fp.write("\n")
            fp.write("\\.\n;")

            fp.write("\n\n\n--\n-- SINGLE COLUMN STATS\n--\n")
            fp.write("set client_encoding = '%s'; \nCOPY %s.pg_statistic_stat FROM STDIN WITH ENCODING '%s';\n" \
                            % (encoding, restore_scheam_name, encoding))
            fp.flush()

            # copy single column pg_statistic
            sql = "set client_encoding = '%s'; COPY (SELECT relname, rn.nspname as relnspname, " \
                  "staattnum, starelkind,  typname,  tn.nspname as typnspname, stainherit, stanullfrac, " \
                  "stawidth, stadistinct, stakind1, stakind2, stakind3, stakind4, stakind5, " \
                  "staop1, staop2, staop3, staop4, staop5, stanumbers1::text, " \
                  "stanumbers2::text, stanumbers3::text, stanumbers4::text, stanumbers5::text, " \
                  "stavalues1::text, stavalues2::text, stavalues3::text, stavalues4::text, stavalues5::text, typelem, stadndistinct " \
                  "FROM pg_statistic s JOIN pg_class c ON(c.oid = s.starelid AND c.relnamespace <>11) " \
                  "JOIN pg_attribute a ON (a.attrelid = s.starelid AND a.attnum = s.staattnum) " \
                  "JOIN pg_namespace rn ON(rn.oid = c.relnamespace AND rn.nspname " \
                  "NOT IN('pg_toast', 'pg_catalog', 'information_schema', 'cstore')) " \
                  "JOIN pg_type t ON (t.oid = a.atttypid) JOIN pg_namespace tn ON(tn.oid = t.typnamespace) " \
                  "WHERE s.starelid NOT IN(SELECT c.oid FROM pg_class c, pg_attribute a, pg_type t " \
                  "WHERE c.relnamespace <>11 AND c.oid=a.attrelid AND c.relnamespace IN(SELECT rn.oid FROM pg_namespace rn " \
                  "WHERE rn.nspname NOT IN('pg_toast', 'pg_catalog', 'information_schema', 'cstore')) AND t.oid = a.atttypid " \
                  "AND t.typname IN('tsvector', 'tsquery', 'gtsvector') GROUP BY c.oid))" \
                  "TO STDOUT WITH ENCODING '%s';" % (encoding, encoding)
            self.logger.debug("SQL for copying pg_statistic data: %s." % sql)
            (status, output) = self.execSQLCommand(sql, self.user, "", port, dbname, option="--quiet")
            if(status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % sql + " Error:\n%s" % output)
            if(len(output)):
                fp.writelines(output)
                fp.write("\n")
            fp.write("\\.\n;")

            fp.write("\n\n\n--\n-- MULTI COLUMN STATS\n--\n")
            fp.write("set client_encoding = '%s'; \nCOPY %s.pg_statistic_stat_ext FROM STDIN WITH ENCODING '%s';\n" \
                            % (encoding, restore_scheam_name, encoding))
            fp.flush()

            # copy multi column pg_statistic
            sql = "set client_encoding = '%s'; COPY (SELECT relname, rn.nspname as relnspname, " \
                  "stakey, starelkind, stainherit, stanullfrac, stawidth, stadistinct, " \
                  "stadndistinct, stakind1, stakind2, stakind3, stakind4, stakind5, " \
                  "staop1, staop2, staop3, staop4, staop5, stanumbers1::text, " \
                  "stanumbers2::text, stanumbers3::text, stanumbers4::text, stanumbers5::text, " \
                  "stavalues1::text, stavalues2::text, stavalues3::text, stavalues4::text, stavalues5::text " \
                  "FROM pg_statistic_ext s JOIN pg_class c ON(c.oid = s.starelid AND c.relnamespace <>11) " \
                  "JOIN pg_namespace rn ON(rn.oid = c.relnamespace AND rn.nspname " \
                  "NOT IN('pg_toast', 'pg_catalog', 'information_schema', 'cstore'))) " \
                  "TO STDOUT WITH ENCODING '%s';" % (encoding, encoding)
            self.logger.debug("SQL for copying pg_statistic_ext data: %s." % sql)
            (status, output) = self.execSQLCommand(sql, self.user, "", port, dbname, option="--quiet")
            if(status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % sql + " Error:\n%s" % output)
            if(len(output)):
                fp.writelines(output)
                fp.write("\n")

            fp.write("\\.\n;\n\n\n--\n-- EXECUTE STATS\n--\n" \
                     "analyze pg_catalog.pg_statistic;\n" \
                     "analyze pg_catalog.pg_class;\n" \
                     "analyze pg_catalog.pg_type;\n" \
                     "analyze pg_catalog.pg_attribute; \n" \
                     "analyze pg_catalog.pg_namespace; \n\n")
            fp.flush()

            fp.write("\n\n\n--\n-- UPDATE pg_class\n--\n" \
                     "UPDATE pg_class SET (relpages, reltuples, relallvisible) = (t.relpages, t.reltuples, t.relallvisible) FROM %s.pg_class_stat t \n" \
                     "    WHERE oid = (SELECT oid FROM pg_class WHERE relnamespace = (SELECT oid FROM pg_namespace WHERE nspname = t.nspname) and relname = t.relname);\n\n" \
                     % restore_scheam_name)

            fp.write("\n\n\n--\n-- INSERT pg_statistic\n--\n")


            fp.write("INSERT INTO pg_statistic (starelid, starelkind, staattnum, stainherit, stanullfrac, stawidth, stadistinct, stadndistinct, \n" \
               "    stakind1, stakind2, stakind3, stakind4, stakind5, staop1, staop2, staop3, staop4, staop5, stanumbers1, stanumbers2, stanumbers3, stanumbers4, stanumbers5, \n" \
               "    stavalues1, stavalues2, stavalues3, stavalues4, stavalues5) \n"\
               "    SELECT (SELECT oid FROM pg_class WHERE relname=t.relname AND relnamespace=(SELECT oid FROM pg_namespace WHERE nspname=t.relnspname)) relid, t.starelkind, \n" \
               "    t.staattnum, t.stainherit, t.stanullfrac, t.stawidth, t.stadistinct, t.stadndistinct, stakind1, stakind2, stakind3, stakind4, stakind5, staop1, staop2, staop3, staop4, staop5, \n" \
               "    stanumbers1::real[], stanumbers2::real[], stanumbers3::real[], stanumbers4::real[], stanumbers5::real[], \n" \
               "    array_in(stavalues1::cstring, CASE WHEN stakind1=4 AND typelem <> 0 THEN typelem ELSE (SELECT oid FROM pg_type WHERE typname=t.typname AND typnamespace=(SELECT oid FROM pg_namespace WHERE nspname=t.typnspname) LIMIT 1) END, -1), \n" \
               "    array_in(stavalues2::cstring, CASE WHEN stakind2=4 AND typelem <> 0 THEN typelem ELSE (SELECT oid FROM pg_type WHERE typname=t.typname AND typnamespace=(SELECT oid FROM pg_namespace WHERE nspname=t.typnspname) LIMIT 1) END, -1), \n" \
               "    array_in(stavalues3::cstring, CASE WHEN stakind3=4 and typelem <> 0 THEN typelem ELSE (SELECT oid FROM pg_type WHERE typname=t.typname AND typnamespace=(SELECT oid FROM pg_namespace WHERE nspname=t.typnspname) LIMIT 1) END, -1), \n" \
               "    array_in(stavalues4::cstring, CASE WHEN stakind4=4 and typelem <> 0 THEN typelem ELSE (SELECT oid FROM pg_type WHERE typname=t.typname AND typnamespace=(SELECT oid FROM pg_namespace WHERE nspname=t.typnspname) LIMIT 1) END, -1), \n" \
               "    array_in(stavalues5::cstring, CASE WHEN stakind5=4 and typelem <> 0 THEN typelem ELSE (SELECT oid FROM pg_type WHERE typname=t.typname AND typnamespace=(SELECT oid FROM pg_namespace WHERE nspname=t.typnspname) LIMIT 1) END, -1) \n" \
               "    FROM %s.pg_statistic_stat t WHERE relid IS NOT NULL;\n\n" % restore_scheam_name)


            fp.write("\n\n\n--\n-- INSERT pg_statistic_ext\n--\n")


            fp.write("INSERT INTO pg_statistic_ext (starelid, starelkind, stakey, stainherit, stanullfrac, stawidth, stadistinct, stadndistinct, \n" \
               "    stakind1, stakind2, stakind3, stakind4, stakind5, staop1, staop2, staop3, staop4, staop5, stanumbers1, stanumbers2, stanumbers3, stanumbers4, stanumbers5, \n" \
               "    stavalues1, stavalues2, stavalues3, stavalues4, stavalues5) \n"\
               "    SELECT (SELECT oid FROM pg_class WHERE relname=t.relname AND relnamespace=(SELECT oid FROM pg_namespace WHERE nspname=t.relnspname)) relid, t.starelkind, \n" \
               "    t.stakey, t.stainherit, t.stanullfrac, t.stawidth, t.stadistinct, t.stadndistinct, stakind1, stakind2, stakind3, stakind4, stakind5, staop1, staop2, staop3, staop4, staop5, \n" \
               "    stanumbers1::real[], stanumbers2::real[], stanumbers3::real[], stanumbers4::real[], stanumbers5::real[], \n" \
               "    complex_array_in(stavalues1::cstring, relid, t.stakey), \n" \
               "    complex_array_in(stavalues2::cstring, relid, t.stakey), \n" \
               "    complex_array_in(stavalues3::cstring, relid, t.stakey), \n" \
               "    complex_array_in(stavalues4::cstring, relid, t.stakey), \n" \
               "    complex_array_in(stavalues5::cstring, relid, t.stakey) \n" \
               "    FROM %s.pg_statistic_stat_ext t WHERE relid IS NOT NULL;\n\n" % restore_scheam_name)


            # Connect to the database and find the table name and schema name for the table 
            # which containing the tsvector, gtsvector and tsquery type column.or the index depends on 
            # the tsvector, gtsvector and tsquery type column
            # Parses the execution results, create the analyze statement, and write it into gs_upgrade_dump_statistic.sql
            # do analyze for the table
            fp.write("\n--\n-- EXECUTE ANALYZE\n--\n")
            sql = "SELECT c.relname, rn.nspname " \
                  "FROM pg_class c " \
                  "JOIN pg_namespace rn ON(c.relnamespace <> 11 AND c.relkind <> 'i' AND rn.oid = c.relnamespace AND rn.nspname " \
                  "NOT IN('pg_toast', 'pg_catalog', 'information_schema', 'cstore')) " \
                  "JOIN pg_attribute a ON(c.oid=a.attrelid) " \
                  "JOIN pg_type t ON(t.oid = a.atttypid and t.typname IN('tsvector', 'tsquery', 'gtsvector')) " \
                  "GROUP BY c.relname, rn.nspname order by c.relname"
            self.logger.debug("SQL for doing analyze for the table: %s." % sql)
            (status, output) = self.execSQLCommand(sql, self.user, "", port, dbname)
            if(status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % sql + " Error:\n%s" % output)

            output = output.strip()
            for line in output.split('\n'):
                if(len(line.strip()) and line.split('|')==2):
                    relname = line.split('|')[0].strip()
                    nspname = line.split('|')[1].strip()
                    fp.write("analyze \"%s\".\"%s\";\n" % (nspname, relname))
            fp.flush()

            # do analyze for the table which the index depends on
            sql = "SELECT cl.relname, np.nspname " \
                  "FROM pg_class cl, pg_namespace np " \
                  "WHERE np.oid = cl.relnamespace AND " \
                  "cl.oid IN (SELECT d.refobjid " \
                  "FROM pg_class c " \
                  "JOIN pg_namespace rn ON(c.relnamespace <> 11 AND c.relkind = 'i' AND rn.oid = c.relnamespace AND rn.nspname " \
                  "NOT IN('pg_toast', 'pg_catalog', 'information_schema', 'cstore')) " \
                  "JOIN pg_attribute a ON(c.oid=a.attrelid) " \
                  "JOIN pg_type t ON(t.oid = a.atttypid and t.typname IN('tsvector', 'tsquery', 'gtsvector')) " \
                  "JOIN pg_depend d ON(d.classid = 'pg_class'::regclass AND d.refobjsubid > 0 AND d.deptype = 'a' AND d.objid = c.oid)) " \
                  "GROUP BY cl.relname, np.nspname order by cl.relname"
            self.logger.debug("SQL for doing analyze for the table which the index depends on: %s." % sql)
            (status, output) = self.execSQLCommand(sql, self.user, "", port, dbname)
            if(status != 0):
                raise Exception(ErrorCode.GAUSS_513["GAUSS_51300"] % sql + " Error:\n%s" % output)

            output = output.strip()
            for line in output.split('\n'):
                if(len(line.strip()) and line.split('|')==2):
                    relname = line.split('|')[0].strip()
                    nspname = line.split('|')[1].strip()
                    fp.write("analyze \"%s\".\"%s\";\n" % (nspname, relname))

            fp.write("\nDROP SCHEMA IF EXISTS %s CASCADE;\n\n" % restore_scheam_name)
            fp.flush()

        fp.write("\n\n\n--\n-- DO CHECKPOINT as this is last operation\n--\nCHECKPOINT;\n\n")
        fp.close()

    def changedDumpSqlFile(self, sqlFile):
        """
        function: change dump sql file
        input : sqlFile
        output: NA
        """
        # clean the tmp file
        newSqlFile = "%s_bak" % sqlFile
        DefaultValue.cleanTmpFile(newSqlFile)

        fpOld = None
        fpNew = None
        number = 0
        # split flag
        splitFormat = "-- Name: "
        # read sqlFile, change it, write to new_sqlFile
        # Use "-- Name: " as a delimiter to count the number of records
        try:
            fpOld = open(sqlFile, "r")
            g_file.createFileInSafeMode(newSqlFile)
            fpNew = open(newSqlFile, "w")

            strLine = fpOld.readline()
            if (not strLine):
                raise Exception("Failed to read dump file [%s]." % sqlFile)

            fpNew.write("START TRANSACTION;\n")
            fpNew.write(strLine)
            fpNew.flush()

            while True:
                strLine = fpOld.readline()
                if (not strLine):
                    break
                if (strLine.strip().startswith(splitFormat)):
                    number = number + 1
                    if (int(number)%1000 == int(0)):
                        fpNew.write("COMMIT;\n")
                        fpNew.write(strLine)
                        fpNew.write("START TRANSACTION;\n")
                    else:
                        fpNew.write(strLine)
                else:
                    fpNew.write(strLine)
            fpNew.write("COMMIT;\n")
            fpNew.flush()

            DefaultValue.cleanTmpFile(sqlFile, fpOld)
            if (fpNew):
                fpNew.close()
            
            # give permission to newSqlFile
            g_file.changeMode("shell", DefaultValue.KEY_FILE_MODE, newSqlFile)

            # remove old sql file, rename new sql file to old sql file
            os.rename(newSqlFile, sqlFile)

        except Exception, e:
            if (fpOld):
                fpOld.close()
            DefaultValue.cleanTmpFile(newSqlFile, fpNew)
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50230"] % "dump file" + " Error:\n%s" % str(e))
        
    def fixDumpFile(self, dumpSqlFile, port):
        """
        function: get the dump file prefix
                  1.check the dump directory have file or not
                  2.traverse the directory get dump file prefix
        input : dumpSqlFile, port
        output: NA
        """
        sqlFilePre = dumpSqlFile[:-4]
        databases = self.getAllDatabase()
        sqlFile = ""
        for database in databases:
            sqlFile = "%s_%s.sql" % (sqlFilePre, database)
            self.changedDumpSqlFile(sqlFile)
        
    def IgnorePatternFunction_InitialUser(self):
        """
        function: ignore pattern function initial user
        input : NA
        output: re compile result
        """
        # ignore user information 
        INITIAL_USER_IGNORE_PATTERN = "^ALTER (ROLE|USER) (%s|\"%s\"|'%s')" % (self.user, self.user, self.user)
        return re.compile(INITIAL_USER_IGNORE_PATTERN)

    def IgnorePatternFunction_CstoreSchema(self):
        """
        function: ignore pattern function cstore schema
        input : NA
        output: re compile result
        """
        # ignore shema information 
        CSTORE_SCHEMA_IGNORE_PATTERN = "^CREATE SCHEMA cstore;"
        return re.compile(CSTORE_SCHEMA_IGNORE_PATTERN)

    def getIgnorePatterns(self):
        """
        function: match the IgnorePatternFunction_ 
                  and append it to listname
        input : NA
        output: listname
        """
        listname = []
        for i in dir(self):
            if (i.startswith("IgnorePatternFunction_")):
                self.logger.debug("Find function %s." % i)
                listname.append(getattr(self, i)())
        return listname

    def needToBeClean(self, line, patterns):
        """
        function: check the sql need to be clean or not
        input : line, patterns
        output: True/False
        """
        # verify the SQL needs to be cleaned
        line = line.strip()
        for pattern in patterns:
            if (pattern.match(line) is not None):
                return True
        return False

    def needToBeReplace(self, line):
        """
        function: check the sql need to be replace or not
        input : line
        output: True/False
        """
        line = line.strip()
        # replace the create user SQL command
        PATTERN = "^CREATE (ROLE|USER) (%s|\"%s\"|'%s') PASSWORD" % (self.user, self.user, self.user)
        pattern = re.compile(PATTERN)
        if (pattern.match(line) is not None):
            return True
        return False

    def modifySchemaFile(self, desc, filename, isAlterUser = True):
        """
        function: clean the schema file
        input : desc, filename
            isAlterUser, The value isAlterUser of True indicates that the password must be updated when performing an increase in CN, gs_expand and gs_replace
        output: NA
        """
        self.logger.log("Cleaning %s schema file." % desc)
        self.logger.log("File: %s." % filename)
        try:
            patterns = self.getIgnorePatterns()
            fp = open(filename, "r")
            readLines = fp.readlines()
            fp.close()

            writeLines = []
            for line in readLines:
                if (self.needToBeClean(line, patterns)):
                    self.logger.log("This line will not been added to schema file:\n%s." % (line.strip("\n")))
                    continue
                if (self.needToBeReplace(line)):
                    if(isAlterUser):
                        self.logger.log("This line will be replaced in schema file:\n%s." % (line.strip("\n")))
                        replace_pattern = "CREATE (ROLE|USER) (%s|\"%s\"|'%s') PASSWORD" % (self.user, self.user, self.user)
                        line = re.sub(replace_pattern, "UPDATE pg_catalog.pg_authid SET rolpassword =", line)
                        line = line.replace(';', ' WHERE oid = 10;')
                        self.logger.log("Replaced line:\n%s." % (line.strip("\n")))
                        writeLines.append(line)
                    else:
                        self.logger.log("This line will not been added to schema file:\n%s." % (line.strip("\n")))
                    continue
                writeLines.append(line)

            fp = open(filename, "w")
            fp.writelines(writeLines)
            fp.close()
            
            g_file.changeMode("shell", DefaultValue.KEY_FILE_MODE, filename)
        except Exception as e:
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50207"] % ("[%s] schema file" % desc) + " Error:\n%s"% str(e))
        self.logger.log("Successfully cleaned [%s] schema file." % desc)
        
    def createTableSpaceDir(self, sqlFile):
        """
        function: Check if table space dir exists before restore
                  if not, create it first.
        input : sqlFile
        output: NA
        """
        # get 'CREATE TABLESPCE' SQL command from sqlFile
        #     0        get it
        #     256      there is no tablespace
        #     other    failed to execute the shell command
        matchMode = '^ *CREATE TABLESPACE .* LOCATION .*'
        cmd = "%s -G '%s' %s 2>/dev/null" % (g_Platform.getGrepCmd(), matchMode, sqlFile)
        self.logger.debug("Command for searching tablespace directory: %s" % cmd)
        (status, output) = commands.getstatusoutput(cmd)
        if(status == 0):
            pass
        elif(status == 256):
            self.logger.debug("There is no tablespace in the current sql file.")
            return
        else:
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50219"] % "tablespace directory" + " Error:\n%s" % output)

        # parse the result. If the string contains an escape character, the result wil add an escaped character again.
        sqlList = output.split('\n')
        dirNum = len(sqlList)
        for i in range(dirNum):
            if(re.findall(matchMode, sqlList[i]) == []):
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50222"] % sqlFile)

        # the tablespace path is between ''
        # and maybe the tablespace path including ' ' or '\'
        # so we should do special treatment for it
        for i in range(len(sqlList)):
            tablespaceDir = ""
            number = sqlList[i].find('\'')
            if (number < 0 ):
                continue
            resList = sqlList[i][number:].strip()
            endres  = resList[1:].find('\'')
            tablespaceDir = resList[1:endres+1]
            # the tablespace path string contains two levels of escaping, undoing the layer
            tablespaceDir = tablespaceDir.replace('\\\\','\\')
            # create the tablespace directory
            # grant owner to tablespaceDir
            # give permission to tablespaceDir
            self.logger.debug("Tablespace directory is %s." % tablespaceDir)
            g_file.createDirectory(tablespaceDir)
            g_file.changeMode("shell", DefaultValue.KEY_FILE_MODE, tablespaceDir)
            g_file.changeOwner("shell", self.user, tablespaceDir)

            # check tablespaceDir is writeable
            if (not g_file.checkDirWriteable(tablespaceDir)):
                raise Exception(ErrorCode.GAUSS_501["GAUSS_50102"] % ( tablespaceDir, "the database administrator"))

            if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_COODINATOR):
                instancename = "cn_%s" % self.instInfo.instanceId
            else:
                instancename = "dn_%s_%s" % (self.instInfo.instanceId, str(int(self.instInfo.instanceId) + 1))
            instanceTblspcDir = "%s/%s_%s" % (tablespaceDir, DefaultValue.TABLESPACE_VERSION_DIRECTORY, instancename)
            
            
            # remove tablespace directory
            if(os.path.exists(instanceTblspcDir)):
                g_file.removeDirectory(instanceTblspcDir)
        
    def restoreInstance(self):
        """
        function: Build new instance.Restore from dump file
        input : sqlFile
        output: NA
        """
        sqlFile = ""
        if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_COODINATOR):
            sqlFile = self.schemaCoordinatorFile
            jobFile = self.coordinatorJobDataFile
            statFile = self.coordinatorStatisticsDataFile
        elif (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_DATANODE):
            sqlFile = self.schemaDatanodeFile
        else:
            self.logger.debug("Don't need to restore: %s." % self.instInfo.datadir)
            return

        # creating tablespace
        self.logger.debug("Creating tablespace.")
        self.createTableSpaceDir(sqlFile)

        # restore the global object using dump file
        (status, output) = self.execSQLFile(sqlFile, self.user, "", self.instInfo.port, DefaultValue.DEFAULT_DB_NAME)
        self.logger.debug(output)
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50220"] % "instance" + " Data: \n%s. Output: \n%s." % (self.instInfo.datadir, output))

        # get the database which we need to be restored
        sqlFilePre = sqlFile[:-4]
        databases = self.getAllDatabase()

        if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_COODINATOR):
            for database in databases:
                sqlFile = "%s_%s.sql" % (sqlFilePre, database)
                database = database.replace('$', '\$')
                (status, output) = self.execSQLFile(sqlFile, self.user, "", self.instInfo.port, DefaultValue.DEFAULT_DB_NAME)
                if (status != 0):
                    raise Exception(ErrorCode.GAUSS_502["GAUSS_50220"] % "instance" + " Data: \n%s. Output: \n%s." % (self.instInfo.datadir, output))
        elif (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_DATANODE):
            fileList = []
            tmpDir = DefaultValue.getTmpDirFromEnv()
            fileNames = os.listdir(tmpDir)
            if(len(fileNames)>0):
                for name in fileNames:
                    if(name.startswith("dump_output_datanode")and name.endswith(".sql")):
                        fileList.append(name)
            for tableFile in fileList:
                database = (tableFile[:-4]).split('dump_output_datanode_')[1]
                sqlFile = "%s/%s" % (tmpDir, tableFile)
                if (database == 'template1'):
                    (status, output) = self.execSQLFile(sqlFile, self.user, "", self.instInfo.port, database)
                else:
                    (status, output) = self.execSQLFile(sqlFile, self.user, "", self.instInfo.port, DefaultValue.DEFAULT_DB_NAME)
                self.logger.debug(output)
                if (status != 0):
                    raise Exception(ErrorCode.GAUSS_502["GAUSS_50220"] % "instance" + " Data: \n%s. Output: \n%s." % (self.instInfo.datadir, output))
            
        if (self.instInfo.instanceRole == DefaultValue.INSTANCE_ROLE_COODINATOR):
            # restore the job data using jobFile 
            (status, output) = self.execSQLFile(jobFile, self.user, "", self.instInfo.port, DefaultValue.DEFAULT_DB_NAME)
            self.logger.debug("Complete to restore the job data. Output:\n%s" % output)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50220"] % "instance" + " Data: \n%s. Output: \n%s." % (self.instInfo.datadir, output))

            # restore the statistics data
            (status, output) = self.execSQLFile(statFile, self.user, "", self.instInfo.port, DefaultValue.DEFAULT_DB_NAME)
            self.logger.debug("Complete to restore the statistics data. Output:\n%s" % output)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50220"] % "instance" + " Data: \n%s. Output: \n%s." % (self.instInfo.datadir, output))



