#!/usr/bin/env python
#-*- coding:utf-8 -*-


try:
    import sys
    reload(sys)
    sys.setdefaultencoding('utf-8')
    import os
    import psutil
    from gspylib.common.Common import DefaultValue
    from gspylib.inspection.common import SharedFuncs
    from gspylib.inspection.common.CheckItem import BaseItem
    from gspylib.inspection.common.CheckResult import ResultStatus
    from gspylib.hardware.gsdisk import g_disk
    from gspylib.common.Common import ClusterCommand
    from gspylib.common.ErrorCode import ErrorCode
except ImportError, e:
    raise Exception("[GAUSS-52200] : Unable to import module: %s." % str(e))


class CheckSpaceForShrink(BaseItem):
    def __init__(self):
        super(CheckSpaceForShrink, self).__init__(self.__class__.__name__)
        self.Threshold_NG = None
        self.cluster_LC = None

    def preCheck(self):
        # check current node contains cn instances if not raise  exception
        super(CheckSpaceForShrink, self).preCheck()
        # check the threshold was set correctly
        if (not (self.threshold.has_key('Threshold_NG') )):
            raise Exception("The threshold Threshold_NG can not be empty.")
        self.Threshold_NG = int(self.threshold['Threshold_NG'])
        
    def obtainDataDir(self, nodeInfo):
        dataDirList = {}
        for inst in nodeInfo.datanodes:
            if(inst.hostname not in self.ShrinkNodes):
                dataDirList[inst.datadir] = ["DN",0]
        return dataDirList

    def obtainDiskDir(self):
        cmd = "df -h -P | awk '{print $NF}'"
        output = SharedFuncs.runShellCmd(cmd)
        allDiskPath = output.split('\n')[1:]
        return allDiskPath

    def initCluster_LC(self):
        if(self.LCName):
            from gspylib.common.DbClusterInfo import dbClusterInfo
            self.cluster_LC = dbClusterInfo()
            filename = DefaultValue.getEnv("GAUSSHOME") + "/bin/%s.cluster_static_config"%self.LCName
            self.cluster_LC.initFromStaticConfig(self.user,filename,True)
        else:
            self.cluster_LC = self.cluster

    def checkLcGroupNameExist(self, lcName ,cnport):
        """
        function: Check if the logical cluster name exists
        input :NA
        output:NA
        """
        sql = "SELECT count(*) FROM pg_catalog.pgxc_group \
               WHERE group_name='%s' and group_kind = 'v';" % lcName
        (status, output) = ClusterCommand.remoteSQLCommand(sql,
                                                           self.user, 
                                                           cnport.hostname,
                                                           cnport.port,
                                                           ignoreError = False)
        if (status != 0 or not output.isdigit()):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % sql + " Error:\n%s" % str(output))
        elif (int(output) == 0):
            raise Exception(ErrorCode.GAUSS_500["GAUSS_50410"] + " The specified logical cluster name does not exist in the cluster.")

    def doCheck(self):
        flag = "Normal"
        resultStr = ""
        DiskInfoDict = {}
        pathDisk = {}
        diskList = {}
        pathList = []
        #get cn port
        cns = None
        for dbnode in self.cluster.dbNodes:
            cns = self.cluster.getDbNodeByName(dbnode.name).coordinators
            if(cns):
                break
        if(not cns):
            raise Exception("Error to get the coordinator")
        cnport = cns[0]
        #check if the last shrink was completed
        sql_query = "SELECT count(*) FROM pg_catalog.pgxc_group \
                     WHERE group_kind='v' and in_redistribution='t'"
        (status,output) = ClusterCommand.remoteSQLCommand(sql_query,
                                                           self.user, 
                                                           cnport.hostname,
                                                           cnport.port,
                                                           ignoreError = False)
        if (status != 0 or not output.isdigit()):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % sql_query + " Error:\n%s" % str(output))
        elif (int(output) == 1):
            self.result.rst = ResultStatus.OK
            return
        elif (int(output) != 0):
            raise Exception(ErrorCode.GAUSS_500["GAUSS_50410"] + " More than one cluster in redistributing.")
        #get dn's path list except shrink dns
        self.checkLcGroupNameExist(self.LCName,cnport)
        self.initCluster_LC()
        if (self.cluster):
            nodeinfo = self.cluster_LC.getDbNodeByName(self.host)
            if(nodeinfo):
                pathDisk = self.obtainDataDir(nodeinfo)
                pathList = pathDisk.keys()
            #no dn in this host , no need to check
            if(not pathList):
                self.result.rst = ResultStatus.NA
                return
        else:
            pathList = self.obtainDiskDir()
        #get the max dn size in cluster
        sql_query = "select pgxc_max_datanode_size('%s')"%self.LCName
        (status,output) = ClusterCommand.remoteSQLCommand(sql_query,
                                                           self.user, 
                                                           cnport.hostname,
                                                           cnport.port,
                                                           ignoreError = False)
        if(0 != status or "" == output):
            raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % sql_query + " Error:\n%s" % str(output))
        dnsize_max = int(output.split("\n")[0].strip())
        #dns in each host is same,so data added in each dn is dnsize_max*num_shrink/num_remain
        dnname_all = self.cluster_LC.getClusterNodeNames()
        num_shrink = len(self.ShrinkNodes.split(','))
        num_remain = len(dnname_all) - num_shrink
        deltasize = float(dnsize_max) * num_shrink / num_remain
        '''
        get all dn's disk and dn num in each disk
        if a disk has N dns 
        the data added of this disk is N*deltasize
        '''
        for path in pathList:
            diskName = g_disk.getMountPathByDataDir(path)
            if(diskName not in diskList.keys()):
                diskList[diskName] = [path,1]
            else:
                diskList[diskName][1] += 1
        #get the usage after shrink
        if diskList:
            for key,value in diskList.items():
                usageInfo = round(g_disk.getDiskSpaceForShrink(value[0],deltasize*value[1]), 2)
                diskInfo = "%s %s%%" % (diskName, usageInfo)
                DiskInfoDict[usageInfo] = diskInfo
                rateNum = usageInfo
            if (rateNum > self.Threshold_NG):
                resultStr += "The usage of the device disk space[%s:%d%%] cannot be greater than %d%%.\n" % (
                            diskName, rateNum, self.Threshold_NG)
                flag = "Error"
        else:
            resultStr += "error to get disk list of dns"
            flag = "Error"
        #set result
        self.result.val = resultStr
        if (flag == "Normal"):
            self.result.rst = ResultStatus.OK
        else:
            self.result.rst = ResultStatus.NG
        #set raw and val
        if(DiskInfoDict):
            keys = DiskInfoDict.keys()
            keys.sort()
            MaxDisk = map(DiskInfoDict.get, keys)[-1]
            MinDisk = map(DiskInfoDict.get, keys)[0]
            self.result.val += "\nDisk     Filesystem spaceUsage\nMax free %s\nMin free %s" % (MaxDisk, MinDisk)
            for diskInfo in map(DiskInfoDict.get, keys):
                self.result.raw += "\n%s" % diskInfo
