#!/usr/bin/env python
#-*- coding:utf-8 -*-
#############################################################################
#Copyright (c): 2010-2020, Huawei Tech. Co., Ltd.
#FileName     : gsplatform.py
#Version      : Gauss200 
#Date         : 2018-03-26
#Description  : gsplatform.py is a utility to do something for platform information.
#############################################################################

""" The following platform framework is used to handle any differences between
    the platform's we support.  The GenericPlatform class is the base class
    that a supported platform extends from and overrides any of the methods
    as necessary.
"""

try:
    import os
    import sys
    import commands
    import platform
    import socket
    import time
    localDirPath = os.path.dirname(os.path.realpath(__file__))
    sys.path.insert(0, localDirPath + "/../../../lib")
    try:
        import psutil
    except ImportError, e:
        #mv psutil mode .so file by python version
        pythonVer = sys.version[:3]
        psutilLinux = os.path.join(localDirPath, "./../../../lib/psutil/_psutil_linux.so")
        psutilPosix = os.path.join(localDirPath, "./../../../lib/psutil/_psutil_posix.so")
        psutilLinuxBak = "%s_%s" % (psutilLinux, pythonVer)
        psutilPosixBak = "%s_%s" % (psutilPosix, pythonVer)

        glo_cmd = "rm -rf '%s' && cp -r '%s' '%s' " % (psutilLinux, psutilLinuxBak, psutilLinux)
        glo_cmd += " && rm -rf '%s' && cp -r '%s' '%s' " % (psutilPosix, psutilPosixBak, psutilPosix)
        psutilFlag = True
        for psutilnum in range(3):
            (status_mvPsutil, output_mvPsutil) = commands.getstatusoutput(glo_cmd)
            if(status_mvPsutil != 0):
                psutilFlag = False
                time.sleep(1)
            else:
                psutilFlag = True
                break
        if(not psutilFlag):
            print ("Failed to execute cmd: %s. Error:\n%s" % (glo_cmd, output_mvPsutil))
            sys.exit(1)
        #del error import and reload psutil
        del sys.modules['psutil._common']
        del sys.modules['psutil._psposix']
        import psutil
    sys.path.append(sys.path[0] + "/../../")
    from gspylib.common.ErrorCode import ErrorCode
    localDirPath = os.path.dirname(os.path.realpath(__file__))
    sys.path.insert(0, localDirPath + "/../../../lib/netifaces/")
    sys.path.append(localDirPath + "/../inspection/lib/netifaces/")
    try:
        from netifaces import interfaces, ifaddresses, AF_INET, AF_INET6
    except ImportError, e:
        #get python unicode value. The current environment python is compiled with UCS2 or UCS4.
        #1114111 is UCS4
        #65535 is UCS2
        flagNum = 4 if sys.maxunicode == 1114111 else 2
        omToolsNetifacesPath = os.path.join(localDirPath, "./../../../lib/netifaces/netifaces.so")
        inspectToolsNetifacesPath = os.path.join(localDirPath, "./../../../script/gspylib/inspection/lib/netifaces/netifaces.so")
        newPythonDependNetifacesPath = "%s_UCS%d" % (omToolsNetifacesPath, flagNum)
        glo_cmd = "rm -f '%s' && cp -f -p '%s' '%s' " % (omToolsNetifacesPath
                                                        , newPythonDependNetifacesPath
                                                        , omToolsNetifacesPath)
        glo_cmd += " && rm -f '%s' && cp -f -p '%s' '%s' " % (inspectToolsNetifacesPath
                                                            , newPythonDependNetifacesPath
                                                            , inspectToolsNetifacesPath)
        flagExce = True
        for retryNum in range(3):
            (statusExec, outputExec) = commands.getstatusoutput(glo_cmd)
            if(statusExec != 0):
                flagExce = False
                time.sleep(1)
            else:
                flagExce = True
                break
        if(not flagExce):
            raise Exception("Failed to execute cmd: %s. Error:\n%s" % (glo_cmd, outputExec))
        from netifaces import interfaces, ifaddresses, AF_INET, AF_INET6
except Exception, e:
    raise Exception("[GAUSS-52200] : Unable to import module: %s." % str(e))

#---------------platforms--------------------
#global variable for our platform
SUSE = "suse"
REDHAT = "redhat"
CENTOS = "centos"
EULEROS = "euleros"
ORACLE = "oracle"
KYLIN = "kylin"
SUPPORT_WHOLE_PLATFORM_LIST = [SUSE, REDHAT, CENTOS, EULEROS, ORACLE, KYLIN]
# RedhatX platform
SUPPORT_RHEL_SERIES_PLATFORM_LIST = [REDHAT, CENTOS]
SUPPORT_RHEL6X_VERSION_LIST = ["6.4", "6.5", "6.6", "6.7", "6.8", "6.9"]
SUPPORT_RHEL7X_VERSION_LIST = ["7.0", "7.1", "7.2", "7.3", "7.4", "7.5", "7.6"]
SUPPORT_RHEL_SERIES_VERSION_LIST = (SUPPORT_RHEL6X_VERSION_LIST + SUPPORT_RHEL7X_VERSION_LIST)
# EulerOS 2.3 -> 2.0 SP3
SUPPORT_EULEROS_VERSION_LIST = ["2.0"]
SUPPORT_ORACLE_VERSION_LIST = ["7.4"]
SUPPORT_KYLIN_VERSION_LIST = ["4.0", "7.5", "7.6"]
# SuSE platform
SUSE11 = "11"
SUSE12 = "12"
SUPPORT_SUSE_VERSION_LIST = [SUSE11, SUSE12]
SUPPORT_SUSE11X_VERSION_LIST = ["1", "2", "3", "4"]
SUPPORT_RHEL12X_VERSION_LIST = ["0", "1", "2", "3"]
BIT_VERSION = "64bit"

#---------------command path--------------------
CMD_PATH = ['/bin', '/usr/local/bin', '/usr/bin', '/sbin', '/usr/sbin']
CMD_CACHE = {}
BLANK_SPACE = " "
COLON = ":"

#######################################################


class CommandNotFoundException(Exception):
    """
    """
    def __init__(self, cmd, paths):
        self.cmd = cmd
        self.paths = paths

    def __str__(self):
        return "Could not locate command: '%s' in this set of paths: %s" % (self.cmd, repr(self.paths))


def findCmdInPath(cmd, additionalPaths = [], printError=True):
    """
    """
    global CMD_CACHE
    if cmd not in CMD_CACHE:
        # Search additional paths and don't add to cache.
        for p in additionalPaths:
            f = os.path.join(p, cmd)
            if os.path.exists(f):
                return f

        for p in CMD_PATH:
            f = os.path.join(p, cmd)
            if os.path.exists(f):
                CMD_CACHE[cmd] = f
                return f

        if cmd == "killall":
            gphome = os.getenv("GPHOME")
            if not gphome:
                gphome = os.path.realpath(os.path.join(os.path.realpath(__file__), "../../../../"))
            gphome = gphome.replace("\\", "\\\\").replace('"', '\\"\\"')
            for rac in ["|", ";", "&", "$", "<", ">", "`", "\\", "'", "\"", "{", "}", "(", ")", "[", "]", "~", "*",
                        "?", " ", "!", "\n"]:
                if rac in gphome:
                    raise Exception(
                        ErrorCode.GAUSS_502["GAUSS_50219"] % gphome + " There are illegal characters in the path.")
            if gphome != "" and os.path.exists(os.path.join(gphome, "script/killall")):
                return os.path.join(gphome, "script/killall")
            else:
                raise Exception(ErrorCode.GAUSS_502["GAUSS_50201"] % "killall")

        if printError:
            print('Command %s not found' % cmd)
        search_path = CMD_PATH[:]
        search_path.extend(additionalPaths)
        raise CommandNotFoundException(cmd, search_path)
    else:
        return CMD_CACHE[cmd]

#Requirements:
#1. ulimit, ntpq, source, kerberos is not found under system path


class GenericPlatform(object):
    """
    manage OS command,config or service for muti-platform
    """
    def __init__(self):
        pass

    def echoCmdWithNoReturn(self, line, filePath):
        cmd = "echo %s >> '%s' 2>/dev/null" % (line, filePath)
        return cmd

    def getCreateFileCmd(self, path):
        cmd = "touch '%s'" % path
        return cmd

    def getMoveFileCmd(self, src, dest):
        cmd = "mv '%s' '%s'" % (src, dest)
        return cmd

    def get_machine_arch_cmd(self):
        return 'uname -i'

    def getDefaultLocale(self):
        return 'en_US.utf-8'

    def getRemoveCmd(self, pathType):
        opts = " "
        if(pathType == "file"):
            opts = " -f "
        elif(pathType == "directory"):
            opts = " -rf "
        return findCmdInPath('rm') + opts

    def getChmodCmd(self, Permission, src, recursive = False):
        return findCmdInPath('chmod') + (" -R " if recursive else BLANK_SPACE) + Permission + BLANK_SPACE + src

    def getChownCmd(self, owner, group, src, recursive = False):
        return findCmdInPath('chown') + (" -R " if recursive else BLANK_SPACE) + owner + COLON + group + BLANK_SPACE + src

    def getCopyCmd(self, src, dest, pathType = ""):
        opts = " "
        if(pathType == "directory"):
            opts = " -r "
        return findCmdInPath('cp') + " -p  -f " + opts + BLANK_SPACE + "'" + src + "'" + BLANK_SPACE + "'" + dest + "'"
    
    def getRemoteCopyCmd(self, src, dest, remoteHost, copyTo = True, pathType = "", otherHost=None):
        opts = ""
        if(pathType == "directory"):
            opts = "-x -r"
        if(copyTo):
            return "pscp %s -H %s %s %s " % (opts.strip() , remoteHost,  src, dest)
        else:
            localhost = self.getLocalIp()
            if (otherHost is not None):
                localhost = otherHost
            return "pssh -s -H %s \" pscp %s -H %s %s %s \" " % (remoteHost, opts.strip(), localhost, src, dest)

    def getHostName(self):
        """
        function : Get host name
        input : NA
        output: string
        """
        hostCmd = findCmdInPath('hostname')
        (status, output) = commands.getstatusoutput(hostCmd)
        #if cmd failed, then exit
        if (status != 0):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50219"] % "host name")
        return output
    
    def getLocalIp(self):
        """
        function: Obtaining the local IP address
        input: NA
        output: NA
        """
        from gspylib.common.Common import DefaultValue
        env_dist = os.environ
        if "HOST_IP" not in env_dist.keys():
            hostName = self.getHostName()
            return hostName
        host_ip = env_dist.get("HOST_IP")
        if (host_ip is not None):
            if (DefaultValue.isIpValid(host_ip)):
                return host_ip

        sockInfo = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sockInfo.connect(('192.168.0.0', 80))
        host_ip = sockInfo.getsockname()[0]
        return host_ip
    
    def getScpCmd(self):
        """
        Get scp cmd for special remotely copy, just like remote to remote or remote to local.
        :return: str
        """
        return "pscp -H "

    def getUseraddCmd(self, user, group):
        return findCmdInPath('useradd') + " -m " + user + " -g " + group
 
    def getUserdelCmd(self, user):
        return findCmdInPath('userdel') + " -r " + user

    def getGroupaddCmd(self, group):
        return findCmdInPath('groupadd') + " " + group

    def getGroupdelCmd(self, group):
        return findCmdInPath('groupdel') + " " + group
    
    def getMoveCmd(self, src, dest):
        return findCmdInPath('mv') + " -f " + "'" + src + "'" + BLANK_SPACE + "'" + dest + "'"
        
    def getMakeDirCmd(self, src, recursive = False):
        return findCmdInPath('mkdir') + (" -p " if recursive else BLANK_SPACE) + "'" + src + "'"

    def getPingCmd(self, host, count, interval, packetSize = 56):
        opts = " "
        if(int(packetSize) != int(56)):
            opts = " -s " + str(packetSize)
        return findCmdInPath('ping') + BLANK_SPACE + host + " -c " + count + " -i " + interval + opts

    def getWcCmd(self):
        return findCmdInPath('wc')

    def getTarCmd(self):
        return findCmdInPath('tar')
    
    def getZipCmd(self):
        return findCmdInPath('zip')
    
    def getUnzipCmd(self):
        return findCmdInPath('unzip')
    
    def getEchoCmd(self, echoString):
        cmdStr = '%s "%s"' % (findCmdInPath('echo'), echoString)
        return cmdStr
  
    def getSedCmd(self):
        return findCmdInPath('sed')

    def getGrepCmd(self):
        return findCmdInPath('grep')
    
    def getLsofCmd(self):
        return findCmdInPath('lsof') + " -i:"
        
    def getIfconfigCmd(self):
        return findCmdInPath('ifconfig')

    def getIpCmd(self):
        return findCmdInPath('ip')

    def getDateCmd(self):
        return findCmdInPath('date')

    def getAwkCmd(self):
        return findCmdInPath('awk')
 
    def getFindCmd(self):
        return findCmdInPath('find')

    def getTouchCmd(self, filename):
        return findCmdInPath('touch') + BLANK_SPACE + filename

    def getListCmd(self):
        return findCmdInPath('ls')

    def getSHA256Cmd(self):
        return findCmdInPath('sha256sum')

    def getProcessCmd(self):
        return findCmdInPath('ps')
    
    def getCatCmd(self):
        return findCmdInPath('cat')

    def getDdCmd(self):
        return findCmdInPath('dd')
    
    def getCdCmd(self, path):
        return 'cd' + BLANK_SPACE + "'" + path + "'"

    def getAllCrontabCmd(self):
        cmd = findCmdInPath('crontab') + BLANK_SPACE + " -l"
        return cmd

    def getCrontabCmd(self):
        return findCmdInPath('crontab')

    def getKillProcessCmd(self, signal, pid):
        return findCmdInPath('kill') + " -" + signal + BLANK_SPACE + pid

    def getKillallCmd(self):
        return findCmdInPath('killall')

    def getKillallProcessCmd(self, signal, username, procName = ""):
        if(procName != ""):
            return findCmdInPath('killall') + " -s " + signal + " -u " + username + BLANK_SPACE + procName
        else:
            return findCmdInPath('killall') + " -s " + signal + " -u " + username

    def getXargsCmd(self):
        return findCmdInPath('xargs')

    def getDeleteSemaphoreCmd(self, user):
        ipcs = findCmdInPath('ipcs')
        ipcrm = findCmdInPath('ipcrm')
        xargs = findCmdInPath('xargs')
        awk = findCmdInPath('awk')
        return "%s -s | %s '/ %s /{print $2}' | %s -n1 %s -s" % (ipcs, awk, user, xargs, ipcrm)

    def getProcessIdByKeyWordsCmd(self, keywords):
        ps = findCmdInPath('ps') 
        grep = findCmdInPath('grep')
        awk = findCmdInPath('awk')
        return "%s -ef| %s -F '%s' | %s -F -v 'grep'| %s '{print $2}'" % (ps, grep, keywords, grep, awk)

    def getSysctlCmd(self):
        return findCmdInPath('sysctl')

    def getServiceCmd(self, serviceName, action):
        return findCmdInPath('service') + BLANK_SPACE + serviceName + BLANK_SPACE + action

    def getSystemctlCmd(self, serviceName, action):
        return findCmdInPath('systemctl') + BLANK_SPACE + action + BLANK_SPACE + serviceName

    def getUlimitCmd(self):
        return 'ulimit'

    def getGetConfValueCmd(self):
        return findCmdInPath('getconf') + " PAGESIZE "

    def getBlockdevCmd(self, device, value = "", isSet = False):
        if (isSet and value != ""):
            return findCmdInPath('blockdev') + " --setra " + value + BLANK_SPACE + device
        else:
            return findCmdInPath('blockdev') + " --getra " + device

    def getSysModManagementCmd(self, OperType, module):
        """
        OperType: list     --list system module
              load     --load system module
              insert   --insert system module by force
              remove   --remove system module
              dep      --generate modules.dep and map files
        """
        if(OperType == "list"):
            return findCmdInPath('lsmod') + BLANK_SPACE + module
        elif(OperType == "load"):
            return findCmdInPath('modprobe') + BLANK_SPACE + module
        elif(OperType == "insert"):
            return findCmdInPath('insmod') + BLANK_SPACE + module
        elif(OperType == "remove"):
            return findCmdInPath('rmmod') + BLANK_SPACE + module
        elif(OperType == "dep"):
            return findCmdInPath('depmod') + BLANK_SPACE + module

    def getMountCmd(self):
        return findCmdInPath('mount')

    def getLocaleCmd(self):
        return findCmdInPath('locale')

    def getPasswordExpiresCmd(self, user):
        return findCmdInPath('chage') + " -l " + user

    def getIOStatCmd(self):
        return findCmdInPath('iostat') + " -xm 2 3 "

    def getEthtoolCmd(self):
        return findCmdInPath('ethtool')

    def getTailCmd(self):
        return findCmdInPath('tail')

    def getSshCmd(self, address):
        return "pssh -s -H " + BLANK_SPACE + address 

    def getChkconfigCmd(self, OperType, service=""):
        if(OperType == "list"):
            return findCmdInPath('chkconfig') + " --list "
        elif(OperType == "delete" and service):
            return findCmdInPath('chkconfig') + " --del " + service

    def getNtpqCmd(self):
        return "/usr/sbin/ntpq -p "

    def getManageKerberosCmd(self, OperType):
        """
        OperType: init      --init kerberos
              destory   --destory kerberos
        """
        if(OperType == "init"):
            return "kinit -k -t "
        elif(OperType == "destory"):
            return 'kdestroy'

    def getManageSSDCmd(self):
        pass

    def getPythonCmd(self):
        return findCmdInPath('python')

    def getShellCmd(self):
        return findCmdInPath('sh')

    def getSourceCmd(self):
        return 'source'

    def getTestCmd(self):
        """
        Linux test cmd
        example: test -f /etc/profile && echo 1 || echo 2
        """
        return findCmdInPath("test")

    def getPgrepCmd(self):
        """
        Linux pgrep cmd
        """
        return findCmdInPath("pgrep")

    def getExportCmd(self, key="", value=""):
        """
        Linux export cmd
        """
        cmd = findCmdInPath("export")
        if key:
            cmd += " %s=%s" % (key, value)
        return cmd


class LinuxPlatform(GenericPlatform):
    """
    manage Linux command,config or service for muti-platform
    """
    def __init__(self):
        pass

    def getCpuInfoFile(self):
        return "/proc/cpuinfo" 

    def getMemInfoFile(self):
        return "/proc/meminfo" 

    def getBondInfoPath(self):
        return "/proc/net/bonding/"

    def getSysctlFile(self):
        return "/etc/sysctl.conf"

    def getMtablFile(self):
        return "/etc/mtab"

    def getInterruptFile(self):
        return "/proc/interrupts"

    def getHostsFile(self):
        return "/etc/hosts"

    def getName(self):
        return "linux"

    def getDefaultLocale(self):
        return 'en_US.utf8'

    def getDiskFreeCmd(self, Mounted = "", inode = False):
        # -P is for POSIX formatting.  Prevents error 
        # on lines that would wrap
        return findCmdInPath('df') + " -Pk " + (" -i " if inode else " -h ") + Mounted

    def getDirSizeCmd(self, path, unit = ""):
        # -s only shows the total size
        # unit specify the output size unit
        return findCmdInPath('du') + " -s " + (" -B %s " % unit if unit else " -h ") + path

    def getSadcCmd(self, interval, outFileName):
        cmd = "/usr/lib64/sa/sadc -F -d " + str(interval) + " " + outFileName
        return cmd

    def getCompressFilesCmd(self, tarName, fileSrc):
        cmd = "%s -zvcf '%s' %s" % (self.getTarCmd(), tarName, fileSrc)
        return cmd

    def getDecompressFilesCmd(self, srcPackage, dest):
        cmd = "%s -zxvf '%s' -C '%s'" % (self.getTarCmd(), srcPackage, dest)
        return cmd

    def getCompressZipFilesCmd(self, zipName, fileSrc):
        cmd = "cd %s && %s -r '%s.zip' ./*" % (fileSrc, self.getZipCmd(), zipName)
        return cmd

    def getDecompressZipFilesCmd(self, srcPackage, dest):
        cmd = "%s -o '%s' -d '%s'" % (self.getUnzipCmd(), srcPackage, dest)
        return cmd

    def getReplaceFileLineContentCmd(self, oldLine, newLine, path):
        cmd = "%s -i \"s/%s/%s/g\" '%s'" % (self.getSedCmd(), oldLine, newLine, path)
        return cmd

    def getDirPermissionCmd(self, dirPath):
        cmd = "%s -ld '%s' | %s -F\" \" '{print $1}' " % (self.getListCmd(), dirPath, self.getAwkCmd())
        return cmd

    def getFileSHA256Cmd(self, fileName):
        cmd = "%s '%s' | %s -F\" \" '{print $1}' " % (self.getSHA256Cmd(), fileName, self.getAwkCmd())
        return cmd

    def getExecuteCmdWithUserProfile(self, user, userProfile, executeCmd, ignoreError=True):
        """
        """
        if ((user != "") and (os.getuid() == 0)):
            cmd = "su - %s -c '%s %s; %s'" % (user, self.getSourceCmd(), userProfile, executeCmd)
        else:
            cmd = "%s %s; %s" % (self.getSourceCmd(), userProfile, executeCmd)
        if ignoreError:
            cmd += " 2>/dev/null"
        return cmd

    def getUserHomePath(self):
        """
        """
        # converts the relative path to an absolute path
        cmd = "echo ~ 2>/dev/null"
        (status, output) = commands.getstatusoutput(cmd)
        if(status != 0):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50219"] % "user home")
        return output

    def checkProcExists(self, procName):
        """
        """
        for pid in psutil.pids():
            p = psutil.Process(pid)
            if (procName == p.name()):
                return True
        return False

    def checkProcAlive(self, procPid):
        """
        """
        try:
            os.kill(procPid, 0)
        except OSError:
            return False
        else:
            return True

    def getIpAddressAndNIC(self, ipType = "ipv4"):
        """
        """
        if (ipType == "ipv4"):
            key = AF_INET
        else:
            key = AF_INET6

        for iface in interfaces():
            if (key in ifaddresses(iface)):
                ipAddress = ifaddresses(iface)[key][0]['addr']
                yield(iface, ipAddress)

    def getIpAddressAndNICList(self, ipType = "ipv4"):
        """
        """
        return list(self.getIpAddressAndNIC(ipType))

    def getNetworkNumByIPAddr(self, ipAddress, ipType = "ipv4"):
        """
        """
        try:
            mappingList = self.getIpAddressAndNICList(ipType)
            for mapInfo in mappingList:
                if (mapInfo[1] == ipAddress):
                    return mapInfo[0]
            raise Exception(ErrorCode.GAUSS_506["GAUSS-50612"] % ipAddress)
        except Exception, e:
            raise Exception(str(e))

    def getHostNameByIPAddr(self, ipAddress):
        """
        """
        try:
            return socket.gethostbyaddr(ipAddress)[0]
        except Exception, e:
            raise Exception(str(e))
        
    def getLinuxNetworkConfigFile(self, networkConfPath, networkCardNum):
        """
        """
        try:
            networkConfFile = "%sifcfg-%s" % (networkConfPath, networkCardNum)
            #Network configuration file does not exist 
            if (not os.path.exists(networkConfFile)):
                cmd = "%s %s -iname 'ifcfg-*-%s' -print" % (self.getFindCmd(), networkConfFile, networkCardNum)
                (status, output) = commands.getstatusoutput(cmd)
                if (status != 0 or output.strip() == "" or len(output.split('\n')) != 1):
                    raise Exception(ErrorCode.GAUSS_502["GAUSS-50201"] % networkConfFile)
                networkConfFile = output.strip()
            return networkConfFile
        except Exception, e:
            raise Exception(str(e))

    def getNetworkBondModeByBondConfigFile(self, bondingConfFile):
        """
        """
        try:
            #Check the bond mode
            cmd = "%s -w '\<Bonding Mode\>' %s | %s  -F ':' '{print $NF}'" % (self.getGrepCmd(), bondingConfFile, self.getAwkCmd())
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)
            return "BondMode %s" % output.strip()
        except Exception, e:
            raise Exception(str(e))

    def getNetworkBondModeInfo(self, networkConfFile, networkCardNum):
        """
        """
        #Get the bond profile
        if (not os.path.isfile(networkConfFile)):
            return "BondMode Null"

        bondingConfFile = "%s%s" % (self.getBondInfoPath(), networkCardNum)
        cmd = "%s -i 'BONDING_OPTS\|BONDING_MODULE_OPTS' %s" % (self.getGrepCmd(), networkConfFile)
        (status, output) = commands.getstatusoutput(cmd)
        #Analysis results
        if ((output.strip() != "")):
            if ((output.find("mode") > 0) and os.path.exists(bondingConfFile)):
                bondInfo = self.getNetworkBondModeByBondConfigFile(bondingConfFile)
            else:
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)
        elif (os.path.exists(bondingConfFile)):
            bondInfo = self.getNetworkBondModeByBondConfigFile(bondingConfFile)
            bondInfo += "\nNo 'BONDING_OPTS' or 'BONDING_MODULE_OPTS' in bond config file[%s]." % networkConfFile
        else:
            bondInfo = "BondMode Null"
        return bondInfo

    def getNetworkMaskByNICNum(self, networkCardNum, ipType = "ipv4"):
        """
        """
        if (ipType == "ipv4"):
            return ifaddresses(networkCardNum)[AF_INET][0]["netmask"]
        else:
            return ifaddresses(networkCardNum)[AF_INET6][0]["netmask"]

    def getNetworkMTUValueByNICNum(self, networkCardNum):
        """
        """
        return psutil.net_if_stats()[networkCardNum].mtu

    def getNetworkRXTXValueByNICNum(self, networkCardNum, valueType):
        """
        """
        try:
            cmd = "%s -g %s | %s '%s:' | %s -n 1" % (self.getEthtoolCmd(), networkCardNum, self.getGrepCmd(), valueType.upper(), self.getTailCmd())
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)
            value = output.split(':')[-1].split(' ')[0].strip()
            if (not str(value).isdigit()):
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)
            return int(value)
        except Exception, e:
            raise Exception(str(e))

    def setNetworkRXTXValue(self, networkCardNum, rxValue = 8192, txValue = 8192):
        """
        """
        try:
            cmd = "%s -G %s rx %s tx %s" % (self.getEthtoolCmd(), networkCardNum, rxValue, txValue)
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                if (output.find("no ring parameters changed, aborting") < 0):
                    raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)
        except Exception, e:
            raise Exception(str(e))

    def getNetworkSpeedByNICNum(self, networkCardNum):
        """
        """
        keyWord = "Speed: "
        speedUnit = "Mb/s"
        try:
            cmd = "%s %s | grep '%s'" % (self.getEthtoolCmd(), networkCardNum, keyWord)
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0 or output == ""):
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)
            if (len(output.split('\n')) >= 1):
                for line in output.split('\n'):
                    if (line.find(keyWord) >= 0 and line.find(speedUnit) >= 0):
                        return int(line.split(':')[-1].strip()[:-4])
            return 0
        except Exception, e:
            raise Exception(str(e))

    def checkNetworkInterruptByNIC(self, networkCardNum):
        """
        """
        try:
            interruptConfFile = self.getInterruptFile()
            numberedListCmd = "%s %s | %s '%s-' | %s -F ' ' '{print $1}' | %s -F ':' '{print $1}'" % (
                self.getCatCmd(), interruptConfFile, self.getGrepCmd(), networkCardNum, self.getAwkCmd(), self.getAwkCmd())
            irqCmd = "%s /proc/irq/$i/smp_affinity" % self.getCatCmd()
            cmd = "for i in `%s`; do %s ; done" % (numberedListCmd, irqCmd)
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)
        except Exception, e:
            raise Exception(str(e))

        #cpu core number followed by 1 2 4 8,every 4 left shift one
        Mapping = {0: "1", 1: "2", 2: "4", 3: "8"}
        flag = True
        for index, eachLine in enumerate(output.split()):
            #Remove the ','
            eachLine = eachLine.replace(",", "")
            #Replace 0000,00001000 to 1,Remove invalid content
            validValue = eachLine.replace("0", "")
            #Convert the row index to the expected value
            expandNum = Mapping[index % 4]
            #Convert line index to expected position
            expandBit = index / 4 * -1 - 1
            #value and position is correct
            if (eachLine[expandBit] == expandNum and validValue == expandNum):
                continue
            else:
                print("Network card [%s] multi-queue support is not enabled.\n" % networkCardNum)
                flag = False
                break
        return flag

    def getInterruptCountNum(self, networkCardNum):
        """
        function : We can makesure that all dev names is startwith 'ethX-' and endwith '-X'
        input  : String
        output : Int
        """
        try:
            interruptConfFile = self.getInterruptFile()
            cmd = "%s %s | %s '%s-' | %s -l" % (self.getCatCmd(), interruptConfFile, self.getGrepCmd(), networkCardNum, self.getWcCmd())
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0):
                raise Exception("Failed to obtain network card interrupt count numbers. \
Commands for getting interrupt count numbers: %s." % cmd)
            if (not str(output.strip()).isdigit()):
                return 0
            return int(output.strip())
        except Exception, e:
            raise Exception(str(e))

    def setNetworkInterruptByNIC(self, networkCardNum):
        """
        """
        # check process irqbalance
        procName = "irqbalance"
        if self.checkProcExists(procName):
            print("        Warning: irqbalance is running and will likely override this script's affinitization. \
Please stop the irqbalance service and/or execute 'killall irqbalance'.")
            killCmd = "%s %s" % (self.getKillallCmd(), procName)
            # we don't care about the return value
            os.system(killCmd)

        count = int(self.getInterruptCountNum(networkCardNum))
        i = 0
        while (i < count):
            # the dev name type like this: eth1-1, eth1-rx-1, eth1-tx-1, eth1-TxRx-1
            interruptConfFile = self.getInterruptFile()
            numberedListCmd = "%s %s | %s '%s.*-%s$' | %s -F ' ' '{print $1}' | %s -F ':' '{print $1}'" % (
                self.getCatCmd(), interruptConfFile, self.getGrepCmd(), networkCardNum, str(i), self.getAwkCmd(), self.getAwkCmd())
            (status, output) = commands.getstatusoutput(numberedListCmd)
            if (status != 0 or output.strip() == ""):
                print("Failed to obtain network card interrupt value. \
Commands for getting interrupt value: %s." % numberedListCmd)
            else:
                IRQ = output.strip()
                print("The network '%s' interrupt configuration path: /proc/irq/%s/smp_affinity." % (networkCardNum, IRQ))

                cmdTrans = "MASK=$((1<<%s)); echo $MASK" % i
                (status, output) = commands.getstatusoutput(cmdTrans)
                if (status != 0 or not str(output.strip()).isdigit()):
                    print("Failed to change decit to binary. Commands for translating: %s." % cmdTrans)
                else:
                    # Under SuSE platform, when the length is greater than 8, the ',' must be used.
                    value = str(hex(int(output.strip())))[2:]
                    if (len(value) <= 8):
                        resultValue = value
                    else:
                        resultValue = value[:-8] + ',' + value[len(value) - 8:]

                    cmdSet = "%s > /proc/irq/%s/smp_affinity" % (self.getEchoCmd(resultValue), IRQ) 
                    (status, output) = commands.getstatusoutput(cmdSet)
                    if (status != 0):
                        print("Failed to set network '%s' IRQ. Commands for setting: %s." % (networkCardNum, cmdSet))
                    else:
                        print("Set network card '%s' IRQ to \"%s\"." % (networkCardNum, resultValue))
            i += 1

    def getPackageFile(self, distName, version, packageVersion, productVersion, fileType = "tarFile"):
        """
        function : Get the path of binary file version.
        input : NA
        output : String
        """
        distname, version, idnum = platform.dist()
        distname = distname.lower()
        machineType = platform.machine()
        if (machineType == "aarch64"):
            redString = "REDHAT"
        else:
            redString = "REDHAT"
        dirName = os.path.dirname(os.path.realpath(__file__))
        prefixStr = productVersion
        if (fileType == "tarFile"):
            postfixStr = "tar.gz"
        elif (fileType == "binFile"):
            postfixStr = "bin"
        elif (fileType == "sha256File"):
            postfixStr = "sha256"
        else:
            raise Exception("The value of fileType is incorrect.")

        # RHEL and CentOS have the same kernel version, 
        # So RHEL cluster package can run directly on CentOS.
        if (distname in (REDHAT, ORACLE)):
            fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, redString, BIT_VERSION, postfixStr))
        elif (distname in CENTOS):
            if os.path.isfile(os.path.join("/etc", "euleros-release")):
                fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, "EULER", BIT_VERSION, postfixStr))
                if not os.path.isfile(fileName):
                    fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, redString, BIT_VERSION, postfixStr))
            else:
                fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, redString, BIT_VERSION, postfixStr))
        elif (distName in KYLIN):
            if (version[0:3] == "4.0"):
                # YinHe Kylin use EULER package.
                fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, "EULER", BIT_VERSION, postfixStr))
            else:
                # ZhongBiao Kylin use Redhat package.
                fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, "REDHAT", BIT_VERSION, postfixStr))
        elif (distname == SUSE and version.split('.')[0] in ("11", "12")):
            fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, "SUSE11", BIT_VERSION, postfixStr))
        elif (distname in (EULEROS) and (idnum in ["SP2", "SP3", "SP5"])):
            fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, "EULER", BIT_VERSION, postfixStr))
            if not os.path.isfile(fileName):
                fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, redString, BIT_VERSION, postfixStr))
        elif (distname in (EULEROS) and (idnum == "SP8")):
            fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, "EULER", BIT_VERSION, postfixStr))
        elif (distname in (EULEROS)):
            fileName = os.path.join(dirName, "./../../../", "%s-%s-%s-%s.%s" % (prefixStr, packageVersion, redString, BIT_VERSION, postfixStr))
        else:
            raise Exception(ErrorCode.GAUSS_519["GAUSS_51900"] + "Supported platforms are: %s." % str(SUPPORT_WHOLE_PLATFORM_LIST))

        fileName = os.path.normpath(fileName)
        if (not os.path.exists(fileName)):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50201"] % fileName)
        if (not os.path.isfile(fileName)):
            raise Exception(ErrorCode.GAUSS_502["GAUSS_50210"] % fileName)
        return fileName

    def setKeyValueInSshd(self, key, value):
        """
        function: Set a (key, value) pair into /etc/ssh/sshd_config, before "Match" section.
                "Match" section in sshd_config should always places in the end.
                Attention: you need to remove the old (key, value) from sshd_config manually.
        input:
            key: the configuration name of sshd_config
            value: the configuration value(Only single line string permitted here).
        output:
            void
        """
        sshd_config = '/etc/ssh/sshd_config'
        cmd = "grep -E '^\<Match\>' %s" % sshd_config
        (status, output) = commands.getstatusoutput(cmd)

        if status == 0:
            cmd = "sed -i '/^\<Match\>.*/i %s %s' %s" % (key, value, sshd_config)
        else:
            if output is not None and len(output.strip()) != 0:
                raise Exception(ErrorCode.GAUSS_503["GAUSS_50321"] % "Match section" + "Command: %s, Error: %s" % (cmd, output))
            cmd = "echo '' >> %s ; echo '%s %s' >> %s" % (sshd_config, key, value, sshd_config)

        (status, output) = commands.getstatusoutput(cmd)
        if status != 0:
            raise Exception((ErrorCode.GAUSS_503["GAUSS_50320"] % (key, value)) + ("Command: %s, Error: %s" % (cmd, output)))

class SLESPlatform(LinuxPlatform):
    """
    manage SUSE Linux Enterprise Server command,config or service for muti-platform
    """

    def __init__(self):
        self.NetWorkConfPath = "/etc/sysconfig/network/"
        self.SuSEReleaseFile = "/etc/SuSE-release"
        self.OSReleaseFile = "/etc/SuSE-release"

    def isPlatFormEulerOSOrRHEL7X(self):
        return False

    def getManageFirewallCmd(self, action):
        return findCmdInPath('SuSEfirewall2') + BLANK_SPACE + action

    def getLinuxFirewallStatus(self):
        """
        """
        try:
            cmd = self.getManageFirewallCmd("status")
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0 or output == ""):
                raise Exception("Failed to execute command %s. " % cmd + " Error: \n%s " % output)

            if (output.strip().find("SuSEfirewall2 not active") > 0):
                firewallStatus = "disabled"
            else:
                firewallStatus = "enabled"
            return firewallStatus
        except Exception, e:
            raise Exception(str(e))

    def getManageCrondCmd(self, action):
        return self.getServiceCmd("cron", action)

    def getManageSshdCmd(self, action):
        return self.getServiceCmd("sshd", action)

    def getManageSyslogCmd(self, action):
        return self.getServiceCmd("syslog", action)

    def getManageRsyslogCmd(self, action):
        return self.getServiceCmd("rsyslog", action)

    def getManageSystemdJournaldCmd(self, action):
        return self.getServiceCmd("systemd-journald", action)

    def getManageGsOsServerCmd(self, action):
        try:
            raise Exception("gs-OS-set service only can be supported on RHEL/CentOS Platform. ") 
        except Exception, e:
            raise Exception(str(e))

    def getCurrentPlatForm(self):
        """
        """
        try:
            distName, version, currentId = platform.dist()
            bits, linkage = platform.architecture()
            if (distName.lower() != SUSE or version not in SUPPORT_SUSE_VERSION_LIST):
                raise Exception("Platform %s%s is not supported. " % (distName.lower(), version))

            # os-release is added since SLE 12; SuSE-release will be removed in a future service pack or release
            if os.path.exists(self.SuSEReleaseFile):
                cmd = "%s -i 'PATCHLEVEL' %s  | %s -F '=' '{print $2}'" % (self.getGrepCmd(), self.SuSEReleaseFile, self.getAwkCmd())
            else:
                cmd = "%s -i 'VERSION_ID' %s  | %s -F '.' '{print $2}' | %s 's/\"//'" % (self.getGrepCmd(), self.OSReleaseFile, self.getAwkCmd(), self.getSedCmd())
            (status, output) = commands.getstatusoutput(cmd)
            if (status == 0 and output != ""):
                patchlevel = output.strip()
            else:
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)

            if (bits == BIT_VERSION and 
                ((version == SUSE11 and patchlevel in SUPPORT_SUSE11X_VERSION_LIST) or 
                 (version == SUSE12 and patchlevel in SUPPORT_RHEL12X_VERSION_LIST))):
                platformVersion = "%s.%s" % (version, patchlevel)
                return distName.lower(), platformVersion
            else:
                raise Exception(ErrorCode.GAUSS_519["GAUSS_51900"] + " The current system is: %s%s.%s" % (distName.lower(), version, patchlevel))
        except Exception, e:
            raise Exception(str(e))

    def getNetworkConfigFileByNICNum(self, networkCardNum):
        """
        """
        return self.getLinuxNetworkConfigFile(self.NetWorkConfPath, networkCardNum)

    def getNetworkConfigFileByIPAddr(self, ipAddress):
        """
        """
        networkCardNum = self.getNetworkNumByIPAddr(ipAddress)
        return self.getNetworkConfigFileByNICNum(networkCardNum)


class RHELPlatform(LinuxPlatform):
    """
    manage Red Hat Enterprise Linux command,config or service for muti-platform
    """
    def __init__(self):
        self.NetWorkConfPath = "/etc/sysconfig/network-scripts/"

    def isSupportSystemctl(self):
        distName, version, currentId = platform.dist()
        if ((distName.lower() == EULEROS and version[0:3] in SUPPORT_EULEROS_VERSION_LIST) or
                (distName.lower() in SUPPORT_RHEL_SERIES_PLATFORM_LIST and version[0:3] in SUPPORT_RHEL7X_VERSION_LIST) or
                ((distName.lower() == ORACLE and version[0:3] in SUPPORT_ORACLE_VERSION_LIST) or
                (distName.lower() == KYLIN and version[0:3] in SUPPORT_KYLIN_VERSION_LIST)) or
                (distName.lower() == CENTOS and version[0:3] in SUPPORT_EULEROS_VERSION_LIST and os.path.isfile(os.path.join("/etc", "euleros-release")))):
            return True
        else:
            return False

    def isPlatFormEulerOSOrRHEL7X(self):
        """
        """
        return self.isSupportSystemctl()

    def getManageFirewallCmd(self, action):
        if self.isSupportSystemctl():
            return self.getSystemctlCmd("firewalld.service", action)
        else:
            return self.getServiceCmd("iptables", action)

    def getLinuxFirewallStatus(self):
        """
        """
        try:
            cmd = self.getManageFirewallCmd("status")
            (status, output) = commands.getstatusoutput(cmd)
            if (status != 0 and output == ""):
                raise Exception(ErrorCode.GAUSS_514["GAUSS_51400"] % cmd + " Error: \n%s " % output)

            if self.isSupportSystemctl():
                if (output.strip().find("Active: active (running)") > 0):
                    firewallStatus = "enabled"
                else:
                    firewallStatus = "disabled"
            else:
                if (output.strip().find("Firewall is not running") > 0):
                    firewallStatus = "disabled"
                else:
                    firewallStatus = "enabled"
            return firewallStatus
        except Exception, e:
            raise Exception(str(e))

    def getManageCrondCmd(self, action):
        if self.isSupportSystemctl():
            return self.getSystemctlCmd("crond.service", action)
        else:
            return self.getServiceCmd("crond", action)

    def getManageSshdCmd(self, action):
        if self.isSupportSystemctl():
            return self.getSystemctlCmd("sshd.service", action)
        else:
            return self.getServiceCmd("sshd", action)

    def getManageGsOsServerCmd(self, action):
        if self.isSupportSystemctl():
            return self.getSystemctlCmd("gs-OS-set.service", action)
        else:
            return self.getServiceCmd("gs-OS-set", action)

    def getManageSyslogCmd(self, action):
        try:
            raise Exception("Syslog service only can be supported on SuSE Platform. ") 
        except Exception, e:
            raise Exception(str(e))

    def getManageRsyslogCmd(self, action):
        try:
            raise Exception("Rsyslog service only can be supported on SuSE Platform. ") 
        except Exception, e:
            raise Exception(str(e))

    def getManageSystemdJournaldCmd(self, action):
        try:
            raise Exception("systemd-journald service only can be supported on SuSE Platform. ") 
        except Exception, e:
            raise Exception(str(e))

    def getCurrentPlatForm(self):
        """
        """
        try:
            distName, version, currentId = platform.dist()
            bits, linkage = platform.architecture()
            if ((bits == BIT_VERSION and 
                ((distName.lower() == EULEROS and version[0:3] in SUPPORT_EULEROS_VERSION_LIST) or
                 (distName.lower() in SUPPORT_RHEL_SERIES_PLATFORM_LIST and version[0:3] in SUPPORT_RHEL_SERIES_VERSION_LIST) or
                 (distName.lower() == ORACLE and version[0:3] in SUPPORT_ORACLE_VERSION_LIST)) or
                 (distName.lower() == KYLIN and version[0:3] in SUPPORT_KYLIN_VERSION_LIST)
                )):
                return (distName.lower(), version[0:3])
            else:
                if distName.lower() == CENTOS and os.path.isfile(os.path.join("/etc", "euleros-release")) and (version[0:3] in SUPPORT_EULEROS_VERSION_LIST):
                    return (EULEROS, version[0:3])
                if (distName.lower() == EULEROS):
                    raise Exception(ErrorCode.GAUSS_519["GAUSS_51900"] + " The current system is: %s%s%s" % (distName.lower(), version[0:3], currentId))
                else:
                    raise Exception(ErrorCode.GAUSS_519["GAUSS_51900"] + " The current system is: %s%s" % (distName.lower(), version[0:3]))
        except Exception, e:
            raise Exception(str(e))

    def getNetworkConfigFileByIPAddr(self, ipAddress):
        """
        """
        networkCardNum = self.getNetworkNumByIPAddr(ipAddress)
        return self.getLinuxNetworkConfigFile(self.NetWorkConfPath, networkCardNum)

    def getNetworkConfigFileByNICNum(self, networkCardNum):
        """
        """
        return self.getLinuxNetworkConfigFile(self.NetWorkConfPath, networkCardNum)


class UserPlatform():
    """
    manage Red Hat Enterprise Linux command,config or service for muti-platform
    """
    def __init__(self):
        """
        function : Check support OS version and init OS class
        """
        # now we support this platform:
        #     RHEL/CentOS     "6.4", "6.5", "6.6", "6.7", "6.8", "6.9", "7.0", "7.1", "7.2", "7.3", "7.4", "7.5 "64bit
        #     EulerOS         "2.0", "2.3" 64bit
        #     SuSE11          sp1/2/3/4 64bit
        #     SuSE12          sp0/1/2/3 64bit
        distName, version, idNum = platform.dist()
        if (distName.lower() not in SUPPORT_WHOLE_PLATFORM_LIST):
            raise Exception(ErrorCode.GAUSS_519["GAUSS_51900"] + "Supported platforms are: %s." % str(SUPPORT_WHOLE_PLATFORM_LIST))

        if (distName.lower() == SUSE):
            #SuSE11.X SUSE12.X
            self.userPlatform = SLESPlatform()
        elif (distName.lower() in SUPPORT_RHEL_SERIES_PLATFORM_LIST):
            #RHEL6.X RHEL7.X
            self.userPlatform = RHELPlatform()
        else:
            #EULEROS 2.0/2.3
            self.userPlatform = RHELPlatform()
        try: 
            self.userPlatform.getCurrentPlatForm()
        except Exception as e:
            raise Exception(str(e))

# global platform class
g_Platform = UserPlatform().userPlatform
